goog.provide('game.logic.testForMysterySymbol');


(function() {
	"use strict";

	var testForMysterySymbol = function( platformInterface, config, betResults, slotGame ) {

		var isWinningSpin = betResults[ 'win' ].greaterThan( 0 ) || ( betResults['bonus'] && betResults['bonus'].length > 0 );

		// winning game is a 100% chance of getting a mystery symbol.
		// losing games has a 31% chance in main game, 62% in free spins.
		var mysterySymbolChance = platformInterface.getRandomNumber( 'IA-MysterySymbol-DisplayChance', 0, 9999 );
		if ( !isWinningSpin ) {
			if ( config[ 'betSelection' ][ 'checkpointId' ] === 0 ) {
				if ( mysterySymbolChance >= 3100 ) {
					return;
				}
			} else {
				if ( mysterySymbolChance >= 6200 ) {
					return;
				}
			}
		}

		var buildPotentialClumpList = function() {

			var reelIndex, rowIndex;
			var observedCells = [];
			for ( reelIndex=0; reelIndex<game.logic.GameData.reels.nreels; ++reelIndex ) {
				var reelArray = [];
				for ( rowIndex=0; rowIndex<game.logic.GameData.reels.nrows; ++rowIndex ) {
					reelArray.push( 0 );
				}
				observedCells.push( reelArray );
			}

			var checkNeighbouringSymbols = function( reel, row, clumpData ) {
				observedCells[ reel ][ row ] = 1;
				var symbol = slotGame.screen_symbols[ reel ][ row ];
				clumpData = clumpData || { 'positions': [ { 'reel': reel, 'row': row } ], 'symbol': symbol };

				var neighboursToTest = [];

				if ( reel-1 >= 0 ) {
					neighboursToTest.push( { reel: reel-1, row: row } );
				}
				if ( reel+1 < game.logic.GameData.reels.nreels ) {
					neighboursToTest.push( { reel: reel+1, row: row } );
				}
				if ( row-1 >= 0 ) {
					neighboursToTest.push( { reel: reel, row: row-1 } );
				}
				if ( row+1 < game.logic.GameData.reels.nrows ) {
					neighboursToTest.push( { reel: reel, row: row+1 } );
				}

				neighboursToTest.forEach( function( neighbour ) {
					var neighbourSymbol = slotGame.screen_symbols[ neighbour.reel ][ neighbour.row ];
					if( neighbourSymbol === symbol && observedCells[ neighbour.reel ][ neighbour.row ] === 0 ) {
						// found neighbouring symbol same type as previous, add it to clump data
						clumpData['positions'].push( { 'reel': neighbour.reel, 'row': neighbour.row } );
						// then check that symbol for neighbours too
						checkNeighbouringSymbols( neighbour.reel, neighbour.row, clumpData );
					}
				} );

				return clumpData;
			};

			// search reel window for possible symbol clumps.
			var potentialClumpsList = [];
			for ( reelIndex=0; reelIndex<game.logic.GameData.reels.nreels; ++reelIndex ) {
				for ( rowIndex=0; rowIndex<game.logic.GameData.reels.nrows; ++rowIndex ) {

					if ( observedCells[ reelIndex ][ rowIndex ] === 0 ) {
						var clumpData = checkNeighbouringSymbols( reelIndex, rowIndex );
						potentialClumpsList.push( clumpData );
					}
				}
			}
			return potentialClumpsList;
		};


		var combineWildClumps = function( potentialClumpsList ) {
			// Now we have a list of possible symbol clumps (including those that are only 1 symbol big)
			// Next group together any wild clumps that appear within the first 3 reels.
			// This'll mean multiple wild clumps will animate at the same time in the game instead of separately, and will include single wild symbols
			var combinedClump = { 'positions': [], 'symbol': 'WI' };
			potentialClumpsList.forEach( function( clump ) {
				if ( clump['symbol'] === 'WI' ) {
					var clumpInFirst3Reels = clump['positions'].some( function( pos ) { return pos['reel'] < 3; } );
					if ( clumpInFirst3Reels ) {
						combinedClump['positions'] = combinedClump['positions'].concat( clump['positions'] );
						clump['positions'].length = 0;
					}
				}
			} );

			// remove any empty clumps
			potentialClumpsList = potentialClumpsList.filter( function( clump ) {
				return clump['positions'].length > 0;
			} );

			if ( combinedClump['positions'].length > 0 ) {
				potentialClumpsList.unshift( combinedClump );
			}
		};


		var removeSingleSymbolsOnReelClumps = function( potentialClumpsList ) {

			// remove any clump symbols that do not have a consecutive symbol above or below it on the reel
			potentialClumpsList.forEach( function( clump ) {
				clump['positions'] = clump['positions'].filter( function( pos ) {
					if ( pos['row'] - 1 >= 0 ) {
						var aboveSymbol = slotGame.screen_symbols[ pos['reel'] ][ pos['row'] - 1 ];
						if ( aboveSymbol === clump['symbol'] ) {
							return true;
						}
					}
					if ( pos['row'] + 1 < game.logic.GameData.reels.nrows ) {
						var belowSymbol = slotGame.screen_symbols[ pos['reel'] ][ pos['row'] + 1 ];
						if ( belowSymbol === clump['symbol'] ) {
							return true;
						}
					}
					return false;
				} );
			} );
		};


		var discardInvalidClumps = function( potentialClumpsList, isWinningSpin ) {

			var isValidClump = function( clump ) {
				// discard those less than 2 symbols big
				if ( clump['positions'].length < 2 ) {
					return false;
				}

				// discard if any clump symbols appear above or below a freespins symbol
				var nearFreeSpinsSymbols = clump['positions'].some( function( pos ) {
					var symbol;
					var aboveIndex = pos['row'] - 1;
					var belowIndex = pos['row'] + 1;
					if ( aboveIndex >= 0 ) {
						symbol = slotGame.screen_symbols[ pos['reel'] ][ aboveIndex ];
						if ( symbol === 'FS' ) {
							return true;
						}
					}
					if ( belowIndex < game.logic.GameData.reels.nrows ) {
						symbol = slotGame.screen_symbols[ pos['reel'] ][ belowIndex ];
						if ( symbol === 'FS' ) {
							return true;
						}
					}
					return false;
				} );

				if ( nearFreeSpinsSymbols ) {
					return false;
				}

				// discard if in freespins and a single reel has both wild symbols and one of the clump symbols (as in freespins the wild symbols will be stacked)
				if ( config[ 'betSelection' ][ 'checkpointId' ] > 0 ) {
					var clumpOnWildStackReel = clump['positions'].some( function( pos ) {

						for ( var rowIndex=0; rowIndex< slotGame.screen_symbols[ pos['reel'] ].length; ++rowIndex ) {
							var symbol = slotGame.screen_symbols[ pos['reel'] ][ rowIndex ];
							if ( symbol === 'WI' ) {
								return true;
							}
						}
						return false;
					} );
					if ( clumpOnWildStackReel ) {
						return false;
					}
				}

				if ( isWinningSpin ) {
					// if it's a winning spin, make sure there are at least 2 winlines containing the clump symbol
					var winCount = 0;
					var contains2Winlines = betResults[ 'reels' ][ 'winline' ].some( function( win ) {

						var containsClump = clump['positions'].some( function( pos ) {

							var lineDef = game.logic.GameData.reels.winLineDefinitionArray[ win[ 'line' ] ];
							for ( var reel=0; reel<win.run; ++reel ) {
								var row = lineDef[ reel ];
								if ( pos['row'] === row && pos['reel'] === reel ) {
									return true;
								}
							}
							return false;
						} );
						if ( containsClump ) {
							winCount++;
						}
						return winCount >= 2;
					} );
					if ( !contains2Winlines ) {
						return false;
					}
				} else {
					// if it's a losing spin, count the number of clumped symbols that occur within the first 3 reels.
					// Only allow clumps with 2 or more symbols in the first 3 reels OR if there are at least 2 wild symbols in each of 2 rows of the first 3 reels
					var symbolCount = 0;
					for ( var reelIndex=0; reelIndex<3; ++reelIndex ) {
						var wildCount = 0;
						for ( var rowIndex=0; rowIndex<game.logic.GameData.reels.nrows; ++rowIndex ) {

							var containsClump = clump['positions'].some( function( pos ) {
								return pos['reel'] === reelIndex && pos['row'] === rowIndex;
							} );

							var isWild = slotGame.screen_symbols[ reelIndex ][ rowIndex ] === 'WI';
							if ( isWild ) {
								wildCount++;
							}

							if ( containsClump || wildCount >= 2 ) {
								symbolCount++;
								break;
							}
						}
					}
					if ( symbolCount < 2 ) {
						return false;
					}
				}

				return true;
			};

			var newClumpList = potentialClumpsList.filter( isValidClump );
			var clumpLimit = 5;

			// Limit number of clumps (very rare to have it above this number)
			while ( newClumpList.length > clumpLimit ) {
				newClumpList.pop();
			}

			// Those that are a ruby symbol or less need to pass a 50/50 chance of being accepted.
			var passChances = platformInterface.getRandomNumber( 'IA-MysterySymbol-ClumpChance', 0, 1, clumpLimit );
			var passIndex = 0;
			newClumpList = newClumpList.filter( function( clump ) {
				var chance = passChances[ passIndex ];
				passIndex++;
				if ( clump['symbol'] === 'R' || clump['symbol'] === 'E' || clump['symbol'] === 'S' ) {
					if ( chance === 0 ) {
						return false;
					}
				}
				return true;
			} );

			return newClumpList;
		};

		var potentialClumpsList = buildPotentialClumpList();
		combineWildClumps( potentialClumpsList );
		removeSingleSymbolsOnReelClumps( potentialClumpsList );
		var finalClumpList = discardInvalidClumps( potentialClumpsList, isWinningSpin );

		if ( finalClumpList.length > 0 ) {
			// All clumps must reveal the same symbol type. Remove any clumps that are not equal to the first symbol
			var firstClump = finalClumpList[0];
			finalClumpList = finalClumpList.filter( function( clump ) {
				return clump['symbol'] === firstClump['symbol'];
			} );

			var forcedSymbolsArray = [];
			var formattedClumpArray = [];
			var clumpChange = [];
			for ( var ci=0; ci<finalClumpList.length; ++ci ) {
				for ( var pi=0; pi<finalClumpList[ci]['positions'].length; ++pi ) {
					clumpChange.push( { 'reel': finalClumpList[ci]['positions'][ pi ]['reel'], 'row': finalClumpList[ci]['positions'][ pi ]['row'], 'symbol': finalClumpList[ci]['symbol'] } );
					forcedSymbolsArray.push( { 'reel': finalClumpList[ci]['positions'][ pi ]['reel'], 'row': finalClumpList[ci]['positions'][ pi ]['row'], 'symbol': 'MS' } );
				}
			}
			formattedClumpArray.push( clumpChange );
			if ( !betResults['bonus'] ) {
				betResults['bonus'] = [];
			}
			betResults['bonus'].push( { 'name':"hiddenWilds", 'win': new BigNumber( 0 ), 'changes': formattedClumpArray, 'isWinningSpin': ( isWinningSpin ? 1 : 0 ) } );

			betResults['reels']['force_symbols'] = forcedSymbolsArray;
		}
	};

	game.logic.testForMysterySymbol = testForMysterySymbol;


}());





goog.provide('game.logic.BuildBetResults');
goog.provide('game.logic.FormatBetResults');
goog.provide('game.logic');

goog.require('libs.server.SlotGameLogic');
//goog.require('game.logic.testForPopAndDrop');
goog.require('game.logic.testForMysterySymbol');


(function() {
	"use strict";

	var addScatter = function( betResult, positions) {
		if ( !betResult[ 'reels' ][ 'scatter' ] ) {
			betResult[ 'reels' ][ 'scatter' ] = [];
		}
		betResult[ 'reels' ][ 'scatter' ].push( { 'win':0, 'position': positions } );
	};


	var addBonus = function( betResult, bonusElement ) {
		if ( !betResult[ 'bonus' ] ) {
			betResult[ 'bonus' ] = [];
		}
		betResult[ 'bonus' ].push( bonusElement );
	};


	var saveLastFreeSpinInfo = function( config, count, trigger, awarded, lockedReels, lockedReelSet ) {
	
		config[ 'serverState' ][ 'game' ][ 'freeSpinsBonus' ] = {
			'count': count,
			'trigger': trigger,
			'awarded': awarded,
			'lockedReels': lockedReels,
			'lockedReelSet': lockedReelSet
		};
	};


	var getLastFreeSpinInfo = function( config ) {
	
		if ( config[ 'serverState' ][ 'game' ] && config[ 'serverState' ][ 'game' ][ 'freeSpinsBonus' ] ) {
			// Format: { count: lastFreeSpinsCount, trigger: lastFreeSpinsTrigger, awarded: lastFreeSpinsAwarded, lockedReels: lastFreeSpinsLockedReels, lockedReelSet: lastFreeSpinsLockedReelSet }
			return config[ 'serverState' ][ 'game' ][ 'freeSpinsBonus' ];
		}
		return null;
	};


	var doFreeSpinsCalculation = function( config, reelSetIndex, betResults, platformInterface, freespinInfo, slotGame, symbolSet) {

		// test for retriggers - 1 in 32 chance of being awarded 5 coins
		var retriggerWon = false;
		var naturalRetrigger = false;
		var spinsAwarded = freespinInfo.awarded;
		var retriggerCount = 0;
		var triggerString = freespinInfo[ 'trigger' ].toString();

		var coinsArray = platformInterface.getRandomNumbers( 'IA-FreeSpin-Retrigger', 0, 1, 5 );
		var coinsLost = coinsArray.some( function( coin ) { return coin === 0; } );

		// if this is the last spin of the free spins session and the player has won less than the threshold amount,
		// automatically awarded retrigger
		if ( freespinInfo[ 'count' ] === (spinsAwarded-1) ) {
			var thresholdCashValue = config[ 'betSelection' ][ 'total_stake' ].times( game.logic.GameData.freeSpins.lastSpinRetriggerThreshold[ triggerString ] );
			var wonSoFar = config[ 'serverState' ][ 'game' ][ 'totalWin' ].plus( betResults[ 'win' ] );
			if ( thresholdCashValue.greaterThan( wonSoFar ) ) {
				retriggerWon = true;
				coinsArray = [ 1, 1, 1, 1, 1 ];
			}
		}

		if ( !retriggerWon && !coinsLost ) {
			naturalRetrigger = true;
			retriggerWon = true;
		}

		if ( retriggerWon ) {
			retriggerCount = game.logic.GameData.freeSpins.retriggerAmount[ triggerString ];
			spinsAwarded += retriggerCount;
		}

		// Analyse reel screen to see if there are any wilds on the top 3 rows. Any reels with a wild symbol will be locked. If a wild
		// symbol is on the 4th row, it will not be locked.
		// This is an array of stop positions. -1 for unlocked. >= 0 locked.
		var lockedReels = [ -1, -1, -1, -1, -1 ];
		var screen_symbols = slotGame.screen_symbols; //[nreels][nrows]
		for (var reel = 0; reel < screen_symbols.length; reel++) {
			for (var row = 0; row < ( screen_symbols[ reel ].length-1 ); row++) {

				var symbol = screen_symbols[reel][row];
				if ( symbol === 'WI' ) {
					lockedReels[ reel ] = slotGame.slots_position[ reel ];
				}
			}
		}

		var free_spins_inprogress = freespinInfo[ 'count' ] < ( spinsAwarded-1 );

		freespinInfo[ 'count' ]++;

		var bonusElement = {
			'id': "Free Spins",
			'name': "freeSpinCollect",
			'isFreeSpinCollect': true,
			'win': betResults[ 'win' ], // This is the amount of the winnings that are attributed to the bonus feature, this is for stats 
			'count': freespinInfo[ 'count' ],
			'trigger': freespinInfo[ 'trigger' ],
			'awarded': spinsAwarded,
			'coinsState': coinsArray,
			'retrigger': retriggerCount,
			'lockedReels': lockedReels,
			'lockedReelSet': reelSetIndex,
			'accumulativeWin': config[ 'serverState' ][ 'game' ][ 'totalWin' ]
		};
		saveLastFreeSpinInfo( config, freespinInfo[ 'count' ], freespinInfo[ 'trigger' ], spinsAwarded, lockedReels, reelSetIndex );
		if ( retriggerCount > 0 ) {
			bonusElement[ 'retriggerType' ] = naturalRetrigger ? 'n' : 'f';
		}
		// if ( !free_spins_inprogress ) {
			// // last spin - add free-spins element for William Hill's log parser (this is not used by the client)
			// // As defined by William Hill https://williamhill.jira.com/browse/IG-148
			// var summaryElement = { _id: 'Free Spin Summary' };
			// summaryElement[ 'free-spins' ] = {};
			// summaryElement[ 'free-spins' ][ '_extra-spins' ] = freespinInfo.count;
			// addBonus( betResults, summaryElement );
		// }
		addBonus( betResults, bonusElement );

		return free_spins_inprogress;
	};


	var spinOperation = function( platformInterface, config, betResults ) {

		var isNewGame = config[ 'betSelection' ][ 'checkpointId' ] === 0;
		
		var currCode = platformInterface.getCurrencyCode(); // currency code from Vegas server
		var currRate = game.logic.GameData.williamHill.currencyCodeRate[ currCode ];
		var slotGame = new libs.server.SlotGameLogic( game.logic.GameData.reels.nreels, game.logic.GameData.reels.nrows );

		// betSelection._sequence is now config[ 'betSelection' ][ 'checkpointId' ]
		var freespinInfo = null;
		if ( config[ 'betSelection' ][ 'checkpointId' ] > 0 ) {
			freespinInfo = getLastFreeSpinInfo( config );
		}
		
		// Decide reel_set
		var reel_set = 0;
		if ( !isNewGame ) {
			if ( !freespinInfo || !isFinite( freespinInfo.trigger ) || freespinInfo.trigger < 3 || freespinInfo.trigger > 5 ) {
				throw { message: 'freespinInfo.trigger not defined or outside of range!' };
			}
			// do not change reel set if there are any locked reels - keep the previous one
			var isLocked = false;
			if ( freespinInfo.lockedReels && freespinInfo.lockedReels.length > 0 ) {
				isLocked = freespinInfo.lockedReels.some( function( reelPos ) { return reelPos >= 0; } );
			}
			if ( isLocked ) {
				reel_set = freespinInfo.lockedReelSet;
			} else {
				reel_set = Number( game.logic.GameData.reels.freeSpinsReelSetOutcomeTable.pickOutcome( platformInterface, ( config[ 'betSelection' ][ 'stake' ].div( currRate ).toN() ) ) );
			}
		} else {
			reel_set = Number( game.logic.GameData.reels.primaryReelSetOutcomeTable.pickOutcome( platformInterface, ( config[ 'betSelection' ][ 'stake' ].div( currRate ).toN() ) ) );
		}

		betResults[ 'reels' ][ 'reelset' ] = reel_set;
		game.logic.GameData.createReelSets();
		var symbolSet = game.logic.GameData.reelSets[ reel_set ];
		slotGame.setSymbolSet( symbolSet );

		if ( !isNewGame && freespinInfo.lockedReels ) {
			// free spin games may have locked positions if wilds are on screen from previous spin.
			// step them down 1 from their previous position.
			var forcePositionArray = [];
			for ( var lockedIndex=0; lockedIndex < freespinInfo.lockedReels.length; ++lockedIndex ) {

				var lockedPos = freespinInfo.lockedReels[ lockedIndex ];
				if ( lockedPos >= 0 ) {
					var newPosition = lockedPos - 1;
					if ( newPosition < 0 ) {
						newPosition += symbolSet.getBand( lockedIndex ).getLength();
					}
					forcePositionArray.push( newPosition );
				} else {
					forcePositionArray.push( '*' );
				}
			}

			slotGame.setPosition( platformInterface, forcePositionArray, "IA-slots-setPosition");

		} else {
			// normal spin ( or this first free spin ) picks normal random numbers for each reel
			slotGame.setPosition( platformInterface, null, "IA-slots-setPosition");
		}

		betResults[ 'win' ] = new BigNumber(0);
		betResults[ 'reels' ][ 'stops' ] = slotGame.slots_position.slice(0);
		betResults[ 'reels' ][ 'screen' ] = slotGame.screen_symbols.slice(0);

		// Test line wins
		var winlineArray = libs.server.SlotGameLogic.analyseAllLineWins(
			slotGame.screen_symbols,
			game.logic.GameData.reels.linewinAwardTable,
			config[ 'betSelection' ][ 'stake' ],
			config[ 'betSelection' ][ 'lines' ],
			game.logic.GameData.reels.winLineDefinitionArray,
			3,
			game.logic.GameData.reels.matchable_symbols_list,
			[ 'WI' ]
		);
			
		// replace wild winlines with GA symbol
		for ( var winlineIndex=0; winlineIndex<winlineArray.length; ++winlineIndex ) {

			var wl = winlineArray[ winlineIndex ];
			if ( wl[ 'symbol' ] === 'WI' ) {
				wl[ 'symbol' ] = 'GA';
			}
			betResults[ 'win' ] = betResults[ 'win' ].plus( wl[ 'win' ] );
		}
		
		betResults[ 'reels' ][ 'winline' ] = winlineArray;
		betResults[ 'winCount' ] += winlineArray.length;

		var free_spins_inprogress = false;

		// Test for awarding freespins
		if ( isNewGame ) {
			betResults[ 'checkpointName' ] = 'NormalSpin'; // Required for WH history logging
			// Test scatters
			var testResultScatter = libs.server.SlotGameLogic.screenScatterWin( slotGame.screen_symbols, 3, [ 'FS' ] );
			if ( testResultScatter.pass )
			{
				free_spins_inprogress = true;
				addScatter( betResults, testResultScatter.positions );
				var trigger = testResultScatter.got;

				// Dont multiply bonus by currate as total stake has already been multiplied
				var entryBonus = config[ 'betSelection' ][ 'total_stake' ].times( game.logic.GameData.freeSpins.entryBonus[ testResultScatter.got ] );
				var awarded = game.logic.GameData.freeSpins.awarded[ testResultScatter.got ];
				addBonus( betResults, {
					'name': "freeSpinWin",
					'isFreeSpinWin': true,
					'win': entryBonus,
					'is_full': false,
					'count': 0,
					'awarded': awarded,
					'trigger': trigger,
					'entry_bonus': entryBonus,
					'accumulativeWin': entryBonus,
					'additionalWin': entryBonus
				} );
				saveLastFreeSpinInfo( config, 0, trigger, awarded, '', -1 );

				betResults[ 'win' ] = betResults[ 'win' ].plus( entryBonus );
				betResults[ 'winCount' ]++;
			}
		} else {
			betResults[ 'checkpointName' ] = 'FreeSpin'; // Required for WH history logging
			free_spins_inprogress = doFreeSpinsCalculation( config, reel_set, betResults, platformInterface, freespinInfo, slotGame, symbolSet );
		}

		game.logic.testForMysterySymbol( platformInterface, config, betResults, slotGame );

		// Cap win at the end before adding jackpots
		if ( game.logic.GameData.financial.maxPayoutPerSpin ) {
			var maxWin = ( new BigNumber( game.logic.GameData.financial.maxPayoutPerSpin ) ).times( currRate );
			if ( betResults[ 'win' ].greaterThan( maxWin ) ) {
				betResults[ 'win' ] = maxWin;
			}
		}
		
		// test for jackpot win
		var matchable_symbols_list_jackpots = [ 'SP', 'GP' ]; // jackpot symbols cannot be substituted by wilds
		var jackpotIds = [ 1, 0 ];

		for ( var jackpotSymbolIndex=0; jackpotSymbolIndex<matchable_symbols_list_jackpots.length; ++jackpotSymbolIndex ) {

			var jackpotSymbol = matchable_symbols_list_jackpots[ jackpotSymbolIndex ];
			var testJackpotResult = libs.server.SlotGameLogic.screenScatterWin( slotGame.screen_symbols, 5, [jackpotSymbol], [] );
			if (testJackpotResult.pass)
			{
				addScatter( betResults, testJackpotResult.positions );
				
				var wonId = jackpotIds[ jackpotSymbolIndex ];
				var jackpotWinValue = platformInterface.claimJackpot( wonId, config[ 'betSelection' ][ 'total_stake' ] );

				// jackpotWinInfo = { id: 0, win: 0 }
				if ( !betResults[ 'jackpotWins' ] ) {
					betResults[ 'jackpotWins' ] = [];
				}
				betResults[ 'jackpotWins' ].push( { 'id': wonId, 'win':jackpotWinValue } );
				betResults['win'] = betResults['win'].plus( jackpotWinValue );
				betResults[ 'winCount' ]++;
			}
		}

		betResults[ 'expectCheckpoint' ] = free_spins_inprogress;
	};
	

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	// game.logic.buildBetResults is the entry point for determining a game logic result.
	// 'platformInterface' is the interface as to which random numbers can be requested and jackpots reported as won.
	// 'config' is a JSON object which contains the following:
	//		'betSelection' - Client game configuration - contains the stake per line, total stake, number of lines (if a reels game) and any other information specific to the game being played.
	//		'serverState': { - Optional
	//			'player' - Server saved player state - contains any data that should be preserved permanently over multiple games. This could be used for 'levelling up' to get extra on certain bonuses over multiple games for instance.
	//			'game' - Server saved game state - contains data that should be preserved over a game cycle, e.g. if player is in free spins or a bonus that requires checkpointing
	//		}

	var buildBetResults = function( platformInterface, config ) {

		if ( !config[ 'serverState' ] ) {
			config[ 'serverState' ] = {};
		}
		if ( !config[ 'serverState' ][ 'game' ] ) {
			config[ 'serverState' ][ 'game' ] = {};
		}
		if ( !config[ 'serverState' ][ 'player' ] ) {
			config[ 'serverState' ][ 'player' ] = {};
		}
		if ( config[ 'betSelection' ][ 'checkpointId' ] === undefined ) {
			config[ 'betSelection' ][ 'checkpointId' ] = 0;
		}

		// Initialise default bet result values
		var betResults = {};

		betResults[ 'win' ] = new BigNumber(0);
		betResults[ 'winCount' ] = 0;
		betResults[ 'reels' ] = {}; // .reelset .stops
		betResults[ 'bet_stake' ] = config[ 'betSelection' ][ 'stake' ];
		betResults[ 'checkpointId' ] = config[ 'betSelection' ][ 'checkpointId' ];

		spinOperation( platformInterface, config, betResults );
		betResults[ 'RTP' ] = 95.0;
		
		// Add up total winnings this game cycle
		var totalWin = config[ 'serverState' ][ 'game' ][ 'totalWin' ] || new BigNumber( 0 );
		totalWin = totalWin.plus( betResults[ 'win' ] );
		config[ 'serverState' ][ 'game' ][ 'totalWin' ] = betResults[ 'totalWin' ] = totalWin;
		
		var totalWinCount = config[ 'serverState' ][ 'game' ][ 'totalWinCount' ] || new BigNumber( 0 );
		totalWinCount = totalWinCount.plus( betResults[ 'winCount' ] );
		config[ 'serverState' ][ 'game' ][ 'totalWinCount' ] = betResults[ 'totalWinCount' ] = totalWinCount;

		// save next expected checkpoint ID for verification next spin
		config[ 'serverState' ][ 'game' ][ 'nextCheckpointId' ] = config[ 'betSelection' ][ 'checkpointId' ] + 1;

		return betResults;
	};


	game.logic.BuildBetResults = buildBetResults;


}());