/**
 * The core components needed for the client
 *
 * @module libs_ClientCore
 */
goog.provide( 'libs.client.core.clientInformation' );

goog.requireThirdParty( 'screenfull' );
goog.require( 'game.deviceCompatibility' );

(function() {
	"use strict";

	/**
	 * A class containing all the relevant functions to determine a device's information and it's capabilities
	 * @class ClientInformation
	 * @constructor
	 */
	var ClientInformation = function() {
		var that = this;

		this.userAgent = navigator.userAgent;

		libs.log( 'Game Version: ' + BUILD_VERSION );
		libs.log( 'User Agent: ' + this.userAgent );

		this._graphicalQuality = null;
		this._audioQuality = null;
		this._has3d = null;

		this._IOSVersion = 0;
		if ( this.isIOS() ) {
			this._IOSVersion = (function() {
				var match = (that.userAgent).match( /OS (\d+)_(\d+)?/ ),
					version = [
						parseInt( match[1], 10 ),
						parseInt( match[2], 10 )
					];
				return parseFloat( version.join( '.' ) );
			})();
		}

		this._androidVersion = 0;
		if ( this.isAndroid() ) {
			this._androidVersion = 4.3; // covers HTC one, which report as Linux
			this._androidVersion = (function() {
				var match = that.userAgent.match( /Android\s([0-9\.]*)/ );
				return parseFloat( match ? match[1] : 0 );
			})();
		}

		this._chromeVersion = 0;
		if ( this.isChrome() ) {
			this._chromeVersion = (function() {
				var match = that.userAgent.match( /Chrome\/(\d+)\./ );
				return parseInt( match ? match[1] : 0 );
			})();
		}

		this._IEVersion = 0;
		if ( this.isIE() ) {
			this._IEVersion = (function() {
				var match = that.userAgent.match( /MSIE ([0-9]{1,}[\.0-9]{0,})/ );
				if ( !match ) {
					match = that.userAgent.match( /rv:([0-9]{1,}[\.0-9]{0,})/ );
				}
				return parseInt( match ? match[1] : 0 );
			})();
		}
	};

	/**
	 * Checks if the given string is present in the current user agent
	 * @private
	 * @method _isInUA
	 * @param {string} lookFor
	 * @return {boolean}
	 */
	var _isInUA = function( lookFor ) {
		return navigator.userAgent.indexOf( lookFor ) !== -1;
	};

	/**
	 * Checks if all of the given array of strings is present in the current user agent
	 * @private
	 * @method _areAllStringsInUA
	 * @param {array} stringArray
	 * @return {boolean}
	 */
	var _areAllStringsInUA = function( stringArray ) {
		for ( var i = 0; i < stringArray.length; ++i ) {
			var string = stringArray[i];
			if ( !_isInUA( string ) ) {
				return false;
			}
		}

		return true;
	};

	/**
	 * Checks if any of the given array of strings is present in the current user agent
	 * @private
	 * @method _areAnyStringsInUA
	 * @param {array} stringArray
	 * @return {boolean}
	 */
	var _areAnyStringsInUA = function( stringArray ) {
		for ( var i = 0; i < stringArray.length; ++i ) {
			var string = stringArray[i];
			if ( _isInUA( string ) ) {
				return true;
			}
		}

		return false;
	};

	/**
	 * Checks if the current device is in the given device array
	 * @private
	 * @method _isDeviceOnList
	 * @param {array} deviceArray
	 * @return {boolean}
	 */
	var _isDeviceOnList = function( deviceArray ) {
		for ( var i = 0; i < deviceArray.length; ++i ) {
			var device = deviceArray[i];
			if ( _areAllStringsInUA( device.uaAND ) ) {
				if ( device.uaOR === undefined || device.uaOR.length === 0 || _areAnyStringsInUA( device.uaOR ) ) {
					return true;
				}
			}
		}

		return false;
	};

	/**
	 * Returns either "mobile" or "desktop", depending on the device type
	 * @public
	 * @method getDeviceType
	 * @return {string}
	 */
	ClientInformation.prototype.getDeviceType = function() {
		//This is handled by the game and then referenced
		return game.deviceCompatibility.deviceType;
	};

	// OS detection

	/**
	 * Checks whether the device reports iPhone or iPod capabilities
	 * @public
	 * @method isIPhoneLike
	 * @return {boolean}
	 */
	ClientInformation.prototype.isIPhoneLike = function() {
		return _isInUA( "iPhone" ) || _isInUA( "iPod" );
	};

	/**
	 * Checks whether the device reports that it is an iPad
	 * @public
	 * @method isIPad
	 * @return {boolean}
	 */
	ClientInformation.prototype.isIPad = function() {
		return _isInUA( "iPad" );
	};

	/**
	 * Checks whether the device is running iOS
	 * @public
	 * @method isIOS
	 * @return {boolean}
	 */
	ClientInformation.prototype.isIOS = function() {
		return this.isIPhoneLike() || this.isIPad();
	};

	/**
	 * Checks whether the device is an Android device
	 * @public
	 * @method isAndroid
	 * @return {boolean}
	 */
	ClientInformation.prototype.isAndroid = function() {
		return _areAnyStringsInUA( ["Android", "Linux"] );
	};

	/**
	 * Checks whether the device is running WebOS
	 * @public
	 * @method isWebOS
	 * @return {boolean}
	 */
	ClientInformation.prototype.isWebOS = function() {
		return _isInUA( "webOS" );
	};

	/**
	 * Checks whether the device is a Blackberry
	 * @public
	 * @method isBlackberry
	 * @return {boolean}
	 */
	ClientInformation.prototype.isBlackberry = function() {
		return _isInUA( "BlackBerry" );
	};

	/**
	 * Checks whether the device is a Windows Phone
	 * @public
	 * @method isWindowsPhone
	 * @return {boolean}
	 */
	ClientInformation.prototype.isWindowsPhone = function() {
		return _isInUA( "Windows Phone" );
	};

	// OS version detection

	/**
	 * Checks whether the device is an Android device running a version of Android less than version 4
	 * @public
	 * @method isOldAndroid
	 * @return {boolean}
	 */
	ClientInformation.prototype.isOldAndroid = function() {
		return this.isAndroid() && this._androidVersion < 4;
	};

	/**
	 * Checks whether the device is an Android device running either version 2 or 3 of Android
	 * @public
	 * @deprecated Use {@link ClientInformation#isOldAndroid isOldAndroid}
	 * @method isAndroid2Or3
	 * @return {boolean}
	 */
	ClientInformation.prototype.isAndroid2Or3 = function() {
		return _isInUA( "Android 2" ) || _isInUA( "Android 3" );
	};

	/**
	 * Checks whether the device is Android 4
	 * @public
	 * @method isAndroid4
	 * @return {boolean}
	 */
	ClientInformation.prototype.isAndroid4 = function() {
		return _isInUA( "Android 4" );
	};

	/**
	 * Checks whether the device is running a version of iOS less than version 6
	 * @public
	 * @method isOldIOS
	 * @return {boolean}
	 */
	ClientInformation.prototype.isOldIOS = function() {
		return this.isIOS() && this._IOSVersion < 6;
	};

	/**
	 * Checks whether the device is running iOS version 6
	 * @public
	 * @method isIOS6
	 * @return {boolean}
	 */
	ClientInformation.prototype.isIOS6 = function() {
		return this.isIOS() && _isInUA( "OS 6" );
	};

	/**
	 * Checks whether the device is running iOS version 7
	 * @public
	 * @method isIOS7
	 * @return {boolean}
	 */
	ClientInformation.prototype.isIOS7 = function() {
		return this.isIOS() && _isInUA( "OS 7" );
	};

	/**
	 * Checks whether the device is running iOS version 8
	 * @public
	 * @method isIOS8
	 * @return {boolean}
	 */
	ClientInformation.prototype.isIOS8 = function() {
		return this.isIOS() && _isInUA( "OS 8" );
	};

	/**
	 * Returns the version of iOS that the device is running
	 * @public
	 * @method getIOSVersion
	 * @return {string}
	 */
	ClientInformation.prototype.getIOSVersion = function() {
		return this._IOSVersion;
	};

	/**
	 * Returns the version of Android that the device is running
	 * @public
	 * @method getAndroidVersion
	 * @return {string}
	 */
	ClientInformation.prototype.getAndroidVersion = function() {
		return this._androidVersion;
	};

	// browser detection

	/**
	 * Checks if the current device is running a manufacturer browser
	 * @public
	 * @method isNativeAndroidBrowser
	 * @return {boolean}
	 */
	ClientInformation.prototype.isNativeAndroidBrowser = function() {
		return this.isAndroid() && !this.isHardwareAcceleratedChrome() && !this.isFirefox();
	};

	/**
	 * Checks if the current browser is Chrome
	 * @public
	 * @method isChrome
	 * @return {boolean}
	 */
	ClientInformation.prototype.isChrome = function() {
		return _areAnyStringsInUA( ["Chrome", "CriOS"] );
	};

	/**
	 * Checks if the current browser is Firefox
	 * @public
	 * @method isFirefox
	 * @return {boolean}
	 */
	ClientInformation.prototype.isFirefox = function() {
		return _isInUA( "Firefox" );
	};

	/**
	 * Checks if the current browser is Internet Explorer
	 * @public
	 * @method isIE
	 * @return {boolean}
	 */
	ClientInformation.prototype.isIE = function() {
		return _areAnyStringsInUA( ["MSIE", "Trident", "IEMobile"] );
	};

	/**
	 * Checks if the current browser is Internet Explorer running on a desktop
	 * @public
	 * @method isIEDesktop
	 * @return {boolean}
	 */
	ClientInformation.prototype.isIEDesktop = function() {
		return _areAnyStringsInUA( ["MSIE", "Trident"] ) && !this.isIEMobile();
	};

	/**
	 * Checks if the current browser is Internet Explorer running on a mobile device
	 * @public
	 * @method isIEMobile
	 * @return {boolean}
	 */
	ClientInformation.prototype.isIEMobile = function() {
		return _isInUA( "IEMobile" );
	};

	/**
	 * Checks if the current browser is Opera Mini
	 * @public
	 * @method isOperaMini
	 * @return {boolean}
	 */
	ClientInformation.prototype.isOperaMini = function() {
		return _isInUA( "Opera Mini" );
	};

	/**
	 * Checks if the current browser is Opera Mobile
	 * @public
	 * @method isOperaMobile
	 * @return {boolean}
	 */
	ClientInformation.prototype.isOperaMobile = function() {
		return _isInUA( "Opera Mobi" );
	};

	// browser version detection

	/**
	 * Gets the version of Internet Explorer
	 * @public
	 * @method getIEVersion
	 * @return {string}
	 */
	ClientInformation.prototype.getIEVersion = function() {
		return this._IEVersion;
	};

	/**
	 * Checks if Internet Explorer is running in compatibility mode
	 * @public
	 * @method isIEInCompatabiltyMode
	 * @return {boolean}
	 */
	ClientInformation.prototype.isIEInCompatabiltyMode = function() {
		return _isInUA( "compatible" );
	};

	/**
	 * Checks if the current Internet Explorer compatibility mode is compatible
	 * @public
	 * @method isIEDocumentModeCompatible
	 * @return {boolean}
	 */
	ClientInformation.prototype.isIEDocumentModeCompatible = function() {
		if ( document.documentMode ) {
			return Number( document.documentMode ) >= 9;
		}

		return true;
	};

	/**
	 * Gets the version of Chrome
	 * @public
	 * @method isIEDocumentModeCompatible
	 * @return {string}
	 */
	ClientInformation.prototype.getChromeVersion = function() {
		return this._chromeVersion;
	};

	/**
	 * Checks if the current version of Chrome can be hardware accelerated
	 * @public
	 * @method isHardwareAcceleratedChrome
	 * @return {boolean}
	 */
	ClientInformation.prototype.isHardwareAcceleratedChrome = function() {
		return this.isChrome() && this._chromeVersion >= 20;
	};

	/**
	 * Checks if the current version of Chrome is greater than version 29
	 * @public
	 * @method isRecentChrome
	 * @return {boolean}
	 */
	ClientInformation.prototype.isRecentChrome = function() {
		return this.isChrome() && this._chromeVersion >= 30;
	};

	// manufacturer detection

	/**
	 * Checks if the device is a Samsung device
	 * @public
	 * @method isSamsung
	 * @return {boolean}
	 */
	ClientInformation.prototype.isSamsung = function() {
		return _isInUA( "GT-" );
	};

	/**
	 * Checks if the device is a Samsung Galaxy S3
	 * @public
	 * @method isGalaxyS3
	 * @return {boolean}
	 */
	ClientInformation.prototype.isGalaxyS3 = function() {
		return _isInUA( "GT-I930" );
	};

	/**
	 * Checks if the device is a Samsung Galaxy S2
	 * @public
	 * @method isGalaxyS2
	 * @return {boolean}
	 */
	ClientInformation.prototype.isGalaxyS2 = function() {
		return _isInUA( "GT-I910" );
	};


	/**
	 * Checks if the device is an HTC One
	 * @public
	 * @method isHTCOne
	 * @return {boolean}
	 */
	ClientInformation.prototype.isHTCOne = function() {
		return _areAnyStringsInUA( ["Chrome/11.0.696.34 Safari/534.24", "HTC_One"] );
	};

	/**
	 * Checks if the device is a Sony Xperia Z
	 * @public
	 * @method isSonyXperiaZ
	 * @return {boolean}
	 */
	ClientInformation.prototype.isSonyXperiaZ = function() {
		return _isInUA( "Sony Xperia Z" );
	};

	// capability detection

	/**
	 * Checks if the device supports the full screen API
	 * @public
	 * @method supportsFullscreenAPI
	 * @return {boolean}
	 */
	ClientInformation.prototype.supportsFullscreenAPI = function() {
		// s4 reports it can do fullscreenapi, but can't, so freezes game. It's browser is based on chrome 28...
		// so restricting it to a definitely working version and above will solve potential issue on other devices too
		if ( this.isChrome() && this._chromeVersion < 33 ) {
			return false;
		}

		return screenfull && screenfull.enabled;
	};

	/**
	 * Checks if the device supports web audio API
	 * @public
	 * @method hasWebAudioAPI
	 * @return {boolean}
	 */
	ClientInformation.prototype.hasWebAudioAPI = function() {
		return (typeof(window.AudioContext) !== "undefined" && window.AudioContext !== null);
	};

	/**
	 * Checks if the device is running Safari on a mobile device
	 * @public
	 * @method isIOSBrowser
	 * @return {boolean}
	 */
	ClientInformation.prototype.isIOSBrowser = function() {
		return ( this.isIOS() && _isInUA( "Safari" ) && !window.navigator.standalone );
	};

	/**
	 * Checks if the device is running within a wrapper on iOS
	 * @public
	 * @method isIOSBrowser
	 * @return {boolean}
	 */
	ClientInformation.prototype.isIOSNativeApp = function() {
		return ( this.isIOS() && !_isInUA( "Safari" ) && !window.navigator.standalone );
	};

	/**
	 * Checks if the device is running in home screen on iOS
	 * @public
	 * @method isIOSBrowser
	 * @return {boolean}
	 */
	ClientInformation.prototype.isIOSWebApp = function() {
		return ( this.isIOS() && window.navigator.standalone );
	};

	// performance options

	/**
	 * Checks if the device needs to use CSS hardware acceleration
	 * @public
	 * @method requiresForceCSSHardwareAcceleration
	 * @return {boolean}
	 */
	ClientInformation.prototype.requiresForceCSSHardwareAcceleration = function() {
		// The Samsung S3 native browser doesn't do rendering well unless we can force the webkit transform hack
		// Only effect the S3 ON 4.1.2
		return (this.isGalaxyS3() && this.getAndroidVersion() === 4.1) || (this.isGalaxyS2() && this.getAndroidVersion() === 4.0);
	};

	/**
	 * Checks if the device can handle blurred reels
	 * @public
	 * @method useBlurredReels
	 * @return {boolean}
	 */
	ClientInformation.prototype.useBlurredReels = function() {
		return this.getGraphicalQuality() === "LQ";
	};

	/**
	 * Checks if the device needs to use HTML audio
	 * @public
	 * @method forceUseHTMLAudio
	 * @return {boolean}
	 */
	ClientInformation.prototype.forceUseHTMLAudio = function() {
		if ( game.deviceCompatibility.deviceType === 'mobile' ) {
			if ( this.isAndroid() && this.isChrome() && !this.isRecentChrome() ) {
				return true;
			}
		} else if ( game.deviceCompatibility.deviceType === 'desktop' ) {
			if ( this.isFirefox() === true ) {
				return true;
			}
		}

		return false;
	};

	/**
	 * Gets the current devices audio capability and returns it in the form of "LQ", "MQ" or "HQ"
	 * @public
	 * @method getAudioQuality
	 * @return {string}
	 */
	ClientInformation.prototype.getAudioQuality = function() {
		if ( this._audioQuality === null ) {
			this._audioQuality = "HQ";

			// if ( this.isAndroid() ) {
			// 	if ( this.getAndroidVersion() < 4.1 ) {
			// 		this._audioQuality = "LQ";
			// 	} else if ( this.isHTCOne() && !this.isRecentChrome() ) {
			// 		this._audioQuality = "LQ";
			// 	} else if ( this.isGalaxyS3() && this.getAndroidVersion() === 4.1 && !this.isRecentChrome() ) {
			// 		this._audioQuality = "LQ";
			// 	} else if ( !this.hasWebAudioAPI() ) {
			// 		this._audioQuality = "MQ";
			// 	} else if ( this.isChrome() && !this.isRecentChrome() ) {
			// 		this._audioQuality = "MQ";
			// 	}
			// } else if ( this.isIEMobile() ) {
			// 	this._audioQuality = "LQ";
			// } else if ( this.isOldIOS() ) {
			// 	this._audioQuality = "LQ";
			// }

			// libs.log( "Audio Quality: " + this._audioQuality );
		}

		return this._audioQuality;
	};

	/**
	 * Gets the current devices graphics capability and returns it in the form of "LQ", "MQ" or "HQ"
	 * @public
	 * @method getGraphicalQuality
	 * @return {string}
	 */
	ClientInformation.prototype.getGraphicalQuality = function() {
		if ( this._graphicalQuality === null ) {
			this._graphicalQuality = "MQ";

			// if ( this.isAndroid() ) {
			// 	if ( this.isAndroid2Or3() || this.requiresForceCSSHardwareAcceleration() ) {
			// 		this._graphicalQuality = "LQ";
			// 	} else if ( !this.isHardwareAcceleratedChrome() && !this.isFirefox() ) {
			// 		if ( window.devicePixelRatio <= 1 ) {
			// 			this._graphicalQuality = "LQ";
			// 		} else {
			// 			this._graphicalQuality = "MQ";
			// 		}
			// 	}
			// } else if ( this.isIOS() ) {
			// 	if ( this.isOldIOS() ) {
			// 		this._graphicalQuality = "LQ";
			// 	} else if ( window.devicePixelRatio <= 1 ) {
			// 		if ( this.isIPhoneLike() ) { // detect a 3GS
			// 			this._graphicalQuality = "LQ";
			// 		} else { // ipad
			// 			this._graphicalQuality = "MQ";
			// 		}
			// 	} else if ( this.isIPhoneLike() && this.isIOS6() && $( window ).innerWidth() <= 480 ) {
			// 		this._graphicalQuality = "MQ"; // iphone 4 and 4s on IOS6
			// 	}
			// }

			libs.log( "Graphical Quality: " + this._graphicalQuality );
		}

		return this._graphicalQuality;
	};

	/**
	 * Checks if the current device supports 3D
	 * @public
	 * @method is3dSupported
	 * @return {boolean}
	 */
	ClientInformation.prototype.is3dSupported = function() {
		if ( this._has3d === null ) {

			var el = document.createElement( 'p' );
			var transforms = {
				'WebkitTransform': '-webkit-transform',
				'OTransform': '-o-transform',
				'MSTransform': '-ms-transform',
				'MozTransform': '-moz-transform',
				'transform': 'transform'
			};

			document.body.appendChild( el );
			for ( var t in transforms ) {
				if ( el.style[t] !== undefined ) {
					el.style[t] = 'matrix3d(1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1)';
					this._has3d = window.getComputedStyle( el ).getPropertyValue( transforms[t] );
				}
			}
			document.body.removeChild( el );

			this._has3d = this._has3d !== 'none';
		}

		return this._has3d;
	};

	/**
	 * Checks if the current device is on the White List
	 * @public
	 * @method isDeviceOnWhitelist
	 * @return {boolean}
	 */
	ClientInformation.prototype.isDeviceOnWhitelist = function() {
		return _isDeviceOnList( game.deviceCompatibility.whitelist );
	};

	/**
	 * Checks if the current device is on the Black List
	 * @public
	 * @method isDeviceOnBlacklist
	 * @return {boolean}
	 */
	ClientInformation.prototype.isDeviceOnBlacklist = function() {
		return _isDeviceOnList( game.deviceCompatibility.blacklist );
	};

	/**
	 * Checks if current browser is webkit
	 * @public
	 * @method isWebkitBrowser
	 * @returns {boolean}
	 */
	ClientInformation.prototype.isWebkitBrowser = function() {
		return 'WebkitAppearance' in document.documentElement.style;
	};

	/**
	 * Returns a 'best guess' at what kind of device we are running on.
	 * @public
	 * @method buildDeviceString
	 * @return {string}
	 */
	ClientInformation.prototype.buildDeviceString = function() {

		var browser;

		if ( this.isNativeAndroidBrowser() ) {
			browser = 'native Android browser';
		} else if ( this.isChrome() ) {
			browser = 'Chrome ' + this.getChromeVersion();
		} else if ( this.isFirefox() ) {
			browser = 'Firefox';
		} else if ( this.isIE() ) {
			browser = 'Internet Explorer ' + this.getIEVersion();
		} else if ( this.isOperaMini() ) {
			browser = 'Opera Mini';
		} else if ( this.isOperaMobile() ) {
			browser = 'Opera Mobile';
		} else {
			browser = 'unknown browser';
		}

		if ( this.getDeviceType() === 'mobile' ) {
			// mobile
			if ( this.isAndroid() ) {
				var androidVersion = this.getAndroidVersion();

				if ( this.isSamsung() ) {
					if ( this.isGalaxyS3() ) {
						return "Galaxy S3 Android [" + androidVersion + "] " + browser;
					} else {
						return "Samsung Android [" + androidVersion + "] " + browser;
					}
				} else if ( this.isHTCOne() ) {
					return "HTC One Android [" + androidVersion + "] " + browser;
				} else if ( this.isSonyXperiaZ() ) {
					return "Sony Xperia Z Android [" + androidVersion + "] " + browser;
				} else {
					return "Android [" + androidVersion + "] " + browser;
				}
			} else if ( this.isIOS() ) {

				var iosVersion = this.getIOSVersion();

				if ( this.isIPad() ) {
					return "iPad [" + iosVersion + "] " + browser;
				} else if ( this.isIPhoneLike() ) {
					return "iPhone-like [" + iosVersion + "] " + browser;
				} else {
					return "IOS [" + iosVersion + "] " + browser;
				}

			} else if ( this.isBlackberry() ) {
				return "Blackberry " + browser;
			} else if ( this.isWindowsPhone() ) {
				return "Windows phone " + browser;
			} else {
				return "Unknown mobile device " + browser;
			}
		} else {
			// desktop
			return 'Desktop ' + browser;
		}
	};


	libs.client.core.clientInformation = new ClientInformation();

}());