goog.provide( 'libs.client.core.DebugServerInterface' );


(function() {
	"use strict";

	var defaultSettings = {
		logFlushInterval: 15000
	};


	var g_cookieName = 'ingg-debug-server-id';


	var readInTestOutcomes = function( callback ) {
		var path = libs.client.assetUrlPrefix + 'toolsData/testOutcomes.json';
		var data;
		$.getJSON( path, function( dataArg ) {
			data = dataArg;
		} ).always( function() {
			callback( data );
		} );
	};

	var logMaximumLines = 5000;
	var g_logArray = [];

	var addToLog = function( msg ) {

		g_logArray.push( msg );
		if ( g_logArray.length > logMaximumLines ) {
			g_logArray.shift();
		}
	};

	var onWindowError = function( errorMsg, url, lineNumber ) {

		var msg = '';
		if ( url ) {
			msg += url;
		}
		if ( isFinite( lineNumber ) ) {
			msg += ":" + lineNumber;
		}
		if ( errorMsg ) {
			msg += " " + errorMsg;
		}
		addToLog( msg );
		return false;
	};


	window.onerror = onWindowError;
	libs.onLog( addToLog );


	var DebugServerInterface = function( overrideSettings, gameSetupFlags ) {

		var that = this;
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		this._gameSetupFlags = gameSetupFlags;
		this._connected = false;
		this._serverPlayerId = null;
		this._serverSessionId = null;
		this._preBetResultCalculationHandle = null;
		this._getPersistentData();

		readInTestOutcomes( function( outcomeData ) {
			that._outcomeData = outcomeData;
			if ( that._outcomeData && that._outcomeData.debugServer && that._outcomeData.debugServer.url ) {
				that._sendGameDetails( true );
				libs.globalEventBus.connectActivityStart( 'libs.activity.localServer.preBetResultCalculation', function( activity, currentCheckpointId ) { that._onPreBetResultCalculation( activity, currentCheckpointId ); } );
			} else {
				libs.log( "testOutcomes.json is missing or could not be loaded. Debug server connection will now be disabled..." );
			}
		} );
	};


	DebugServerInterface.prototype._onPreBetResultCalculation = function( activity, currentCheckpointId ) {

		var that = this;
		if ( this._connected && currentCheckpointId === 0 ) {
			this._preBetResultCalculationHandle = activity.join();
			// send request to debug server to see if we should force a game outcome for the next spin
			that._postMessage( 'client_check_forced_outcome', {}, function( success, responseObject ) {
				if ( success && responseObject.rng ) {
					libs.client.hillsVegas.LocalServerPlatformInterface.forceRng( responseObject.rng );
				}
				that._preBetResultCalculationHandle.leave();
			} );
		}
	};


	DebugServerInterface.prototype._getPersistentData = function() {

		var findCookie = function( name ) {
			var nameEQ = name + "=";
			var ca = document.cookie.split( ';' );
			for ( var i = 0; i < ca.length; i++ ) {
				var c = ca[i];
				while ( c.charAt( 0 ) === ' ' ) {
					c = c.substring( 1, c.length );
				}
				if ( c.indexOf( nameEQ ) === 0 ) {
					return c.substring( nameEQ.length, c.length );
				}
			}
			return null;
		};

		var jsonString = findCookie( g_cookieName );
		if ( jsonString ) {
			try {
				var dataObject = JSON.parse( jsonString );
				this._serverPlayerId = dataObject['playerId'];
				this._serverSessionId = dataObject['sessionId'];
			}
			catch ( err ) {
				libs.log( "Failed to parse debug server cookie " + err );
			}
		}
	};


	DebugServerInterface.prototype._sendGameDetails = function( isFirstTimeThisSession ) {

		// send outcome force data if it exists
		var that = this;

		var dataObj = {
			'testOutcomeData': this._outcomeData,
			'game': {
				'setup': this._gameSetupFlags,
				'friendlyName': game.logic.GameData.friendlyGameName,
				'version': BUILD_VERSION,
				'guessedDeviceName': libs.client.core.clientInformation.buildDeviceString(), // use ClientInformation to take a guess at what device type we are on
				'virgoGameUID': game.logic.GameData.virgo.gameUID  // Virgo games are required to send the game UID so the debug server can link a client to a server
			}
		};

		this._postMessage( 'client_game_details', dataObj, function( success, responseObject ) {
			if ( success ) {
				that._onGameDetailsSuccess( responseObject );
			} else {
				libs.log( "Failed to connect to debug server at '" + that._outcomeData.debugServer.url + "'" );
			}
		}, isFirstTimeThisSession ); // dont send stored session ID on game first load - let the server create a new session entry
	};


	DebugServerInterface.prototype._onGameDetailsSuccess = function( responseObject ) {

		if ( !this._connected ) {
			// Do this only on the response to the first message sent to the server
			this._connected = true;
			libs.log( "Successfully connected to debug server at '" + this._outcomeData.debugServer.url + "' using player ID '" + this._serverPlayerId + "' and session ID '" + this._serverSessionId + "'" );
			this._startLogFlushTimeout();
		}
	};


	DebugServerInterface.prototype._savePersistentData = function( responseObject ) {

		// Save off new player ID in a cookie
		var playerId = responseObject['playerId'];
		var sessionId = responseObject['sessionId'];

		var needsSaving = ( playerId && this._serverPlayerId !== playerId ) || ( sessionId && this._serverSessionId !== sessionId );
		if ( needsSaving ) {
			this._serverPlayerId = playerId;
			this._serverSessionId = sessionId;
			var farAwayInTheFuture = new Date();
			farAwayInTheFuture.setDate( 2147483647 ); // January 2038 - Maximum value of a 31 bit integer
			var cookieString = JSON.stringify( {
				'playerId': this._serverPlayerId,
				'sessionId': this._serverSessionId
			} );
			document.cookie = g_cookieName + '=' + cookieString + '; expires=' + farAwayInTheFuture.toUTCString() + "; path=/";
		}
	};


	DebugServerInterface.prototype._postMessage = function( msgName, msgData, callback, excludeSessionId ) {

		var that = this;
		var success = false;
		var responseObject;
		var dataObject = JSON.stringify( {
			'msgType': msgName,
			'msgData': msgData || {},
			'playerId': this._serverPlayerId,
			'sessionId': ( excludeSessionId ? undefined : this._serverSessionId )
		} );
		$.ajax( {
			'type': 'POST',
			'url': this._outcomeData.debugServer.url,
			'data': dataObject,
			'dataType': 'json',
			'success': function( responseArg ) {
				success = true;
				responseObject = responseArg;
				that._savePersistentData( responseObject );
				if ( responseObject['gameDetailsRequest'] ) {
					// Server may sometimes respond to a message asking for the game details so it can properly create a player's session entry in the database
					that._sendGameDetails();
				}
			}
		} )
			.fail( function( xhr, textStatus, errorThrown ) {
				//libs.log( "Handshake failed: " + xhr.statusText + " : " + textStatus );
				success = false;
			} )
			.always( function() {
				if ( callback ) {
					callback( success, responseObject );
				}
			} );
	};


	DebugServerInterface.prototype._startLogFlushTimeout = function() {
		var that = this;
		libs.setTimeout( function() { that._flushLog(); }, this._settings.logFlushInterval );
	};


	DebugServerInterface.prototype._flushLog = function() {

		var that = this;
		if ( g_logArray.length > 0 ) {
			this._postMessage( 'client_log', { 'log': g_logArray }, function( success ) {
				if ( success ) {
					g_logArray.length = 0; // clear sent data array on success, keep it if it fails
				}
				that._startLogFlushTimeout();
			} );
		} else {
			// nothing to log, wait another x seconds till next flush
			this._startLogFlushTimeout();
		}
	};


	libs.client.core.DebugServerInterface = DebugServerInterface;

}());