/**
 * The core components needed for the client
 *
 * @module libs_ClientCore
 */

goog.provide( 'libs.client.core.InputController' );

goog.require( 'libs.globalEventBus' );


(function() {
	"use strict";

	var defaultSettings = {};

	/**
	 * The InputController class is used to handle all user input and propagate the data to the client
	 * @class InputController
	 * @constructor
	 */
	var InputController = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._interactiveSurface = null;
		this._interactivejCanvas = null;

		$( document ).bind( 'keydown', function( e ) {
			if ( BUILD_TYPE !== 'dev' ) {
				e.preventDefault();
				e.stopPropagation();
			}
			var keyChar = String.fromCharCode( e.keyCode );
			libs.globalEventBus.invoke( 'libs.event.notify.keyPressed', e.keyCode, keyChar );
		} );

		libs.globalEventBus.connect( 'libs.event.notify.interactiveSurface', function( canvas ) {
			that._onNotifyInteractiveSurface( canvas );
		} );
	};

	/**
	 * Attaches input events onto the canvas object that is passed in
	 * @private
	 * @method _onNotifyInteractiveSurface
	 * @param {object} surface
	 */
	InputController.prototype._onNotifyInteractiveSurface = function( surface ) {
		var that = this;

		this._interactiveSurface = surface;
		this._interactivejCanvas = $( surface.canvas );

		this._vmousedownHandler = libs.callback( function( e ) {
			that._vmousedown( e );
		} );
		this._vmouseupHandler = libs.callback( function( e ) {
			that._vmouseup( e );
		} );
		this._vmousemoveHandler = libs.callback( function( e ) {
			that._vmousemove( e );
		} );
		this._vclickHandler = libs.callback( function( e ) {
			that._vmouseclick( e );
		} );

		libs.globalEventBus.connectActivityStart( 'libs.activity.system.tickManagerActive', function( activity ) {
			that._onSystemTickManagerActiveStart( activity );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.system.tickManagerActive', function() {
			that._onSystemTickManagerActiveEnd();
		} );
	};

	/**
	 * Binds the input events to the canvas when the game starts running
	 *
	 * @private
	 * @method _onSystemTickManagerActiveStart
	 * @param {object} activity
	 */
	InputController.prototype._onSystemTickManagerActiveStart = function( activity ) {
		if ( this._interactivejCanvas ) {
			this._interactivejCanvas.bind( 'vmousedown', this._vmousedownHandler );
			this._interactivejCanvas.bind( 'vmouseup', this._vmouseupHandler );
			this._interactivejCanvas.bind( 'vmouseup', this._vmousemoveHandler );
			this._interactivejCanvas.bind( 'vclick', this._vclickHandler );
		}
		else if ( BUILD_TYPE === 'dev' ) {
			libs.logWarn( 'No Interactive Surface has been set.' );
		}
	};

	/**
	 * Unbinds the input events on the canvas when the update manager has been stopped
	 *
	 * @private
	 * @method _onSystemTickManagerActiveEnd
	 */
	InputController.prototype._onSystemTickManagerActiveEnd = function() {
		if ( this._interactivejCanvas ) {
			this._interactivejCanvas.unbind( 'vmousedown', this._vmousedownHandler );
			this._interactivejCanvas.unbind( 'vmouseup', this._vmouseupHandler );
			this._interactivejCanvas.unbind( 'vmouseup', this._vmousemoveHandler );
			this._interactivejCanvas.unbind( 'vclick', this._vclickHandler );
		}
		else if ( BUILD_TYPE === 'dev' ) {
			libs.logWarn( 'No Interactive Surface has been set.' );
		}
	};

	/**
	 * Notifies the event bus that the down event is fired. It passes the event X and Y coordinates of the event
	 * @private
	 * @method _vmousedown
	 */
	InputController.prototype._vmousedown = function( e ) {
		libs.globalEventBus.invoke( 'libs.event.notify.canvasPointerDown', this._toPixelXY( e ) );
	};

	/**
	 * Notifies the event bus that the up event has fired. It passes the event X and Y coordinates of the event
	 * @private
	 * @method _vmouseup
	 */
	InputController.prototype._vmouseup = function( e ) {
		libs.globalEventBus.invoke( 'libs.event.notify.canvasPointerUp', this._toPixelXY( e ) );
	};

	/**
	 * Notifies the event bus that the move event has fired. It passes the event X and Y coordinates of the event
	 * @private
	 * @method _vmousemove
	 */
	InputController.prototype._vmousemove = function( e ) {
		e.preventDefault();
		libs.globalEventBus.invoke( 'libs.event.notify.canvasPointerMove', this._toPixelXY( e ) );
	};

	/**
	 * Notifies the event bus that a click event has fired. It passes the event X and Y coordinates of the event
	 * @private
	 * @method _vmouseclick
	 */
	InputController.prototype._vmouseclick = function( e ) {
		libs.globalEventBus.invoke( 'libs.event.notify.canvasPointerClick', this._toPixelXY( e ) );
	};

	/**
	 * Calculates the X and Y positions from a mouse event
	 * @private
	 * @method _toPixelXY
	 * @return {object} Object containing X and Y coordinates
	 */
	InputController.prototype._toPixelXY = function( e ) {
		var offset = this._interactivejCanvas.offset();
		var pixelX, pixelY, srcX, srcY;
		srcX = e.pageX - offset.left;
		srcY = e.pageY - offset.top;

		pixelX = srcX * (this._interactiveSurface.canvas.width / this._interactiveSurface.styleWidth);
		pixelX = Math.floor( pixelX );

		pixelY = srcY * (this._interactiveSurface.canvas.height / this._interactiveSurface.styleHeight);
		pixelY = Math.floor( pixelY );

		return { x: pixelX, y: pixelY };
	};


	libs.client.core.InputController = InputController;

}());