/**
 * The core components needed for the client
 *
 * @module libs_ClientCore
 */

goog.provide( 'libs.client.core.Preloader' );

goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );


(function() {
	"use strict";

	var defaultSettings = {
		maxConnections: 5
	};

	var clientInfo = libs.client.core.clientInformation;
	libs.client.assetUrlPrefix = window['EXPORTED_VIRGO_ROOT_PATH'] || '';

	/**
	 * The Preloader class handles the loading of assets
	 * @class Preloader
	 * @constructor
	 */
	var Preloader = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._manifest = [];
		this._loadedImages = {};

		this._assetsLoadedDOM = false;
		this._assetsLoadedCanvas = false;

		this._handleGameLoading = null;

		$( window ).load( libs.callback( function() {
			that._assetsLoadedDOM = true;
			that._checkAllAssetsLoaded();
		} ) );

		libs.globalEventBus.connect( 'libs.event.notify.manifestsToLoad', function( manifestArray ) {
			that._onManifestsToLoad( manifestArray );
		} );
		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gameLoading', function( activity ) {
			that._onGameLoadingStart( activity );
		} );
	};

	/**
	 * Takes an array of manifests and combines them into a single array of assets to load
	 * @private
	 * @method _onManifestsToLoad
	 * @param {array} manifestArray
	 */
	Preloader.prototype._onManifestsToLoad = function( manifestArray ) {
		for ( var i = 0; i < manifestArray.length; ++i ) {
			var array = manifestArray[i].map( function( obj ) {
				var src = obj.src;
				if ( !src.match( /^(#|http\:)/i ) ) {
					obj.src = libs.client.assetUrlPrefix + src;
				}
				return obj;
			} );
			this._manifest = this._manifest.concat( array );
		}
	};

	/**
	 * Attaches to the GameLoadingStart activity and stats the downloading of files specified in the manifests
	 *
	 * @private
	 * @method _onGameLoadingStart
	 * @param {object} activity
	 */
	Preloader.prototype._onGameLoadingStart = function( activity ) {
		var that = this;

		if ( that._manifest.length > 0 ) {
			this._handleGameLoading = activity.join();


			this._loadJsonFiles( that._manifest, function() {
				that._loadManifest( that._manifest, that._loadedImages );
			} );
		} else {
			libs.logError( 'Preloader._onGameLoadingStart: nothing in manifest to load' );
		}
	};

	/**
	 * Loads in an array of JSON files and adds the assets to the global manifest, firing a callback when completed
	 *
	 * @private
	 * @method _loadJsonFiles
	 * @param {array} manifest
	 * @param {function} completeCallback
	 */
	Preloader.prototype._loadJsonFiles = function( manifest, completeCallback ) {
		var getFileDirectory = function( filePath ) {
			if ( filePath.indexOf( "/" ) === -1 ) {
				return filePath.substring( 0, filePath.lastIndexOf( '\\' ) );
			}
			else {
				return filePath.substring( 0, filePath.lastIndexOf( '/' ) );
			}
		};

		// load JSON files first that contain further images to load
		var jsonManifest = [];
		for ( var i = 0; i < manifest.length; ++i ) {
			var obj = manifest[i];

			//IE 8 workaround for type undefined error
			for ( var key in obj ) {
				if ( key === 'type' ) {
					if ( obj[key] === createjs.LoadQueue.JSON ) {
						jsonManifest.push( obj );
						manifest[i] = undefined; // leaves an empty slot in array, doesn't change the size
					}
				}
			}
		}

		if ( jsonManifest.length > 0 ) {
			var jsonLoader = new createjs.LoadQueue();
			jsonLoader.setMaxConnections( this._settings.maxConnections );

			jsonLoader.addEventListener( "error", function( e ) {
				libs.log( "ERROR: Could not load JSON asset '" + e.item.src + "' [" + e.text + "]" );
			} );

			jsonLoader.addEventListener( "fileload", function( e ) {
				if ( e.result instanceof String ) {
					libs.log( "ERROR: Failed to parse JSON file '" + e.item.src + "' [" + e.result + "]" );
				} else if ( e.result instanceof Array ) {
					e.result.forEach( function( sub ) {
						if ( sub.jsonType === "textureAtlas" ) {
							// add loaded object to the original manifest
							sub.type = createjs.LoadQueue.IMAGE;
							// change the image path to point to the same location as the JSON file
							sub.src = getFileDirectory( e.item.src ) + '/' + sub.src;
							manifest.push( sub );
						}
					} );
				} else {
					libs.log( "ERROR: Unknown type encountered whilst loading JSON files ('" + e.result.toString() + "')" );
				}
			} );

			jsonLoader.addEventListener( 'complete', function( e ) {
				completeCallback();
			} );

			jsonLoader.loadManifest( jsonManifest );
		} else {
			completeCallback();
		}
	};

	/**
	 * Loads all the items in the manifest an images arrays
	 *
	 * @private
	 * @method _loadJsonFiles
	 * @param {array} manifest
	 * @param {array} images
	 */
	Preloader.prototype._loadManifest = function( manifest, images ) {
		var that = this;
		var createLoader = new createjs.LoadQueue( false, null, true );

		if ( clientInfo.getAudioQuality() === "LQ" ) {
			libs.log( "Using [AudioTag] audio technology" );
		} else {
			createLoader.installPlugin( createjs.Sound );
			if ( clientInfo.forceUseHTMLAudio() ) {
				if ( !createjs.Sound.registerPlugins( [createjs.HTMLAudioPlugin] ) ) {
					libs.log( "WARNING: SoundJS will not work in this browser" );
				}
			} else {
				if ( !createjs.Sound.initializeDefaultPlugins() ) {
					libs.log( "WARNING: SoundJS will not work in this browser" );
				}
			}

			libs.log( "Using " + createjs.Sound.activePlugin + " audio technology" );

			createjs.Sound.alternateExtensions = ["mp3"];
		}

		var lastPercent = 0;

		var imageDataLoading = 0;
		var allLoaded = false;
		var checkForEnd = function() {
			if ( !imageDataLoading && allLoaded ) {
				libs.globalEventBus.invoke( 'libs.event.notify.loadingProgress', 100 );
				that._assetsLoadedCanvas = true;
				that._checkAllAssetsLoaded();
			}
		};

		createLoader.setMaxConnections( this._settings.maxConnections );

		createLoader.addEventListener( "complete", function() {
			allLoaded = true;
			checkForEnd();
		} );

		createLoader.addEventListener( "progress", function( e ) {
			var percent = Math.round( e.loaded * 100 );
			if ( percent > lastPercent ) {
				lastPercent = percent;
				libs.globalEventBus.invoke( 'libs.event.notify.loadingProgress', percent );
			}
		} );

		createLoader.addEventListener( "error", function( e ) {
			libs.logError( "Could not load asset '" + e.item.src + "' [" + e.text + "]" );

			// do not show error message for sound
			if ( e.item.src.indexOf( "sound" ) === -1 ) {
				var positiveButton = {
					func: function() {
						window.location.reload( true );
					},
					text: libs.getTranslation( 'RETRY' )
				};
				libs.globalEventBus.invoke( 'libs.event.request.showCustomError', libs.getTranslation( 'Server_communcation_lost' ), null, positiveButton );
			}
		} );

		createLoader.addEventListener( "fileload", function( e ) {
			var itemType = e.item.type;

			if ( createjs.LoadQueue.JAVASCRIPT === itemType ) {
				document.body.appendChild( e.result ); //Eval the javascript
			} else if ( createjs.LoadQueue.IMAGE === itemType ) {
				if ( e.item.data ) {
					e.item.data.forEach( function( sub ) {
						// Android 2.X doesn't support toDataUrl so you want to
						/// use the canvas as an image. Drawing canvases is very
						// slow on iOS so we want to convert the canvas to an image
						// if possible.

						var img = new Image();
						var canvas;
						var isSubImage = true;
						var func = sub.func || e.item.func;
						if ( func ) {
							canvas = func( e.result, sub, e.item );

							var canvasData = canvas.toDataURL( "image/png" );

							if ( canvasData.indexOf( "data:image/png" ) === 0 ) {
								img.onload = function() {
									imageDataLoading--;
									checkForEnd();
								};
								imageDataLoading++;
								img.src = canvasData;
							}
							else {
								img = canvas;
							}
							isSubImage = false;
						}

						images[sub.id] = {
							type: "metagu_sub_image",
							image: func ? img : e.result,
							x: isSubImage ? sub.x : 0,
							y: isSubImage ? sub.y : 0,
							width: sub.width,
							height: sub.height,
							trimmedOffsetX: sub.trimmedOffsetX || 0,
							trimmedOffsetY: sub.trimmedOffsetY || 0,
							untrimmedSize: sub.untrimmedSize
						};
					} );
				} else {
					var img = e.result;
					img.trimmedOffsetX = 0;
					img.trimmedOffsetY = 0;
					images[e.item.id] = img;
				}
			}
		} );

		createLoader.loadManifest( manifest );
	};

	/**
	 * Checks to see if all assets have loaded then removes itself from the activity set in
	 *
	 * @private
	 * @method _checkAllAssetsLoaded
	 */
	Preloader.prototype._checkAllAssetsLoaded = function() {
		$( window ).orientationchange();
		if ( this._assetsLoadedDOM && this._assetsLoadedCanvas ) {
			libs.loadedImages = this._loadedImages;
			libs.globalEventBus.invoke( 'libs.event.notify.imagesLoaded' );
			this._handleGameLoading.leave();
			this._handleGameLoading = null;
		}
	};

	libs.client.core.Preloader = Preloader;
}());