/**
 * The core components needed for the client
 *
 * @module libs_ClientCore
 */

goog.provide( 'libs.setTimeoutTick' );
goog.provide( 'libs.clearTimeoutTick' );
goog.provide( 'libs.setIntervalTick' );
goog.provide( 'libs.clearIntervalTick' );
goog.provide( 'libs.setIntervalCountTick' );
goog.provide( 'libs.clearIntervalCountTick' );

/**
 * Handles all the library timer events
 *
 * @class timeoutTicks
 * @method _tickTimerEvent
 */
(function() {
	"use strict";

	var timeProperties = {
		tickTimerList: [],
		tickTimerId: 0,
		tickManagerActive: false
	};

	/**
	 * This is called on each tick, and passed the delta time to each of the functions in timeProperties.tickTimerList
	 *
	 * @private
	 * @method _tickTimerEvent
	 * @param {number} timeDelta
	 */
	var _tickTimerEvent = function( timeDelta ) {
		var index = 0;
		var length = timeProperties.tickTimerList.length;
		do {
			var timer = timeProperties.tickTimerList[index];
			if ( !timer ) {
				continue;
			}
			if ( timer.count === 0 ) {
				timeProperties.tickTimerList.splice( index, 1 );
			} else {
				if ( timer.trackTime > timer.time ) {
					timer.trackTime = 0;
					timer.count--;
					timer.callback( timer.count ); // arguments and the closure are bound to the function
				}
			}
			timer.trackTime += timeDelta;
		} while ( ++index < length );
	};

	/**
	 * Used to log an error if any errors occur in a timer
	 *
	 * @private
	 * @method _timerTickManagerError
	 * @param {string} name
	 */
	var _timerTickManagerError = function( name ) {
		if ( !timeProperties.tickManagerActive ) {
			libs.logError( name + ' cannot run because \'tickManager\' isn\'t active' );
			return true;
		}
		return false;
	};

	/**
	 * Creates a setTimeout that uses the game time instead of the system time
	 *
	 * @public
	 * @method libsSetTimeoutTick
	 * @param {function} callback - The function to call when complete
	 * @param {number} time - The time in milliseconds of how long the time should run for
	 * @param {object} [closure] - The context of the function
	 */
	var libsSetTimeoutTick = function( callback, time, closure ) {
		if ( _timerTickManagerError( 'setTimeoutTick' ) ) {
			return 0;
		}
		var argv = Array.apply( null, arguments );
		argv.splice( 0, 3 );
		timeProperties.tickTimerId++;
		timeProperties.tickTimerList.push( {
			count: 1,
			id: timeProperties.tickTimerId,
			callback: callback.apply.bind( callback, closure || this, argv ),
			time: time,
			trackTime: 0
		} );
		return timeProperties.tickTimerId;
	};

	/**
	 * Creates a setInterval that uses the game time instead of the system time
	 *
	 * @public
	 * @method libsSetIntervalTick
	 * @param {function} callback - The function to call when complete
	 * @param {number} time - The time in milliseconds of how long the time should run for
	 * @param {object=} closure - The context of the function
	 */
	var libsSetIntervalTick = function( callback, time, closure ) {
		if ( _timerTickManagerError( 'setIntervalTick' ) ) {
			return 0;
		}
		var argv = Array.apply( null, arguments );
		argv.splice( 0, 3 );
		timeProperties.tickTimerId++;
		timeProperties.tickTimerList.push( {
			id: timeProperties.tickTimerId,
			count: -1,
			callback: callback.apply.bind( callback, closure || this, argv ),
			trackTime: 0,
			time: time
		} );
		return timeProperties.tickTimerId;
	};

	/**
	 * Creates a setInterval that uses the game time instead of the system time and runs for the passed number of times
	 *
	 * @public
	 * @method libsSetIntervalCountTick
	 * @param {function} callback - The function to call when complete
	 * @param {number} time - The time in milliseconds of how long the time should run for
	 * @param {number} count - The number of times to run the setInterval
	 * @param {object} closure - The context of the function
	 */
	var libsSetIntervalCountTick = function( callback, time, count, closure ) {
		if ( _timerTickManagerError( 'setIntervalCountTick' ) ) {
			return 0;
		}
		var argv = Array.apply( null, arguments );
		argv.splice( 0, 4 );
		timeProperties.tickTimerId++;
		timeProperties.tickTimerList.push( {
			id: timeProperties.tickTimerId,
			count: count || 1,
			callback: callback.apply.bind( callback, closure || this, argv ),
			trackTime: 0,
			time: time
		} );
		return timeProperties.tickTimerId;
	};

	/**
	 * Clears a timer with the given ID
	 *
	 * @public
	 * @method libsClearTimerTick
	 * @param {number} id - The ID of the timer to clear
	 */
	var libsClearTimerTick = function( id ) {
		var index = 0;
		var length = timeProperties.tickTimerList.length;
		do {
			if ( !timeProperties.tickTimerList[index] ) {
				continue;
			}
			if ( timeProperties.tickTimerList[index].id === id ) {
				timeProperties.tickTimerList.splice( index, 1 );
			}
		} while ( ++index < length );
	};

	/**
	 * Waits for the event bus to be initialized, and then connects to the relevant activities
	 *
	 * @private
	 * @method waitForEventBus
	 */
	var waitForEventBus = setInterval( function() {
		if ( !libs.globalEventBus ) {
			return;
		}
		libs.globalEventBus.connect( 'libs.event.notify.tick', _tickTimerEvent );
		libs.globalEventBus.connectActivityStart( 'libs.activity.system.tickManagerActive', function() {
			timeProperties.tickManagerActive = true;
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.system.tickManagerActive', function() {
			timeProperties.tickManagerActive = false;
		} );
		clearInterval( waitForEventBus );
	}, 100 );


	libs.setTimeoutTick = libsSetTimeoutTick;
	libs.clearTimeoutTick = libsClearTimerTick;
	libs.setIntervalTick = libsSetIntervalTick;
	libs.clearIntervalTick = libsClearTimerTick;
	libs.setIntervalCountTick = libsSetIntervalCountTick;
	libs.clearIntervalCountTick = libsClearTimerTick;

}());