/**
 * The core components needed for the client
 *
 * @module libs_ClientCore
 */

goog.provide( 'libs.client.core.UpdateManager' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.core.clientInformation' );


(function() {
	"use strict";

	var defaultSettings = {};

	var clientInfo = libs.client.core.clientInformation;
	var isMobileDevice = clientInfo.getDeviceType() === 'mobile';
	var isDesktopDeviceType = clientInfo.getDeviceType() === 'desktop';
	var isChrome = clientInfo.isChrome();

	var UpdateManager = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._hasPausedForScreenRotation = false;
		this._hasPausedForBlur = false;

		this._unFreezeTimeoutId = 0;

		this._lastFrame = 0;
		this._lastFrameTime = 0;

		this._timeDelta = 1;
		this._timeDeltaPausedFrame = 0;

		this._debugDeltaMultiplier = 1;

		if ( libs.client.core.clientInformation.isNativeAndroidBrowser() ) {
			// The s3 version of the android browser doesn't restart the event loop when the window regains focus.
			// Once this is called it restarts the normal loop as well. This should do nothing on the other browsers.
			$( window ).bind( "focus", libs.callback( function() {
					that._restartGameHack();
				}
			) );
		}

		$( window ).bind( "blur", libs.callback( function() {
				that._hasPausedForBlur = true;
				that._pause();
			}
		) );

		this._handleSystemTickManager = null;

		this._defaultUpdateFunctionsSet = false;
		this._defaultUpdateFunctions = {
			update: function() {},
			drawAllBase: function() {},
			drawDirtyBase: function() {},
			drawAllTop: function() {},
			drawDirtyTop: function() {},
			drawOverlay: function() {}
		};

		this._currentUpdateFunctions = this._defaultUpdateFunctions;

		this._needFullDrawBase = true;
		this._needFullDrawTop = true;
		this._canvasOverlayModeEnabled = false;
		this._framesToPerformFullRender = 0;

		libs.globalEventBus.connectActivityStart( 'libs.activity.system.tickManagerActive', function( activity ) {
			that._onSystemTickManagerActiveStart( activity );
		} );
		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gamePlaying', function( activity ) {
			that._onGamePlayingStart( activity );
		} );

		libs.globalEventBus.connect( 'libs.event.notify.keyPressed', function( keyCode, keyChar ) { that._onKeyPressed( keyCode, keyChar ); } );

		libs.globalEventBus.connectActivityStart( 'libs.activity.display.error', function( activity, isFatal ) {
			that._onDisplayErrorStart( activity, isFatal );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.display.error', function() {
			that._onDisplayErrorEnd();
		} );
		libs.globalEventBus.connect( 'libs.event.notify.orientationChanged', function( orientation ) {
			that._onNotifyOrientationChanged( orientation );
		} );

		libs.globalEventBus.connect( 'libs.event.notify.defaultUpdateFunctions', function( updateFunctions ) {
			that._onNotifyDefaultUpdateFunctions( updateFunctions );
		} );
		libs.globalEventBus.connect( 'libs.event.request.useOverrideUpdateFunctions', function( updateFunctions ) {
			that._onRequestUseOverrideUpdateFunctions( updateFunctions );
		} );
		libs.globalEventBus.connect( 'libs.event.request.clearOverrideUpdateFunctions', function() {
			that._onRequestClearOverrideUpdateFunctions();
		} );

		libs.globalEventBus.connect( 'libs.event.request.canvasFullDraw', function( fullDrawBase, fullDrawTop ) {
			that._onRequestCanvasFullDraw( fullDrawBase, fullDrawTop );
		} );
		libs.globalEventBus.connect( 'libs.event.request.performFullRender', function( framesToPerformFullRender ) {
			that._onRequestPerformFullRender( framesToPerformFullRender );
		} );
		libs.globalEventBus.connect( 'libs.event.request.enableCanvasOverlayMode', function() {
			that._onRequestEnableCanvasOverlayMode();
		} );
		libs.globalEventBus.connect( 'libs.event.request.disableCanvasOverlayMode', function() {
			that._onRequestDisableCanvasOverlayMode();
		} );
	};

	UpdateManager.prototype._onSystemTickManagerActiveStart = function( activity ) {
		this._handleSystemTickManager = activity.join();
	};

	UpdateManager.prototype._onGamePlayingStart = function( activity ) {
		libs.globalEventBus.invokeActivity( 'libs.activity.system.tickManagerActive' );

		this._run();
	};

	UpdateManager.prototype._onKeyPressed = function( keyCode, keyChar ) {
		if ( BUILD_TYPE === 'dev' ) {
			if ( keyCode === 111 ) { // '/' key
				this._debugDeltaMultiplier /= 2;
				libs.log( 'UpdateManager: game is running at ' + this._debugDeltaMultiplier + 'x normal speed' );
			} else if ( keyCode === 106 ) { // '*' key
				this._debugDeltaMultiplier *= 2;
				libs.log( 'UpdateManager: game is running at ' + this._debugDeltaMultiplier + 'x normal speed' );
			}
		}
	};

	UpdateManager.prototype._onDisplayErrorStart = function( activity, isFatal ) {
		if ( libs.globalEventBus.isActivityRunning( 'libs.activity.systemState.gamePlaying' ) ) {
			this._pause();
		}
	};

	UpdateManager.prototype._onDisplayErrorEnd = function() {
		if ( libs.globalEventBus.isActivityRunning( 'libs.activity.systemState.gamePlaying' ) ) {
			this._resume();
		}
	};

	UpdateManager.prototype._onNotifyOrientationChanged = function( orientation ) {
		if ( libs.globalEventBus.isActivityRunning( 'libs.activity.systemState.gamePlaying' ) ) {
			if ( isMobileDevice ) {
				if ( orientation === 'landscape' ) {
					this._hasPausedForScreenRotation = false;
					this._resume( true );
				} else {
					this._hasPausedForScreenRotation = true;
					this._pause();
				}
			}
		}
	};

	UpdateManager.prototype._onNotifyDefaultUpdateFunctions = function( updateFunctions ) {
		if ( !this._defaultUpdateFunctionsSet ) {
			this._defaultUpdateFunctions = {
				update: updateFunctions.update || function() { },
				drawAllBase: updateFunctions.drawAllBase || function() {},
				drawDirtyBase: updateFunctions.drawDirtyBase || function() {},
				drawAllTop: updateFunctions.drawAllTop || function() {},
				drawDirtyTop: updateFunctions.drawDirtyTop || function() {},
				drawOverlay: updateFunctions.drawOverlay || function() {}
			};
			this._currentUpdateFunctions = this._defaultUpdateFunctions;
			this._defaultUpdateFunctionsSet = true;
		} else {
			libs.logError( 'Cannot overwrite default update functions once set' );
		}
	};

	UpdateManager.prototype._onRequestUseOverrideUpdateFunctions = function( updateFunctions ) {
		this._currentUpdateFunctions = {
			update: updateFunctions.update || function() {},
			drawAllBase: updateFunctions.drawAllBase || function() {},
			drawDirtyBase: updateFunctions.drawDirtyBase || function() {},
			drawAllTop: updateFunctions.drawAllTop || function() {},
			drawDirtyTop: updateFunctions.drawDirtyTop || function() {},
			drawOverlay: updateFunctions.drawOverlay || function() {}
		};

		this._framesToPerformFullRender = 2;
	};

	UpdateManager.prototype._onRequestClearOverrideUpdateFunctions = function() {
		this._currentUpdateFunctions = this._defaultUpdateFunctions;
		this._framesToPerformFullRender = 2;
	};

	UpdateManager.prototype._onRequestCanvasFullDraw = function( fullDrawBase, fullDrawTop ) {
		if ( fullDrawBase ) {
			this._needFullDrawBase = true;
		}

		if ( fullDrawTop ) {
			this._needFullDrawTop = true;
		}
	};

	UpdateManager.prototype._onRequestPerformFullRender = function( framesToPerformFullRender ) {
		this._framesToPerformFullRender = framesToPerformFullRender;
	};

	UpdateManager.prototype._onRequestEnableCanvasOverlayMode = function() {
		this._canvasOverlayModeEnabled = true;

		this._currentUpdateFunctions.drawAllBase();
		this._currentUpdateFunctions.drawAllTop();

		libs.surfaceBase.ctx.drawImage( libs.surfaceTop.canvas, 0, 0 );

		libs.globalEventBus.invoke( 'libs.event.request.changeTopCanvasZIndex', libs.surfaceSettings.top.zIndexMovedToFront );
	};

	UpdateManager.prototype._onRequestDisableCanvasOverlayMode = function() {
		this._canvasOverlayModeEnabled = false;
		this._framesToPerformFullRender = 2;

		libs.globalEventBus.invoke( 'libs.event.request.changeTopCanvasZIndex', libs.surfaceSettings.top.zIndexNormal );
	};

	UpdateManager.prototype._run = function() {
		this._timeDelta = 1;

		// Using a callback proxy allows the requestAnimationFrame() callback to be executed within this click touch event closure started
		// by the 'sound enabled' question box. This means sounds will be able to play at all points on older versions of chrome.
		// http://www.createjs.com/tutorials/Mobile%20Safe%20Approach/
		window.requestAnimationFrame( libs.callback( this._run, this ) );

		if ( !(this._handleSystemTickManager && this._handleSystemTickManager.isJoined()) ) {
			if ( this._hasPausedForBlur ) {
				this._hasPausedForBlur = false;
				this._resume();
			}

			return;
		}

		this._lastFrameTime = this._lastFrame;
		this._lastFrame = Date.now();

		if ( this._lastFrameTime ) {
			this._timeDelta = this._lastFrame - this._lastFrameTime;
			this._timeDelta *= this._debugDeltaMultiplier;
		}

		libs.globalEventBus.invoke( 'libs.event.notify.tick', this._timeDelta );

		this._currentUpdateFunctions.update( this._timeDelta );

		if ( this._framesToPerformFullRender > 0 ) {
			this._needFullDrawBase = true;
			this._needFullDrawTop = true;
			$( '#viewport' ).redrawGentle();

			--this._framesToPerformFullRender;
		}

		if ( this._canvasOverlayModeEnabled ) {
			this._currentUpdateFunctions.drawOverlay();
		} else {
			// force full draw on desktop chrome, as otherwise it goes into lower res rendering mode
			if ( this._needFullDrawBase || (isDesktopDeviceType && isChrome) ) {
				this._currentUpdateFunctions.drawAllBase();
				this._needFullDrawBase = false;
			} else {
				this._currentUpdateFunctions.drawDirtyBase();
			}

			if ( this._needFullDrawTop || (isDesktopDeviceType && isChrome) ) {
				this._currentUpdateFunctions.drawAllTop();
				this._needFullDrawTop = false;
			} else {
				this._currentUpdateFunctions.drawDirtyTop();
			}
		}
	};

	UpdateManager.prototype._pause = function() {
		if ( this._handleSystemTickManager && this._handleSystemTickManager.isJoined() ) {
			if ( this._unFreezeTimeoutId ) {
				this._unFreezeTimeoutId = libs.clearTimeout( this._unFreezeTimeoutId );
			}

			this._timeDeltaPausedFrame = Date.now() - this._lastFrame;
			this._handleSystemTickManager.leave();
			this._handleSystemTickManager = null;
		}
	};

	UpdateManager.prototype._resume = function( pausedForScreenRotation ) {
		if ( !(this._handleSystemTickManager && this._handleSystemTickManager.isJoined()) ) {
			if ( !this._hasPausedForBlur && !this._hasPausedForScreenRotation ) {
				var that = this;

				var unfreezeTime = 0;
				if ( isMobileDevice && pausedForScreenRotation ) {
					unfreezeTime = 1400;
				}

				// give time for orientation to finish before resuming the game
				this._unFreezeTimeoutId = libs.setTimeout( function() {
					that._lastFrame = Date.now() - that._timeDeltaPausedFrame;
					libs.globalEventBus.invokeActivity( 'libs.activity.system.tickManagerActive' );
					libs.globalEventBus.invoke( 'libs.event.request.performFullRender', 1 );
				}, unfreezeTime );
			}
		}
	};

	UpdateManager.prototype._restartGameHack = function() {
		window.requestAnimationFrame( function() {
		} );
	};


	libs.client.core.UpdateManager = UpdateManager;

}());