goog.provide( 'libs.client.displayHandler' );
goog.provide( 'libs.client.displayHandler.DesktopDisplayHandler' );

goog.require( 'libs.globalEventBus' );


(function() {
	"use strict";

	var defaultSettings = {
		gameWidth: undefined,
		gameHeight: undefined
	};

	var selectors = {
		body: $( '#body' ),
		header: $( '#header' ),
		footer: $( '#footer' ),
		main: $( '#main' ),
		backgroundImages: $( '.background img' ),
		introVideoContainer: $( '#intro-video-container' )
	};

	var updateSelectors = function() {
		selectors.backgroundImages = $( '.background img' );
		selectors.backgroundDivContainer = $( '.background div' );
	};

	var DesktopDisplayHandler = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._currentFontScale = 1;

		this._widthToHeight = this._settings.gameWidth / this._settings.gameHeight;

		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.systemInit', function( activity ) { that._onSystemInitStart( activity ); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.systemState.systemInit', function() { that._onSystemInitEnd(); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gamePlaying', function( activity ) { that._onGamePlayingStart( activity ); } );
	};

	DesktopDisplayHandler.prototype._onSystemInitStart = function( activity ) {
		var that = this;

		$( window ).bind( 'resize', function( event ) {
			// called 3 times in a row, as, for some reason, that sorts out background images positions not being correct
			for ( var i = 0; i < 3; ++i ) {
				that._resizeCanvas();
			}
		} );
	};

	DesktopDisplayHandler.prototype._onSystemInitEnd = function() {
		$( window ).trigger( 'resize' );
	};

	DesktopDisplayHandler.prototype._onGamePlayingStart = function( activity ) {
		$( window ).trigger( 'resize' );

		// in case any elements on screen on game load effect page dimensions
		libs.setImmediate( function() {
			$( window ).trigger( 'resize' );
		} );
	};

	DesktopDisplayHandler.prototype._resizeCanvas = function() {
		var newWidth = $( window ).width();
		var headerHeight = (!selectors.header.length || !selectors.header.is( ':visible' )) ? 0 : selectors.header.height();
		var footerHeight = (!selectors.footer.length || !selectors.footer.is( ':visible' )) ? 0 : selectors.footer.height();
		var newHeight = window.innerHeight - headerHeight - footerHeight;
		var newWidthToHeight = newWidth / newHeight;
		var emCapValue = 0.5816; // taken from body css added below, this before is the point text becomes unreadable;

		if ( newWidthToHeight > this._widthToHeight ) {
			newWidth = newHeight * this._widthToHeight;
			selectors.main.height( newHeight );
			selectors.main.width( newWidth );
		} else { // window height is too high relative to desired game height
			newHeight = newWidth / this._widthToHeight;
			selectors.main.width( newWidth );
			selectors.main.height( newHeight );
		}

		if ( selectors.introVideoContainer.length ) {
			selectors.introVideoContainer.height( newHeight );
		}

		this._resizeBackgroundImages( newHeight, headerHeight );
		this._resizeBackgroundDivs( newHeight, headerHeight, newWidthToHeight );

		var newFontSizeScale = emCapValue;
		if ( (newWidth / this._settings.gameWidth) > emCapValue ) {
			newFontSizeScale = newWidth / this._settings.gameWidth;
		}

		if ( newFontSizeScale !== this._currentFontScale ) {
			selectors.body.css( 'fontSize', newFontSizeScale + 'em' );
			libs.globalEventBus.invoke( 'libs.event.notify.fontScaleChanged', newFontSizeScale );
			this._currentFontScale = newFontSizeScale;
		}

		libs.setTimeout( function() {
			libs.globalEventBus.invoke( 'libs.event.notify.displayHeightChanged', window.innerHeight );
		}, 10 );
	};

	DesktopDisplayHandler.prototype._resizeBackgroundImages = function( newHeight, headerHeight ) {
		updateSelectors();

		var viewport = window;

		var bgW = selectors.backgroundImages.width();
		selectors.backgroundImages.css( {
			'left': ($( viewport ).width() / 2) - (bgW / 2),
			'position': 'absolute',
			'height': newHeight,
			'top': headerHeight
		} );
	};

	DesktopDisplayHandler.prototype._resizeBackgroundDivs = function( newHeight, headerHeight, newWidth ) {
		updateSelectors();

		var bgW = selectors.backgroundImages.width();

		selectors.backgroundDivContainer.css( {
			'left': ($( window ).width() / 2) - (bgW / 2),
			'position': 'absolute',
			'height': newHeight,
			'top': headerHeight,
			'width': bgW
		} );
	};

	libs.client.displayHandler.DesktopDisplayHandler = DesktopDisplayHandler;

}());