goog.provide( 'libs.client.displayHandler' );
goog.provide( 'libs.client.displayHandler.MobileDisplayHandler' );

goog.requireThirdParty( 'screenfull' );
goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.utils.DOM' );


(function() {
	"use strict";

	var defaultSettings = {
		gameWidth: undefined,
		gameHeight: undefined,
		resizeForFooter: false,
		text: {
			orientation: {
				fontSize: 50
			}
		}
	};

	var selectors = {
		body: $( '#body' ),
		viewport: $( '#viewport' ),
		footer: $( '#footer' ),
		main: $( '#main' ),
		backgroundElements: $( '.background' ),
		backgroundImages: $( '.background img' ),
		swipeUpContainer: $( '#swipe-up-container' ),
		orientationContainer: $( '#orientation-container' ),
		title: $( '#orientation-title' )
	};

	var updateSelectors = function() {
		selectors.backgroundElements = $( '.background' );
		selectors.backgroundImages = $( '.background img' );
		selectors.backgroundDivContainer = $( '.background div' );
	};

	var doubleCheckOrientationDelay = 400;
	var resizeCanvasDelay = 1000;
	var scrollToDelay = 500;
	var disablingScrollEventDelay = 1;
	var disableScrollEventTime = 1000;

	var clientInfo = libs.client.core.clientInformation;

	var logMessage = function( message ) {
		if ( false ) {
			libs.log( message );
		}
	};

	var MobileDisplayHandler = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._widthToHeight = this._settings.gameWidth / this._settings.gameHeight;

		this._orientationHandlingEnabled = true;

		this._lastOrientation = null;
		this._currentOrientation = null;
		this._doubleCheckOrientation = null;
		this._orientationChangeInProgress = false;
		this._orientationTimeoutId = 0;

		this._scrollEventStatus = 'enabled';
		this._scrollTimeoutId = 0;
		this._useScrollToURLBarFix = false;
		if ( clientInfo.isNativeAndroidBrowser() ) {
			this._useScrollToURLBarFix = true;
		} else if ( clientInfo.isIPhoneLike() && clientInfo.isIOS6() && clientInfo.isIOSBrowser() ) {
			this._useScrollToURLBarFix = true;
		}

		this._requireSwipeUp = (clientInfo.getIOSVersion() === 7 || clientInfo.isIOS8()) && clientInfo.isIPhoneLike() && ( clientInfo.isIOSBrowser() || clientInfo.isChrome() );

		this._windowInnerWidth = window.innerWidth;
		this._windowInnerHeight = window.innerHeight;
		this._snugVerticalFit = false;

		this._supportsFullscreenAPI = clientInfo.supportsFullscreenAPI();
		this._isIEMobile = clientInfo.isIEMobile();

		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.systemInit', function( activity ) { that._onSystemInitStart( activity ); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.systemState.systemInit', function() { that._onSystemInitEnd(); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gamePlaying', function( activity ) { that._onGamePlayingStart( activity ); } );
		libs.globalEventBus.connect( 'libs.event.notify.displayHeightChanged', function( height ) { that._onNotifyDisplayHeightChanged( height );} );
		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );
	};

	MobileDisplayHandler.prototype._onSystemInitStart = function( activity ) {
		var that = this;

		var jWin = $( window );

		jWin.bind( 'orientationchange', function( event ) { that._onOrientationChange( event ); } );

		if ( this._supportsFullscreenAPI ) {
			document.addEventListener( screenfull.raw.fullscreenchange, function() { $( window ).orientationchange(); } );
		}

		if ( window.visibilityChangeEventName && clientInfo.isChrome() ) {
			document.addEventListener( window.visibilityChangeEventName, function() {
				// a bug in the fullscreen api for chrome means that on version 33 and under, pressing the home screen then coming back
				// into the game will stop the screen from being centred correctly when in fullscreen. So come out of it in these cases
				if ( clientInfo.getChromeVersion() < 34 ) {
					screenfull.exit();
				}

				// force an orientation change when regaining focus to the game to
				// fix nexus 5 issue of it allowing game to be played in portrait
				if ( document[window.visibilityHiddenName] === false ) {
					libs.setTimeout( function() {
						$( window ).orientationchange();
					}, 1000 );
					libs.setTimeout( function() {
						$( window ).orientationchange();
					}, 2000 );
				}
			} );
		}

		if ( this._useScrollToURLBarFix ) {
			jWin.bind( 'scroll',
				function( event ) {
					if ( that._scrollEventStatus !== 'disabled' ) {
						libs.setTimeout( function() {
							that._resizeCanvas( false, false );
						}, 250 );
					}
				}
			);
		}

		if ( clientInfo.isIPhoneLike() ) {
			updateSelectors();
			selectors.backgroundElements.css( 'min-height', '320px' );
		}

		if ( clientInfo.getIOSVersion() >= 7 && clientInfo.isIPhoneLike() && clientInfo.isIOSBrowser() ) {
			selectors.body.addClass( 'iOS7OrAbove' );
		}

		if ( !this._requireSwipeUp ) {
			selectors.swipeUpContainer.remove();
		}

		libs.setInterval( function() {
			that._screenSizeWatcher();
		}, 1000 );
	};

	MobileDisplayHandler.prototype._onSystemInitEnd = function() {
		$( window ).orientationchange();
	};

	MobileDisplayHandler.prototype._onGamePlayingStart = function( activity ) {
		if ( this._currentOrientation === 'portrait' ) {
			libs.globalEventBus.invoke( 'libs.event.notify.orientationChanged', 'portrait' );
		} else { // landscape
			libs.globalEventBus.invoke( 'libs.event.notify.orientationChanged', 'landscape' );
		}

		libs.globalEventBus.invoke( 'libs.event.notify.snugVerticalFit', this._snugVerticalFit );
	};

	MobileDisplayHandler.prototype._onOrientationChange = function( event ) {
		this._orientationChangeInProgress = true;

		this._screenSizeWatcher();

		var jqueryOrientation = event.orientation;
		var windowOrientation = this._windowInnerHeight > this._windowInnerWidth ? 'portrait' : 'landscape';

		logMessage( 'orientationchange jquery: ' + jqueryOrientation );
		logMessage( 'orientationchange window: ' + windowOrientation );
		logMessage( 'windowInnerHeight: ' + this._windowInnerHeight + ' - windowInnerWidth: ' + this._windowInnerWidth );

		if ( this._doubleCheckOrientation === null ) {
			this._doubleCheckOrientation = true;
		}

		this._currentOrientation = windowOrientation;

		var that = this;
		libs.setTimeout( function() {
			that._resizeCanvas( true, true );
		}, resizeCanvasDelay );

		this._updateOrientation();

		if ( this._doubleCheckOrientation ) {
			libs.setTimeout( function() {
				that._doubleCheckOrientation = false;
				$( window ).orientationchange();
			}, doubleCheckOrientationDelay );
		} else {
			this._orientationChangeInProgress = false;
			this._doubleCheckOrientation = null;
		}
	};

	MobileDisplayHandler.prototype._onNotifyDisplayHeightChanged = function( height ) {
		if ( !this._orientationChangeInProgress ) {
			if ( this._requireSwipeUp ) {
				this._resizeCanvas( true, true );
				this._slideHandler();
			} else if ( !this._isIEMobile ) {
				var hideURLBar = this._useScrollToURLBarFix && this._currentOrientation === 'landscape';
				this._resizeCanvas( hideURLBar, false );
			}
		}
	};

	MobileDisplayHandler.prototype._screenSizeWatcher = function() {
		var newWindowInnerWidth = window.innerWidth;
		if ( newWindowInnerWidth !== this._windowInnerWidth ) {
			logMessage( 'innerWidth: old:' + this._windowInnerWidth + ' - new:' + newWindowInnerWidth );
			this._windowInnerWidth = newWindowInnerWidth;
			libs.globalEventBus.invoke( 'libs.event.notify.displayWidthChanged', this._windowInnerWidth );
		}

		var newWindowInnerHeight = window.innerHeight;
		if ( newWindowInnerHeight !== this._windowInnerHeight ) {
			logMessage( 'innerHeight: old:' + this._windowInnerHeight + ' - new:' + newWindowInnerHeight );
			this._windowInnerHeight = newWindowInnerHeight;
			libs.globalEventBus.invoke( 'libs.event.notify.displayHeightChanged', this._windowInnerHeight );
		}
	};

	MobileDisplayHandler.prototype._updateOrientation = function() {
		if ( this._requireSwipeUp ) {
			if ( this._currentOrientation === 'portrait' || this._lastOrientation === 'portrait' && this._currentOrientation === 'landscape' ) {
				this._showSwipeUp( false );
			}
		}

		if ( this._lastOrientation !== this._currentOrientation ) {
			if ( this._currentOrientation === 'portrait' ) {
				this._showOrientationMessage( true );
				libs.globalEventBus.invoke( 'libs.event.notify.orientationChanged', 'portrait' );
			} else { // landscape
				this._showOrientationMessage( false );
				libs.globalEventBus.invoke( 'libs.event.notify.orientationChanged', 'landscape' );
			}

			this._lastOrientation = this._currentOrientation;
		}
	};

	MobileDisplayHandler.prototype._resizeCanvas = function( doUrlBarFix, canHideUrlBarResizeCanvas ) {
		var that = this;
		var footerHeight = 0;

		var newWidth = $( window ).innerWidth();
		var newHeight = (clientInfo.getIOSVersion() >= 7) ? $( window ).innerHeight() : window.innerHeight;

		var newWidthToHeight = newWidth / newHeight;

		if ( this._settings.resizeForFooter ) {
			//For some reason the actual footer height was not enough, by doubling we get extra space. 
			footerHeight = (!selectors.footer.length || !selectors.footer.is( ':visible' )) ? 0 : selectors.footer.height() * 2;
		}

		if ( clientInfo.isIOS7() ) {
			newHeight = newWidth / this._widthToHeight;
			newHeight -= footerHeight;
			// when in native or web app, reduce height of game to account for status bar at the top
			// (only needed on widescreen IOS phones as narrow screen have enough height already due to aspect ratio)
			if ( clientInfo.isIPhoneLike() && newWidth > 480 && (clientInfo.isIOSWebApp() || clientInfo.isIOSNativeApp() || clientInfo.isChrome()) ) {
				newHeight -= 20; // status bar is 20 pixels high for IOS7; doesn't exist for IOS 6 or 8
				newWidth = newHeight * this._widthToHeight;
			}
		} else {
			if ( newWidthToHeight > this._widthToHeight ) {
				newHeight -= footerHeight;
				newWidth = newHeight * this._widthToHeight;
			} else { // window height is too high relative to desired game height
				newHeight = newWidth / this._widthToHeight;
				newHeight -= footerHeight;
			}
		}

		var snugVerticalFit = newWidthToHeight > this._widthToHeight;
		if ( snugVerticalFit !== this._snugVerticalFit ) {
			this._snugVerticalFit = snugVerticalFit;
			libs.globalEventBus.invoke( 'libs.event.notify.snugVerticalFit', this._snugVerticalFit );
		}

		newWidth = Math.round( newWidth );
		newHeight = Math.round( newHeight );

		selectors.main.width( newWidth );
		selectors.main.height( newHeight );

		selectors.main.css( 'top', ((window.innerHeight / 2) - (selectors.main.height() / 2)) );
		selectors.main.css( 'left', 0 );

		var newFontSizeScale = newWidth / this._settings.gameWidth;

		if ( newFontSizeScale !== this._currentFontScale ) {
			selectors.body.css( 'fontSize', newFontSizeScale + 'em' );
			libs.globalEventBus.invoke( 'libs.event.notify.fontScaleChanged', newFontSizeScale );
			this._currentFontScale = newFontSizeScale;
		}

		if ( doUrlBarFix ) {
			this._hideUrlBar( canHideUrlBarResizeCanvas );
		}

		libs.setIntervalCount( function() {
			that._resizeBackgroundImages();
			that._resizeBackgroundDivs();
		}, 125, 12 );
	};

	MobileDisplayHandler.prototype._resizeBackgroundImages = function() {
		updateSelectors();

		var bgW = selectors.backgroundImages.width();
		var bgH = selectors.backgroundImages.height();

		var bgLeft = ($( window ).width() / 2) - (bgW / 2);
		var bgTop = (window.innerHeight / 2) - (bgH / 2);

		if ( !this._snugVerticalFit ) {
			selectors.backgroundImages.css( {
				'left': bgLeft,
				'top': bgTop,
				'width': "auto",
				'height': "100%"
			} );
		} else {
			selectors.backgroundImages.css( {
				'left': bgLeft,
				'top': bgTop,
				'width': "100%",
				'height': "auto"
			} );
		}
	};

	MobileDisplayHandler.prototype._resizeBackgroundDivs = function() {
		updateSelectors();

		var bgW = selectors.backgroundImages.width();
		var bgH = selectors.backgroundImages.height();

		var bgLeft = ($( window ).width() / 2) - (bgW / 2);
		var bgTop = (window.innerHeight / 2) - (bgH / 2);

		selectors.backgroundDivContainer.css( {
			'left': bgLeft,
			'top': bgTop,
			'width': bgW,
			'height': bgH
		} );
	};


	MobileDisplayHandler.prototype._disableActingOnScroll = function() {
		var that = this;

		if ( this._scrollEventStatus === 'enabled' ) {
			this._scrollEventStatus = 'disabling';
			libs.setTimeout( function() {
				that._scrollEventStatus = 'disabled';
				libs.setTimeout( function() {
					that._scrollEventStatus = 'enabled';
				}, disableScrollEventTime );
			}, disablingScrollEventDelay );
		}
	};

	MobileDisplayHandler.prototype._hideUrlBar = function( canHideUrlBarResizeCanvas ) {
		var that = this;

		var doCanvasResize = this._useScrollToURLBarFix && canHideUrlBarResizeCanvas;

		libs.clearTimeout( this._scrollTimeoutId );
		if ( doCanvasResize ) {
			document.body.style.height = '' + (window.outerHeight + 100) + 'px';
		}

		this._scrollTimeoutId = libs.setTimeout( function() {
			that._disableActingOnScroll();
			if ( clientInfo.isIOS() || (!that._supportsFullscreenAPI && !that._isIEMobile) ) {
				window.scrollTo( 0, 0 );
			}
			libs.setTimeout( function() {
				if ( doCanvasResize && that._currentOrientation === 'landscape' ) {
					document.body.style.height = '0px';
				}
				if ( doCanvasResize ) {
					that._resizeCanvas( true, false );
				}
			}, 100 );
		}, scrollToDelay );
	};

	MobileDisplayHandler.prototype._slideHandler = function() {
		var wH = $( window ).height(),
			wHi = window.innerHeight;
		if ( wH > wHi ) {
			this._showSwipeUp( true );
		} else {
			this._showSwipeUp( false );
		}
	};

	MobileDisplayHandler.prototype._showSwipeUp = function( show ) {
		if ( show === undefined || show === true ) {
			libs.client.utils.DOM.fadeIn( selectors.swipeUpContainer, 150 );
		} else {
			libs.client.utils.DOM.fadeOut( selectors.swipeUpContainer, 150,
				function() {
					$( this )['css']( 'opacity', 0.8 );
				}
			);
			$( window ).trigger( 'resize' );
		}
	};

	MobileDisplayHandler.prototype._showBackgrounds = function( show ) {
		if ( show === undefined || show === true ) {
			selectors.backgroundElements.show();
		} else {
			selectors.backgroundElements.hide();
		}
	};

	MobileDisplayHandler.prototype._showOrientationMessage = function( show ) {
		if ( this._orientationHandlingEnabled && (show === undefined || show === true) ) {
			this._showBackgrounds( false );

			selectors.viewport.css( 'visibility', 'hidden' );
			selectors.main.css( 'visibility', 'hidden' );

			libs.clearTimeout( this._orientationTimeoutId );
			this._orientationTimeoutId = libs.setTimeout( function() {
				selectors.orientationContainer.show();
				selectors.viewport.css( 'visibility', 'visible' );
			}, 1400 );
		} else {
			selectors.viewport.css( 'visibility', 'hidden' );
			selectors.orientationContainer.hide();

			this._showBackgrounds( true );

			libs.clearTimeout( this._orientationTimeoutId );
			this._orientationTimeoutId = libs.setTimeout( function() {
				selectors.orientationContainer.hide();
				selectors.viewport.css( 'visibility', 'visible' );
				selectors.main.css( 'visibility', 'visible' );
			}, 1400 );
		}
	};

	MobileDisplayHandler.prototype._onNotifyLanguageChanged = function( langCode ) {
		libs.client.utils.DOM.setText( selectors.title, libs.getTranslation( 'Please_rotate_your_device_to_landscape' ), this._settings.text.orientation );
	};


	libs.client.displayHandler.MobileDisplayHandler = MobileDisplayHandler;

}());