goog.provide( 'libs.client.hillsVegas.createLiveServer' );

goog.require( 'libs.getDefaultFormatForCurrencyCode' );
goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.utils.misc' );


(function() {
	"use strict";

	var initUrl = "../Engine/init.asp";
	var betUrl = "../Engine/bet.asp";
	var closeUrl = "../Engine/close.asp";
	var pollUrl = "../Engine/poll.asp";
	var historyURL = "../Engine/history.asp";

	var pollTime = 5000;
	var pollWaitForResponseTimeout = 20000;
	var pollDisableTimeAfterBetFinished = 2000;


	var LiveServer = function() {
		var that = this;
		this._pollTimerId = null;
		this._pollResponseTimeoutId = null;
		this._onInitCallback = null;
		this._startingReelSet = 0;
		this._lastReceivedTimestamp = new BigNumber( 0 );

		this._comms = createHttpComms();
		this._comms.onReplyReceived( function( req, good, error ) {that._replyReceived( req, good, error );} );

		this._reelSets = [];

		this._seqNumber = 0;

		this._inBet = false;
		this._ignorePollAfterBetFinished = false;
		this._ignorePollTimeoutId = null;

		this._languageCode = 'en_GB';
		this._currencyCode = 'GBP';
		this._currencyFormatting = libs.getDefaultFormatForCurrencyCode( this._currencyCode ); // Format to GBP until we receive handshake response

		libs.globalEventBus.connect( 'libs.event.notify.betRecoveryStarted', function() { that._onNotifyBetRecoveryStarted(); } );

		this._startNextPoll();
	};

	LiveServer.prototype._onNotifyBetRecoveryStarted = function() {
		this._inBet = true;
	};

	LiveServer.prototype.onInit = function( callback ) {
		this._onInitCallback = callback;
	};

	LiveServer.prototype.connect = function() {
		this._sendMessage( initUrl, {}, this._initReceived, this._networkError );
	};

	LiveServer.prototype.getReels = function() {
		return this._reelSets;
	};

	LiveServer.prototype.getStartingReelSet = function() {
		return this._startingReelSet;
	};

	LiveServer.prototype.startBet = function( stake, lines, postBackData ) {
		this._inBet = true;
		var jpId = 0;
		var jackpotObject = this.jackpots.map( function( jpValue ) { return { 'id': jpId++, 'money': jpValue }; } );
		var jsonRequest = {
			'betSelection': { 'stake': stake, 'lines': lines, 'checkpointId': this._seqNumber },
			'jackpots': jackpotObject,
			'postData': postBackData
		};
		this._sendMessage( betUrl, jsonRequest, this._betReceived, this._networkError );
	};

	LiveServer.prototype.betFinished = function( inFreeSpin ) {
		var that = this;

		if ( inFreeSpin ) {
			this._seqNumber++;
			libs.globalEventBus.invoke( 'libs.event.notify.serverBetFinished' );
		} else {
			this._seqNumber = 0;
			this._sendMessage( closeUrl, {}, this._closeReceived, this._networkError );
		}

		this._inBet = false;

		libs.clearTimeout( this._ignorePollTimeoutId );
		this._ignorePollAfterBetFinished = true;
		this._ignorePollTimeoutId = libs.setTimeout( function() {
			that._ignorePollAfterBetFinished = false;
		}, pollDisableTimeAfterBetFinished );
	};

	LiveServer.prototype.loadHistory = function() {

		this._sendMessage( historyURL, {}, this._historyReceived, this._networkError );
	};

	LiveServer.prototype._networkError = function( status ) {
		libs.globalEventBus.invoke( 'libs.event.request.showError', libs.getTranslation( 'Connection_failed_please_restart_the_game' ), true );
	};

	LiveServer.prototype._getError = function( reply ) {
		var errorMsg = null;
		var obj = reply['error'];
		if ( obj ) {
			errorMsg = {
				id: parseInt( obj['id'], 10 ),
				text: obj['text']
			};
		}
		return errorMsg;
	};

	LiveServer.prototype._startNextPoll = function( sendImmediately ) {
		var that = this;
		this._clearPollTimeouts();
		this._pollTimerId = libs.setTimeout( function() {
			that._pollTimerId = null;

			// Set up timeout wait timer
			that._pollResponseTimeoutId = libs.setTimeout( function() {
				// If we don't receive a response in X seconds, do another poll
				//libs.log( "No response, restarting poll timer..." );
				that._pollResponseTimeoutId = null;
				that._startNextPoll( true );
			}, pollWaitForResponseTimeout );

			// Send poll message
			that._sendMessage( pollUrl, {}, that._pollReceived );

		}, ( sendImmediately ? 1 : pollTime ) );
	};

	LiveServer.prototype._clearPollTimeouts = function() {
		if ( this._pollTimerId ) {
			libs.clearTimeout( this._pollTimerId );
			this._pollTimerId = null;
		}
		if ( this._pollResponseTimeoutId ) {
			libs.clearTimeout( this._pollResponseTimeoutId );
			this._pollResponseTimeoutId = null;
		}
	};

	LiveServer.prototype._handleError = function( error ) {
		var fatal = error.id !== 1;
		libs.globalEventBus.invoke( 'libs.event.request.showError', error.text, fatal );
		if ( !fatal ) {
			this._inBet = false;
			this._startNextPoll( true );
		}
	};

	LiveServer.prototype._initReceived = function( reply ) {
		try {
			var error = this._getError( reply );
			if ( error ) {
				this._handleError( error );
			} else {
				this._readInit( reply );
			}
		}
		catch ( err ) {
			libs.logException( err );
			this._handleError( { id: 0, text: 'Invalid response received from server' } );
		}
	};

	LiveServer.prototype._readInit = function( reply ) {

		var initObj = reply['data'];
		this._reelSets = initObj['reel_set'];
		this._startingReelSet = initObj['startingReelSet'] || 0;
		this._readCommonState( reply );

		this._currencyCode = initObj['currency']['code'];
		this._currencyFormatting = libs.getDefaultFormatForCurrencyCode( this._currencyCode );

		this.availableStakesPerLine = initObj['stake']['steps'].map( function( stake ) { return new BigNumber( stake ); } );
		this.startingStake = new BigNumber( initObj['stake']['starting'] );

		var res = null;
		if ( initObj['bet'] ) {
			res = this._readAllBetInfo( initObj['bet'] );
			this.startingStake = res['bet_stake'];
			this._seqNumber = res['checkpointId'];
			this.balance = this.balance.minus( res['win'] );
		}
		var playerPersistentState = initObj['playerServerState'];

		if ( this._onInitCallback ) {
			libs.globalEventBus.invoke( 'libs.event.request.changeLanguage', this._languageCode );
			libs.globalEventBus.invoke( 'libs.event.request.changeCurrency', this._currencyCode, this._currencyFormatting );
			this._onInitCallback( res, playerPersistentState );
		}
	};


	LiveServer.prototype._betReceived = function( reply ) {
		try {
			var error = this._getError( reply );
			if ( error ) {
				this._handleError( error );
				libs.globalEventBus.invoke( 'libs.event.notify.serverBetAborted' );
			} else {
				this._readBet( reply );
			}
		}
		catch ( err ) {
			libs.logException( err );
			this._handleError( { id: 0, text: 'Invalid response received from server' } );
		}
	};

	LiveServer.prototype._readBet = function( reply ) {
		this._readCommonState( reply );
		var betResult = this._readAllBetInfo( reply['data'] );
		libs.globalEventBus.invoke( 'libs.event.notify.serverBetResult', betResult );
	};

	LiveServer.prototype._readAllBetInfo = function( reply ) {
		return reply;
	};

	LiveServer.prototype._closeReceived = function( reply ) {
		try {
			var error = this._getError( reply );

			if ( error ) {
				this._handleError( error );
			} else {
				this._readClose( reply );
			}
		}
		catch ( err ) {
			libs.logException( err );
			this._handleError( { id: 0, text: 'Invalid response received from server' } );
		}
	};

	LiveServer.prototype._readClose = function( reply ) {
		this._readCommonState( reply );
		libs.globalEventBus.invoke( 'libs.event.notify.serverBetFinished' );
	};

	LiveServer.prototype._historyReceived = function( reply ) {
		try {
			var error = this._getError( reply );

			if ( error ) {
				this._handleError( error );
			} else {
				this._readHistory( reply );
			}
		}
		catch ( err ) {
			libs.logException( err );
			this._handleError( { id: 0, text: 'Invalid response received from server' } );
		}
	};

	LiveServer.prototype._readHistory = function( reply ) {

		libs.globalEventBus.invoke( 'libs.event.notify.serverBetHistory', reply.data );
	};

	LiveServer.prototype._pollReceived = function( reply ) {
		var that = this;
		that._startNextPoll();

		if ( that._inBet || that._ignorePollAfterBetFinished ) {
			return;
		}
		try {
			var error = that._getError( reply );

			if ( error ) {
				//Don't raise error in polls.
			} else {
				that._readPoll( reply );
			}
		}
		catch ( err ) {
			libs.logException( err );
			//this._handleError( { id: 0, text: 'Invalid response received from server' } );
		}
	};

	LiveServer.prototype._readPoll = function( reply ) {
		this._readCommonState( reply );

		libs.globalEventBus.invoke( 'libs.event.notify.serverPolled' );
	};

	LiveServer.prototype._sendMessage = function( url, jsonObject, good, error ) {

		var xmlString = libs.xmlUtils.wrapJsonInXml( "jsonRequest", jsonObject );
		this._comms.sendMessage( url, xmlString, good, error );
	};

	LiveServer.prototype._replyReceived = function( req, good, error ) {

		if ( req.status === 200 && good ) {
			try {
				var jsonObject = libs.xmlUtils.unwrapJsonFromXml( "jsonResponse", req.responseText );
				good.call( this, jsonObject );
			}
			catch ( err ) {
				console.error( "Error parsing response from server" );
				console.error( err );
				error.call( this, 0 );
			}
		} else if ( error ) {
			error.call( this, req.status );
		}
	};


	LiveServer.prototype._readCommonState = function( reply ) {

		var state = reply;

		// get timestamp, only update values if it's a more recent message
		var timeStamp = new BigNumber( state['timestamp'] );

		// Removed some operands as the variable is instantiated in the constructor
		if ( this._lastReceivedTimestamp.equals( 0 ) || timeStamp.greaterThan( this._lastReceivedTimestamp ) ) {

			this._lastReceivedTimestamp = timeStamp;
			this.balance = libs.client.utils.misc.toBigNumber( state["money"] );
			if ( state['jackpots'] ) {
				this.jackpots = state['jackpots'].map( function( jackpot ) {
					return libs.client.utils.misc.toBigNumber( jackpot["money"] );
				} );
			}
		}
	};


	/*
	 * Connection classes.
	 */

	var HttpComms = function() {
		this._replyReceivedCallback = null;
	};

	HttpComms.prototype.onReplyReceived = function( callback ) {
		this._replyReceivedCallback = callback;
	};

	HttpComms.prototype.sendMessage = function( url, xmlStr, good, error ) {
		var req = new XMLHttpRequest();
		var that = this;

		req.onreadystatechange = function() {
			if ( req.readyState === 4 && that._replyReceivedCallback ) {
				that._replyReceivedCallback( req, good, error );
			}
		};

		req.open( "POST", url );
		req.timeout = 60000; //This is a Hills requirement
		req.setRequestHeader( "Content-Type", "application/x-www-form-urlencoded" );

		req.send( "Data=" + encodeURIComponent( xmlStr ).replace( "%20", "+" ) );
	};


	var createHttpComms = function() {
		return new HttpComms();
	};


	libs.client.hillsVegas.createLiveServer = function() {
		return new LiveServer();
	};


}());
