goog.provide( 'libs.client.hillsVegas.LocalServerPlatformInterface' );

goog.require( 'libs.MersenneTwister' );

goog.require( 'game.logic' );


(function() {
	"use strict";

	var LocalServerPlatformInterface = function( settings ) {
		settings = settings || {};
		this._settings = settings;

		this._languageCode = ( settings ? settings.languageCode : null ) || 'en_GB'; // default to en-GB language code
		this._currencyCode = ( settings ? settings.currencyCode : null ) || 'XXX'; // default to XXX currency code (points)
		this._currentGameRngString = '';
		this._balance = new BigNumber( 0 );
		this._winningsThisGame = new BigNumber( 0 );
		this._freeSpinThisGame = false;
		this._jackpotValues = [];
		this._mersenneTwister = new libs.MersenneTwister(); // Automatically seeds to time

		// Jackpot mode determines jackpot behaviour.
		// 'zero': all jackpots permanently set to zero
		// 'average': will always return the pre-calculated value for each jackpot according to the GD spreadsheet
		// 'maxvalue': (default) will always use the maximum possible value for the jackpots (required by WH in demo play)
		// 'emulate': emulates jackpot accumulation and payout
		this._jackpotMode = settings.jackpotMode || 'average';
		this.connect();
	};

	LocalServerPlatformInterface.forcedRngString = '';
	LocalServerPlatformInterface.forcedStaticDataString = '';

	///////////////////////// Setup

	LocalServerPlatformInterface.prototype.connect = function() {
		this._jackpotValues = [];
		if ( game.logic.GameData.jackpots && game.logic.GameData.jackpots.list ) {
			for ( var jackpotIndex = 0; jackpotIndex < game.logic.GameData.jackpots.list.length; ++jackpotIndex ) {
				var jp = game.logic.GameData.jackpots.list[jackpotIndex];
				var seedValue = jp.maximumValue || 0;
				if ( this._jackpotMode === 'average' ) {
					seedValue = jp.spreadsheetAverage || 0;
				} else if ( this._jackpotMode === 'emulate' ) {
					seedValue = jp.initialValue || 0;
				} else if ( this._jackpotMode === 'zero' ) {
					seedValue = 0;
				}
				seedValue = new BigNumber( seedValue );
				this._jackpotValues.push( { 'id': jackpotIndex, 'value': seedValue } );
			}
		}
		this._balance = new BigNumber( this._settings.initialBalance || 100000 );
	};

	LocalServerPlatformInterface.prototype.getBalance = function() {
		return this._balance;
	};

	LocalServerPlatformInterface.prototype.getJackpotValue = function( jackpotId ) {
		return new BigNumber( this._jackpotValues[jackpotId]['value'] );
	};

	LocalServerPlatformInterface.prototype.getJackpotCount = function() {
		return this._jackpotValues.length;
	};

	LocalServerPlatformInterface.prototype._commitOutstandingWin = function() {
		if ( this._winningsThisGame.greaterThan( 0 ) ) {
			this._balance = this._balance.plus( this._winningsThisGame );
			this._winningsThisGame = new BigNumber( 0 );
		}
	};

	LocalServerPlatformInterface.prototype.takeStake = function( currentSequence, stakeValue, win, eligibleForJackpot ) {

		if ( currentSequence === 0 ) {
			this._freeSpinThisGame = false;
			this._balance = this._balance.minus( stakeValue );

			if ( eligibleForJackpot ) {
				// add stake contributions to jackpots if in emulate mode
				if ( this._jackpotMode === 'emulate' ) {
					if ( game.logic.GameData.jackpots && game.logic.GameData.jackpots.list ) {
						for ( var jackpotIndex = 0; jackpotIndex < game.logic.GameData.jackpots.list.length; ++jackpotIndex ) {
							var jp = game.logic.GameData.jackpots.list[jackpotIndex];

							var contrib = new BigNumber( jp.stakeContribution );
							var newValue = this._jackpotValues[jackpotIndex]['value'].plus( contrib );

							if ( newValue.greaterThan( jp.maximumValue ) ) {
								newValue = jp.maximumValue;
							}
							this._jackpotValues[jackpotIndex]['value'] = newValue;
						}
					}
				}
			}
		} else {
			this._freeSpinThisGame = true;
		}

		this._commitOutstandingWin();
		this._winningsThisGame = win;
	};

	///////////////////////// Jackpots

	LocalServerPlatformInterface.prototype.claimJackpot = function( jackpotId ) {

		if ( game.logic.GameData.jackpots && game.logic.GameData.jackpots.list ) {
			var jp = game.logic.GameData.jackpots.list[jackpotId];
			var won = this.getJackpotValue( jackpotId );
			if ( this._jackpotMode === 'emulate' ) {
				this._jackpotValues[jackpotId]['value'] = new BigNumber( jp.reseedValue || 0 );
			}
			return won;
		} else {
			throw new Error( "Logic error: LocalServerPlatformInterface.prototype.claimJackpot() called but GameData contains no jackpots section!" );
		}
	};

	///////////////////////// Cashing out

	LocalServerPlatformInterface.prototype.getLanguageCode = function() {
		return this._languageCode;
	};

	LocalServerPlatformInterface.prototype.getCurrencyCode = function() {

		return this._currencyCode;
	};

	///////////////////////// Random number retrieval


	LocalServerPlatformInterface.prototype._checkForcedRandomNumbers = function( type, lowestInclusive, highestInclusive, numberCount ) {

		var numbers = [];

		if ( LocalServerPlatformInterface.forcedRngString && LocalServerPlatformInterface.forcedRngString.length > 0 ) {
			var setRngsArr = LocalServerPlatformInterface.forcedRngString.split( "," );
			var numbersToGet = Math.min( setRngsArr.length, numberCount );
			for ( var i = 0; i < numbersToGet; ++i ) {
				var chance = Number( setRngsArr.shift() );
				if ( (chance < lowestInclusive) || (chance > highestInclusive) ) {
					chance = lowestInclusive + ( chance % ( highestInclusive - lowestInclusive ) ); // in the forced version, just modulus the number so it comes within range.
				}
				numbers.push( chance );
			}
			LocalServerPlatformInterface.forcedRngString = setRngsArr.join( "," );
		}

		return numbers;
	};


	LocalServerPlatformInterface.prototype.getRandomNumber = function( type, lowestInclusive, highestInclusive ) {
		return this.getRandomNumbers( type, lowestInclusive, highestInclusive, 1 )[0];
	};

	LocalServerPlatformInterface.prototype.getRandomNumbers = function( type, lowestInclusive, highestInclusive, numberCount ) {

		numberCount = numberCount || 1;
		var numbers = this._checkForcedRandomNumbers( type, lowestInclusive, highestInclusive, numberCount );
		var numbersLeft = numberCount - numbers.length;
		var range = ( highestInclusive - lowestInclusive ) + 1;
		for ( var i = 0; i < numbersLeft; ++i ) {
			var mtResult = this._mersenneTwister.generateInt() % range;
			var num = mtResult + lowestInclusive;
			numbers.push( num );
		}

		if ( numbers.length > 0 ) {
			if ( this._currentGameRngString.length > 0 ) {
				this._currentGameRngString += ",";
			}
			this._currentGameRngString += numbers.join( "," );
		}

		return numbers;
	};

	LocalServerPlatformInterface.prototype.closeBet = function() {
		this._commitOutstandingWin();

		// reset balance so it never runs out
		if ( !this._freeSpinThisGame ) {
			var balanceThreshold = this._settings.balanceResetThreshold || 10;
			if ( this._balance.lessThan( balanceThreshold ) ) {
				this._balance = new BigNumber( this._settings.initialBalance || 100000 );
			}
		}
	};

	LocalServerPlatformInterface.prototype.startFirstBetOperation = function() {
		this._currentGameRngString = '';
	};

	LocalServerPlatformInterface.prototype.getPreviousRngString = function() {
		return this._currentGameRngString;
	};

	///////////////////////// SDK only static methods

	LocalServerPlatformInterface.setupSDKJackpots = function() {

	};

	LocalServerPlatformInterface.forceCreateSession = function() {

		return true;
	};

	LocalServerPlatformInterface.forceRng = function( rngString ) {
		LocalServerPlatformInterface.forcedRngString = rngString;
	};


	LocalServerPlatformInterface.forceStaticData = function( staticDataString ) {
		LocalServerPlatformInterface.forcedStaticDataString = staticDataString;
	};


	LocalServerPlatformInterface.getForcedStaticData = function() {
		return LocalServerPlatformInterface.forcedStaticDataString;
	};

	libs.client.hillsVegas.LocalServerPlatformInterface = LocalServerPlatformInterface;

}());