/**
 * The core components needed for the client
 *
 * @module libs_ClientCore
 */

goog.provide( 'libs.client.languages.LanguageController' );
goog.provide( 'libs.getTranslation' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.languages.languagePack' );


/**
 * A controller for listening to events that request a language change
 * and sends whatever is listening the appropriate translations
 *
 * @author Kelvin Bland <kelvin.bland@ingg.com>
 *
 */

(function() {
	"use strict";

	var defaultSettings = {};

	var currentLanguage = '';
	var currentTranslations = libs.client.languages.languagePack['en_GB'];

	/**
	 * Handles the changing of languages and notifies the game of the changes
	 * @class LanguageController
	 * @constructor
	 * @static
	 */
	var LanguageController = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;
		this._onRequestChangeLanguage( 'en_GB' );

		libs.globalEventBus.connect( 'libs.event.request.changeLanguage', function( langCode ) {
			that._onRequestChangeLanguage( langCode );
		} );
		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.systemInit', function( activity ) {
			that._onSystemInitStart( activity );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.systemState.gameInit', function() {
			that._onGameInitEnd();
		} );
	};

	/**
	 * A function sending out a notification that the language preference has been set
	 *
	 * Searches through the list of available language dictionaries and sends these out to be picked up
	 * by any component listening for this change.
	 *
	 *
	 * @param {string} langCode - The language preference being set. Language codes are in the form of ISO language then country code, concatenated by a dash e.g. en_GB
	 * @private
	 */
	LanguageController.prototype._onRequestChangeLanguage = function( langCode ) {
        // WeR: Fixed language (gb) for Marketing client
        if (! game.settings.WeR) {
            if ( libs.client.languages.languagePack[langCode] ) {
                currentLanguage = langCode;
            } else {
                if ( BUILD_TYPE === 'dev' ) {
                    libs.logError( 'LanguageController: Failed to change to the language [' + langCode + '] so defaulting to en_GB' );
                }
                currentLanguage = 'en_GB';
            }
            currentTranslations = libs.client.languages.languagePack[currentLanguage];
            libs.globalEventBus.invoke( 'libs.event.notify.languageChanged', currentLanguage );
        }
	};

	/**
	 * This is called as the game starts it's initialization and sets the game language. We fire this first so we can
	 * set up strings for things like the orientation handler and error screen
	 *
	 * @private
	 * @method _onSystemInitStart
	 * @param {object} activity
	 */
	LanguageController.prototype._onSystemInitStart = function( activity ) {
		libs.globalEventBus.invoke( 'libs.event.notify.languageChanged', currentLanguage );
	};

	/**
	 * This is called when the server has been initialized and sets the game language. This is called a second time because
	 * the currency is not made available to the game until the server has been initialized
	 *
	 * @private
	 * @method _onGameInitEnd
	 */
	LanguageController.prototype._onGameInitEnd = function() {
		libs.globalEventBus.invoke( 'libs.event.notify.languageChanged', currentLanguage );
	};

	libs.client.languages.LanguageController = LanguageController;

	/**
	 * Get translation from library
	 *
	 * @param {string} mainKey
	 * @param {Array} [keyArr]
	 * @return {string}
	 */
	libs.getTranslation = function( mainKey, keyArr ) {
		if ( currentTranslations[mainKey] && currentTranslations[mainKey] !== '' ) {
			mainKey = currentTranslations[mainKey];
		} else {
			if ( BUILD_TYPE === 'dev' ) {
				libs.logError( 'LanguageController: Translation key [' + mainKey + '] not found for language ' + currentLanguage );
				mainKey = "@&%$";
			} else if ( libs.client.languages.languagePack['en_GB'][mainKey] ) {
				mainKey = libs.client.languages.languagePack['en_GB'][mainKey];
			}
		}

		if ( keyArr === undefined || keyArr === null ) {
			return mainKey;
		}
		return libs.formatString( mainKey, keyArr );
	};

}());