goog.provide( 'libs.client.presentation.BigWinnerFountain' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.rendering.LegacyAnimatedSprite' );
goog.require( 'libs.client.rendering.SpriteGroup' );
goog.require( 'libs.client.utils.misc' );


(function() {
	"use strict";

	var defaultSettings = {
		gravity: 0.0004,
		particleFrequencies: [
			100,
			50,
			25
		],
		xSpeed: {
			min: -0.2,
			max: 0.2
		},
		ySpeed: {
			min: -0.675,
			max: -0.575
		}
	};

	var defaultSettingsMobile = {
		maxX: 960,
		minX: -20,
		xStart: {
			min: 350,
			max: 450
		},
		yStart: {
			min: 550,
			max: 620
		}
	};

	var defaultSettingsDesktop = {
		maxX: 780,
		minX: 0,
		xStart: {
			min: 250,
			max: 450
		},
		yStart: {
			min: 500,
			max: 540
		}
	};

	var isDesktopDeviceType = libs.client.core.clientInformation.getDeviceType() === 'desktop';
	defaultSettings = libs.client.utils.misc.mergeSettings( defaultSettings, isDesktopDeviceType ? defaultSettingsDesktop : defaultSettingsMobile, true );

	var BigWinnerFountain = function( overrideSettings, images ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._featureActive = false;
		this._images = images;

		this._particles = new libs.client.rendering.SpriteGroup();

		this._particleFrequency = this._settings.particleFrequencies[0];
		this._timeSinceLastParticle = 0;
		this._maxImageIndex = 0;

		libs.globalEventBus.connect( 'libs.event.notify.tick', function( timeDelta ) { that._onNotifyTick( timeDelta ); } );
	};

	BigWinnerFountain.prototype._onNotifyTick = function( timeDelta ) {
		if ( this._featureActive ) {
			this._particles.update( timeDelta );
			this._maybeAddParticle( timeDelta );
		}
	};

	BigWinnerFountain.prototype._maybeAddParticle = function( timeDelta ) {
		var particleImages;
		this._timeSinceLastParticle += timeDelta;
		if ( this._timeSinceLastParticle > this._particleFrequency ) {
			particleImages = this._pickImageSet();
			this._addParticle( particleImages );
			this._timeSinceLastParticle = 0;
		}
	};

	BigWinnerFountain.prototype._pickImageSet = function() {
		var imageIndex = libs.client.utils.misc.getRandomInt( 0, this._maxImageIndex + 1 );
		return this._images[imageIndex];
	};

	BigWinnerFountain.prototype._addParticle = function( images ) {
		var particle, settings;

		settings = {
			xSpeed: this._randInRange( this._settings.xSpeed.min, this._settings.xSpeed.max ),
			ySpeed: this._randInRange( this._settings.ySpeed.min, this._settings.ySpeed.max ),
			x: this._randInRange( this._settings.xStart.min, this._settings.xStart.max ),
			y: this._randInRange( this._settings.yStart.min, this._settings.yStart.max ),
			gravity: this._settings.gravity,
			minX: this._settings.minX,
			maxX: this._settings.maxX
		};

		particle = new Particle( images, settings );
		this._particles.sprites.push( particle );
	};

	BigWinnerFountain.prototype._randInRange = function( bottom, top ) {
		var distance = top - bottom;
		return bottom + (Math.random() * distance);
	};

	BigWinnerFountain.prototype.isDirty = function() {
		return this._featureActive;
	};

	BigWinnerFountain.prototype.draw = function( ctx ) {
		if ( this._featureActive ) {
			this._particles.draw( ctx );
		}
	};

	BigWinnerFountain.prototype.start = function( bigWinLevel ) {
		this._featureActive = this._images.length > 0; // feature can't be active if no images to use
		this._timeSinceLastParticle = 0;
		this._particleFrequency = this._settings.particleFrequencies[bigWinLevel];
		this._maxImageIndex = bigWinLevel;
	};

	BigWinnerFountain.prototype.end = function() {
		this._featureActive = false;
		this._particles.sprites = [];
		this._timeSinceLastParticle = 0;
	};


	var Particle = function( images, settings ) {
		this._sprite = new libs.client.rendering.LegacyAnimatedSprite( images,
			{ fps: 25, autoplay: true, timesToLoop: -1 },
			{ x: settings.x, y: settings.y } );

		this._sprite.update( 0 );
		this._sprite.update( Math.floor( Math.random() * 1000 ) );

		this._xSpeed = settings.xSpeed;
		this._ySpeed = settings.ySpeed;
		this._minX = settings.minX;
		this._maxX = settings.maxX;

		this._gravity = settings.gravity;
	};

	Particle.prototype.draw = function( ctx ) {
		this._sprite.draw( ctx );
	};

	Particle.prototype.update = function( timeDelta ) {
		this._sprite.update( timeDelta );

		this._sprite.y += this._ySpeed * timeDelta;
		this._ySpeed += this._gravity * timeDelta;

		this._sprite.x += this._xSpeed * timeDelta;
		this._checkForBounce();
		//this._checkForFade();
	};

	Particle.prototype._checkForBounce = function() {
		if ( this._minX !== undefined ) {
			if ( this._sprite.x <= this._minX ) {
				this._sprite.x = this._minX + (this._minX - this._sprite.x);
				this._xSpeed = -this._xSpeed;
			}
		}

		if ( this._maxX !== undefined ) {
			var maxSpriteX = this._maxX - this._sprite.width;
			if ( this._sprite.x >= maxSpriteX ) {
				this._sprite.x = maxSpriteX - (this._sprite.x - maxSpriteX);
				this._xSpeed = -this._xSpeed;
			}
		}
	};

//	Particle.prototype._checkForFade = function(){
//		if(this._minX !== undefined){
//			if(this._sprite.x <= this._minX){
//				this._sprite.fadeOut(100);
//			}
//		}
//
//		if(this._maxX !== undefined){
//			var maxSpriteX = this._maxX - this._sprite.width;
//			if(this._sprite.x >= maxSpriteX){
//				this._sprite.fadeOut(100);
//			}
//		}
//	};

	libs.client.presentation.BigWinnerFountain = BigWinnerFountain;

}());