goog.provide( 'libs.client.presentation.ButtonBaseClass' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.presentation.HtmlButton' );


(function() {
	"use strict";

	var defaultSettings = {
		name: undefined,
		fadeInTime: 0,
		fadeOutTime: 0,
		keyMapping: {
			keyCode: null,
			keyChar: null
		},
		model: {
			useBaseModel: true,
			updateEvents: {
				activityStartAndEnd: [],
				activityStart: [],
				activityEnd: [],
				event: []
			},
			downEvent: null,
			upEvent: null,
			shouldBeEnabled: function() { return true; },
			shouldBeVisible: function() { return true; }
		},
		popupSafe: false,
		isMultiLine: true,
		text: {
			title: {
				multiline: {
					maxLines: 2,
					reduceSpaceBetweenLinesBy: 10,
					marginTop: 10
				}
			}
		},
		translations: {
			title: null
		}
	};

	var ButtonBaseClass = function( overrideSettings, selectors ) {
		if ( this.constructor === ButtonBaseClass ) {
			libs.logError( 'Cannot instantiate abstract ButtonBaseClass class' );
		}

		if ( !defaultSettings || !selectors ) {
			libs.logError( 'Required parameters for ButtonBaseClass not supplied' );
		}

		var that = this;

		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		this._selectors = selectors;

		this._button = new libs.client.presentation.HtmlButton( this._selectors.container, this._settings.popupSafe,
			this._settings.model.downEvent, this._settings.model.upEvent );

		this._baseModel = {
			updateEvents: {
				activityStartAndEnd: [],
				activityStart: [],
				activityEnd: [],
				event: []
			},
			downEvent: null,
			upEvent: null,
			shouldBeEnabled: function() { return true; },
			shouldBeVisible: function() { return true; }
		};

		this._isEnabled = true;
		this._isVisible = false;

		libs.globalEventBus.connect( 'libs.event.notify.buttonBaseModel', function( baseModel ) { that._onNotifyButtonBaseModel( baseModel ); } );
		libs.globalEventBus.connect( 'libs.event.notify.keyPressed', function( keyCode, keyChar ) { that._onNotifyKeyPressed( keyCode, keyChar ); } );
		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );
	};

	ButtonBaseClass.prototype._onNotifyButtonBaseModel = function( baseModel ) {
		var that = this;

		if ( this._settings.model.useBaseModel ) {
			this._baseModel = baseModel;
		}

		var updateEvents = this._settings.model.updateEvents;
		jQuery.merge( updateEvents.activityStartAndEnd, this._baseModel.updateEvents.activityStartAndEnd );
		jQuery.merge( updateEvents.activityStart, this._baseModel.updateEvents.activityStart );
		jQuery.merge( updateEvents.activityEnd, this._baseModel.updateEvents.activityEnd );
		jQuery.merge( updateEvents.event, this._baseModel.updateEvents.event );
		jQuery.merge( updateEvents.activityStart, updateEvents.activityStartAndEnd );
		jQuery.merge( updateEvents.activityEnd, updateEvents.activityStartAndEnd );
		jQuery.unique( updateEvents.activityStart );
		jQuery.unique( updateEvents.activityEnd );
		jQuery.unique( updateEvents.event );

		var i;
		var activityName;
		var updateButtonFunc = function() {
			that._updateButton();
		};

		for ( i = 0; i < updateEvents.activityStart.length; ++i ) {
			activityName = updateEvents.activityStart[i];
			libs.globalEventBus.connectActivityStart( activityName, updateButtonFunc );
		}
		for ( i = 0; i < updateEvents.activityEnd.length; ++i ) {
			activityName = updateEvents.activityEnd[i];
			libs.globalEventBus.connectActivityEnd( activityName, updateButtonFunc );
		}
		for ( i = 0; i < updateEvents.event.length; ++i ) {
			var eventName = updateEvents.event[i];
			libs.globalEventBus.connect( eventName, updateButtonFunc );
		}

		this._updateButton();
	};

	ButtonBaseClass.prototype._onNotifyKeyPressed = function( keyCode, keyChar ) {
		if ( this._button.isEnabled() && this._settings.model.downEvent ) {
			if ( keyChar === this._settings.keyMapping.keyChar || keyCode === this._settings.keyMapping.keyCode ) {
				this._settings.model.downEvent();
			}
		}
	};

	ButtonBaseClass.prototype._onNotifyLanguageChanged = function( langCode ) {
		if ( this._settings.translations.title ) {
			var translation = libs.getTranslation( this._settings.translations.title );
			var textSettings = this._settings.text.title;
			if ( this._selectors.container ) {
				libs.client.utils.DOM.setMultiLineText( this._selectors.container, translation, textSettings );
			}
		}
	};

	ButtonBaseClass.prototype._updateButton = function() {
		var shouldBeEnabled = this._baseModel.shouldBeEnabled();
		if ( shouldBeEnabled ) {
			shouldBeEnabled = this._settings.model.shouldBeEnabled();
		}

		if ( this._isEnabled !== shouldBeEnabled ) {
			if ( shouldBeEnabled ) {
				this._button.enable();
			} else {
				this._button.disable();
			}
			this._isEnabled = shouldBeEnabled;
		}

		var shouldBeVisible = this._baseModel.shouldBeVisible();
		if ( shouldBeVisible ) {
			shouldBeVisible = this._settings.model.shouldBeVisible();
		}

		if ( this._isVisible !== shouldBeVisible ) {
			if ( shouldBeVisible ) {
				this._button.show( this._settings.fadeInTime );
			} else {
				this._button.hide( this._settings.fadeOutTime );
			}
			this._isVisible = shouldBeVisible;
		}
	};


	libs.client.presentation.ButtonBaseClass = ButtonBaseClass;

}());