goog.provide( 'libs.client.presentation.ErrorScreen' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.utils.DOM' );
goog.require( 'libs.client.utils.misc' );


(function() {
	"use strict";

	var defaultSettings = {
		links: undefined,
		text: {
			error: {
				fontSize: 24,
				maxHeight: 90,
				maxWidth: 510
			},
			buttons: {
				fontSize: 18,
				maxWidth: 135
			}
		}
	};

	var selectors = {
		container: $( '#error-screen-container' ),
		message: $( '#error-screen-message' ),
		positiveButton: {
			container: $( '#error-screen-button-positive-container' ),
			title: $( '#error-screen-button-positive-title' )
		},
		negativeButton: {
			container: $( '#error-screen-button-negative-container' ),
			title: $( '#error-screen-button-negative-title' )
		}
	};

	selectors.container.hide();

	var clientInfo = libs.client.core.clientInformation;

	var ErrorScreen = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._handleDisplayError = null;
		this._errorScreenEnabled = true;

		this._customNegativeFunc = null;
		this._customPositiveFunc = null;

		this._showErrorScreen( false );

		libs.globalEventBus.connectActivityStart( 'libs.activity.display.error', function( activity ) { that._onDisplayErrorStart( activity ); } );

		libs.globalEventBus.connect( 'libs.event.request.showError', function( errorText, isFatal ) { that._onRequestShowError( errorText, isFatal ); } );
		libs.globalEventBus.connect( 'libs.event.request.showCustomError', function( errorText, negativeButton, positiveButton ) { that._onRequestShowCustomError( errorText, negativeButton, positiveButton ); } );
		libs.globalEventBus.connect( 'libs.event.notify.orientationChanged', function( orientation ) { that._onOrientationChanged( orientation ); } );

		// TODO: Reinstate this for during initial testing phase of a game. Think of a nice way to handle this.
		//libs.onLogException( function( msg, exception ) { that._onShowException( exception ); } );
	};

	ErrorScreen.prototype._onDisplayErrorStart = function( activity ) {
		this._handleDisplayError = activity.join();
	};

	ErrorScreen.prototype._onRequestShowError = function( errorText, isFatal ) {
		libs.globalEventBus.invokeActivity( 'libs.activity.display.error', isFatal );

		if ( isFatal ) {
			this._setErrorScreenText( [errorText, libs.getTranslation( 'OK' ), libs.getTranslation( 'OK' )] );
		} else {
			this._setErrorScreenText( [errorText, libs.getTranslation( 'EXIT' ), libs.getTranslation( 'OK' )] );
		}

		this._setErrorScreenPositiveButtonFunction( null );
		this._setErrorScreenNegativeButtonFunction( null );

		var that = this;
		if ( isFatal ) {
			if ( clientInfo.getDeviceType() === 'mobile' ) {
				this._setErrorScreenNegativeButtonFunction( function() { libs.client.utils.misc.openUrl( that._settings.links.error ); } );
			} else if ( clientInfo.getDeviceType() === 'desktop' ) {
				this._setErrorScreenNegativeButtonFunction( function() { window.open( window.location, '_parent' ).close(); } );
			}
			this._showErrorScreenPositiveButton( false );
			this._showErrorScreenNegativeButton( true );
		} else {
			this._setErrorScreenPositiveButtonFunction( function() { that._hideErrorScreen(); } );
			this._showErrorScreenPositiveButton( true );
			this._showErrorScreenNegativeButton( false );
		}

		this._updateVisibility();
	};

	ErrorScreen.prototype._onRequestShowCustomError = function( errorText, negativeButton, positiveButton ) {
		libs.globalEventBus.invokeActivity( 'libs.activity.display.error', false );

		var negativeButtonText = "";
		var positiveButtonText = "";

		this._customNegativeFunc = null;
		this._customPositiveFunc = null;
		this._setErrorScreenPositiveButtonFunction( null );
		this._setErrorScreenNegativeButtonFunction( null );

		var that = this;
		if ( negativeButton ) {
			negativeButtonText = negativeButton.text || null;
			this._customNegativeFunc = negativeButton.func || function() {};
			this._setErrorScreenNegativeButtonFunction( function() {
				that._customNegativeFunc();
				that._hideErrorScreen();
			} );
			this._showErrorScreenNegativeButton( true );
		} else {
			this._showErrorScreenNegativeButton( false );
		}

		if ( positiveButton ) {
			positiveButtonText = positiveButton.text || null;
			this._customPositiveFunc = positiveButton.func || function() {};
			this._setErrorScreenPositiveButtonFunction( function() {
				that._customPositiveFunc();
				that._hideErrorScreen();
			} );
			this._showErrorScreenPositiveButton( true );
		} else {
			this._showErrorScreenPositiveButton( false );
		}

		this._setErrorScreenText( [errorText, negativeButtonText, positiveButtonText] );

		this._updateVisibility();
	};

	ErrorScreen.prototype._onShowException = function( exception ) {
		var errorText = 'Error';
		if ( typeof exception === 'string' ) {
			errorText += ': ' + exception;
		}
		else if ( typeof exception === 'object' && (exception.name || exception.message) ) {
			if ( exception.name ) {
				errorText += ': ' + exception.name + '\n';
			}
			if ( exception.message && !exception.name ) {
				errorText += ': ';
			}
			if ( exception.message ) {
				errorText += exception.message;
			}
		}
		else {
			errorText += ': ' + exception;
		}//.toString();

		this._onRequestShowError( errorText, true );
	};

	ErrorScreen.prototype._onOrientationChanged = function( orientation ) {
		if ( clientInfo.getDeviceType() === 'mobile' ) {
			this._errorScreenEnabled = orientation === 'landscape';
			this._updateVisibility();
		}
	};

	ErrorScreen.prototype._updateVisibility = function() {
		if ( this._errorScreenEnabled && this._handleDisplayError && this._handleDisplayError.isJoined() ) {
			libs.globalEventBus.invoke( 'libs.event.request.showBackdrop', 'body', {
				backdropClass: 'backdrop-errorScreen',
				zIndex: 580,
				animationTime: 200
			} );

			this._showErrorScreen( true );
		} else {
			libs.globalEventBus.invoke( 'libs.event.request.hideBackdrop', 'body', 'backdrop-errorScreen' );
			this._showErrorScreen( false );
		}
	};

	ErrorScreen.prototype._hideErrorScreen = function() {
		if ( this._handleDisplayError ) {
			this._handleDisplayError.leave();
		}
		this._handleDisplayError = null;

		this._updateVisibility();

		this._customNegativeFunc = function() {};
		this._customPositiveFunc = function() {};
	};

	ErrorScreen.prototype._showErrorScreen = function( show ) {
		if ( show === undefined || show === true ) {
			libs.client.utils.DOM.fadeIn( selectors.container, 200 );
		} else {
			libs.client.utils.DOM.fadeOut( selectors.container, 200 );
		}
	};

	ErrorScreen.prototype._setErrorScreenText = function( textArray ) {
		libs.client.utils.DOM.setText( selectors.message, textArray[0], this._settings.text.error );
		libs.client.utils.DOM.setText( selectors.negativeButton.title, textArray[1], this._settings.text.buttons );
		libs.client.utils.DOM.setText( selectors.positiveButton.title, textArray[2], this._settings.text.buttons );
	};

	ErrorScreen.prototype._setErrorScreenPositiveButtonFunction = function( touchFunc ) {
		if ( touchFunc ) {
			selectors.positiveButton.container.downEvent( touchFunc );
		} else {
			selectors.positiveButton.container.unbind();
		}
	};

	ErrorScreen.prototype._setErrorScreenNegativeButtonFunction = function( touchFunc ) {
		if ( touchFunc ) {
			selectors.negativeButton.container.downEvent( touchFunc );
		} else {
			selectors.negativeButton.container.unbind();
		}
	};

	ErrorScreen.prototype._showErrorScreenPositiveButton = function( show ) {
		if ( show === undefined || show === true ) {
			selectors.positiveButton.container.css( "display", "inline-block" );
		} else {
			selectors.positiveButton.container.css( "display", "none" );
		}
	};

	ErrorScreen.prototype._showErrorScreenNegativeButton = function( show ) {
		if ( show === undefined || show === true ) {
			selectors.negativeButton.container.css( "display", "inline-block" );
		} else {
			selectors.negativeButton.container.css( "display", "none" );
		}
	};


	libs.client.presentation.ErrorScreen = ErrorScreen;

}());