goog.provide( 'libs.client.presentation.JackpotMeterBaseClass' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.utils.DOM' );


(function() {
	"use strict";

	var defaultSettings = {
		name: undefined,
		fadeInTime: 0,
		fadeOutTime: 0,
		jackpotIndex: 0,
		model: {
			useBaseModel: true,
			updateEvents: {
				activityStartAndEnd: [],
				activityStart: [],
				activityEnd: [],
				event: []
			},
			shouldBeVisible: function() { return true; }
		},
		text: {
			title: null,
			value: null
		},
		translations: {
			title: null
		},
		hideForMessageBar: false
	};

	var JackpotMeterBaseClass = function( overrideSettings, selectors ) {
		if ( this.constructor === JackpotMeterBaseClass ) {
			libs.logError( 'Cannot instantiate abstract JackpotMeterBaseClass class' );
		}

		if ( !defaultSettings || !selectors ) {
			libs.logError( 'Required parameters for JackpotMeterBaseClass not supplied' );
		}

		var that = this;

		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		this._selectors = selectors;

		this._baseModel = {
			updateEvents: {
				activityStartAndEnd: [],
				activityStart: [],
				activityEnd: [],
				event: []
			},
			shouldBeVisible: function() { return true; }
		};

		this._value = new BigNumber( 0 );
		this._isVisible = false;
		this._numberOfPauseRequests = 0;
		this._forceHiddenForMessageBar = false;

		libs.globalEventBus.connect( 'libs.event.notify.meterBaseModel', function( baseModel ) { that._onNotifyMeterBaseModel( baseModel ); } );
		libs.globalEventBus.connect( 'libs.event.notify.jackpotValues', function( jackpotValues ) { that._onJackpotValues( jackpotValues ); } );
		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );
		libs.globalEventBus.connect( 'libs.event.notify.currencyChanged', function( currencyCode, formatForCurrency ) { that._onNotifyCurrencyChanged( currencyCode, formatForCurrency ); } );
		libs.globalEventBus.connect( 'libs.event.request.pauseMeterUpdating', function( metersArray ) { that._onRequestPauseMeterUpdating( metersArray ); } );
		libs.globalEventBus.connect( 'libs.event.request.resumeMeterUpdating', function( metersArray ) { that._onRequestResumeMeterUpdating( metersArray ); } );

		//Hide jackpots when a message bar is shown
		libs.globalEventBus.connect( 'libs.event.notify.messageBarState', function( state ) { that._onNotifyMessageBarState( state ); } );
	};

	JackpotMeterBaseClass.prototype._onNotifyMeterBaseModel = function( baseModel ) {
		var that = this;

		if ( this._settings.model.useBaseModel ) {
			this._baseModel = baseModel;
		}

		var updateEvents = this._settings.model.updateEvents;
		jQuery.merge( updateEvents.activityStartAndEnd, this._baseModel.updateEvents.activityStartAndEnd );
		jQuery.merge( updateEvents.activityStart, this._baseModel.updateEvents.activityStart );
		jQuery.merge( updateEvents.activityEnd, this._baseModel.updateEvents.activityEnd );
		jQuery.merge( updateEvents.event, this._baseModel.updateEvents.event );
		jQuery.merge( updateEvents.activityStart, updateEvents.activityStartAndEnd );
		jQuery.merge( updateEvents.activityEnd, updateEvents.activityStartAndEnd );
		jQuery.unique( updateEvents.activityStart );
		jQuery.unique( updateEvents.activityEnd );
		jQuery.unique( updateEvents.event );

		var i;
		var activityName;
		var updateMeterFunc = function() { that._updateMeter(); };

		for ( i = 0; i < updateEvents.activityStart.length; ++i ) {
			activityName = updateEvents.activityStart[i];
			libs.globalEventBus.connectActivityStart( activityName, updateMeterFunc );
		}
		for ( i = 0; i < updateEvents.activityEnd.length; ++i ) {
			activityName = updateEvents.activityEnd[i];
			libs.globalEventBus.connectActivityEnd( activityName, updateMeterFunc );
		}
		for ( i = 0; i < updateEvents.event.length; ++i ) {
			var eventName = updateEvents.event[i];
			libs.globalEventBus.connect( eventName, updateMeterFunc );
		}

		this._updateMeter();
	};

	JackpotMeterBaseClass.prototype._onJackpotValues = function( jackpotValues ) {
		this._value = jackpotValues[this._settings.jackpotIndex];
		if ( this._selectors.value && this._numberOfPauseRequests === 0 ) {
			this._updateDOM();
		}
	};

	JackpotMeterBaseClass.prototype._onNotifyLanguageChanged = function( langCode ) {
		if ( this._selectors.title && this._settings.translations.title ) {
			libs.client.utils.DOM.setText( this._selectors.title, libs.getTranslation( this._settings.translations.title ), this._settings.text.title );
		}
	};

	JackpotMeterBaseClass.prototype._onNotifyCurrencyChanged = function( currencyCode, formatForCurrency ) {
		this._updateDOM();
	};

	JackpotMeterBaseClass.prototype._onRequestPauseMeterUpdating = function( metersArray ) {
		for ( var i = 0; i < metersArray.length; ++i ) {
			if ( metersArray[i].match( this._settings.name ) ) {
				++this._numberOfPauseRequests;
				break;
			}
		}
	};

	JackpotMeterBaseClass.prototype._onRequestResumeMeterUpdating = function( metersArray ) {
		for ( var i = 0; i < metersArray.length; ++i ) {
			if ( metersArray[i].match( this._settings.name ) ) {
				--this._numberOfPauseRequests;
				if ( this._numberOfPauseRequests <= 0 ) {
					this._numberOfPauseRequests = 0;
					this._updateDOM();
				}
				break;
			}
		}
	};

	JackpotMeterBaseClass.prototype._updateDOM = function() {
		if ( this._selectors.value ) {
			var text = libs.moneyFormat( this._value, this._settings.alwaysIncludeSubunit );
			libs.client.utils.DOM.setText( this._selectors.value, text, this._settings.text.value );
		}
	};

	JackpotMeterBaseClass.prototype._updateMeter = function() {
		var shouldBeVisible = this._baseModel.shouldBeVisible();
		if ( shouldBeVisible ) {
			shouldBeVisible = this._settings.model.shouldBeVisible();
		}

		if ( this._isVisible !== shouldBeVisible ) {
			if ( !this._forceHiddenForMessageBar && shouldBeVisible ) {
				// updated by We R - hide jackpot display during free spins round
				// libs.client.utils.DOM.fadeIn( this._selectors.container, this._settings.fadeInTime );
			} else {
				libs.client.utils.DOM.fadeOut( this._selectors.container, this._settings.fadeOutTime );
			}
			this._isVisible = shouldBeVisible;
		}
	};

	JackpotMeterBaseClass.prototype._onNotifyMessageBarState = function( state ) {
		var that = this;

		var shouldBeVisible = this._baseModel.shouldBeVisible();
		if ( shouldBeVisible ) {
			shouldBeVisible = this._settings.model.shouldBeVisible();
		}

		if ( this._settings.hideForMessageBar && state ) {
			libs.client.utils.DOM.fadeOut( this._selectors.container, 1000 );
			this._forceHiddenForMessageBar = true;
		} else if ( this._settings.hideForMessageBar && shouldBeVisible && !state ) {
			libs.client.utils.DOM.fadeIn( that._selectors.container, 1000 );
			this._forceHiddenForMessageBar = false;
		}
	};


	libs.client.presentation.JackpotMeterBaseClass = JackpotMeterBaseClass;

}());