goog.provide( 'libs.client.presentation.MeterBaseClass' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.utils.DOM' );


(function() {
	"use strict";

	var defaultSettings = {
		name: undefined,
		alwaysIncludeSubunit: true,
		canBeNegativeValue: false,
		fadeInTime: 0,
		fadeOutTime: 0,
		hideTextOnZeroValue: false,
		updateValueEvent: null,
		model: {
			useBaseModel: true,
			updateEvents: {
				activityStartAndEnd: [],
				activityStart: [],
				activityEnd: [],
				event: []
			},
			shouldBeVisible: function() { return true; }
		},
		text: {
			title: null,
			value: null
		},
		translations: {
			title: null
		},
		animateToValue: false,
		animateToValueTimings: {
			valueThresholds: [0, 100, 1000, 100000, 1000000],
			animationSpeeds: [500, 1000, 2500, 5000, 7500]
		}
	};

	var MeterBaseClass = function( overrideSettings, selectors ) {
		if ( this.constructor === MeterBaseClass ) {
			libs.logError( 'Cannot instantiate abstract MeterBaseClass class' );
		}

		if ( !defaultSettings || !selectors ) {
			libs.logError( 'Required parameters for MeterBaseClass not supplied' );
		}

		var that = this;

		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		this._selectors = selectors;

		this._baseModel = {
			updateEvents: {
				activityStartAndEnd: [],
				activityStart: [],
				activityEnd: [],
				event: []
			},
			shouldBeVisible: function() { return true; }
		};

		this._value = new BigNumber( 0 );
		this._isVisible = false;
		this._numberOfPauseRequests = 0;
		this._stateMachine = new libs.StateMachine();
		this._countTime = 0;
		this._isAnimating = false;

		if ( this._settings.updateValueEvent ) {
			libs.globalEventBus.connect( this._settings.updateValueEvent, function( value, animate ) { that._onUpdateValueEvent( value, animate ); } );
		}

		libs.globalEventBus.connect( 'libs.event.notify.meterBaseModel', function( baseModel ) { that._onNotifyMeterBaseModel( baseModel ); } );
		libs.globalEventBus.connect( 'libs.event.notify.tick', function( timeDelta ) { that._onNotifyTick( timeDelta ); } );
		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );
		libs.globalEventBus.connect( 'libs.event.notify.currencyChanged', function( currencyCode, formatForCurrency ) { that._onNotifyCurrencyChanged( currencyCode, formatForCurrency ); } );
		libs.globalEventBus.connect( 'libs.event.request.pauseMeterUpdating', function( metersArray ) { that._onRequestPauseMeterUpdating( metersArray ); } );
		libs.globalEventBus.connect( 'libs.event.request.resumeMeterUpdating', function( metersArray ) { that._onRequestResumeMeterUpdating( metersArray ); } );
	};

	MeterBaseClass.prototype._onNotifyMeterBaseModel = function( baseModel ) {
		var that = this;

		if ( this._settings.model.useBaseModel ) {
			this._baseModel = baseModel;
		}

		var updateEvents = this._settings.model.updateEvents;
		jQuery.merge( updateEvents.activityStartAndEnd, this._baseModel.updateEvents.activityStartAndEnd );
		jQuery.merge( updateEvents.activityStart, this._baseModel.updateEvents.activityStart );
		jQuery.merge( updateEvents.activityEnd, this._baseModel.updateEvents.activityEnd );
		jQuery.merge( updateEvents.event, this._baseModel.updateEvents.event );
		jQuery.merge( updateEvents.activityStart, updateEvents.activityStartAndEnd );
		jQuery.merge( updateEvents.activityEnd, updateEvents.activityStartAndEnd );
		jQuery.unique( updateEvents.activityStart );
		jQuery.unique( updateEvents.activityEnd );
		jQuery.unique( updateEvents.event );

		var i;
		var activityName;
		var updateMeterFunc = function() { that._updateMeter(); };

		for ( i = 0; i < updateEvents.activityStart.length; ++i ) {
			activityName = updateEvents.activityStart[i];
			libs.globalEventBus.connectActivityStart( activityName, updateMeterFunc );
		}
		for ( i = 0; i < updateEvents.activityEnd.length; ++i ) {
			activityName = updateEvents.activityEnd[i];
			libs.globalEventBus.connectActivityEnd( activityName, updateMeterFunc );
		}
		for ( i = 0; i < updateEvents.event.length; ++i ) {
			var eventName = updateEvents.event[i];
			libs.globalEventBus.connect( eventName, updateMeterFunc );
		}

		this._updateMeter();
	};

	MeterBaseClass.prototype._onUpdateValueEvent = function( value, animate ) {
		var that = this;
		this._finalValue = value;
		if ( this._selectors.value && this._numberOfPauseRequests === 0 ) {
			if ( (this._settings.animateToValue || animate) && !this._finalValue.equals( 0 ) ) {
				this._isAnimating = true;
				this._stateMachine.stateGoto( function( timeDelta ) { that._statePlay( timeDelta ); } );
			} else {
				this._value = this._finalValue;
				this._updateDOM();
			}
		}
	};

	MeterBaseClass.prototype._onNotifyLanguageChanged = function( langCode ) {
		if ( this._selectors.title && this._settings.translations.title ) {
			libs.client.utils.DOM.setText( this._selectors.title, libs.getTranslation( this._settings.translations.title ), this._settings.text.title );
		}
	};

	MeterBaseClass.prototype._onNotifyCurrencyChanged = function( currencyCode, formatForCurrency ) {
		this._updateDOM();
	};

	MeterBaseClass.prototype._onRequestPauseMeterUpdating = function( metersArray ) {
		for ( var i = 0; i < metersArray.length; ++i ) {
			if ( metersArray[i].match( this._settings.name ) ) {
				++this._numberOfPauseRequests;
				break;
			}
		}
	};

	MeterBaseClass.prototype._onRequestResumeMeterUpdating = function( metersArray ) {
		for ( var i = 0; i < metersArray.length; ++i ) {
			if ( metersArray[i].match( this._settings.name ) ) {
				--this._numberOfPauseRequests;
				if ( this._numberOfPauseRequests <= 0 ) {
					this._numberOfPauseRequests = 0;
					this._updateDOM();
				}
				break;
			}
		}
	};

	MeterBaseClass.prototype._updateDOM = function() {
		if ( this._selectors.value ) {
			var text;
			if ( this._value.equals( 0 ) && this._settings.hideTextOnZeroValue ) {
				text = '';
			} else if ( this._value.lessThan( 0 ) && !this._settings.canBeNegativeValue ) {
				text = libs.moneyFormat( new BigNumber( 0 ), this._settings.alwaysIncludeSubunit );
				if ( BUILD_TYPE === 'dev' ) {
					libs.logError( 'Meter ' + this._settings.translations.title + ' attempted to show a negative value when not allowed: ' );
				}
			} else {
				text = libs.moneyFormat( this._value, this._settings.alwaysIncludeSubunit );
			}

			libs.client.utils.DOM.setText( this._selectors.value, text, this._settings.text.value );
		}
	};

	MeterBaseClass.prototype._onNotifyTick = function( timeDelta ) {
		this._stateMachine.update( timeDelta );
	};

	MeterBaseClass.prototype._statePlay = function() {
		if ( this._stateMachine.phaseOnce() ) {
			var that = this;
			this._startingValue = this._value;
			//Find the time to countTo dependant on the length of the number. 
			this._countTime = 0;
			this._settings.animateToValueTimings.valueThresholds.forEach( function( animationThreshold, index ) {
				if ( animationThreshold < that._finalValue.minus( that._value ).toN() ) {
					that._countTime = that._settings.animateToValueTimings.animationSpeeds[index];
				}
			} );
		}

		if ( this._stateMachine.phaseTimed( this._countTime ) ) {
			//With small time increments we need to round to 4 decimal places.
			this._progress = Math.round( this._stateMachine.phaseTimedGetProgress() * 1000 ) / 1000;
			//Set it to a percentage of the value we want to increment to & then update the DOM. 
			this._value = this._finalValue.minus( this._startingValue ).times( this._progress ).plus( this._startingValue ).round();
			this._updateDOM();
		}

		if ( this._stateMachine.phaseOnce() ) {
			this._isAnimating = false;
		}
	};

	MeterBaseClass.prototype.isAnimating = function() {
		return this._isAnimating;
	};

	MeterBaseClass.prototype._updateMeter = function() {
		var shouldBeVisible = this._baseModel.shouldBeVisible();
		if ( shouldBeVisible ) {
			shouldBeVisible = this._settings.model.shouldBeVisible();
		}

		if ( this._isVisible !== shouldBeVisible ) {
			if ( shouldBeVisible ) {
				libs.client.utils.DOM.fadeIn( this._selectors.container, this._settings.fadeInTime );
			} else {
				libs.client.utils.DOM.fadeOut( this._selectors.container, this._settings.fadeOutTime );
			}
			this._isVisible = shouldBeVisible;
		}
	};


	libs.client.presentation.MeterBaseClass = MeterBaseClass;

}());