goog.provide( 'libs.client.reels.AutoplaySelector' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.utils.DOM' );


(function() {
	"use strict";

	var defaultSettings = {
		anchorClassList: [],
		textClassList: {},
		untilBonusClass: 'autoplay-selector-until-bonus',
		allowedValueCount: 5, // Number of selection values that are allowed. Code at the moment doesn't rescale so must be 5
		availableAutoplayValues: libs.getExternalSetting( 'autoplaySteps', "10,20,30,40,untilBonus" )
	};

	var defaultSettingsDesktop = {
		anchorClassList: ['button', 'blue', 'rectangle-small'],
		textClassList: {
			singleLine: [
				'button-text-single-line'
			],
			doubleLine: [
				'button-text-double-line-top',
				'button-text-double-line-bottom'
			]
		},
		translations: {
			untilBonusTextValue: [
				'UNTIL',
				'BONUS'
			]
		}
	};

	var defaultSettingsMobile = {
		translations: {
			untilBonusTextValue: [
				'UNTIL_BONUS'
			]
		}
	};

	var selectors = {
		backdropClass: '.stake',
		autoplaySelectorContainer: $( '#autoplay-steps-container' ),
		autoPlaySteps: $( '.autoplayStep' )
	};

	var updateSelectors = function() {
		selectors.autoPlaySteps = $( '.autoplayStep' );
	};

	selectors.autoplaySelectorContainer.hide();

	// windows phones seem to display a glitch when fading in an element with a box shadow
	// therefore we will remove it for those phones
	if ( libs.client.core.clientInformation.isIEMobile() ) {
		selectors.autoplaySelectorContainer.css( 'box-shadow', 'none' );
	}

	var isMobileDeviceType = libs.client.core.clientInformation.getDeviceType() === 'mobile';
	var isDesktopDeviceType = libs.client.core.clientInformation.getDeviceType() === 'desktop';
	defaultSettings = libs.client.utils.misc.mergeSettings( defaultSettings, isDesktopDeviceType ? defaultSettingsDesktop : defaultSettingsMobile, true );

	var AutoplaySelector = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._handleDisplayAutoplaySelector = null;

		libs.globalEventBus.connect( 'libs.event.request.autoplaySelector', function() { that._onRequestAutoplaySelector(); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.autoplaySelector', function( activity ) { that._onDisplayAutoplaySelectorStart( activity ); } );
		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );

		this._buildAutoplaySelector( this._settings.availableAutoplayValues );
	};

	AutoplaySelector.prototype._onRequestAutoplaySelector = function() {
		var that = this;

		libs.globalEventBus.invokeActivity( 'libs.activity.display.autoplaySelector' );
		libs.globalEventBus.invoke( 'libs.event.request.disableReelDragger' );

		var opacity;
		if ( isMobileDeviceType ) {
			opacity = 0.8;
		} else {
			opacity = 0.4;
		}

		libs.globalEventBus.invoke( 'libs.event.request.showBackdrop', 'body', {
			backdropClass: 'backdrop-autoplay',
			zIndex: 300,
			animationTime: 200,
			closeOnClick: true,
			opacity: opacity,

			clickEvent: function() {
				that._hideAutoplaySelector( null );
			}
		} );

		libs.client.utils.DOM.fadeIn( selectors.autoplaySelectorContainer, 100 );
	};

	AutoplaySelector.prototype._onDisplayAutoplaySelectorStart = function( activity ) {
		this._handleDisplayAutoplaySelector = activity.join();
	};

	AutoplaySelector.prototype._hideAutoplaySelector = function( numberOfAutoplays ) {
		libs.globalEventBus.invoke( 'libs.event.request.hideBackdrop', 'body', 'backdrop-autoplay' );

		libs.client.utils.DOM.fadeOut( selectors.autoplaySelectorContainer, 10 );

		if ( numberOfAutoplays ) {
			libs.globalEventBus.invoke( 'libs.event.request.playHaptic', 'HQ', 1 );
			libs.globalEventBus.invoke( 'libs.event.request.startAutoplay', numberOfAutoplays );
		}

		if ( this._handleDisplayAutoplaySelector ) {
			this._handleDisplayAutoplaySelector.leave();
			this._handleDisplayAutoplaySelector = null;

			libs.globalEventBus.invoke( 'libs.event.request.enableReelDragger' );
		}
	};


	AutoplaySelector.prototype._buildAutoplaySelector = function( availableAutoplayValues ) {
		var that = this;

		selectors.autoplaySelectorContainer.empty();

		var valueArray = availableAutoplayValues.split( "," );
		if ( valueArray.length !== this._settings.allowedValueCount ) {
			var errorText = "Configuration error: Available autoplay values must have a length of " + this._settings.allowedValueCount;
			libs.globalEventBus.invoke( 'libs.event.request.showError', errorText, true );
			return;
		}

		var errorOccurred = false;

		var autoplayValues = valueArray.map( function( val ) {
			var valNum = parseInt( val, 10 );
			if ( !isNaN( valNum ) && isFinite( valNum ) ) {
				return valNum;
			} else {
				if ( val === 'untilBonus' ) {
					return val;
				} else {
					errorOccurred = true;
					var errorText = "Configuration error: Available autoplay values contain an invalid value '" + val + "'";
					libs.globalEventBus.invoke( 'libs.event.request.showError', errorText, true );
				}
			}
		} );

		if ( errorOccurred ) {
			return;
		}

		if ( isMobileDeviceType ) {
			// add list in reverse; helps some CSS issues
			autoplayValues.reverse();
		}

		autoplayValues.forEach( function( autoplayValue, index ) {
			var isValueUntilBonus = autoplayValue === 'untilBonus';
			var valueText = isValueUntilBonus ? that._settings.translations.untilBonusTextValue : [autoplayValue.toString()];
			var classes = valueText.length > 1 ? that._settings.textClassList.doubleLine : that._settings.textClassList.singleLine;
			if ( !classes ) {
				classes = [''];
			}
			if ( isValueUntilBonus ) {
				for ( var i = 0; i < classes.length; ++i ) {
					classes[i] = that._settings.untilBonusClass + ' ' + classes[i];
				}
			}
			var valueTag = [];

			valueText.forEach( function( textVal, i ) {
				valueTag.push( $( '<span/>', {
					'class': classes ? classes[i] : ''
				} )
					.append( isValueUntilBonus ? libs.getTranslation( textVal ) : textVal ) );
			} );

			var anchorTag = $( '<a/>', {
				'class': 'autoplayStep ' + that._settings.anchorClassList.join( ' ' ),
				'data-value': ( autoplayValue === 'untilBonus' ? -1 : autoplayValue )
			} )
				.append( valueTag );


			$( '<div/>', {
				'class': 'autoplayStepDiv',
				'id': 'autoPlayStep-' + index
			} )
				.append( anchorTag )
				.appendTo( selectors.autoplaySelectorContainer );
		} );

		updateSelectors();
		selectors.autoPlaySteps.downEvent( function() { that._hideAutoplaySelector( $( this ).data( 'value' ) ); } );
	};

	AutoplaySelector.prototype._onNotifyLanguageChanged = function( langCode ) {
		var untilBonusElems = $( '.' + this._settings.untilBonusClass );
		for ( var i = 0; i < untilBonusElems.length; ++i ) {
			libs.client.utils.DOM.setText( $( '.' + this._settings.untilBonusClass + ':eq(' + i + ')' ), libs.getTranslation( this._settings.translations.untilBonusTextValue[i] ) );
		}
	};


	libs.client.reels.AutoplaySelector = AutoplaySelector;

}());