goog.provide( 'libs.client.reels.BellAlarm' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.StateMachine' );


(function() {
	"use strict";

	var defaultSettings = {
		waitTime: 2300,
		soundEffect: 'SndBigWinBellRingLoop',
		stakeMultiplierThreshold: 20,
		graphicalQualities: ['MQ', 'HQ'],
		elementAnim: {
			elementShakeInterval: 50,
			shakeMagnitude: 0.7
		},
		shakeAnim: {
			duration: 15,
			distance: 1.5,
			count: 30
		}
	};


	var BellAlarm = function( overrideSettings ) {

		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		this._shakeAnimation = [];
		for ( var i = 0; i < this._settings.shakeAnim.count; i++ ) {
			this._shakeAnimation.push( {
				duration: this._settings.shakeAnim.duration,
				x: this._settings.shakeAnim.distance,
				y: 0
			} );
			this._shakeAnimation.push( {
				duration: this._settings.shakeAnim.duration,
				x: -this._settings.shakeAnim.distance,
				y: this._settings.shakeAnim.distance
			} );
			this._shakeAnimation.push( {
				duration: this._settings.shakeAnim.duration,
				x: this._settings.shakeAnim.distance,
				y: 0
			} );
			this._shakeAnimation.push( {
				duration: this._settings.shakeAnim.duration,
				x: -this._settings.shakeAnim.distance,
				y: -this._settings.shakeAnim.distance
			} );
		}

		var that = this;

		this._stateMachine = new libs.StateMachine();
		this._activityHandle = null;
		this._betResult = null;
		this._shakeSprites = [];
		this._shakeElements = [];

		libs.globalEventBus.connect( 'libs.event.notify.betResult', function( betResult ) { that._onNotifyBetResult( betResult ); } );
		libs.globalEventBus.connect( 'libs.event.notify.tick', function( timeDelta ) { that._onNotifyTick( timeDelta ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.reelsState.preDisplayReelsWin', function( activity ) { that._onPreDisplayReelsWin( activity ); } );
	};

	BellAlarm.prototype._elementShakeAnimation = function( element, animTime, count, magnitude ) {
		count = count || 12;
		magnitude = magnitude || 1;
		var translationIndex = 0;
		var translationCompilation = [];
		var translations = [
			{ 'translateX': 1, 'translateY': 1, 'rotateZ': 0 },
			{ 'translateX': -1, 'translateY': -2, 'rotateZ': -1 },
			{ 'translateX': -3, 'translateY': 0, 'rotateZ': 1 },
			{ 'translateX': 0, 'translateY': 2, 'rotateZ': 0 },
			{ 'translateX': 1, 'translateY': -1, 'rotateZ': 1 },
			{ 'translateX': -1, 'translateY': 2, 'rotateZ': -1 },
			{ 'translateX': -3, 'translateY': 1, 'rotateZ': 0 },
			{ 'translateX': 2, 'translateY': 1, 'rotateZ': -1 },
			{ 'translateX': -1, 'translateY': -1, 'rotateZ': 1 },
			{ 'translateX': 2, 'translateY': 2, 'rotateZ': 0 },
			{ 'translateX': 1, 'translateY': -2, 'rotateZ': -1 },
			{ 'translateX': 0, 'translateY': 0, 'rotateZ': 0 }
		];

		var translationReset = { 'translateX': 0, 'translateY': 0, 'rotateZ': 0 };

		for ( var i = 0; i < count; i++ ) {
			if ( i === count - 1 ) {
				translationCompilation.push( translationReset );
				break;
			}
			translationCompilation.push( translations[translationIndex] );
			translationIndex++;
			if ( translationIndex === translations.length ) {
				translationIndex = 0;
			}
		}

		var onComplete = function() {
			libs.setTimeoutTick( function() {
				element.css( 'transform', '' );
			}, animTime * 2 );
		};

		translationCompilation.forEach( function( translation, i ) {
			for ( var key in translation ) {
				if ( translation.hasOwnProperty( key ) ) {
					translation[key] *= magnitude;
				}
			}
			if ( i === translationCompilation.length - 1 ) {
				translation = libs.client.utils.misc.mergeSettings( translation, { complete: onComplete } );
			}
			element.velocity( translation, animTime );
		} );
	};

	BellAlarm.prototype.addShakeElement = function( element ) {
		if ( !Array.isArray( element ) ) {
			this._shakeElements.push( element );
		} else {
			this._shakeElements = this._shakeElements.concat( element );
		}
	};

	BellAlarm.prototype.addShakeSprite = function( sprite ) {

		if ( !Array.isArray( sprite ) ) {
			this._shakeSprites.push( sprite );
		} else {
			this._shakeSprites = this._shakeSprites.concat( sprite );
		}
	};


	BellAlarm.prototype._onNotifyBetResult = function( betResult ) {
		this._betResult = betResult;
	};


	BellAlarm.prototype._onNotifyTick = function( timeDelta ) {
		this._stateMachine.update( timeDelta );
	};


	BellAlarm.prototype._shouldWePlay = function() {
		// Only play animation on MQ / HQ gfx
		var gfxQuality = libs.client.core.clientInformation.getGraphicalQuality();
		if ( this._settings.graphicalQualities && this._settings.graphicalQualities.indexOf( gfxQuality ) < 0 ) {
			return false;
		}

		return true;
	};


	BellAlarm.prototype._areWinningsBigEnough = function() {
		// get what was won just on the reels, not the entire game as that would include feature winnings
		var won = new BigNumber( 0 );

		if ( this._betResult['bet_total_stake'] && this._betResult['reels'] && Array.isArray( this._betResult['reels']['winline'] ) ) {
			var winlines = this._betResult['reels']['winline'];
			for ( var winlineIndex = 0; winlineIndex < winlines.length; ++winlineIndex ) {
				var wl = winlines[winlineIndex];
				won = won.plus( wl['win'] );
			}
		}

		var threshold = this._betResult['bet_total_stake'].times( this._settings.stakeMultiplierThreshold );
		return won.gte( threshold );
	};


	BellAlarm.prototype._onPreDisplayReelsWin = function( activity ) {

		var that = this;
		if ( this._shouldWePlay() && this._areWinningsBigEnough() ) {
			this._activityHandle = activity.join();
			this._stateMachine.stateGoto( function( timeDelta ) { that._statePlay( timeDelta ); } );
		}
	};


	BellAlarm.prototype._statePlay = function( timeDelta ) {

		var that = this;

		if ( this._stateMachine.phaseOnce() ) {
			libs.globalEventBus.invoke( 'libs.event.request.playHaptic', this._settings.waitTime );
			libs.globalEventBus.invoke( 'libs.event.request.playSound', ['MQ', 'HQ'], this._settings.soundEffect );

			// start reel frame shake animation
			this._shakeSprites.forEach( function( sprite ) {
				sprite.moveBy( that._shakeAnimation );
			} );

			var shakeCycles = Math.round( this._settings.waitTime / that._settings.elementAnim.elementShakeInterval ) + 1;

			this._shakeElements.forEach( function( element ) {
				that._elementShakeAnimation( element,
					that._settings.elementAnim.elementShakeInterval,
					shakeCycles,
					that._settings.elementAnim.shakeMagnitude );
			} );
		}

		if ( this._stateMachine.phaseTimed( this._settings.waitTime ) ) {

		}

		if ( this._stateMachine.phaseOnce() ) {
			this._shakeSprites.forEach( function( sprite ) {
				sprite.stop();
			} );
			this._activityHandle.leave();
		}
	};


	libs.client.reels.BellAlarm = BellAlarm;

}());