goog.provide( 'libs.client.reels.LogoAnimation' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.rendering.LegacyAnimatedSprite' );
goog.require( 'libs.client.utils.misc' );


(function() {
	"use strict";

	var defaultSettings = {
		animName: 'RS_Logo',
		animSettings: {
			fps: 25,
			resetToFrameZeroOnEnd: true
		},
		drawSettings: {
			x: undefined,
			y: undefined,
			width: undefined,
			height: undefined
		},
		model: {
			updateEvents: {
				activityStartAndEnd: [ 'libs.activity.display.freespinsUI' ],
				activityStart: [],
				activityEnd: [],
				event: []
			},
			shouldBeVisible: function() { return !libs.globalEventBus.isActivityRunning( 'libs.activity.display.freespinsUI' ); }
		},
		fadeInTime: 400,
		fadeOutTime: 400
	};

	var LogoAnimation = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		this._settings.imageName = this._settings.animName; // for legacy reasons

		var that = this;

		this._isVisible = false;
		this._shouldBeVisible = false;
		this._forcedVisibleState = null;

		this._spinsSinceLastAnimated = 0;

		var updateEvents = this._settings.model.updateEvents;
		jQuery.merge( updateEvents.activityStart, updateEvents.activityStartAndEnd );
		jQuery.merge( updateEvents.activityEnd, updateEvents.activityStartAndEnd );
		jQuery.unique( updateEvents.activityStart );
		jQuery.unique( updateEvents.activityEnd );
		jQuery.unique( updateEvents.event );

		var updateLogoFunc = function() { that._updateLogo(); };
		var i;
		var activityName;
		for ( i = 0; i < updateEvents.activityStart.length; ++i ) {
			activityName = updateEvents.activityStart[i];
			libs.globalEventBus.connectActivityStart( activityName, updateLogoFunc );
		}
		for ( i = 0; i < updateEvents.activityEnd.length; ++i ) {
			activityName = updateEvents.activityEnd[i];
			libs.globalEventBus.connectActivityEnd( activityName, updateLogoFunc );
		}
		for ( i = 0; i < updateEvents.event.length; ++i ) {
			var eventName = updateEvents.event[i];
			libs.globalEventBus.connect( eventName, updateLogoFunc );
		}

		this._animation = new libs.client.rendering.Sprite( this._settings );

		this._animation.play();
		this._animation.stop();

		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gamePlaying', function( activity ) { that._onGamePlayingStart( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.betState.preBet', function( activity ) { that._onPreBetStart( activity ); } );
		libs.globalEventBus.connect( 'libs.event.notify.messageBarState', function( isShown ) { that._onNotifyMessageBarState( isShown ); } );
		libs.globalEventBus.connect( 'libs.event.request.changeLogoAnimation', function( imageName ) { that._onRequestChangeLogoAnimation( imageName ); } );
		libs.globalEventBus.connect( 'libs.event.notify.tick', function( stake ) { that._onNotifyTick( stake ); } );
	};

	LogoAnimation.prototype._onNotifyTick = function( timeDelta ) {
		this._doFade();
	};

	LogoAnimation.prototype._onGamePlayingStart = function( activity ) {
		this._changeVisible( true );
		if ( this._isVisible ) {
			this._animation.play();
		}
	};

	LogoAnimation.prototype._onPreBetStart = function( activity ) {
		if ( this._isVisible ) {
			++this._spinsSinceLastAnimated;
			if ( this._spinsSinceLastAnimated > 2 ) {
				this._animation.play();
				this._spinsSinceLastAnimated = 0;
			}
		}
	};

	LogoAnimation.prototype._onNotifyMessageBarState = function( isShown ) {
		if ( !libs.globalEventBus.isActivityRunning( 'libs.activity.display.freespinsUI' ) ) {
			this._changeVisible( !isShown );
		}
	};

	LogoAnimation.prototype.getDrawSettings = function() {
		return this._settings.drawSettings;
	};

	LogoAnimation.prototype.isDirty = function() {
		return this._animation.isDirty();
	};

	LogoAnimation.prototype.draw = function( ctx ) {
		this._animation.draw( ctx );
	};

	LogoAnimation.prototype._updateLogo = function() {
		var shouldBeVisible = this._settings.model.shouldBeVisible();
		this._changeVisible( shouldBeVisible );
	};

	LogoAnimation.prototype._changeVisible = function( shouldBeVisible ) {
		this._shouldBeVisible = shouldBeVisible;
	};

	LogoAnimation.prototype._doFade = function() {
		var shouldBeVisible = this._shouldBeVisible;
		if ( this._forcedVisibleState !== null ) {
			shouldBeVisible = this._forcedVisibleState;
		}
		if ( this._isVisible !== shouldBeVisible ) {
			this._animation.stop();
			if ( shouldBeVisible ) {
				this._animation.fadeTo( { duration: this._settings.fadeInTime, alpha: 1.0 } );
			} else {
				this._animation.fadeTo( { duration: this._settings.fadeOutTime, alpha: 0.0 } );
			}
			this._isVisible = shouldBeVisible;
		}
	};


	LogoAnimation.prototype._onRequestChangeLogoAnimation = function( imageName ) {

		this._settings.imageName = imageName ? imageName : this._settings.animName;
		this._animation.reset( this._settings );
	};


	libs.client.reels.LogoAnimation = LogoAnimation;

}());