goog.provide( 'libs.client.reels.Paytable' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.utils.DOM' );
goog.require( 'libs.getTranslation' );


(function() {
	"use strict";

	var defaultSettings = {
		containerFade: 10,
		resizeIntervalCount: 10,
		resizeIntervalTime: 125,
		links: undefined,
		text: {
			title: {
				fontSize: 32,
				maxWidth: 600
			},
			captionLeftToRight: {
				fontSize: 22,
				maxWidth: 600
			},
			captionWinsAreMultipliedBy: {
				fontSize: 16,
				maxWidth: 350
			},
			buildVersion: {
				fontSize: 10,
				maxWidth: 120
			}
		},
		translations: {
			winsAreMultipliedBy: 'Wins_are_multiplied_by_current_line_bet',
			winningLinesHeader: 'WINNING_LINES',
			winsPayLeftToRight: 'ALL_WINS_PAY_LEFT_TO_RIGHT'
		}
	};

	var selectors = {
		ruleContentClass: '.rule-content',
		container: $( '#paytable-container' ),
		stakePerLine: $( '#paytable-stake-per-line' ),
		winsAreMultipliedBy: $( '#paytable-wins-are-multiplied-by' ),
		winsAreMultipliedByCaption: $( '#paytable-wins-are-multiplied-by-caption' ),
		winningLinesHeader: $( '#paytable-winning-lines-header' ),
		winsPayLeftToRight: $( '#paytable-wins-pay-left-to-right' ),
		winsPayLeftToRightCaption: $( '#paytable-wins-pay-left-to-right-caption' ),
		version: $( '#paytable-version' )
	};

	/**
	 * Updates selectors, for use after DOM has been updated
	 */
	var updateSelectors = function() {
		selectors.stakePerLine = $( '#paytable-stake-per-line' );
		selectors.container = $( '#paytable-paytable-container' );
		selectors.winsAreMultipliedBy = $( '#paytable-wins-are-multiplied-by' );
		selectors.winsAreMultipliedByCaption = $( '#paytable-wins-are-multiplied-by-caption' );
		selectors.winningLinesHeader = $( '#paytable-winning-lines-header' );
		selectors.winsPayLeftToRight = $( '#paytable-wins-pay-left-to-right' );
		selectors.winsPayLeftToRightCaption = $( '#paytable-wins-pay-left-to-right-caption' );
		selectors.version = $( '#paytable-version' );
	};

	selectors.container.hide();

	var clientInfo = libs.client.core.clientInformation;

	/**
	 * Paytable Object
	 * @param overrideSettings
	 * @constructor
	 */
	var Paytable = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._currentLanguage = null;
		this._stakePerLine = new BigNumber( 0 );
		this._stakePerLineFontSize = this._settings.text.captionWinsAreMultipliedBy.fontSize;
		this._slider = null;
		this._payRules = [];
		this._handleDisplayPaytable = null;

		libs.client.utils.DOM.fadeOut( selectors.container, this._settings.containerFade );

		libs.globalEventBus.connect( 'libs.event.notify.stakePerLine', function( stakePerLine ) { that._onNotifyStakePerLine( stakePerLine ); } );
		libs.globalEventBus.connect( 'libs.event.request.openPaytable', function() { that._onRequestOpenPaytable(); } );
		libs.globalEventBus.connect( 'libs.event.request.closePaytable', function() { that._onRequestClosePaytable(); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.paytable', function( activity ) { that._onDisplayPaytableStart( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.error', function( activity, isFatal ) { that._onDisplayErrorStart( activity, isFatal ); } );

		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );
		libs.globalEventBus.connect( 'libs.event.notify.orientationChanged', function( orientation ) { that._onNotifyOrientationChanged( orientation ); } );
	};

	/**
	 *
	 * @param stakePerLine
	 * @private
	 */
	Paytable.prototype._onNotifyStakePerLine = function( stakePerLine ) {
		this._stakePerLine = stakePerLine;
		this._updateStakePerLineText();
	};

	/**
	 * Opens paytable overlay and shows slider
	 * Will setup slider if it doesn't already exist
	 * @private
	 */
	Paytable.prototype._onRequestOpenPaytable = function() {
		libs.globalEventBus.invokeActivity( 'libs.activity.display.paytable' );
		libs.globalEventBus.invoke( 'libs.event.request.disableReelDragger' );

		libs.globalEventBus.invoke( 'libs.event.request.showBackdrop', 'body', {
			backdropClass: 'backdrop-payTable',
			zIndex: 300,
			animationTime: 200
		} );

		libs.client.utils.DOM.fadeIn( selectors.container, this._settings.containerFade );
		if ( !this._slider ) {
			this._setupSlider();
		} else {
			this._slider.setIndex( 0 );
			this._slider.open();
		}
	};

	/**
	 * Listens to be notified when pay table should close
	 * @private
	 */
	Paytable.prototype._onRequestClosePaytable = function() {
		this._closePaytable();
	};

	/**
	 * Sets display paytable var on start
	 * @param activity
	 * @private
	 */
	Paytable.prototype._onDisplayPaytableStart = function( activity ) {
		this._handleDisplayPaytable = activity.join();
	};

	/**
	 * resizes slider once height has changed
	 * @param height
	 * @private
	 */
	Paytable.prototype._onNotifyDisplayHeightChanged = function( height ) {
		if ( this._slider ) {
			this._slider.resize();
		}
	};

	Paytable.prototype._onNotifyLanguageChanged = function( langCode ) {
		var that = this;

		if ( langCode !== this._currentLanguage ) {
			this._updateTranslatableText();
			$( selectors.ruleContentClass + ' img' ).each( function() {
				var currentImgSrc = $( this ).attr( 'src' );
				var newImgSrc = currentImgSrc.replace( new RegExp( that._currentLanguage, 'g' ), langCode );
				$( this ).attr( 'src', newImgSrc );
			} );
			this._currentLanguage = langCode;
		}
	};

	/**
	 * Rusn resize on orientation change
	 * @param orientation
	 * @private
	 */
	Paytable.prototype._onNotifyOrientationChanged = function() {
		var that = this;
		if ( this._slider ) {
			libs.clearIntervalCount( this._resizeIntervalId );
			this._resizeIntervalId = libs.setIntervalCount( function() {
				that._slider.resize();
			}, this._settings.resizeIntervalTime, this._settings.resizeIntervalCount );
		}
	};

	/**
	 * Closes paytable on error
	 * @private
	 */
	Paytable.prototype._onDisplayErrorStart = function() {
		if ( libs.globalEventBus.isActivityRunning( 'libs.activity.systemState.gamePlaying' ) ) {
			this._closePaytable();
		}
	};


	/**
	 * Loads in paytable HTML and builds into Slider using scrollingSlider.js
	 * @private
	 */
	Paytable.prototype._setupSlider = function() {
		var that = this;

		$.ajax( {
			dataType: 'html',
			url: that._settings.links.paytable,
			success: function( data ) {
				data = libs.client.utils.misc.replaceAssetPathsInText( data );
				data = libs.client.utils.misc.replaceLanguageCodeInText( data );

				var els = $( data )
					.filter( selectors.ruleContentClass );

				els.each( function( el ) {
					that._payRules.push( els[el] );
				} );

				that._slider = selectors.container.scrollableSelector( 'horizontal', that._payRules, {
					accelerate: clientInfo.is3dSupported() && !clientInfo.isIE(),
					duration: clientInfo.isIOS() ? 1000 : 200		// not working correctly on iOS without this... skips animation
				} );

				updateSelectors();
				that._updateTranslatableText();
				that._updateStakePerLineText();

				libs.client.utils.DOM.setText( selectors.version, BUILD_VERSION, that._settings.text.buildVersion );

				// slider fires event when value is clicked
				that._slider.on( 'hasBeenClosed', function() {
					that._closePaytable();
				} );
			},
			error: function( data ) {
				libs.globalEventBus.invoke( 'libs.event.request.showError', libs.getTranslation( 'Connection_failed_please_restart_the_game' ), false );
			}
		} );
	};

	/**
	 * Closes paytable overlay
	 * @private
	 */
	Paytable.prototype._closePaytable = function() {
		libs.client.utils.DOM.fadeOut( selectors.container, this._settings.containerFade );

		libs.globalEventBus.invoke( 'libs.event.request.hideBackdrop', 'body', 'backdrop-payTable' );
		libs.globalEventBus.invoke( 'libs.event.request.enableReelDragger' );

		if ( this._handleDisplayPaytable ) {
			this._handleDisplayPaytable.leave();
			this._handleDisplayPaytable = null;
		}
	};

	Paytable.prototype._updateTranslatableText = function() {
		libs.client.utils.DOM.setText( selectors.winsAreMultipliedBy, libs.getTranslation( this._settings.translations.winsAreMultipliedBy ), this._settings.text.captionWinsAreMultipliedBy );
		libs.client.utils.DOM.setText( selectors.winningLinesHeader, libs.getTranslation( this._settings.translations.winningLinesHeader ), this._settings.text.title );
		libs.client.utils.DOM.setText( selectors.winsPayLeftToRight, libs.getTranslation( this._settings.translations.winsPayLeftToRight ), this._settings.text.captionLeftToRight );
		this._stakePerLineFontSize = libs.client.utils.DOM.calculateFontSize( libs.getTranslation( this._settings.translations.winsAreMultipliedBy ), this._settings.text.captionWinsAreMultipliedBy );
		this._updateStakePerLineText();
	};

	Paytable.prototype._updateStakePerLineText = function() {
		libs.client.utils.DOM.setText( selectors.stakePerLine, this._stakePerLine.div( 100 ).toFixed( 2 ), this._settings.text.captionWinsAreMultipliedBy );
		selectors.stakePerLine.css('font-size', this._stakePerLineFontSize );
	};

	libs.client.reels.Paytable = Paytable;

}());