goog.provide( 'libs.client.reels.ReelsAnticipation' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.rendering.LegacyAnimatedSprite' );
goog.require( 'libs.client.rendering.SpriteGroup' );

goog.require( 'game.logic' );


(function() {
	"use strict";

	var defaultSettings = {
		animName: 'RS_LoopingSparkle',
		allowAnticipationSkip: true,
		numberOfReels: game.logic.GameData.numberOfReels,
		reelPositions: undefined,
		singleFrame: {
			animSettings: {
				autoplay: true,
				fps: 1,
				resetToFrameZeroOnEnd: false,
				timesToLoop: -1
			}
		},
		multiFrame: {
			animSettings: {
				autoplay: true,
				fps: 25,
				resetToFrameZeroOnEnd: true,
				timesToLoop: -1
			},
			drawOffset: {
				x: 7,
				y: 68
			}
		},
		reel: {
			delayTime: 2000,
			slowDownMinSpeed: 0.65,
			slowDownTween: libs.client.utils.tweens.linear,
			dropInTween: libs.client.utils.tweens.linear,
			dropInMinSpeed: 0.06
		},
		sound: 'SndLongSpinLoop',
		soundSettings: {
			loopCount: 0,
			loopCallback: null,
			pan: 0
		}
	};

	var defaultSettingsMobile = {
		singleFrame: {
			drawOffset: {
				x: 6,
				y: 59
			}
		}
	};

	var defaultSettingsDesktop = {
		singleFrame: {
			drawOffset: {
				x: 6,
				y: 67
			}
		}
	};

	var isDesktopDeviceType = libs.client.core.clientInformation.getDeviceType() === 'desktop';
	defaultSettings = libs.client.utils.misc.mergeSettings( defaultSettings, isDesktopDeviceType ? defaultSettingsDesktop : defaultSettingsMobile, true );

	var ReelsAnticipation = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._betResult = null;
		this._seen = [];
		this._stoppingActivities = [];
		this._activeReelIndex = null;
		this._stateMachine = new libs.StateMachine();
		this._isQuickPlay = false;
		this._dirty = false;

		this._animations = new libs.client.rendering.SpriteGroup( [] );

		var animSettings;
		var drawOffset;
		var drawSettings = jQuery.extend( true, [], this._settings.reelPositions );
		var animationFrames = libs.client.utils.misc.loadAnimationFrames( libs.loadedImages, this._settings.animName );
		if ( animationFrames.length === 1 ) {
			animSettings = this._settings.singleFrame.animSettings;
			drawOffset = this._settings.singleFrame.drawOffset;
		} else {
			animSettings = this._settings.multiFrame.animSettings;
			drawOffset = this._settings.multiFrame.drawOffset;
		}

		for ( var i = 0; i < this._settings.numberOfReels; ++i ) {
			drawSettings[i].x += drawOffset.x;
			drawSettings[i].y += drawOffset.y;
			var anim = new libs.client.rendering.LegacyAnimatedSprite( animationFrames, animSettings, drawSettings[i] );
			this._animations.sprites.push( anim );
		}

		this._handleReelsAnticipation = null;
		this._spinParameters = {};

		libs.globalEventBus.connectActivityStart( 'libs.activity.reels.anticipation', function( activity ) { that._onReelsAnticipationStart( activity ); } );
		libs.globalEventBus.connect( 'libs.event.notify.tick', function( timeDelta ) { that._onNotifyTick( timeDelta ); } );
		libs.globalEventBus.connect( 'libs.event.notify.reelBounced', function( reelIndex, finalSymbols ) { that._onNotifyReelBounced( reelIndex, finalSymbols ); } );
		libs.globalEventBus.connect( 'libs.event.notify.reelStopped', function( reelIndex, finalSymbols ) { that._onNotifyReelStopped( reelIndex, finalSymbols ); } );
		libs.globalEventBus.connect( 'libs.event.notify.betResult', function( betResult ) { that._onNotifyBetResult( betResult ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.reelsState.reelsLanding', function() { that._onReelsLandingStart(); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.reels.reelStoppingTime', function( activity, reelIndex ) { that._onReelStoppingTime( activity, reelIndex ); } );
		libs.globalEventBus.connect( 'libs.event.notify.spinParameters', function( spinParameters ) {
			that._spinParameters = spinParameters;
		} );
		libs.globalEventBus.connect( 'libs.event.notify.quickPlayState', function( enabled ) {
			that._onNotifyQuickPlayState( enabled );
		} );
	};

	ReelsAnticipation.prototype._onNotifyQuickPlayState = function( enabled ) {
		this._isQuickPlay = enabled;
	};

	ReelsAnticipation.prototype._onNotifyBetResult = function( betResult ) {
		this._betResult = betResult;
		this._seen.length = 0;
		this._stoppingActivities.length = 0;
	};


	ReelsAnticipation.prototype._setupSpinParameters = function() {
		var spinParameters = {
			noBounceReels: [],
			noSkipReels: [],
			dropInAnimReels: [],
			insertExtraSymbolsOnLand: {},
			disableExtraSymbolsInsert: {}
		};
		// turn off reel bounce and ability to skip any reels that are being delayed.
		var pastDelayedReel = false;
		if ( this._betResult && this._betResult['reelDelays'] ) {
			for ( var reelIndex = 0; reelIndex < this._betResult['reelDelays'].length; ++reelIndex ) {
				if ( this._shouldDelayReel( reelIndex ) ) {
					pastDelayedReel = true;
					spinParameters.noBounceReels[reelIndex] = true;
					// change the drop-in parameters to create anticipation spin
					spinParameters.dropInAnimReels[reelIndex] = {
						minSpeed: this._settings.reel.dropInMinSpeed,
						tween: this._settings.reel.dropInTween
					};
					spinParameters.disableExtraSymbolsInsert[reelIndex] = true; // Override extra symbols being inserted
				}
				if ( pastDelayedReel ) {
					if ( !this._settings.allowAnticipationSkip ) {
						spinParameters.noSkipReels[reelIndex] = true;
					}
				}
			}
		}

		libs.globalEventBus.invoke( 'libs.event.request.updateSpinParameters', spinParameters );
	};

	ReelsAnticipation.prototype._shouldDelayReel = function( reelIndex ) {
		if ( this._betResult && this._betResult['reelDelays'] ) {
			if ( this._spinParameters.disableAnticipation ) {
				return false; // all anticipation disabled
			}

			if ( this._spinParameters.disableAnticipationReels && this._spinParameters.disableAnticipationReels[reelIndex] ) {
				return false; // single reel anticipation is disabled
			}

			if ( this._spinParameters.lockedReels && this._spinParameters.lockedReels.indexOf( reelIndex ) >= 0 ) {
				return false; // dont anticipate locked reels
			}
			return !!this._betResult['reelDelays'][reelIndex];
		}
		return false;
	};

	ReelsAnticipation.prototype._onReelsAnticipationStart = function( activity ) {
		this._handleReelsAnticipation = activity.join();
	};

	ReelsAnticipation.prototype._onNotifyTick = function( timeDelta ) {
		if ( this._activeReelIndex ) {
			this._animations.sprites[this._activeReelIndex].update( timeDelta );
		}
		this._stateMachine.update( timeDelta );
	};

	ReelsAnticipation.prototype._onReelsLandingStart = function() {

		this._setupSpinParameters();
		// if the first unlocked reel is due to start anticipation, start that one off (as they will be waiting for the bounce message from the previous reel that will never come)
		for ( var reelIndex = 0; reelIndex < this._settings.numberOfReels; ++reelIndex ) {
			var isLocked = this._spinParameters.lockedReels && this._spinParameters.lockedReels.indexOf( reelIndex ) >= 0;
			if ( !isLocked ) {
				if ( this._shouldDelayReel( reelIndex ) ) {
					this._setupAnticipation( -1, reelIndex );
				}
				break;
			}
		}
	};

	ReelsAnticipation.prototype._onNotifyReelBounced = function( reelIndex, finalSymbols ) {
		this._seen[reelIndex] = true;
		this._setupAnticipation( reelIndex );
	};

	ReelsAnticipation.prototype._onNotifyReelStopped = function( reelIndex, finalSymbols ) {
		//Normally everything happens in reel bounce. In turbo mode the reel bounce will never fire so might have to do something here.
		if ( !this._seen[reelIndex] ) {
			this._setupAnticipation( reelIndex );
		}
		this._seen[reelIndex] = true;
	};

	ReelsAnticipation.prototype._setupAnticipation = function( previousReel, nextReel ) {

		if ( this._checkAbort() ) {
			return;
		}

		var that = this;

		if ( nextReel === undefined ) {
			if ( !isFinite( previousReel ) ) {
				throw new Error( "Unexpected anticipation parameter" );
			}
			nextReel = previousReel + 1;
		}

		if ( isFinite( previousReel ) && previousReel >= 0 ) {
			if ( this._shouldDelayReel( previousReel ) ) {
				// stop currently active anticipation
				libs.globalEventBus.invoke( 'libs.event.request.stopSound', ['HQ'], this._settings.sound );
				this._dirty = true;
			}
		}

		if ( this._shouldDelayReel( nextReel ) ) {
			// start next active anticipation
			this._activeReelIndex = nextReel;
			if ( !libs.globalEventBus.isActivityRunning( 'libs.activity.reels.anticipation' ) ) {
				libs.globalEventBus.invokeActivity( 'libs.activity.reels.anticipation' );
			}
			libs.globalEventBus.invoke( 'libs.event.request.playSound', ['HQ'], this._settings.sound, this._settings.soundSettings );
			this._stateMachine.stateGoto( function( timeDelta ) {
				that._stateAnticipate( timeDelta, nextReel );
			} );
		} else if ( nextReel < game.logic.GameData.reels.nreels ) {
			// Run this reel as normal
			this._dirty = true;
			this._activeReelIndex = null;
			if ( this._stoppingActivities[nextReel] ) {
				this._stateMachine.stateGoto( function( timeDelta ) {
					that._stateNormal( timeDelta, nextReel );
				} );
			}
		} else {
			// All reels complete - leave the anticipation activity
			if ( this._handleReelsAnticipation ) {
				this._handleReelsAnticipation.leave();
				this._handleReelsAnticipation = null;
			}
			this._activeReelIndex = null;
			this._stateMachine.stateGoto( null );
		}
	};

	ReelsAnticipation.prototype._checkAbort = function() {
		if ( this._settings.allowAnticipationSkip && this._isQuickPlay ) {
			libs.globalEventBus.invoke( 'libs.event.request.stopSound', ['HQ'], this._settings.sound );
			this._dirty = true;

			if ( this._handleReelsAnticipation ) {
				this._handleReelsAnticipation.leave();
				this._handleReelsAnticipation = null;
			}
			this._activeReelIndex = null;
			this._stateMachine.stateGoto( null );

			for ( var i = 0; i < this._stoppingActivities.length; ++i ) {
				if ( this._stoppingActivities[i] ) {
					this._stoppingActivities[i].leave();
					this._stoppingActivities[i] = null;
				}
			}

			return true;
		}
		return false;
	};

	ReelsAnticipation.prototype._stateAnticipate = function( timeDelta, reelIndex ) {

		if ( this._checkAbort() ) {
			return;
		}

		if ( this._stateMachine.phaseTimed( this._settings.reel.delayTime ) ) {
			var newSpeed = this._settings.reel.slowDownTween( this._stateMachine.phaseTimedGetRevProgress() ) * ( this._spinParameters.maxSpeed - this._settings.reel.slowDownMinSpeed ) + this._settings.reel.slowDownMinSpeed;
			libs.globalEventBus.invoke( 'libs.event.request.changeReelSpeed', reelIndex, newSpeed );
		}

		if ( this._stateMachine.phaseOnce() ) {
			var activity = this._stoppingActivities[reelIndex];
			if ( !activity ) {
				throw new Error( "Anticipation: Unexpected value in _stateAnticipate" );
			}
			activity.leave();
			this._stoppingActivities[reelIndex] = null;
		}
	};

	ReelsAnticipation.prototype._stateNormal = function( timeDelta, reelIndex ) {

		if ( this._checkAbort() ) {
			return;
		}

		if ( this._stateMachine.phaseTimed( this._spinParameters.reelLandingGap ) ) {
		}

		if ( this._stateMachine.phaseOnce() ) {
			var activity = this._stoppingActivities[reelIndex];
			if ( !activity ) {
				throw new Error( "Anticipation: Unexpected value in _stateNormal" );
			}
			activity.leave();
			this._stoppingActivities[reelIndex] = null;
		}
	};

	ReelsAnticipation.prototype._onReelStoppingTime = function( activity, reelIndex ) {

		if ( this._checkAbort() ) {
			return;
		}

		// Find first reel to be delayed, if the current reelIndex is greater or equal to this one join the stopping activity.
		var soonestDelayedReel = -1;
		for ( var i = 0; i < game.logic.GameData.reels.nreels; ++i ) {
			if ( this._shouldDelayReel( i ) ) {
				soonestDelayedReel = i;
				break;
			}
		}

		if ( soonestDelayedReel >= 0 && soonestDelayedReel <= reelIndex ) {
			this._stoppingActivities[reelIndex] = activity.join();
		}
	};

	ReelsAnticipation.prototype.isDirty = function() {
		if ( this._dirty ) {
			return true;
		}

		if ( this._activeReelIndex ) {
			return !!this._animations.sprites[this._activeReelIndex];
		}

		return false;
	};

	ReelsAnticipation.prototype.draw = function( ctx ) {
		if ( this._activeReelIndex ) {
			this._animations.sprites[this._activeReelIndex].draw( ctx );
			this._dirty = false;
		}
	};


	libs.client.reels.ReelsAnticipation = ReelsAnticipation;

}());