goog.provide( 'libs.client.reels.ScrollableStakeSelector' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.moneyFormat' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.utils.DOM' );


(function() {
	"use strict";

	var defaultSettings = {
		containerFade: 10,
		arrowFade: 125,
		resizeIntervalCount: 10,
		resizeIntervalTime: 125,
		text: {
			stakeSteps: {
				fontSize: 26,
				maxCharacters: 10,
				fontSizeDecreasePerExtraChar: 2.3
			}
		},
		sliderConfig: {
			numOfShownItems: 6,
			throwingSpeed: 30,
			throwingAcceleration: 3,
			snappingEnabled: false
		}
	};

	var selectors = {
		container: $( '#stake-selector-container' ),
		scrollArrows: $( '#stake-scroll-up, #stake-scroll-down' ),
		upArrow: $( '#stake-scroll-up' ),
		downArrow: $( '#stake-scroll-down' ),
		buttonClass: '.button.stake',
		values: $( '.stakeSelector .item' ),
		activeStake: $( '.stakeSelector .item.selected' )
	};

	selectors.container.hide();
	selectors.scrollArrows.hide();

	// windows phones seem to display a glitch when fading in an element with a box shadow
	// therefore we will remove it for those phones
	if ( libs.client.core.clientInformation.isIEMobile() ) {
		selectors.container.css( 'box-shadow', 'none' );
	}

	/**
	 * Updates selectors, for use after DOM has been updated
	 */
	var updateSelectors = function() {
		selectors.activeStake = $( '.stakeSelector .item.selected' );
		selectors.values = $( '.stakeSelector .item' );
	};

	var clientInfo = libs.client.core.clientInformation;

	/**
	 * Scrollable Stake Selector
	 * @param overrideSettings {object}
	 * @constructor
	 */
	var ScrollableStakeSelector = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._slider = null;

		this._stakeSelectionTimeoutId = 0;
		this._isFirstSetup = true;

		this._stakeTotal = new BigNumber( 0 );
		this._numberOfWinlines = 1;
		this._availableStakesPerLine = [];

		this._stakeStepsDataArray = [];

		this._handleDisplayStakeSelector = null;
		this._resizeIntervalId = 0;

		this._hideStakeSelector( false );

		libs.globalEventBus.connect( 'libs.event.notify.stakeTotal', function( stakeTotal ) { that._onNotifyStakeTotal( stakeTotal ); } );
		libs.globalEventBus.connect( 'libs.event.notify.availableStakesPerLine', function( availableStakes ) { that._onNotifyAvailableStakesPerLine( availableStakes ); } );
		libs.globalEventBus.connect( 'libs.event.notify.numberOfWinlines', function( numberOfWinlines ) { that._onNotifyNumberOfWinlines( numberOfWinlines ); } );
		libs.globalEventBus.connect( 'libs.event.request.stakeSelector', function() { that._onRequestStakeSelector(); } );
		libs.globalEventBus.connect( 'libs.event.notify.displayHeightChanged', function( height ) { that._onNotifyDisplayHeightChanged( height ); } );
		libs.globalEventBus.connect( 'libs.event.notify.orientationChanged', function( orientation ) { that._onNotifyOrientationChanged( orientation ); } );
		libs.globalEventBus.connect( 'libs.event.notify.currencyChanged', function( currencyCode, formatForCurrency ) { that._onNotifyCurrencyChanged( currencyCode, formatForCurrency ); } );

		libs.globalEventBus.connectActivityStart( 'libs.activity.display.stakeSelector', function( activity ) { that._onDisplayStakeSelectorStart( activity ); } );
	};


	/**
	 * Sets stake total once notifed
	 * @param stakeTotal {BigNumber}
	 * @private
	 */
	ScrollableStakeSelector.prototype._onNotifyStakeTotal = function( stakeTotal ) {
		this._stakeTotal = stakeTotal;
	};

	/**
	 * Updates available stakes per line
	 * @param availableStakes {Array}
	 * @private
	 */
	ScrollableStakeSelector.prototype._onNotifyAvailableStakesPerLine = function( availableStakes ) {
		this._availableStakesPerLine = availableStakes;
	};

	/**
	 * Sets number of win lines once notified
	 * @param numberOfWinlines
	 * @private
	 */
	ScrollableStakeSelector.prototype._onNotifyNumberOfWinlines = function( numberOfWinlines ) {
		this._numberOfWinlines = numberOfWinlines;
	};

	/**
	 * resizes slider once height has changed
	 * @param height
	 * @private
	 */
	ScrollableStakeSelector.prototype._onNotifyDisplayHeightChanged = function( height ) {
		if ( this._slider ) {
			this._slider.resize();
		}
	};

	/**
	 * Rusn resize on orientation change
	 * @param orientation
	 * @private
	 */
	ScrollableStakeSelector.prototype._onNotifyOrientationChanged = function() {
		var that = this;
		if ( this._slider ) {
			libs.clearIntervalCount( this._resizeIntervalId );
			this._resizeIntervalId = libs.setIntervalCount( function() {
				that._slider.resize();
			}, this._settings.resizeIntervalTime, this._settings.resizeIntervalCount );
		}
	};

	/**
	 * Calls setup of stake list and builds into Slider using scrollingSlider.js
	 * @private
	 */
	ScrollableStakeSelector.prototype._setupSlider = function() {
		var that = this;

		this._setupStakeList();

		this._slider = selectors.container.scrollableSelector( 'vertical', this._stakeStepsDataArray, {
			numShownItems: this._settings.sliderConfig.numOfShownItems,
			throwingSpeed: this._settings.sliderConfig.throwingSpeed,
			startingValue: that._stakeTotal,
			throwing: clientInfo.is3dSupported(),
			throwingAcceleration: this._settings.sliderConfig.throwingAcceleration,
			accelerate: clientInfo.is3dSupported() && !clientInfo.isIE(),
			snappingEnabled: this._settings.sliderConfig.snappingEnabled
		} );

		selectors.downArrow.downEvent( function() {
			that._slider.next();
		} );

		selectors.upArrow.downEvent( function() {
			that._slider.prev();
		} );


		// slider fires event when value has changed
		this._slider.on( 'valueSelected', function( e ) {
			if ( that._enableStakeSelection ) {
				var newStake = new BigNumber( e.message.value );
				var newStakePerLine = newStake.dividedBy( that._numberOfWinlines );

				that._stakeTotal = newStake;
				libs.globalEventBus.invoke( 'libs.event.notify.stakeTotal', newStake );
				libs.globalEventBus.invoke( 'libs.event.request.changeStakePerLine', newStakePerLine );

				that._hideStakeSelector( true );
			}
		} );

		updateSelectors();
		selectors.values.css( 'font-size', libs.client.utils.DOM.calculateFontSize( this._getLargestStakeString(), this._settings.text.stakeSteps ) );
		this._hideStakeSelector( false );
	};

	/**
	 * Hides stake selector overlay and closes slider
	 * @param hasStakeChanged {bool} - Plays a sounds if stake has changed
	 * @private
	 */
	ScrollableStakeSelector.prototype._hideStakeSelector = function( hasStakeChanged ) {
		libs.globalEventBus.invoke( 'libs.event.request.hideBackdrop', 'body', 'backdrop-stake' );

		if ( this._slider ) {
			this._slider.close();
		}
		libs.client.utils.DOM.fadeOut( selectors.container, this._settings.containerFade );
		libs.client.utils.DOM.fadeOut( selectors.scrollArrows, this._settings.arrowFade );


		if ( hasStakeChanged ) {
			libs.globalEventBus.invoke( 'libs.event.request.playSound', ['HQ'], 'SndStakeSelect' );
			libs.globalEventBus.invoke( 'libs.event.request.playHaptic', 1 );
		}

		if ( this._handleDisplayStakeSelector ) {
			this._handleDisplayStakeSelector.leave();
			this._handleDisplayStakeSelector = null;

			libs.globalEventBus.invoke( 'libs.event.request.enableReelDragger' );
		}
	};

	/**
	 * Shows stake selectors overlay and opens slider
	 * @private
	 */
	ScrollableStakeSelector.prototype._onRequestStakeSelector = function() {
		var that = this;

		libs.globalEventBus.invokeActivity( 'libs.activity.display.stakeSelector' );

		libs.globalEventBus.invoke( 'libs.event.request.disableAllButtons' );
		libs.globalEventBus.invoke( 'libs.event.request.disableReelDragger' );

		updateSelectors();

		libs.globalEventBus.invoke( 'libs.event.request.showBackdrop', 'body', {
			backdropClass: 'backdrop-stake',
			zIndex: 300,
			animationTime: 200,

			clickEvent: function() {
				that._hideStakeSelector( false );
			}
		} );

		libs.client.utils.DOM.fadeIn( selectors.container, this._settings.containerFade );
		libs.client.utils.DOM.fadeIn( selectors.scrollArrows, this._settings.arrowFade );
		this._slider.open();

		// disable the selection buttons to make in case 'touch' gets remembered from
		// opening stake selection, and then selects stake straight away
		libs.clearTimeout( this._stakeSelectionTimeoutId );
		this._enableStakeSelection = false;
		this._slider.slider.enabled = false;
		this._stakeSelectionTimeoutId = libs.setTimeout( function() {
			that._enableStakeSelection = true;
			that._slider.slider.enabled = true;
		}, 600 );
	};

	/**
	 * Builds stake list from availableStakesPerLine var
	 * @private
	 */
	ScrollableStakeSelector.prototype._setupStakeList = function() {
		for ( var i = this._availableStakesPerLine.length - 1; i >= 0; i-- ) {
			var stake = new BigNumber( this._availableStakesPerLine[i] * this._numberOfWinlines );

			// WE R change - make only stakes of 1 unit or greater available
			var formattedStake = libs.moneyFormat( stake, false );
			if (formattedStake >= 1) {
				this._stakeStepsDataArray.push( {
					data: stake,
					html: formattedStake
				} );
			}
		}
	};

	ScrollableStakeSelector.prototype._onNotifyCurrencyChanged = function( currencyCode, formatForCurrency ) {
		selectors.values.each( function() {
			$( this ).text( libs.moneyFormat( new BigNumber( $( this ).data().value ), false ) );
		} );
	};

	/**
	 * Runs when notified to show selector, runds setup if not set previously
	 * @param activity
	 * @private
	 */
	ScrollableStakeSelector.prototype._onDisplayStakeSelectorStart = function( activity ) {
		if ( this._isFirstSetup ) {
			libs.client.utils.DOM.fadeIn( selectors.container, this._settings.containerFade );
			libs.client.utils.DOM.fadeIn( selectors.scrollArrows, this._settings.arrowFade );

			this._setupSlider();
			this._isFirstSetup = false;
		}
		this._handleDisplayStakeSelector = activity.join();
	};

	/**
	 * Checks for the largest text string in selector list
	 * @returns {string} Longest text in list
	 * @private
	 */
	ScrollableStakeSelector.prototype._getLargestStakeString = function() {
		var longestText = "";
		for ( var i = 0; i < this._stakeStepsDataArray.length; ++i ) {
			var text = this._stakeStepsDataArray[i]['html'];

			if ( text.length > longestText.length ) {
				longestText = text;
			}
		}
		return longestText;
	};


	libs.client.reels.ScrollableStakeSelector = ScrollableStakeSelector;

}());