(function( $, window ) {
	'use strict';

	/**
	 * Base for all sliders
	 * @param el - Container Element
	 * @param contents {Array} - Contents to be populated in slider
	 * @param settings {Object} - Settings object
	 * @constructor
	 */
	var ScrollingSlider = function( el, contents, settings ) {
		var that = this;

		this.settings = settings || {};
		this.element = el;
		this.contents = contents;

		this.selectedIndex = 0;
		this._setStartingValue( this.settings.startingValue || 0 );

		this.numShownItems = this.settings.numShownItems || 10;
		this.items = [];
		this.accelerate = this.settings.accelerate === false ? false : this.settings.accelerate || true;

		this.offset_y = undefined;
		this.offset_x = undefined;
		this.center = undefined;
		this.halfItemHeight = undefined;
		this.min = undefined;

		this.inputDownPos = undefined;
		this.offset = undefined;

		this.hasInput = false;
		this.isMoving = false;
		this.enabled = true;
		this.mouseMoveTimer = null;
		this.isMouseMoving = false;
		this.isActive = false;
		this.scrollDisplayIndex = 0;
		this.snappingEnabled = this.settings.snappingEnabled;

		this.snapAnimation = undefined;
		this.stacked = (typeof this.settings.stacked !== 'boolean') ? true : this.settings.stacked;

		this.throwing = {
			active: typeof this.settings.throwing === 'boolean' ? this.settings.throwing : true,
			velocity: undefined,
			startTime: undefined,
			speed: clamp( (this.settings.throwingSpeed || 50) / 100, 0, 1 ) || 1, // throwing speed is passed as a % - this calc's it to a fraction
			isThrowing: false,
			anim: undefined,
			event: undefined,
			acceleration: this.settings.throwingAcceleration || 5
		};


		/* Build Slider*/
		this._buildContainer();
		contents.forEach( function( content, index ) {
			that.items.push( that._buildSliderContent( content, index ) );
		} );

		if ( this.settings.hasControls ) {
			this._buildControls();
		}

		this.container.appendTo( this.element );
		this._setSelected( this.selectedIndex );
		this._resize();

		$( window ).resize( this._resize.bind( that ) );

		$( this.container )
			.bind( 'touchleave mouseleave touchend mouseup', function( e ) {
				if ( that.isMoving && !that.throwing.isThrowing ) {
					if ( that.throwing.active && that.isMouseMoving ) {
						that._throwScroll( e );
					} else {
						that._snapToIndex( that._getCenterIndex( e ), null, false );
						that.scrollDisplayIndex = that._getCenterIndex( e );
					}
				} else {
					if ( that.hasInput ) {
						$( e.target )
							.click();
					}
				}
				that.hasInput = false;
			} );

		$( this.container )
			.on( 'touchmove mousemove', function( e ) {
				e.preventDefault();
				if ( !that.hasInput ) {
					return;
				}
				var drag = e.originalEvent.touches ?
					(that.stacked ? e.originalEvent.touches[0].pageY : e.originalEvent.touches[0].pageX) :
					(that.stacked ? e.pageY : e.pageX);

				if ( !that.isMouseMoving ) {
					that.inputDownPos = drag;
					that.isMouseMoving = true;
				}


				var delta = (drag - that.inputDownPos) + that.offset,
					min = that.min + (that.center - that.halfItemHeight),
					max = that.center - that.halfItemHeight;

				if ( (delta < min || delta > max) && this.snappingEnabled ) {
				} else {
					if ( drag - that.inputDownPos > 10 || drag - that.inputDownPos < -10 ) {
						that.isMoving = true;
						that.throwing.startTime = Date.now();
						that._draw( delta );
					}
				}

				clearTimeout( that.mouseMoveTimer );
				that.mouseMoveTimer = setTimeout( function() {
					that.isMouseMoving = false;
				}, 100 );
			} );

		$( this.container )
			.on( 'click', '.item', function() {
				if ( that.isMoving || !that.stacked ) {
					return;
				}

				if ( that.enabled ) {
					var index = parseInt( this.attributes['data-index-id'].value );
					that.selectedIndex = index;
					that.setIndex( index );

					// publish an event to say it's been clicked
					$( that.container ).trigger( {
						type: "valueSelected",
						time: new Date(),
						message: {
							value: Number( this.attributes['data-value'].value )
						}
					} );
				}
			} );

		$( this.container )
			.on( 'touchstart mousedown', function( e ) {
				e.preventDefault();
				that.offset = getTopFromMatrix( that.items[0].css( 'transform' ), that.stacked );

				that.inputDownPos = e.originalEvent.touches ?
					(that.stacked ? e.originalEvent.touches[0].pageY : e.originalEvent.touches[0].pageX) :
					(that.stacked ? e.pageY : e.pageX);

				that.hasInput = true;
			} );
	};

	/**
	 * Opens slider and sets to selected index
	 */
	ScrollingSlider.prototype.open = function() {
		this.scrollDisplayIndex = this.selectedIndex;
		this.element.show();
		this._resize();
		this._snapToIndex( this.selectedIndex, 1 );
	};

	/**
	 * Closes slider and publish close event
	 */
	ScrollingSlider.prototype.close = function() {
		this.element.hide();

		// publish an event to say it's been closed
		$( this.container ).trigger( {
			type: "hasBeenClosed",
			time: new Date()
		} );
	};

	/**
	 * Set and snap to index
	 * @param index {int} - Index to select
	 */
	ScrollingSlider.prototype.setIndex = function( index, duration ) {
		duration = duration || null;
		this._setSelected( index );
		this._snapToIndex( index, duration, false );
	};

	/**
	 * Gets value of selected index
	 * @returns {string} - Value of selected index
	 */
	ScrollingSlider.prototype.getValue = function() {
		return this.items[this.selectedIndex].text();
	};

	/**
	 * Gets current selected index
	 * @returns {number} - Selected index
	 */
	ScrollingSlider.prototype.getIndex = function() {
		return this.selectedIndex;
	};

	/**
	 * Sets staring value for setup of slider
	 * @param startingValue
	 * @private
	 */
	ScrollingSlider.prototype._setStartingValue = function( startingValue ) {
		var that = this;
		this.contents.forEach( function( content, index ) {
			if ( Number( startingValue ) === Number( content.data ) ) {
				that.selectedIndex = index;
			}
		} );
	};

	/**
	 * Builds slider container
	 * @private
	 */
	ScrollingSlider.prototype._buildContainer = function() {
		this.container = $( '<ul/>', {
			'css': {
				'height': '100%',
				'width': '80%',
				'display': 'block',
				'list-style-type': 'none',
				'overflow': 'hidden',
				'position': 'relative',
				'margin': '0 auto',
				'user-select': 'none',
				'cursor': 'pointer'
			},
			'class': 'container'
		} );
	};

	/**
	 * Snaps to selected index
	 * @param index {int} - Index to snap to
	 * @param setDuration {int} - Duration of animation
	 * @private
	 */
	ScrollingSlider.prototype._snapToIndex = function( index, setDuration, override ) {
		var that = this,
			duration = setDuration || (this.settings.duration || 300),
			timeStamp = window.performance.now();

		index = clamp( (index === -1 ? this.selectedIndex : index), 0, this.items.length - 1 );

		if ( this.snappingEnabled ) {
			this.selectedIndex = index;
			this._setSelected( index );
		}

		if ( (index === 0 || index === this.items.length - 1) || this.snappingEnabled || override ) {
			this._snap( {
				duration: duration,
				startTime: timeStamp,
				currentTime: timeStamp,
				index: index,
				startingPos: getTopFromMatrix( that.items[0].css( 'transform' ), that.stacked ),
				distanceToSnap: this.center - (getTopFromMatrix( this.items[index].css( 'transform' ), that.stacked ) + that.halfItemHeight)
			} );
		}
		this.enabled = true;
	};

	/**
	 * Sets selected class to selected element
	 * @param index {int} - Index of selected element
	 * @private
	 */
	ScrollingSlider.prototype._setSelected = function( index ) {
		var cssClass;

		this.items.forEach( function( item, i ) {
			cssClass = i !== index ?
				'normal' :
				'selected';

			item.removeClass( 'selected' )
				.addClass( cssClass );
		} );
	};

	/**
	 * snap animation to selected index
	 * @param anim {Object} - Animation Object
	 * @private
	 */
	ScrollingSlider.prototype._snap = function( anim ) {
		var that = this,
			perc = clamp( (anim.currentTime - anim.startTime) / anim.duration, 0, 1 ),
			delta = perc * anim.distanceToSnap + anim.startingPos;

		this._draw( delta );

		if ( perc < 1 ) {
			this.snapAnimation = window.requestAnimationFrame( function( time ) {
				anim.currentTime = time;
				that._snap( anim );
			} );
		} else {
			window.cancelAnimationFrame( this.snapAnimation );
			this.enabled = true;
			this.isMoving = false;
		}
	};

	ScrollingSlider.prototype._resize = function() {};
	ScrollingSlider.prototype._buildControls = function() {};


	/* Bet Slider */
	/**
	 * Bet slider - Inherrits from Base Slider
	 * @constructor
	 */
	function VerticalSlider() {
		ScrollingSlider.apply( this, arguments );
	}

	VerticalSlider.prototype = Object.create( ScrollingSlider.prototype );
	VerticalSlider.prototype.constructor = VerticalSlider;

	/**
	 * Calculates velocity for throwing when scrolling
	 * @param e {Event}
	 * @private
	 */
	VerticalSlider.prototype._throwScroll = function( e ) {
		var throwing = this.throwing,
			inputEndPos = e.originalEvent.changedTouches &&
			(e.type !== 'touchleave' || e.type !== 'touchend' || e.type !== 'mouseleave' || e.type !== 'mouseup') ?
				e.originalEvent.changedTouches[0].pageY :
				e.pageY,
			endTime = Date.now();

		throwing.velocity = (((this.inputDownPos - inputEndPos) / (endTime - throwing.startTime)) * this.throwing.acceleration) * throwing.speed;
		throwing.event = e;

		this._scroll();
	};

	/**
	 * Scrolls and snaps baed on velocity of throw
	 * @private
	 */
	VerticalSlider.prototype._scroll = function() {
		var that = this,
			throwing = this.throwing;

		if ( throwing.isThrowing ) {
			var min = that.min,
				max = that.center + that.halfItemHeight,
				topBlock = getTopFromMatrix( that.items[0].css( 'transform' ), that.stacked );

			if ( topBlock < min || topBlock > max ) {
				throwing.velocity *= 0.1;
				this._snapToIndex( that._getCenterIndex( throwing.event ) );
			}

			this.items.forEach( function( item, i ) {
				var x = 0,
					y = getTopFromMatrix( that.items[i].css( 'transform' ), that.stacked ) - throwing.velocity;

				item.css( {
					'transform': getTranslateString( x, y, that.accelerate )
				} );
			} );
		}

		if ( (throwing.velocity > 1 || throwing.velocity < -1) ) {
			throwing.anim = window.requestAnimationFrame( that._scroll.bind( this ) );

			throwing.velocity *= 0.9;
			throwing.isThrowing = true;

		} else {
			window.cancelAnimationFrame( throwing.anim );

			throwing.velocity = 0;
			throwing.isThrowing = false;
			that.isMoving = false;

			this.scrollDisplayIndex = that._getCenterIndex( throwing.event );
			if ( this.snappingEnabled ) {
				this._snapToIndex( that._getCenterIndex( throwing.event ), null, false );
			}
		}
	};


	/**
	 * Builds content item and appends to Container
	 * @param content
	 * @param index
	 * @returns {*|!jQuery}
	 * @private
	 */
	VerticalSlider.prototype._buildSliderContent = function( content, index ) {
		var that = this;
		return $( '<li/>', {
			'data-index-id': index,
			'data-value': content.data,
			'class': 'item',
			'css': {
				'transform': getTranslateString( 0, 0, that.accelerate ),
				'position': 'absolute',
				'height': 100 / that.numShownItems + '%',
				'vertical-align': 'middle'
			}
		} ).append( $( '<div/>', {
			'css': {
				'position': 'absolute',
				'top': ' 50%',
				'transform': 'translate(0, -50%)',
				'width': '100%',
				'left': '0'
			}
		} ).append( content.html ) ).appendTo( that.container );
	};

	/**
	 * Resize function
	 * @private
	 */
	VerticalSlider.prototype._resize = function() {
		this.offset_y = Number( this.container.height() ) / clamp( this.numShownItems, 0, this.items.length );
		this.offset_x = 0;
		this.halfItemHeight = Number( this.items[0].height() ) / 2;
		this.center = Number( this.container.height() ) / 2;
		this.min = ((this.items.length * this.offset_y) - (this.items[0].height() * 2)) * -1;

		this._redraw();
	};

	/**
	 * Redraw function
	 * @private
	 */
	VerticalSlider.prototype._redraw = function() {
		var delta = -(((this.offset_y * this.scrollDisplayIndex) - this.center) + this.halfItemHeight);
		this._draw( delta );
		this.offset = getTopFromMatrix( this.items[0].css( 'transform' ), this.stacked );
	};

	/**
	 * Draw function
	 * @param delta {int} - Delta position for offset
	 * @private
	 */
	VerticalSlider.prototype._draw = function( delta ) {
		var that = this;
		if ( this.items[0] ) {
			this.items.forEach( function( item, i ) {
				var x = 0,
					y = i * that.offset_y + delta;

				item.css( {
					'transform': getTranslateString( x, y, that.accelerate )
				} );
			} );
		}
	};

	/**
	 * Gets index of the item closest to the center of the slider
	 * @returns {number|*}
	 * @private
	 */
	VerticalSlider.prototype._getCenterIndex = function() {
		var that = this,
			diffArr = [];

		this.items.forEach( function( item ) {
			var num = getTopFromMatrix( item.css( 'transform' ), that.stacked );
			diffArr.push( Math.abs( that.center - num ) );
		} );

		this.isMoving = false;
		return diffArr.indexOf( Math.min.apply( Math, diffArr ) );
	};


	/* Rule Slider */
	/**
	 * Rule slider - Inherrits from Base Slider
	 * @constructor
	 */
	function HorizontalSlider() {
		ScrollingSlider.apply( this, arguments );
	}

	HorizontalSlider.prototype = Object.create( ScrollingSlider.prototype );
	HorizontalSlider.prototype.constructor = HorizontalSlider;

	/**
	 * Opens slider
	 */
	HorizontalSlider.prototype.open = function() {
		this.element.show();
		this.setIndex( this.selectedIndex, 1 );
		this._resize();
	};

	/**
	 * Builds slider container
	 * @private
	 */
	HorizontalSlider.prototype._buildContainer = function() {
		this.container = $( '<ul/>', {
			'css': {
				'height': '100%',
				'width': '100%',
				'overflow': 'hidden',
				'display': 'block',
				'list-style-type': 'none',
				'position': 'relative',
				'user-select': 'none',
				'cursor': 'pointer'
			},
			'class': 'container'
		} );
	};

	/**
	 * Builds controls for slider
	 * @private
	 */
	HorizontalSlider.prototype._buildControls = function() {
		var that = this,
		// dat navigation wrapper
			dotNav = $( '<div/>', {
				'class': 'dot-nav'
			} ),

		// left arrow navigation
			arrowNavLeft = $( '<div/>', {
				'class': 'arrow-nav left'
			} )
				.append( that.settings.arrowLeft || ' &#9664;' )
				.on( 'touchdown mousedown', function( e ) {
					e.stopPropagation();
					e.preventDefault();
					that.selectedIndex = clamp( that.selectedIndex - 1, 0, that.items.length - 1 );
					that.setIndex( that.selectedIndex );
				} ),

		//right arrow navigation
			arrowNavRight = $( '<div/>', {
				'class': 'arrow-nav right'
			} )
				.append( that.settings.arrowRight || '&#9654;' )
				.on( 'touchdown mousedown', function( e ) {
					e.stopPropagation();
					e.preventDefault();
					that.selectedIndex = clamp( that.selectedIndex + 1, 0, that.items.length - 1 );
					that.setIndex( that.selectedIndex );
				} );

		// close button
		$( '<div/>', {
			'class': 'close-button'
		} )
			.append( that.settings.closeText )
			.on( 'touchdown mousedown', function() {
				that.close();
			} )
			.appendTo( this.element );

		// add dots to navigation wrapper
		this.items.forEach( function( item, index ) {
			$( '<span/>', {
				'class': 'dot',
				'data-index-id': index
			} )
				.on( 'touchdown mousedown', function() {
					that.selectedIndex = index;
					that.setIndex( that.selectedIndex );
				} )
				.appendTo( dotNav );
		} );

		this.element
			.append( dotNav )
			.append( arrowNavLeft )
			.append( arrowNavRight );
	};

	/**
	 * Builds content and appends to slider container
	 * @param content
	 * @param index
	 * @returns {*|!jQuery}
	 * @private
	 */
	HorizontalSlider.prototype._buildSliderContent = function( content, index ) {
		var that = this;
		return $( '<li/>', {
			'data-index-id': index,
			'class': 'item',
			'css': {
				'transform': 'translate' + (that.accelerate ? '3d' : '') + '(' + 0 + 'px,' + 0 + 'px' + (that.accelerate ? ',0' : '') + ')',
				'position': 'absolute',
				'width': 80 / that.numShownItems + '%'
			}
		} )
			.append( content )
			.appendTo( that.container );
	};

	/**
	 * Sets selected index and updates navigation
	 * @private
	 */
	HorizontalSlider.prototype._setSelected = function() {
		var el = this.element;

		el.find( '.dot' )
			.removeClass( 'selected' );
		el.find( '.dot[data-index-id="' + this.selectedIndex + '"]' )
			.addClass( 'selected' );

		if ( this.selectedIndex < 1 ) {
			el.find( '.arrow-nav.left' )
				.hide();
		} else {
			el.find( '.arrow-nav.left' )
				.show();
		}
		if ( this.selectedIndex > this.items.length - 2 ) {
			el.find( '.arrow-nav.right' )
				.hide();
		} else {
			el.find( '.arrow-nav.right' )
				.show();
		}
	};

//	/**
//	 * snap animation to selected index
//	 * @param anim {Object} - Animation Object
//	 * @private
//	 */
//	HorizontalSlider.prototype._snap = function (anim) {
//		var that = this,
//			move = null,
//			stop = null,
//			perc = (anim.currentTime - anim.startTime);
//
//
//		if (anim.direction === 'right') {
//			move = anim.startingPos - perc;
//			stop = move >= anim.startingPos + anim.distanceToSnap;
//		} else if (anim.direction === 'left') {
//			move = anim.startingPos + perc;
//			stop = move <= anim.startingPos + anim.distanceToSnap;
//		}
//
//		this._draw(move);
//
//		if (stop) {
//			this.snapAnimation = window.requestAnimationFrame(function (time) {
//				anim.startTime = anim.currentTime;
//				anim.currentTime = time;
//				that._snap(anim);
//			});
//		} else {
//			window.cancelAnimationFrame(this.snapAnimation);
//			this._draw(anim.startingPos + anim.distanceToSnap);
//			this.enabled = true;
//			this.isMoving = false;
//		}
//	};

	/**
	 * Resize slider
	 * @private
	 */
	HorizontalSlider.prototype._resize = function() {
		this.offset_y = 0;

		this.offset_x = Number( this.container.width() ) / clamp( this.numShownItems, 0, this.items.length );
		this.center = Number( this.container.width() ) / 2;

		this.halfItemHeight = Number( this.items[0].width() ) / 2;
		this.min = ((this.items.length * this.offset_x) - this.center - (this.items[0].height() * 2)) * -1;

		this._redraw();
	};

	/**
	 * Redraw slider
	 * @private
	 */
	HorizontalSlider.prototype._redraw = function() {
		var delta = -(((this.offset_x * this.selectedIndex) - this.center) + this.halfItemHeight);
		this._draw( delta );
		this.offset = getTopFromMatrix( this.items[0].css( 'transform' ), this.stacked );
	};

	/**
	 * Draw slider
	 * @param delta {int} - Delta position for the offset
	 * @private
	 */
	HorizontalSlider.prototype._draw = function( delta ) {
		var that = this;
		this.items.forEach( function( item, i ) {
			var x = i * that.offset_x + delta,
				y = that.offset_y;
			item.css( {
				'transform': getTranslateString( x, y, that.accelerate )
			} );
		} );
	};

	/**
	 * Gets index to snap as the center
	 * @param e {Event}
	 * @returns {number} - Index to snap to
	 * @private
	 */
	HorizontalSlider.prototype._getCenterIndex = function( e ) {
		var dir = e.originalEvent.changedTouches &&
		(e.type !== 'touchleave' || e.type !== 'touchend' || e.type !== 'mouseleave' || e.type !== 'mouseup') ?
			(this.stacked ? e.originalEvent.changedTouches[0].pageY : e.originalEvent.changedTouches[0].pageX) :
			(this.stacked ? e.pageY : e.pageX);

		this.isMoving = false;
		return dir < this.inputDownPos ? clamp( this.selectedIndex + 1, 0, this.items.length - 1 ) : clamp( this.selectedIndex - 1, 0, this.items.length - 1 );
	};

	/**
	 * Clamps a value between two given numbers
	 * @method
	 * @param val {number} - Value to clamp
	 * @param min {number} - Minimum value returned
	 * @param max {number} - Maximum value returned
	 * @returns {number} - Clamped value
	 */
	function clamp( val, min, max ) {
		return val > max ? max : val < min ? min : val;
	}

	/**
	 * Gets transform value from first element
	 * @param transform
	 * @param top {bool} - true if vertical slider, false if horizontal
	 * @returns {number} - Transform from top
	 */
	function getTopFromMatrix( transform, top ) {
		var matrixVal = top ? 5 : 4;
		if ( transform === 'none' ) { return; }
		return Number( transform.split( ',' )[matrixVal].replace( ')', '' ) );
	}

	/**
	 * Builds co-oords to css translate string
	 * @param x {int}
	 * @param y {int}
	 * @param accelerate {bool}
	 * @returns {string}
	 */
	function getTranslateString( x, y, accelerate ) {
		return 'translate' + (accelerate ? '3d' : '') + '(' + x + 'px,' + y + 'px' + (accelerate ? ',0' : '') + ')';
	}

	/**
	 * Slider as jQuery Plugin
	 * @param type {string} - Type of slider. 'horizontal' or 'vertical'
	 * @param content {Array} - Content to add to slider
	 * @param options {Object} - Override options
	 * @returns {$.fn}
	 */
	$.fn.scrollableSelector = function( type, content, options ) {
		var settings = options;
		if ( type === 'vertical' ) {

			settings = $.extend( {
				numShownItems: 10,
				hasControls: false,
				accelerate: false,
				type: type,
				snappingEnabled: true
			}, options );

			this.slider = new VerticalSlider( this, content, settings );

			// this methods is only needed for bet slider
			this.getValue = function() {
				return this.slider.getValue();
			};
			this.setValue = function( val ) {
				this.slider.setIndex( this.slider.contents.indexOf( val || 0 ) > -1 ? this.slider.contents.indexOf( val ) : 0 );
			};
		}

		if ( type === 'horizontal' ) {
			// settings required for pay table slider
			settings = $.extend( {
				stacked: false,
				throwing: false,
				numShownItems: 1,
				duration: 500,
				hasControls: true,
				accelerate: false,
				closeText: '&times;',
				type: type,
				snappingEnabled: true
			}, options );

			this.slider = new HorizontalSlider( this, content, settings );
		}

		/**
		 * Open slider
		 */
		this.open = function() {
			this.slider.open();
		};
		/**
		 * Close slider
		 */
		this.close = function() {
			this.slider.close();
		};

		/**
		 * Resize slider
		 */
		this.resize = function() {
			this.slider._resize();
		};
		/**
		 * set new selected index
		 * @param val {int} - Index to set
		 */
		this.setIndex = function( val ) {
			this.slider.setIndex( val );
		};
		/**
		 * get current selectd index
		 * @returns {number|goog.db.Index|?number|*}
		 */
		this.getIndex = function() {
			return this.slider.getIndex();
		};
		/**
		 * skip to next slider index
		 */
		this.next = function() {
			this.slider.scrollDisplayIndex = this.slider.scrollDisplayIndex < this.slider.items.length - 1 ? this.slider.scrollDisplayIndex + 1 : this.slider.scrollDisplayIndex;
			this.slider._snapToIndex( this.slider.scrollDisplayIndex, null, true );
		};
		/**
		 * skip to previous slider index
		 */
		this.prev = function() {
			this.slider.scrollDisplayIndex = this.slider.scrollDisplayIndex > 0 ? this.slider.scrollDisplayIndex - 1 : this.slider.scrollDisplayIndex;
			this.slider._snapToIndex( this.slider.scrollDisplayIndex, null, true );
		};
		return this;
	};

})( jQuery, window );