goog.provide( 'libs.client.reels.SymbolTeaseAnimation' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.StateMachine' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.rendering.LegacyAnimatedSprite' );
goog.require( 'libs.client.rendering.SpriteGroup' );
goog.require( 'libs.client.utils.misc' );


(function() {
	"use strict";


	var defaultSettings = {
		reelsFramePosition: undefined,
		reelPositions: undefined,
		spinSpeed: 1.7,
		offset: {
			x: 18,
			y: 100
		},
		animation: {
			animName: undefined,
			animSettings: {
				autoplay: false,
				timesToLoop: -1,
				fps: 15
			},
			drawSettings: {
				x: 100,
				y: 100
			}
		},
		symbolsToLookAhead: 1,
		verticalSymbolSpacing: undefined,
		symbolRules: [
			// example: { symbol: 'WI', minimumStackCount: 3 }
		],
		sound: null
	};

	var defaultSettingsMobile = {
		clipAtPosY: 470
	};

	var defaultSettingsDesktop = {
		clipAtPosY: 480
	};

	var isDesktopDeviceType = libs.client.core.clientInformation.getDeviceType() === 'desktop';
	defaultSettings = libs.client.utils.misc.mergeSettings( defaultSettings, isDesktopDeviceType ? defaultSettingsDesktop : defaultSettingsMobile, true );


	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


	var SymbolTeaseReelAnimation = function( settings ) {

		var that = this;
		this._settings = settings;
		this._animState = new libs.StateMachine();
		this._isDirty = false;
		this._isAnimating = false;
		this._animationDrawHeight = 0;
		this._totalDistanceTravelled = 0;

		this._animation = null;
		this._animationFrames = libs.client.utils.misc.loadAnimationFrames( libs.loadedImages, this._settings.animation.animName );
		if ( this._animationFrames.length > 0 ) {
			this._animationHeight = this._animationFrames[0].height;
			this._distanceFromWild = this._settings.symbolsToLookAhead * this._settings.verticalSymbolSpacing + this._animationHeight - this._settings.offset.y;
			this._animationDrawHeight = this._animationHeight;
			this._animation = new libs.client.rendering.LegacyAnimatedSprite( this._animationFrames, this._settings.animation.animSettings, this._settings.animation.drawSettings );
			this._animation.alpha = 0;
		} else {
			throw new Error( "SymbolTeaseAnimation: Animation '" + this._settings.animation.animName + "' could not be loaded" );
		}

		libs.globalEventBus.connect( 'libs.event.notify.tick', function( timeDelta ) { that._onNotifyTick( timeDelta ); } );
	};


	SymbolTeaseReelAnimation.prototype.start = function( reelIndex ) {
		var that = this;

		if ( this._animation && !this._isAnimating ) {
			this._isAnimating = true;
			this._reelTop = this._settings.reelsFramePosition.y + this._settings.reelPositions[reelIndex].y;
			this._totalDistanceTravelled = 0;
			this._animation.x = this._settings.reelPositions[reelIndex].x + this._settings.offset.x;
			this._animState.stateGoto( function( td ) { that._stateAnimate( td ); } );
		}
	};


	SymbolTeaseReelAnimation.prototype._boundaryCalculation = function() {
		this._animationHeight = this._animation.image.height;
		this._distanceFromWild = 2 * this._settings.verticalSymbolSpacing + this._animationHeight - this._settings.offset.y;

		this._animation.y = this._reelTop - this._distanceFromWild + this._totalDistanceTravelled;
		if ( this._animation.y < this._reelTop ) {
			this._fireballDrawOffsetY = this._reelTop - this._animation.y;
			this._animationDrawHeight = this._animationHeight - this._fireballDrawOffsetY;
			this._animation.y = this._reelTop;
		}

		// clip fireball animation if it moves outside the reels
		var bottomEdge = this._settings.clipAtPosY - this._animationHeight;
		if ( this._animation.y > bottomEdge ) {
			this._animationDrawHeight = this._animationHeight - ( this._animation.y - bottomEdge );
			if ( this._animationDrawHeight < 0 ) {
				this._animationDrawHeight = 0;
				this._isAnimating = false;
			}
		}
	};


	SymbolTeaseReelAnimation.prototype.isDirty = function() {
		return this._isAnimating || this._isDirty || ( this._animation && this._animation.isDirty() );
	};


	SymbolTeaseReelAnimation.prototype.draw = function( ctx ) {
		this._isDirty = false;
		if ( !this._animation || !this._isAnimating ) {
			return;
		}

		if ( this._animationDrawHeight > 0 ) {
			this._animation.draw( ctx, null, this._animationDrawHeight, undefined, this._fireballDrawOffsetY );
		}
	};


	SymbolTeaseReelAnimation.prototype._onNotifyTick = function( timeDelta ) {
		this._animState.update( timeDelta );
		if ( this._animation ) {
			this._animation.update( timeDelta );
		}

		if ( this._isAnimating ) {
			this._totalDistanceTravelled += timeDelta * this._settings.spinSpeed;
			this._boundaryCalculation();
		}
	};


	SymbolTeaseReelAnimation.prototype._stateAnimate = function( timeDelta ) {
		this._isDirty = true;

		if ( this._animState.phaseOnce() ) {
			this._animation.alpha = 1;
			this._animation.play();
		}

		if ( this._animState.phaseManual() ) {
			if ( !this._isAnimating ) {
				this._animState.phaseStep();
			}
		}

		if ( this._animState.phaseOnce() ) {
			this._animation.alpha = 0;
			this._animState.stateGoto( null );
		}
	};


	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


	var SymbolTeaseAnimation = function( overrideSettings ) {
		var that = this;
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		this._isQuickPlay = false;
		this._reelAnims = [];
		for ( var i = 0; i < game.logic.GameData.numberOfReels; ++i ) {
			this._reelAnims.push( new SymbolTeaseReelAnimation( this._settings ) );
		}

		libs.globalEventBus.connect( 'libs.event.notify.quickPlayState', function( enabled ) { that._isQuickPlay = enabled; } );
		libs.globalEventBus.connect( 'libs.event.notify.symbolPassedWindow', function( reelIndex, symbolId, reelStrip, reelPos, targetStopPos ) { that._onNotifySymbolPassedWindow( reelIndex, symbolId, reelStrip, reelPos, targetStopPos ); } );
	};


	SymbolTeaseAnimation.prototype._doesSymbolPassRule = function( symbolRule, symbolId, reelStrip, reelPos, targetStopPos ) {

		// first check if all symbols we need to check in the stack are the symbol type
		var rootExaminePos = Math.floor( reelPos ) - this._settings.symbolsToLookAhead;
		for ( var examineIndex = 0; examineIndex < ( symbolRule.minimumStackCount || 1 ); ++examineIndex ) {

			var examinePos = rootExaminePos - examineIndex;
			if ( examinePos < 0 ) {
				examinePos += reelStrip.length;
			}
			var examineSymbol = reelStrip[examinePos];

			if ( examineSymbol !== symbolRule.symbol ) {
				return false;
			}
		}

		// then check if the symbol below the examine position is NOT the symbol type we are looking for,
		// so we can identify where the stack starts.
		var bottomOfStackPos = rootExaminePos + 1;
		if ( bottomOfStackPos >= reelStrip.length ) {
			bottomOfStackPos -= reelStrip.length;
		}
		var bottomStackSymbol = reelStrip[bottomOfStackPos];
		if ( bottomStackSymbol === symbolRule.symbol ) {
			return false;
		}

		// Now we've identified we are about to show a stack of the given symbol on screen, make sure it's not above the landing position
		// (otherwise we just see a sole animation drop down the screen after the reels have landed)
		if ( targetStopPos !== null && isFinite( targetStopPos ) ) {
			for ( var i = 0; i <= this._settings.symbolsToLookAhead + 1; ++i ) {
				var lookpos = rootExaminePos + i;
				if ( lookpos >= reelStrip.length ) {
					lookpos -= reelStrip.length;
				}
				if ( lookpos === targetStopPos ) {
					return false;
				}
			}
		}
		return true;
	};


	SymbolTeaseAnimation.prototype._doesSymbolQualify = function( symbolId, reelStrip, reelPos, targetStopPos ) {

		var that = this;
		if ( !this._settings.symbolRules ) {
			return false;
		}
		return this._settings.symbolRules.some( function( symbolRule ) {
			return that._doesSymbolPassRule( symbolRule, symbolId, reelStrip, reelPos, targetStopPos );
		} );
	};


	SymbolTeaseAnimation.prototype._onNotifySymbolPassedWindow = function( reelIndex, symbolId, reelStrip, reelPos, targetStopPos ) {
		if ( !this._isQuickPlay ) {
			// check if the symbol that has passed the window is one of the symbols we are looking for.
			if ( this._doesSymbolQualify( symbolId, reelStrip, reelPos, targetStopPos ) ) {
				// trigger animation if it's not already active
				if ( this._settings.sound ) {
					libs.globalEventBus.invoke( 'libs.event.request.playSound', ['HQ'], this._settings.sound );
				}
				this._reelAnims[reelIndex].start( reelIndex );
			}
		}
	};


	SymbolTeaseAnimation.prototype.isDirty = function() {
		return this._reelAnims.some( function( ra ) {
			return ra.isDirty();
		} );
	};


	SymbolTeaseAnimation.prototype.draw = function( ctx ) {

		this._reelAnims.forEach( function( ra ) {
			ra.draw( ctx );
		} );
	};


	libs.client.reels.SymbolTeaseAnimation = SymbolTeaseAnimation;

}());