goog.provide( 'libs.client.reels.ReelDataModel' );

goog.require( 'libs.globalEventBus' );

goog.require( 'game.logic' );


(function() {
	"use strict";

	var clampReelPosition = function( position, strip ) {
		if ( !isFinite( position ) ) {
			throw( 'clampReelPosition: distance must be a finite number' );
		}
		// Don't use modulus as that'll perform a Math.floor on it - we want floating point support
		while ( position >= strip.length ) {
			position -= strip.length;
		}
		while ( position < 0 ) {
			position += strip.length;
		}
		return position;
	};


	var insertForcedSymbolsIntoVirtualBand = function( reelIndex, virtualBand, forcedSymbols, stop ) {
		// insert forced symbols here
		if ( forcedSymbols !== undefined && forcedSymbols !== null ) {
			for ( var forcedIndex = 0; forcedIndex < forcedSymbols.length; ++forcedIndex ) {
				var sym = forcedSymbols[forcedIndex];
				if ( sym['reel'] === reelIndex ) {
					var forcedPosition = clampReelPosition( stop + sym['row'], virtualBand );
					virtualBand[forcedPosition] = sym['symbol'];
				}
			}
		}
	};


	var checkBandIndexForAdjacencyCompliance = function( symbolAdjacencyRules, virtualBand, index ) {

		if ( !symbolAdjacencyRules ) {
			return true;
		}
		// look down virtual band to make sure all adjacency rules are obeyed
		var allPassed = symbolAdjacencyRules.every( function( rule ) {
			var testSym = virtualBand[index];
			var isRuleSymbol = false;
			if ( Array.isArray( rule.symbol ) ) {
				isRuleSymbol = rule.symbol.indexOf( testSym ) >= 0;
			} else {
				isRuleSymbol = testSym === rule.symbol;
			}

			if ( isRuleSymbol ) {

				if ( rule.block ) {
					// 'block' checks if certain symbols repeat themselves consecutively for longer than the given distance
					var blockLength = 0;
					var longestBlockLength = 0;
					for ( var offset = -rule.distance; offset <= rule.distance; ++offset ) {
						var testIndex = index + offset;
						while ( testIndex < 0 ) {
							testIndex += virtualBand.length;
						}
						while ( testIndex >= virtualBand.length ) {
							testIndex -= virtualBand.length;
						}
						var offsetSym = virtualBand[testIndex];
						if ( offsetSym === testSym ) {
							blockLength++;
						} else {
							if ( longestBlockLength < blockLength ) {
								longestBlockLength = blockLength;
							}
							blockLength = 0;
						}
					}
					if ( rule.must ) {
						return longestBlockLength === rule.distance;
					} else {
						if ( longestBlockLength > rule.distance ) {
							return false;
						}
					}
				}

				// Check 'only' and 'not' rules
				for ( var distance = 1; distance <= rule.distance; ++distance ) {

					var aboveIndex = index - distance;
					if ( aboveIndex < 0 ) {
						aboveIndex += virtualBand.length;
					}
					var belowIndex = ( index + distance ) % virtualBand.length;

					var aboveSym = virtualBand[aboveIndex];
					var belowSym = virtualBand[belowIndex];

					if ( rule.only ) {
						// 'only' method specified. This'll test to make sure ONLY the symbols in the 'only' array appear next to the test symbol.
						if ( rule.only.indexOf( aboveSym ) < 0 || rule.only.indexOf( belowSym ) < 0 ) {
							return false;
						}
					}

					if ( rule.not ) {
						// 'not' method specified. This'll test to make sure that the symbols in the 'not' array do not appear next to the test symbol.
						if ( rule.not.indexOf( aboveSym ) >= 0 || rule.not.indexOf( belowSym ) >= 0 ) {
							return false;
						}
					}
				}
			}
			return true;
		} );
		return allPassed;
	};


	var copyReelBandInto = function( sourcePos, sourceStrip, targetPos, targetStrip, copyLength, copyForwards ) {
		for ( var index = 0; index < copyLength; ++index ) {
			var offset = copyForwards ? index : -index;
			var sourceSymPos = clampReelPosition( sourcePos + offset, sourceStrip );
			var targetSymPos = clampReelPosition( targetPos + offset, targetStrip );
			targetStrip[targetSymPos] = sourceStrip[sourceSymPos];
		}
	};


	// this._allReelStrips[ this._reelSetIndex ].slice( 0 )

	var imposeLandingSymbolCalc = function( attemptsToCreateBandTotal, symbolAdjacencyRules, reelIndex, currentBand, currentFloatingPointStop, targetBand, targetStop,
											forcedSymbols, reverseReels, symbolsAboveReels, symbolsBelowReels, numberOfSymbolsToDisplay,
											verticalSymbolSpacing, extraSymbols ) {

		extraSymbols = extraSymbols || 0;
		var currentStop = Math.floor( currentFloatingPointStop );
		// superimpose the symbols from the new reel strip onto the existing one, starting from 3 symbols away from the on-screen symbols
		// so the user does not notice the change. The new landing symbols will be X symbols away from the change

		var symbolsInView;
		if ( reverseReels ) {
			symbolsInView = symbolsAboveReels + numberOfSymbolsToDisplay;
		} else {
			symbolsInView = symbolsBelowReels + numberOfSymbolsToDisplay;
		}
		var distanceFromCurrentPositionToLanding = symbolsInView + extraSymbols;

		var newSymbolStrip;
		var copyPosition;
		var success = false;
		for ( var attemptsToCreateNewBand = 0; attemptsToCreateNewBand < attemptsToCreateBandTotal; ++attemptsToCreateNewBand ) {

			// Copy the existing symbols in view on the 'virtual' band onto the target band, just above the landing symbols on the new band.
			// Make sure where the 2 bands join, that the reel band adjancency rules are obeyed. If not, make the gap between the old (in-view) symbols
			// and the landing symbols one symbol bigger until the rules are satisfied.
			// This may cause the reels to land out of sequence
			newSymbolStrip = targetBand.slice( 0 );

			// calculate position on new strip where we put the symbols that are currently in view
			if ( reverseReels ) {
				copyPosition = clampReelPosition( targetStop - distanceFromCurrentPositionToLanding, newSymbolStrip );
			} else {
				copyPosition = clampReelPosition( targetStop + distanceFromCurrentPositionToLanding, newSymbolStrip );
			}

			// copy them into target reel set, just before the landing symbols
			copyReelBandInto( currentStop, currentBand, copyPosition, newSymbolStrip, symbolsInView, true );
			// impose forced symbols
			insertForcedSymbolsIntoVirtualBand( reelIndex, newSymbolStrip, forcedSymbols, targetStop );

			// make sure where the symbols join that the adjacency rules are ok - don't need to check the whole band
			var isOk = true;
			var symbolsToCheckCount = distanceFromCurrentPositionToLanding + symbolsInView;
			for ( var count = 0; count < symbolsToCheckCount; ++count ) {
				var symbolIndex = clampReelPosition( copyPosition + count, newSymbolStrip );
				if ( !checkBandIndexForAdjacencyCompliance( symbolAdjacencyRules, newSymbolStrip, symbolIndex ) ) {
					isOk = false;
					break;
				}
			}
			if ( isOk ) {
				success = true;
				break;
			}
			distanceFromCurrentPositionToLanding++;
		}

		if ( !success ) {
			// Emergency: Could not find a suitable landing position which does not break the rules. So, just let the rules be broken for this spin
			newSymbolStrip = targetBand.slice( 0 );
			distanceFromCurrentPositionToLanding = symbolsInView;
			if ( reverseReels ) {
				copyPosition = clampReelPosition( targetStop - distanceFromCurrentPositionToLanding, newSymbolStrip );
			} else {
				copyPosition = clampReelPosition( targetStop + distanceFromCurrentPositionToLanding, newSymbolStrip );
			}
			copyReelBandInto( currentStop, currentBand, copyPosition, newSymbolStrip, symbolsInView, true );
			insertForcedSymbolsIntoVirtualBand( reelIndex, newSymbolStrip, forcedSymbols, targetStop );
		}

		var positionLeftOver;
		if ( reverseReels ) {
			positionLeftOver = 1 - ( currentFloatingPointStop - Math.floor( currentFloatingPointStop ) ) - 1;
		} else {
			positionLeftOver = currentFloatingPointStop - Math.floor( currentFloatingPointStop );
		}

		var distance = ( distanceFromCurrentPositionToLanding + positionLeftOver ) * verticalSymbolSpacing;
		var distanceToLandingSymbolsInView;
		var newPosition = clampReelPosition( copyPosition + ( reverseReels ? -positionLeftOver : positionLeftOver ), newSymbolStrip );
		if ( reverseReels ) {
			distanceToLandingSymbolsInView = ( distanceFromCurrentPositionToLanding + positionLeftOver ) * verticalSymbolSpacing;
		} else {
			distanceToLandingSymbolsInView = ( distanceFromCurrentPositionToLanding + positionLeftOver - symbolsInView ) * verticalSymbolSpacing;
		}
		// returns the distance (in pixels) that the reel has to move in order to reach the landing position,
		// and the distance it has to travel before the first landing symbol rolls into view.
		return {
			landingPos: targetStop,
			distance: distance,
			newBand: newSymbolStrip,
			newPos: newPosition,
			distanceToLandingSymbolsInView: distanceToLandingSymbolsInView,
			success: success
		};
	};


	var defaultSettings = {
		symbolAdjacencyRules: ( game.logic.GameData.reels && game.logic.GameData.reels.symbolAdjacencyRules ) ? game.logic.GameData.reels.symbolAdjacencyRules : null,
		maximumAttemptsToCreateReelBand: 100,

		symbolsToLeaveAboveReels: 1,
		symbolsToLeaveBelowReels: 1

	};


	/*
	 * ReelDataModel - Merges the reel bands from one set to another, without
	 * letting the user see the switch and that it obeys certain rules (such as special symbols needing to be a certain distance from each other,
	 * or ensuring certain symbols always appear in groups of 3)
	 *
	 */

	var ReelDataModel = function( overrideSettings, reelIndex, allReelStrips, numberOfSymbolsToDisplay, verticalSymbolSpacing ) {

		var that = this;

		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		this._reelIndex = reelIndex;
		this._allReelStrips = allReelStrips;
		this._numberOfSymbolsToDisplay = numberOfSymbolsToDisplay;
		this._verticalSymbolSpacing = verticalSymbolSpacing;
		this._reelSetIndex = 0;
		this._spinParameters = {};

		// the virtual reel strip is a made-up reel band combining the original symbols on the reels,
		// the reel set the game is spinning on, and when the stop request comes in it will contain the landing symbols.
		this._virtualStrip = [];
		this._virtualStripPosition = 0;

		this._symbols = [];
		for ( var i = 0; i < numberOfSymbolsToDisplay + 3; ++i ) {// While the reels are moving you can see one extra symbol. Plus the ones just off the reel might overlap.
			this._symbols.push( { id: 0 } );
		}
		this._firstVisibleSymbol = 1; //The second symbol in the list is the top displayed symbol.

		libs.globalEventBus.connect( 'libs.event.notify.spinParameters', function( spinParameters ) {
			that._onNotifySpinParameters( spinParameters );
		} );
	};

	ReelDataModel.prototype._onNotifySpinParameters = function( spinParameters ) {
		this._spinParameters = spinParameters;
	};

	ReelDataModel.prototype._insertForcedSymbolsIntoVirtualBand = function( virtualBand, forcedSymbols, stop ) {
		insertForcedSymbolsIntoVirtualBand( this._reelIndex, virtualBand, forcedSymbols, stop );
	};

	ReelDataModel.prototype._createSymbol = function( stop ) {
		var id = this._virtualStrip[stop];

		if ( !game.logic.GameData.reels.symbols.shortNameExists( id ) ) {
			throw new Error( "ReelDataModel.prototype._createSymbol : Could not find symbol (" + id.toString() + ") in game.logic.GameData.reels.symbols.shortNameExists()" );
		}

		return {
			id: id,
			virtualReelPosition: stop // NOTE: This is the position on the 'virtual' band, not the 'real' reel band! Care should be taken not to mix virtual positions and 'real' positions.
		};
	};

	ReelDataModel.prototype._applySymbolPattern = function( symbolData ) {
		var that = this;

		// make a copy of the existing band
		var virtualBand = new Array( this._virtualStrip.length );
		for ( var i = 0; i < this._virtualStrip.length; ++i ) {
			virtualBand[i] = this._virtualStrip[i];
		}

		var symbolArray = symbolData.data[this._reelIndex];
		var visibleProtectedIndex = clampReelPosition( Math.floor( this._virtualStripPosition ) - this._settings.symbolsToLeaveAboveReels, virtualBand );
		var protectedIndexCount = this._numberOfSymbolsToDisplay + this._settings.symbolsToLeaveBelowReels + this._settings.symbolsToLeaveAboveReels;

		var isProtectedIndex = function( i, pindex, protectedLength ) {
			for ( var a = pindex; a < ( pindex + protectedLength ); ++a ) {
				var clamped = that.clampReelPosition( a, virtualBand );
				if ( clamped === i ) {
					return true;
				}
			}
			return false;
		};

		var startIndex = 0;
		if ( symbolData.randomiseStartIndex ) {
			startIndex = Math.floor( Math.random() * symbolArray.length );
		}
		for ( var index = 0; index < virtualBand.length; ++index ) {
			var symIndex = clampReelPosition( startIndex + index, symbolArray );
			var sym = symbolArray[symIndex];
			if ( sym !== -1 && !isProtectedIndex( index, visibleProtectedIndex, protectedIndexCount ) ) {
				var oldSymbol = virtualBand[index];
				virtualBand[index] = sym;
				// check if band is ok according to adjacency rules, revert to old symbol if it breaks compliance
				if ( !this._checkBandIndexForAdjacencyCompliance( virtualBand, index ) ) {
					virtualBand[index] = oldSymbol;
				}
			}
		}

		this._virtualStrip = virtualBand;
	};

	ReelDataModel.prototype.clampReelPosition = function( position, useStrip ) {
		var strip = useStrip === undefined ? this._virtualStrip : useStrip;
		return clampReelPosition( position, strip );
	};

	ReelDataModel.prototype.setInitStrip = function( reelSetIndex, position ) {
		this._reelSetIndex = reelSetIndex;
		this._virtualStrip = this._allReelStrips[reelSetIndex].slice( 0 );
		this.setStop( position );
	};

	ReelDataModel.prototype.forceSymbols = function( toForce, stopPosition, dontUpdate ) {
		if ( stopPosition === undefined ) {
			stopPosition = this.getStop();
		}
		this._insertForcedSymbolsIntoVirtualBand( this._virtualStrip, toForce, stopPosition );
		if ( !dontUpdate ) {
			this._updateSymbolImages();
		}
	};


	ReelDataModel.prototype._buildSymbolAdjacencyRules = function() {

		if ( this._spinParameters.symbolAdjacencyRules ) {
			return this._settings.symbolAdjacencyRules.concat( this._spinParameters.symbolAdjacencyRules );
		} else {
			return this._settings.symbolAdjacencyRules;
		}
	};


	ReelDataModel.prototype._checkBandIndexForAdjacencyCompliance = function( virtualBand, index ) {

		return checkBandIndexForAdjacencyCompliance( this._buildSymbolAdjacencyRules(), virtualBand, index );
	};


	ReelDataModel.prototype._isNewVirtualBandOkay = function( newBandData ) {

		// look down virtual band to make sure all adjacency rules are obeyed
		for ( var index = 0; index < newBandData.length; ++index ) {
			if ( !this._checkBandIndexForAdjacencyCompliance( newBandData, index ) ) {
				return false;
			}
		}
		return true;
	};


	// merges the existing reel band with the newly selected one, and returns the new virtual position of the
	// landing symbols
	// Passing 'forcedSymbols' here will mean the forced symbols will be inserted into the landing positions,
	// so they will roll into the aperture on landing. Use forceSymbols() method to change displayed symbols
	// after they have landed.
	ReelDataModel.prototype.mergeTargetReelSet = function( reelSetIndex ) {

		if ( this._reelSetIndex !== reelSetIndex ) {
			this._reelSetIndex = reelSetIndex;

			// Merge in new reel set with existing symbols
			var currentStop = this.getStop();

			// superimpose the symbols from the new reel strip onto the existing one, starting from 3 symbols away from the on-screen symbols
			// so the user does not notice the change.
			var symbolsToAdd = this._numberOfSymbolsToDisplay + this._settings.symbolsToLeaveAboveReels + this._settings.symbolsToLeaveBelowReels;

			var newBandData;
			var additionalSymbolsToCopy = 0;
			var destCopyPosition;
			for ( var attemptsToCreateNewBand = 0; attemptsToCreateNewBand < this._settings.maximumAttemptsToCreateReelBand; ++attemptsToCreateNewBand ) {

				newBandData = this._allReelStrips[this._reelSetIndex].slice( 0 );

				// Copy over the symbols that are currently visible onto the new reelband, at the same (clamped) position that it is on at the moment.
				var sourceCopyPosition = clampReelPosition( currentStop - this._settings.symbolsToLeaveAboveReels - additionalSymbolsToCopy, this._virtualStrip );
				var copyLength = Math.min( symbolsToAdd + ( additionalSymbolsToCopy * 2 ), newBandData.length );
				destCopyPosition = clampReelPosition( currentStop - this._settings.symbolsToLeaveBelowReels - additionalSymbolsToCopy, newBandData );
				copyReelBandInto( sourceCopyPosition, this._virtualStrip, destCopyPosition, newBandData, copyLength, true );

				if ( this._isNewVirtualBandOkay( newBandData ) ) {
					break;
				}
				additionalSymbolsToCopy++;
			}
			this._virtualStrip = newBandData;
			this._virtualStripPosition = clampReelPosition( this._virtualStripPosition, this._virtualStrip );

			if ( this._imposedSymbols ) {
				this._applySymbolPattern( this._imposedSymbols );
			}
		}
	};


	// used by the blurred reel spin to immediately change the reel set without doing a merge
	ReelDataModel.prototype.setTargetReelSet = function( reelSetIndex, forcedSymbols, stop, doUpdate ) {
		this._reelSetIndex = reelSetIndex;
		this._virtualStrip = this._allReelStrips[reelSetIndex].slice( 0 );
		this._virtualStripPosition = stop;
		this._insertForcedSymbolsIntoVirtualBand( this._virtualStrip, forcedSymbols, this._virtualStripPosition );
		if ( doUpdate ) {
			this._updateSymbolImages();
		}
	};


	ReelDataModel.prototype.imposeLandingSymbols = function( forcedSymbols, stop, reverseReels ) {

		var extraSymbols = this._spinParameters.insertExtraSymbolsOnLand ? this._spinParameters.insertExtraSymbolsOnLand[this._reelIndex] : 0;
		if ( this._spinParameters.disableExtraSymbolsInsert && this._spinParameters.disableExtraSymbolsInsert[this._reelIndex] ) {
			extraSymbols = 0;
		}
		var result = imposeLandingSymbolCalc( 100, this._buildSymbolAdjacencyRules(), this._reelIndex, this._virtualStrip, this._virtualStripPosition, this._allReelStrips[this._reelSetIndex], stop,
			forcedSymbols, reverseReels, this._settings.symbolsToLeaveAboveReels, this._settings.symbolsToLeaveBelowReels, this._numberOfSymbolsToDisplay,
			this._verticalSymbolSpacing, extraSymbols );
		this._virtualStrip = result.newBand;
		this._virtualStripPosition = result.newPos;
		return result;
	};

	ReelDataModel.prototype.testImposeLandingSymbols = function( forcedSymbols, stop, reverseReels, extraSymbols ) {

		var result = imposeLandingSymbolCalc( 1, this._buildSymbolAdjacencyRules(), this._reelIndex, this._virtualStrip, this._virtualStripPosition, this._allReelStrips[this._reelSetIndex], stop,
			forcedSymbols, reverseReels, this._settings.symbolsToLeaveAboveReels, this._settings.symbolsToLeaveBelowReels, this._numberOfSymbolsToDisplay,
			this._verticalSymbolSpacing, extraSymbols );
		return result.success;
	};

	ReelDataModel.prototype.getSymbolsAtStop = function( stop ) {
		var symbols = [];
		for ( var i = 0; i < this._numberOfSymbolsToDisplay; i++ ) {
			var safeStop = clampReelPosition( stop + i, this._virtualStrip );
			symbols[i] = this._createSymbol( safeStop );
		}
		return symbols;
	};

	ReelDataModel.prototype.getDisplayedSymbols = function() {

		return this._symbols.slice( 0 );
	};

	ReelDataModel.prototype.indexToRow = function( index ) {
		return index - this._firstVisibleSymbol;
	};

	ReelDataModel.prototype._updateSymbolImages = function() {

		var firstIndex = Math.floor( this._virtualStripPosition ) - this._firstVisibleSymbol;
		for ( var i = 0; i < this._symbols.length; i++ ) {
			var symbolStop = clampReelPosition( firstIndex + i, this._virtualStrip );
			this._symbols[i] = this._createSymbol( symbolStop );
		}
	};

	ReelDataModel.prototype.move = function( distance ) {
		if ( !isFinite( distance ) ) {
			throw( 'ReelDataModel.prototype.move: distance must be a finite number' );
		}
		var stopMove = distance / this._verticalSymbolSpacing;
		this._virtualStripPosition = clampReelPosition( this._virtualStripPosition + stopMove, this._virtualStrip );

		this._updateSymbolImages();
	};

	ReelDataModel.prototype.setStop = function( stop ) {
		if ( !isFinite( stop ) ) {
			throw( 'ReelDataModel.prototype.setStop: stop must be a finite number' );
		}
		this._virtualStripPosition = clampReelPosition( Math.floor( stop ), this._virtualStrip );
		this._updateSymbolImages();
	};

	ReelDataModel.prototype.getStop = function() {
		return Math.floor( this._virtualStripPosition );
	};

	ReelDataModel.prototype.getStrip = function() {
		return this._virtualStrip;
	};

	// Returns a values between 0 and 1 multiplied by the symbol spacing for how far back the top displayed symbol should be rendered.
	// subtract from start y position to get position to render top symbol.
	ReelDataModel.prototype.getDisplayOffset = function( symbolSpacing ) {
		var offset = this._virtualStripPosition - Math.floor( this._virtualStripPosition );
		return (offset * symbolSpacing) + (this._firstVisibleSymbol * symbolSpacing);
	};

	ReelDataModel.prototype.imposeSymbolPattern = function( symbolData ) {
		this._imposedSymbols = symbolData;
		this._applySymbolPattern( symbolData );
	};

	ReelDataModel.prototype.getFirstVisibleSymbol = function() {
		return this._firstVisibleSymbol;
	};


	libs.client.reels.ReelDataModel = ReelDataModel;

}());