goog.provide( 'libs.client.reels.ReelModel' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.StateMachine' );
goog.require( 'libs.client.reels.ReelDataModel' );


(function() {
	"use strict";


	var defaultSettings = {};


	var ReelModel = function( overrideSettings, reelsAnimationType, reelIndex, reelStrips, numberOfSymbolsToDisplay, verticalSymbolSpacing, startingReelSet ) {
		var that = this;

		this._spinParameters = {};
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		this._reelIndex = reelIndex;
		this._symbolDisplayCount = numberOfSymbolsToDisplay;
		this.reelDataModel = new libs.client.reels.ReelDataModel( this._settings.dataModel, reelIndex, reelStrips, numberOfSymbolsToDisplay, verticalSymbolSpacing );
		// Most games use reel set 0 (but can be customised through buildinitresponse.js) provided by server in position 0 for each reel for starting game
		this.reelDataModel.setInitStrip( startingReelSet || 0, 0 );

		this._isDirty = true;
		this._reelFullSpeed = false;
		this._isSpinning = false;
		this._reelSpeed = 0.0;
		this._bounceAmount = 0.0;
		this._bouncedSoFar = 0.0;
		this._distanceTravelled = 0.0;
		this._virtualStop = 0;
		this._isBlurred = false;
		this._lastDisplayedSymbols = null;
		this._inFinishSpinState = false;
		this._reelStoppingActivityCompleted = false;
		this._forcedSymbols = [];
		this._realStopPosition = 0;
		this._betResult = null;
		this._distanceUntilInView = 0;
		this._animationStateMachine = new libs.StateMachine();

		this._verticalSymbolSpacing = verticalSymbolSpacing;

		this._handleHeavyDrop = null;

		libs.globalEventBus.connect( 'libs.event.request.forceSymbols', function( symbolsToChange, stopPosition, doNotUpdate ) { that._onRequestForceSymbols( symbolsToChange, stopPosition, doNotUpdate ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.reels.heavyDrop', function( activity, reelIndex ) { that._onHeavyDropStart( activity, reelIndex ); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.reels.reelStoppingTime', function( reelIndex ) { that._onReelStoppingTimeEnd( reelIndex ); } );
		libs.globalEventBus.connect( 'libs.event.request.forceReelSet', function( reelSetIndex, reelPositions ) { that._onRequestForceReelSet( reelSetIndex, reelPositions ); } );
		libs.globalEventBus.connect( 'libs.event.request.changeReelSpeed', function( reelIndex, reelSpeed ) { that._onRequestChangeReelSpeed( reelIndex, reelSpeed ); } );
		libs.globalEventBus.connect( 'libs.event.notify.spinParameters', function( spinParameters ) {
			that._onNotifySpinParameters( spinParameters );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.betResult', function( betResult ) { that._onNotifyBetResult( betResult ); } );
	};

	ReelModel.prototype._onNotifyBetResult = function( betResult ) {
		var that = this;
		this._betResult = betResult;

		if ( this._betResult['reels'] && this._betResult['reels']['stops'] ) {
			this._realStopPosition = this._betResult['reels']['stops'][this._reelIndex];

			if ( this._betResult['reels']['force_symbols'] ) {
				this._forcedSymbols = this._betResult['reels']['force_symbols'].filter( function( force ) {
					return force['reel'] === that._reelIndex;
				} );
			} else {
				this._forcedSymbols = [];
			}
		}
	};

	ReelModel.prototype._onNotifySpinParameters = function( spinParameters ) {
		this._spinParameters = spinParameters;
	};

	ReelModel.prototype.renderBlurred = function() {
		return this._isBlurred;
	};

	ReelModel.prototype.getReelIndex = function() {
		return this._reelIndex;
	};

	ReelModel.prototype.getSymbolDisplayCount = function() {
		return this._symbolDisplayCount;
	};

	ReelModel.prototype._onReelStoppingTimeEnd = function( reelIndex ) {
		if ( this._reelIndex === reelIndex ) {
			this._reelStoppingActivityCompleted = true;
		}
	};

	ReelModel.prototype._onRequestChangeReelSpeed = function( reelIndex, reelSpeed ) {
		if ( this._reelIndex === reelIndex ) {
			this._reelSpeed = reelSpeed;
		}
	};

	ReelModel.prototype._onRequestForceSymbols = function( symbolsToChange, stopPosition, doNotUpdate ) {
		var that = this;
		var changes = symbolsToChange.filter( function( chng ) {
			return chng['reel'] === that._reelIndex;
		} );

		this.reelDataModel.forceSymbols( changes, stopPosition, doNotUpdate );
	};

	ReelModel.prototype._onRequestForceReelSet = function( reelSetIndex, reelPositions ) {

		this.reelDataModel.setTargetReelSet( reelSetIndex, null, reelPositions[ this._reelIndex], true );
	};

	ReelModel.prototype._onHeavyDropStart = function( activity, reelIndex ) {
		if ( reelIndex === this._reelIndex ) {
			this._handleHeavyDrop = activity.join();
		}
	};

	ReelModel.prototype.update = function( timeDelta ) {
		this._animationStateMachine.update( timeDelta );
	};

	ReelModel.prototype.abortAnim = function() {
		this._animationStateMachine.stateGoto( null );
		this._isDirty = true;
	};

	ReelModel.prototype.switchReelSet = function( reelSet ) {
		this.reelDataModel.mergeTargetReelSet( reelSet );
		if ( this._spinParameters.imposeReelSymbols ) {
			this.reelDataModel.imposeSymbolPattern( this._spinParameters.imposeReelSymbols );
		}
	};

	ReelModel.prototype.spinToStop = function() {
		var that = this;
		this._inFinishSpinState = true;
		if ( !this._heavyDropped ) {
			this._animationStateMachine.stateGoto( function( timeDelta ) {
				that._stateFinishSpin( timeDelta );
			} );
		} else {
			this._heavyDropped = false;
			this._finishSpin();
		}
	};

	ReelModel.prototype.blurredStop = function() {
		this._inFinishSpinState = true;
		var that = this;
		this._virtualStop = this._realStopPosition;
		this._isBlurred = false;
		this.reelDataModel.setTargetReelSet( this._betResult['reels']['reelset'], this._forcedSymbols, this._realStopPosition );
		var bounceSpeed = this._spinParameters.maxSpeed;
		var bounceTime = this._spinParameters.bounceBackTime;
		this._animationStateMachine.stateGoto( function( timeDelta ) { that._stateBounce( timeDelta, bounceSpeed, bounceTime ); } );
	};

	ReelModel.prototype.jump = function() {
		// If a 'complete spin' is already in progress, this jumps the animation straight to the given end positions
		if ( this._inFinishSpinState ) {
			var obj = this.reelDataModel.imposeLandingSymbols( this._forcedSymbols, this._realStopPosition, this._spinParameters.reverseReels );
			this._virtualStop = obj.landingPos;
			this._finishSpin();
		}
	};

	ReelModel.prototype.isDirty = function() {
		return this._isDirty;
	};

	ReelModel.prototype.setClean = function() {
		this._isDirty = false;
	};

	ReelModel.prototype.getVirtualBandPositions = function( stopPosition ) {
		if ( stopPosition === undefined ) {
			stopPosition = this._virtualStop;
		}
		var stops = [];
		for ( var index = 0; index < this._symbolDisplayCount; ++index ) {
			stops.push( this.reelDataModel.clampReelPosition( stopPosition + index ) );
		}
		return stops;
	};

	ReelModel.prototype._moveReel = function( move ) {

		if ( this._spinParameters.lockedReels && this._spinParameters.lockedReels.indexOf( this._reelIndex ) > -1 ) {
			// This reel is locked so we ignore updates
			return;
		}

		this.reelDataModel.move( this._spinParameters.reverseReels ? move * -1 : move );
		this._isDirty = true;

		// get symbol passing window
		var currentSymbols = this.reelDataModel.getDisplayedSymbols();
		var hasChanged = false;

		if ( this._lastDisplayedSymbols ) {
			for ( var index = 0; index < currentSymbols.length; ++index ) {
				if ( this._lastDisplayedSymbols[index].id !== currentSymbols[index].id ) {
					hasChanged = true;
					break;
				}
			}
		} else {
			hasChanged = true;
		}
		if ( hasChanged ) {
			this._lastDisplayedSymbols = currentSymbols.slice( 0 );
			libs.globalEventBus.invoke( 'libs.event.notify.symbolPassedWindow', this._reelIndex,
				this._lastDisplayedSymbols[this.reelDataModel.getFirstVisibleSymbol()].id, this.reelDataModel.getStrip(), this.reelDataModel.getStop(), Math.floor( this._virtualStop ) ); // Pass second symbol in array as first in above the reels (not rendered fully)
		}
	};

	ReelModel.prototype.startReelSpin = function() {
		var that = this;

		this._reelFullSpeed = false;
		this._isSpinning = true;

		if ( this._spinParameters.imposeReelSymbols ) {
			this.reelDataModel.imposeSymbolPattern( this._spinParameters.imposeReelSymbols );
		}
		this._animationStateMachine.stateGoto( function( timeDelta ) { that._stateStartSpin( timeDelta ); } );
	};

	ReelModel.prototype.startBlurredSpin = function() {
		var that = this;

		this._isSpinning = true;
		this._animationStateMachine.stateGoto( function( timeDelta ) { that._stateStartBlur( timeDelta ); } );
	};

	ReelModel.prototype.startHeavyDropSpin = function() {
		var that = this;

		this._reelFullSpeed = false;
		this._isSpinning = true;
		this._animationStateMachine.stateGoto( function( timeDelta ) {
			that._stateHeavyDropToPosition( timeDelta, that._spinParameters.heavyDropAnimation.oldPositions[that._reelIndex], that._spinParameters.heavyDropAnimation.distances[that._reelIndex] );
		} );
	};

	ReelModel.prototype._stateStartBlur = function( timeDelta ) {
		// pull back
		if ( !this._spinParameters.noPullBack ) {
			if ( this._animationStateMachine.phaseTimed( this._spinParameters.pullbackTime ) ) {
				this._moveReel( ( this._spinParameters.pullbackSpeed ) * timeDelta );
			}
		}

		// set blurred images
		if ( this._animationStateMachine.phaseOnce() ) {
			this._isBlurred = true;
		}
	};

	// Some games (Amazon Goddess) have a 'heavy drop' position instead of the normal spin.
	// This is used in the free spins feature for GA to lock lines of wilds in position.
	// The new position needs to be known in advance.
	ReelModel.prototype._stateHeavyDropToPosition = function( timeDelta, oldPosition, distance ) {
		if ( this._animationStateMachine.phaseOnce() ) {
			libs.globalEventBus.invokeActivity( 'libs.activity.reels.heavyDrop', this._reelIndex );

			this._heavyDropped = false;
			this._distanceTravelled = 0.0;

			this.reelDataModel.setTargetReelSet( this._spinParameters.heavyDropAnimation.reelsetIndex, null, oldPosition );
			this.reelDataModel.setStop( oldPosition );
			this._virtualStop = oldPosition;
			this._distanceToTravel = distance * this._verticalSymbolSpacing;
		}

		if ( this._animationStateMachine.phaseManual() ) {
			if ( this._moveToLimit( timeDelta, this._spinParameters.heavyDropSpeed, this._distanceToTravel ) ) {
				this._animationStateMachine.phaseStep();
			}
		}

		if ( this._animationStateMachine.phaseOnce() ) {
			this._virtualStop = this.reelDataModel.clampReelPosition( oldPosition - distance );
			this.reelDataModel.setStop( this._virtualStop );
		}

		if ( this._animationStateMachine.phaseOnce() ) {
			this._handleHeavyDrop.leave();
			this._handleHeavyDrop = null;
		}

		if ( this._animationStateMachine.phaseTimed( 200 ) ) {
		}

		if ( this._animationStateMachine.phaseOnce() ) {
			this._heavyDropped = true;
			this._reelFullSpeed = true;
		}
	};

	ReelModel.prototype.getVirtualLandingPosition = function() {
		return this._virtualStop;
	};

	ReelModel.prototype._stateStartSpin = function( timeDelta ) {

		var MAX_SPEED = this._spinParameters.maxSpeed;

		// pull back
		if ( !this._spinParameters.noPullBack && this._spinParameters.pullbackTime > 0 ) {
			if ( this._animationStateMachine.phaseTimed( this._spinParameters.pullbackTime ) ) {
				this._moveReel( ( this._spinParameters.pullbackSpeed ) * timeDelta );
			}
		}

		// accelerate
		if ( this._animationStateMachine.phaseOnce() ) {
			this._reelSpeed = 0.0;
			return; // leave it one frame
		}

		if ( this._animationStateMachine.phaseManual() ) {
			this._reelSpeed += timeDelta * ( this._spinParameters.acceleration );

			var limitHit = false;
			if ( this._reelSpeed > 0 && this._reelSpeed >= MAX_SPEED ) {

				this._reelSpeed = MAX_SPEED;
				limitHit = true;
			}
			var distance = timeDelta * this._reelSpeed;
			this._moveReel( -distance );

			if ( limitHit ) {
				this._animationStateMachine.phaseStep();
			}
		}

		// full speed
		if ( this._animationStateMachine.phaseOnce() ) {
			this._reelFullSpeed = true;
			return;
		}

		if ( this._animationStateMachine.phaseManual() ) {
			this._moveReel( -( timeDelta * this._reelSpeed ) );
		}
	};


	ReelModel.prototype._rollReel = function( timeDelta, speed ) {
		var distance = timeDelta * speed;
		this._moveReel( -distance );
	};


	ReelModel.prototype._stateFinishSpin = function( timeDelta ) {
		var that = this;
		var MAX_SPEED = this._spinParameters.maxSpeed;

		if ( this._animationStateMachine.phaseOnce() ) {
			this._reelStoppingActivityCompleted = false;
			this._bounceAmount = 0.0;
			this._bouncedSoFar = 0.0;
			this._reelSpeed = MAX_SPEED;
			libs.globalEventBus.invokeActivity( 'libs.activity.reels.reelStoppingTime', this._reelIndex );
		}

		// Wait for libs.activity.reels.reelStoppingTime to complete.
		if ( this._animationStateMachine.phaseManual() ) {
			if ( this._reelStoppingActivityCompleted ) {
				this._animationStateMachine.phaseStep();
			} else {
				this._rollReel( timeDelta, this._reelSpeed );
			}
		}

		// super impose finishing symbols at the next available position on the virtual band.
		if ( this._animationStateMachine.phaseOnce() ) {
			this._distanceTravelled = 0;
			var obj = this.reelDataModel.imposeLandingSymbols( this._forcedSymbols, this._realStopPosition, this._spinParameters.reverseReels );
			this._distanceToTravel = obj.distance;
			this._distanceUntilInView = obj.distanceToLandingSymbolsInView;
			this._virtualStop = obj.landingPos;
			return;
		}

		if ( this._animationStateMachine.phaseManual() ) {
			var speed = this._reelSpeed;
			if ( this._spinParameters.dropInAnimReels && this._spinParameters.dropInAnimReels[this._reelIndex] ) {
				var dropInAnim = this._spinParameters.dropInAnimReels[this._reelIndex];
				var dropInTween = dropInAnim.tween || libs.client.utils.tweens.linear;
				var progress = dropInTween( 1.0 - ( this._distanceTravelled / this._distanceToTravel ) );
				speed = ( this._reelSpeed - dropInAnim.minSpeed ) * progress + dropInAnim.minSpeed;
			}
			if ( this._moveToLimit( timeDelta, speed, this._distanceToTravel ) ) {
				this._animationStateMachine.phaseStep();
			}
			if ( this._distanceUntilInView > 0 && this._distanceTravelled >= this._distanceUntilInView ) {
				this._distanceUntilInView = 0;
				var distanceLeft = this._distanceToTravel - this._distanceTravelled;
				var symbolsLeft = distanceLeft / this._verticalSymbolSpacing;
				libs.globalEventBus.invoke( 'libs.event.notify.reelAboutToLand', this._reelIndex, distanceLeft, this._reelSpeed, symbolsLeft );
			}
		}

		if ( this._animationStateMachine.phaseOnce() ) {
			var bounceSpeed = this._reelSpeed;
			var bounceTime = this._spinParameters.bounceBackTime;

			this._animationStateMachine.stateGoto( function( timeDelta ) {
				that._stateBounce( timeDelta, bounceSpeed, bounceTime );
			} );
		}
	};

	ReelModel.prototype._moveToLimit = function( timeDelta, speed, distanceLimit ) {
		var distance = timeDelta * speed;
		var newTravelDistance = this._distanceTravelled + distance;

		var capped = false;
		if ( newTravelDistance >= distanceLimit ) {
			distance = distanceLimit - this._distanceTravelled;
			this._moveReel( -distance );
			this._distanceTravelled = distanceLimit;
			capped = true;
		} else {
			this._moveReel( -distance );
			this._distanceTravelled += distance;
		}

		return capped;
	};


	ReelModel.prototype._stateBounce = function( timeDelta, speed, bounceTime ) {
		var distance = 0;
		var bounceDistance = this._spinParameters.bounceDistance;
		var disallowBounce = this._spinParameters.noBounce || ( this._spinParameters.noBounceReels && this._spinParameters.noBounceReels[this._reelIndex] );

		if ( !disallowBounce ) {
			// bounce overshoot
			if ( this._animationStateMachine.phaseManual() ) {
				distance = timeDelta * speed;
				var oldBounceAmount = this._bounceAmount;
				this._bounceAmount += distance;
				if ( this._bounceAmount > bounceDistance ) {
					distance = bounceDistance - oldBounceAmount;
					this._bounceAmount = bounceDistance;
				}
				this._moveReel( -distance );
				if ( this._bounceAmount >= bounceDistance ) {
					this._animationStateMachine.phaseStep();
				}
			}

			if ( this._animationStateMachine.phaseOnce() ) {
				libs.globalEventBus.invoke( 'libs.event.notify.reelBounced', this._reelIndex, this.reelDataModel.getSymbolsAtStop( this._virtualStop ) );
				return; // leave it one frame before starting bounce back
			}

			// bounce back
			if ( this._animationStateMachine.phaseTimed( bounceTime ) ) {
				var newBounceAmount = this._animationStateMachine.phaseTimedGetProgress() * this._bounceAmount;
				distance = newBounceAmount - this._bouncedSoFar;
				this._bouncedSoFar = newBounceAmount;
				this._moveReel( distance );
			}
		}

		if ( this._animationStateMachine.phaseOnce() ) {
			this._finishSpin();
		}
	};

	ReelModel.prototype._finishSpin = function() {
		this.reelDataModel.setStop( this._virtualStop );
		this._animationStateMachine.stateGoto( null );
		var finalSymbols = this.reelDataModel.getSymbolsAtStop( this._virtualStop );
		this._isSpinning = false;
		this._inFinishSpinState = false;

		libs.globalEventBus.invoke( 'libs.event.notify.reelStopped', this._reelIndex, finalSymbols );
	};

	ReelModel.prototype.isFullSpeed = function() {
		return this._reelFullSpeed;
	};

	ReelModel.prototype.isSpinning = function() {
		return this._isSpinning;
	};

	ReelModel.prototype.testImposeLandingSymbols = function( extraSymbols ) {
		return this.reelDataModel.testImposeLandingSymbols( this._forcedSymbols, this._realStopPosition, this._spinParameters.reverseReels, extraSymbols );
	};


	libs.client.reels.ReelModel = ReelModel;

}());