goog.provide( 'libs.client.reels.ReelView' );
goog.provide( 'libs.client.reels.ReelViewGroup' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.rendering.LegacyAnimatedSprite' );
goog.require( 'libs.client.rendering.SpriteGroup' );

goog.require( 'game.logic' );


( function() {
	"use strict";

	var defaultSettings = {
		numberOfSymbolRows: game.logic.GameData.numberOfSymbolRows,
		verticalSymbolSpacing: undefined,
		reelPositions: undefined,
		symbolsData: undefined,
		symbolImage: {
			width: 199,
			height: 256
		},
		dimmedSymbolBoxes: {
			drawOffset: {
				x: 30,
				y: 83
			},
			width: 140,
			height: 92,
			maxAlpha: 0.7,
			animSpeedIn: 0.0015,
			animSpeedOut: 0.0025
		},
		backgroundImage: null
	};

	var threeRowsDefaultSetting = {
		dimmedSymbolBoxes: {
			drawOffset: {
				y: 67
			},
			height: 123
		}
	};

	var fourRowsDefaultSetting = {
		dimmedSymbolBoxes: {
			drawOffset: {
				y: 83
			},
			height: 92
		}
	};

	var symbolImagesSetup = false;
	var symbolImages = {};
	var symbolImagesBlurred = {};
	var symbolAnimations = {};

	var ReelView = function( overrideSettings, reelModel ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		if ( this._settings.numberOfSymbolRows === 3 ) {
			jQuery.extend( true, this._settings, threeRowsDefaultSetting, overrideSettings );
		} else if ( this._settings.numberOfSymbolRows === 4 ) {
			jQuery.extend( true, this._settings, fourRowsDefaultSetting, overrideSettings );
		}

		this._dimmedSymbolBoxPositions = jQuery.extend( true, [], this._settings.reelPositions );
		for ( var i = 0; i < this._dimmedSymbolBoxPositions.length; ++i ) {
			this._dimmedSymbolBoxPositions[i].x += this._settings.dimmedSymbolBoxes.drawOffset.x;
			this._dimmedSymbolBoxPositions[i].y += this._settings.dimmedSymbolBoxes.drawOffset.y;
		}

		var that = this;

		this.reelModel = reelModel;
		this._forceRenderBackground = false;

		if ( !symbolImagesSetup ) {
			for ( var symbolName in this._settings.symbolsData ) {
				if ( this._settings.symbolsData.hasOwnProperty( symbolName ) ) {
					var symbolData = this._settings.symbolsData[symbolName];
					symbolImages[symbolName] = libs.loadedImages[symbolData['name']];
					symbolImagesBlurred[symbolName] = libs.loadedImages[symbolData['name'] + "_BLUR"];

					var animations = libs.client.utils.misc.loadAnimationFrames( libs.loadedImages, symbolData['name'] );
					//If there are no animations just put the normal symbol in
					if ( animations.length === 0 && libs.loadedImages[symbolData['name']] ) {
						animations.push( libs.loadedImages[symbolData['name']] );
					}
					symbolAnimations[symbolName] = animations;
				}
			}
		}

		this.x = this._settings.reelPositions[this.reelModel.getReelIndex()].x;
		this.y = this._settings.reelPositions[this.reelModel.getReelIndex()].y;

		this._dirtyReelband = false;
		this._drawBackgroundFlag = false;

		this._announcedAnimationFinished = [];
		this._symbolAnimations = new libs.client.rendering.SpriteGroup();
		this._symbolAnimations.sprites = [];
		for ( var row = 0; row < this.reelModel.getSymbolDisplayCount(); ++row ) {
			this._symbolAnimations.sprites[row] = null;
			this._announcedAnimationFinished.push( false );
		}

		this._beforeSymbolDrawCallback = null;
		this._afterSymbolDrawCallback = null;

		this._shouldFadeBackground = false;
		this._shouldForceFadedBackground = false;
		this._forceFadedBackgroundDimmed = false;

		this._backgroundAlpha = 0.0;

		this._symbolRegion = {
			symbolsTop: this._dimmedSymbolBoxPositions[0].y,
			symbolsBottom: this._dimmedSymbolBoxPositions[0].y + (this._settings.dimmedSymbolBoxes.height * this._settings.numberOfSymbolRows )
		};

		this._hiddenSymbols = {};

		libs.globalEventBus.connect( 'libs.event.request.startSymbolAnimations', function( symbolsToAnimate ) { that._onRequestStartSymbolAnimations( symbolsToAnimate ); } );
		libs.globalEventBus.connect( 'libs.event.request.showSymbolOnReel', function( reelIndex, rowIndex ) { that._onRequestShowSymbolOnReel( reelIndex, rowIndex ); } );
		libs.globalEventBus.connect( 'libs.event.request.hideSymbolOnReel', function( reelIndex, rowIndex ) { that._onRequestHideSymbolOnReel( reelIndex, rowIndex ); } );
		libs.globalEventBus.connect( 'libs.event.request.showAllSymbolsOnReel', function( reelIndex ) { that._onRequestShowAllSymbolsOnReel( reelIndex ); } );
		libs.globalEventBus.connect( 'libs.event.request.startForceReelFadedBackground', function( reelIndex, forceDimmed ) { that._onRequestStartForceReelFadedBackground( reelIndex, forceDimmed ); } );
		libs.globalEventBus.connect( 'libs.event.request.stopForceReelFadedBackground', function( reelIndex ) { that._onRequestStopForceReelFadedBackground( reelIndex ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.reelsState.displayReelsWin', function() { that._onActivityStartDisplayReelsWin(); } );
		libs.globalEventBus.connect( 'libs.event.notify.showingAllSymbolsOfWinComplete', function() { that._onNotifyShowingAllSymbolsOfWinComplete(); } );

		libs.globalEventBus.connect( 'libs.event.request.startForcedRenderReelBackground', function( imageName ) { that._onRequestStartForcedRenderReelBackground( imageName ); } );
		libs.globalEventBus.connect( 'libs.event.request.stopForcedRenderReelBackground', function() { that._onRequestStopForcedRenderReelBackground(); } );
	};


	ReelView.prototype._onActivityStartDisplayReelsWin = function() {
		this._drawBackgroundFlag = false;
	};

	ReelView.prototype._onNotifyShowingAllSymbolsOfWinComplete = function() {
		this._drawBackgroundFlag = false;
	};


	ReelView.prototype._onRequestStartForcedRenderReelBackground = function( imageName ) {
		this._forceRenderBackground = true;
		this._backgroundImageName = imageName || this._settings.backgroundImage;
	};


	ReelView.prototype._onRequestStopForcedRenderReelBackground = function() {
		this._forceRenderBackground = false;
	};


	ReelView.prototype._onRequestStartSymbolAnimations = function( symbolsToAnimate ) {
		for ( var i = 0; i < this._announcedAnimationFinished.length; ++i ) {
			this._announcedAnimationFinished[i] = false;
		}

		var startY = this.y;
		var offset = this.reelModel.reelDataModel.getDisplayOffset( this._settings.verticalSymbolSpacing );
		startY -= offset;
		startY += this._settings.verticalSymbolSpacing * this.reelModel.reelDataModel.getFirstVisibleSymbol();

		this.fadeBackground( symbolsToAnimate.length > 0 );

		this._clearAnimations();

		for ( var j = 0; j < symbolsToAnimate.length; j++ ) {
			var symbol = symbolsToAnimate[j];
			var row = symbol['row'];
			if ( symbol['reel'] === this.reelModel.getReelIndex() ) {
				var anim = this._symbolAnimations.sprites[row];

				if ( !this._correctAnimation( anim, symbol ) ) {
					this._symbolAnimations.sprites[row] = this._animateSymbol( symbol, startY + this._settings.verticalSymbolSpacing * row );
				}

				anim = this._symbolAnimations.sprites[row];
				if ( anim && !symbol.freezeAtStart ) {
					this._drawBackgroundFlag = true;
					anim.play();
				}
			}
		}
		libs.globalEventBus.invoke( 'libs.event.request.canvasFullDraw', true, true );
	};

	ReelView.prototype._onRequestHideSymbolOnReel = function( reelIndex, rowIndex ) {
		if ( this.reelModel.getReelIndex() === reelIndex ) {
			this._hiddenSymbols[rowIndex] = true;
			this._dirtyReelband = true;
		}
	};

	ReelView.prototype._onRequestShowSymbolOnReel = function( reelIndex, rowIndex ) {
		if ( this.reelModel.getReelIndex() === reelIndex ) {
			this._hiddenSymbols[rowIndex] = false;
			this._dirtyReelband = true;
		}
	};

	ReelView.prototype._onRequestShowAllSymbolsOnReel = function( reelIndex ) {
		if ( this.reelModel.getReelIndex() === reelIndex ) {
			this._hiddenSymbols = {};
			this._dirtyReelband = true;
		}
	};

	ReelView.prototype._onRequestStartForceReelFadedBackground = function( reelIndex, forceDimmed ) {
		if ( this.reelModel.getReelIndex() === reelIndex ) {
			if ( typeof forceDimmed === 'undefined' ) {
				forceDimmed = true;
			}
			this._shouldForceFadedBackground = true;
			this._forceFadedBackgroundDimmed = forceDimmed;
		}
	};

	ReelView.prototype._onRequestStopForceReelFadedBackground = function( reelIndex ) {
		if ( this.reelModel.getReelIndex() === reelIndex ) {
			this._shouldForceFadedBackground = false;
			this._forceFadedBackgroundDimmed = true;
		}
	};

	ReelView.prototype.onBeforeSymbol = function( callback ) {
		this._beforeSymbolDrawCallback = callback;
	};

	ReelView.prototype.onAfterSymbol = function( callback ) {
		this._afterSymbolDrawCallback = callback;
	};

	ReelView.prototype.update = function( timeDelta ) {
		this._symbolAnimations.update( timeDelta );

		for ( var row = 0; row < this._symbolAnimations.sprites.length; ++row ) {
			var anim = this._symbolAnimations.sprites[row];
			if ( anim && anim.isPlaying === false && this._announcedAnimationFinished[row] === false ) {
				this._dirtyReelband = true;
				libs.globalEventBus.invoke( 'libs.event.notify.reelSymbolAnimationFinished', this.reelModel.getReelIndex(), row );
				this._announcedAnimationFinished[row] = true;
			}
		}

		var forceDimmed = this._shouldForceFadedBackground && this._forceFadedBackgroundDimmed;
		var forceLit = this._shouldForceFadedBackground && !this._forceFadedBackgroundDimmed;

		if ( !forceLit && (forceDimmed || this._shouldFadeBackground) ) {
			if ( this._backgroundAlpha < this._settings.dimmedSymbolBoxes.maxAlpha ) {
				this._backgroundAlpha += this._settings.dimmedSymbolBoxes.animSpeedIn * timeDelta;

				if ( this._backgroundAlpha > this._settings.dimmedSymbolBoxes.maxAlpha ) {
					this._backgroundAlpha = this._settings.dimmedSymbolBoxes.maxAlpha;
				}
			}
		} else {
			if ( this._backgroundAlpha > 0 ) {
				this._backgroundAlpha -= this._settings.dimmedSymbolBoxes.animSpeedOut * timeDelta;

				if ( this._backgroundAlpha < 0 ) {
					this._backgroundAlpha = 0;
				}
			}
		}
	};

	ReelView.prototype.drawFadedBackground = function( ctx ) {
		var startY = this._dimmedSymbolBoxPositions[0].y;
		var x = this._dimmedSymbolBoxPositions[this.reelModel.getReelIndex()].x;

		ctx.save();
		ctx.fillStyle = 'rgba(0,0,0,' + this._backgroundAlpha + ')';
		for ( var row = 0; row < this._symbolAnimations.sprites.length; ++row ) {
			if ( this._symbolAnimations.sprites[row] === null ) {
				ctx.fillRect( x, startY, this._settings.dimmedSymbolBoxes.width, this._settings.dimmedSymbolBoxes.height );
			}

			startY += this._settings.dimmedSymbolBoxes.height;
		}

		ctx.restore();
	};

	ReelView.prototype.isDirtyFadedBackground = function() {
		return this._backgroundAlpha !== 0;
	};

	ReelView.prototype.drawFadedBackgroundIfDirty = function( ctx ) {
		if ( this.isDirtyFadedBackground() ) {
			this.drawFadedBackground( ctx );
		}
	};

	ReelView.prototype._workOutLandingInformation = function( symInfo ) {

		// Works out whether the reels are about to land and how far they are from the landing position.
		// Games can then use this information to either overlay 'fake' data to use for draw under / over a symbol,
		// or real data if the symbols are about to land.
		var landingVirtualPositions = this.reelModel.getVirtualBandPositions( this.reelModel.getVirtualLandingPosition() );

		var info = {};
		info.isLandingSymbol = false;
		for ( var i = 0; i < landingVirtualPositions.length; ++i ) {
			if ( symInfo.virtualReelPosition === landingVirtualPositions[i] ) {
				info.isLandingSymbol = true;
				info.landingSymbolRow = i;
				break;
			}
		}
		info.virtualReelPosition = symInfo.virtualReelPosition;
		return info;
	};

	ReelView.prototype._workOutDrawingInformation = function( x, y ) {
		var info = {};
		info.x = x;
		info.y = y;
		return info;
	};

	ReelView.prototype.drawReelband = function( ctx ) {
		var that = this;
		var orders = Object.keys( this._settings.symbolsData ).map( function( symbolName ) {return that._settings.symbolsData[symbolName].zOrder;} );
		var maxZ = Math.max.apply( Math, orders );

		var symbols = this.reelModel.reelDataModel.getDisplayedSymbols();
		var offset = Math.floor( this.reelModel.reelDataModel.getDisplayOffset( this._settings.verticalSymbolSpacing ) );
		var startY = Math.floor( this.y );
		startY -= offset;

		this._drawUnderReel( ctx, symbols, Math.floor( offset / this._settings.verticalSymbolSpacing ) - 1, startY );

		for ( var zOrder = 0; zOrder <= maxZ; zOrder++ ) {
			this._drawOrderedSymbols( ctx, startY, symbols, zOrder );
		}

		this._drawOverReel( ctx, symbols, startY );

		this._dirtyReelband = false;
		this.reelModel.setClean();
	};


	ReelView.prototype._drawOrderedSymbols = function( ctx, startY, symbols, zOrder ) {
		var avoidAnimation;

		for ( var index = 0; index < symbols.length; index++ ) {
			var symInfo = symbols[index];
			var row = this.reelModel.reelDataModel.indexToRow( index );
			var symbolZOrder = this._settings.symbolsData[symInfo.id].zOrder;
			var isVisible = !this._hiddenSymbols[row];
			if ( !this._isAnimating( row ) && symbolZOrder === zOrder && isVisible ) {
				var symbolImg;
				if ( this.reelModel.renderBlurred() ) {
					symbolImg = symbolImagesBlurred[symInfo['id']];
				} else {
					symbolImg = symbolImages[symInfo['id']];
				}

				avoidAnimation = this._calculateClipRegion( row );
				if ( avoidAnimation ) {
					ctx.save();
					ctx.beginPath();
					ctx.rect( avoidAnimation.x, avoidAnimation.y, avoidAnimation.width, avoidAnimation.height );
					ctx.clip();
				}

				this._drawSymbol( ctx, row, symbolImg, symInfo, startY );
				if ( avoidAnimation ) {
					ctx.restore();
				}
			}
			startY += this._settings.verticalSymbolSpacing;
		}

	};

	ReelView.prototype._isAnimating = function( row ) {
		return this._symbolAnimations.sprites[row] && this._symbolAnimations.sprites[row] !== null;
	};

	ReelView.prototype._calculateClipRegion = function( row ) {
		var aboveAnimate = this._isAnimating( row - 1 );
		var belowAnimate = this._isAnimating( row + 1 );

		if ( !aboveAnimate && !belowAnimate ) {
			return null;
		}

		var clip = {
			x: Math.floor( this._dimmedSymbolBoxPositions[this.reelModel.getReelIndex()].x ),
			y: Math.floor( this._dimmedSymbolBoxPositions[0].y ),
			height: Math.floor( this._settings.dimmedSymbolBoxes.height * 3 ),
			width: Math.floor( this._settings.dimmedSymbolBoxes.width )
		};

		if ( aboveAnimate ) {
			clip.y = Math.floor( clip.y + row * this._settings.dimmedSymbolBoxes.height );
		}
		if ( belowAnimate ) {
			clip.height = Math.floor( (row + 1) * this._settings.dimmedSymbolBoxes.height );
		}

		return clip;
	};

	ReelView.prototype._drawUnderReel = function( ctx, symbols, drawOffset, startY ) {
		var symInfo;

		for ( var index = 0; index < symbols.length; index++ ) {
			symInfo = symbols[index];
			var landingInformation = this._workOutLandingInformation( symInfo );
			var drawingInformation = this._workOutDrawingInformation( this.x, startY );

			if ( this._beforeSymbolDrawCallback ) {
				this._beforeSymbolDrawCallback( ctx, symInfo, landingInformation, drawingInformation );
			}
			startY += this._settings.verticalSymbolSpacing;
		}
	};

	ReelView.prototype._drawOverReel = function( ctx, symbols, startY ) {
		for ( var index = 0; index < symbols.length; index++ ) {
			var symInfo = symbols[index];
			var landingInformation = this._workOutLandingInformation( symInfo );
			var drawingInformation = this._workOutDrawingInformation( this.x, startY );

			if ( this._afterSymbolDrawCallback ) {
				this._afterSymbolDrawCallback( ctx, symInfo, landingInformation, drawingInformation );
			}
			startY += this._settings.verticalSymbolSpacing;
		}
	};

	ReelView.prototype.isDirtyReelband = function() {
		return this.reelModel.isDirty() || this._dirtyReelband === true;
	};

	ReelView.prototype.drawReelbandIfDirty = function( ctx ) {
		if ( this.isDirtyReelband() ) {
			this.drawReelband( ctx );
		}
	};

	ReelView.prototype.drawAnimation = function( ctx ) {

		var that = this;
		this._symbolAnimations.sprites.forEach( function( anim, row ) {
			if ( anim ) {
				that._drawBackground( ctx, row );
			}
		} );

		this._symbolAnimations.draw( ctx );
	};

	ReelView.prototype.isDirtyAnimation = function() {
		for ( var row = 0; row < this._symbolAnimations.sprites.length; ++row ) {
			if ( this._symbolAnimations.sprites[row] ) {
				return true;
			}
		}

		return false;
	};

	ReelView.prototype.drawAnimationIfDirty = function( ctx ) {
		if ( this.isDirtyAnimation() ) {
			this.drawAnimation( ctx );
		}
	};

	ReelView.prototype.fadeBackground = function( shouldFade ) {
		this._shouldFadeBackground = shouldFade;
	};

	ReelView.prototype._clearAnimations = function() {
		for ( var row = 0; row < this._symbolAnimations.sprites.length; ++row ) {
			this._symbolAnimations.sprites[row] = null;
		}
		this._dirtyReelband = true;
	};

	ReelView.prototype._correctAnimation = function( anim, symbol ) {
		return anim && anim !== null && anim.id === symbol.symbol;
	};

	ReelView.prototype._animateSymbol = function( symbol, startY ) {
		var symbolAnim = new libs.client.rendering.LegacyAnimatedSprite( symbolAnimations[symbol['symbol']], this._settings.symbolsData[symbol['symbol']].animSettings );
		symbolAnim.id = symbol['symbol'];
		symbolAnim.x = this.x;
		symbolAnim.y = startY;
		if ( symbolAnim.numberOfFrames > 1 ) {
			symbolAnim.x += this._settings.symbolsData[symbol['symbol']].animOffset.x;
			symbolAnim.y += this._settings.symbolsData[symbol['symbol']].animOffset.y;
		}

		var baseSymbol = symbolImages[symbolAnim.id];
		if ( symbolAnim.image.width > baseSymbol.width ) {
			symbolAnim.x -= (symbolAnim.image.width - baseSymbol.width) / 2;
		} else if ( symbolAnim.image.width < baseSymbol.width ) {
			symbolAnim.x += (baseSymbol.width - symbolAnim.image.width) / 2;
		}

		if ( symbolAnim.image.height > baseSymbol.height ) {
			symbolAnim.y -= (symbolAnim.image.height - baseSymbol.height) / 2;
		} else if ( symbolAnim.image.height < baseSymbol.height ) {
			symbolAnim.y += (baseSymbol.height - symbolAnim.image.height) / 2;
		}

		return symbolAnim;
	};


	ReelView.prototype._drawSymbol = function( ctx, row, symbol, symInfo, startY ) {
		var imageWidth = this._settings.symbolImage.width;
		var imageHeight = this._settings.symbolImage.height;
		var yOffset = 0;
		var xSrcOffset, ySrcOffset, image;

		if ( startY < this._symbolRegion.symbolsTop ) {
			yOffset = this._symbolRegion.symbolsTop - startY;
			imageHeight -= yOffset;
		}

		var symbolBottom = startY + yOffset + imageHeight;
		if ( symbolBottom > this._symbolRegion.symbolsBottom ) {
			imageHeight -= symbolBottom - this._symbolRegion.symbolsBottom;
		}


		if ( imageHeight <= 0 ) {
			return;
		}

		if ( symbol === undefined ) {
			throw( 'ReelView.prototype._drawSymbol : symbol argument is undefined' );
		}

		if ( symbol['type'] !== "metagu_sub_image" ) {
			ctx.drawImage( symbol, Math.floor( this.x ), Math.floor( startY ) );
		} else {
			xSrcOffset = symbol.x;
			ySrcOffset = symbol.y;
			image = symbol.image;

			ctx.drawImage( image,
				Math.floor( xSrcOffset ), Math.floor( ySrcOffset + yOffset ), Math.floor( imageWidth ), Math.floor( imageHeight ),
				Math.floor( this.x ), Math.floor( startY + yOffset ), Math.floor( imageWidth ), Math.floor( imageHeight ) );
		}
	};


	ReelView.prototype._drawBackground = function( ctx, row ) {

		if ( this._forceRenderBackground && this._drawBackgroundFlag && row >= 0 && row < this._settings.numberOfSymbolRows ) {
			var bgImage = libs.loadedImages[this._backgroundImageName];
			var width = this._settings.dimmedSymbolBoxes.width;
			var height = this._settings.dimmedSymbolBoxes.height;
			var x = this._dimmedSymbolBoxPositions[this.reelModel.getReelIndex()].x;
			var y = this._dimmedSymbolBoxPositions[this.reelModel.getReelIndex()].y + height * row;

			if ( bgImage['type'] !== "metagu_sub_image" ) {
				ctx.drawImage( bgImage, Math.floor( x ), Math.floor( y ), width, height );
			} else {
				var xSrcOffset = bgImage.x;
				var ySrcOffset = bgImage.y;
				var xoffset = this._settings.reelPositions[0].x + this._settings.dimmedSymbolBoxes.drawOffset.x;
				var yoffset = this._settings.reelPositions[0].y + this._settings.dimmedSymbolBoxes.drawOffset.y;
				ctx.drawImage( bgImage.image,
					Math.floor( x + xSrcOffset - xoffset ),
					Math.floor( y + ySrcOffset - yoffset ),
					Math.floor( width ),
					Math.floor( height ),
					Math.floor( x ),
					Math.floor( y ),
					Math.floor( width ),
					Math.floor( height ) );
			}
		}
	};


	// Helper to handle a collection of reelViews.
	var ReelViewGroup = function( reelViews ) {
		if ( reelViews ) {
			this.reelViews = reelViews;
		} else {
			this.reelViews = [];
		}
	};

	ReelViewGroup.prototype.each = function( toCall ) {
		for ( var i = 0; i < this.reelViews.length; i++ ) {
			if ( this.reelViews[i] ) {
				toCall( this.reelViews[i] );
			}
		}
	};

	ReelViewGroup.prototype.update = function( timeDelta ) {
		this.each( function( reelView ) { reelView.update( timeDelta ); } );
	};

	ReelViewGroup.prototype.drawFadedBackground = function( ctx ) {
		this.each( function( reelView ) { reelView.drawFadedBackground( ctx ); } );
	};

	ReelViewGroup.prototype.isDirtyFadedBackground = function() {
		return this.reelViews.some( function( reelView ) { return reelView.isDirtyFadedBackground(); } );
	};

	ReelViewGroup.prototype.drawFadedBackgroundIfDirty = function( ctx ) {
		this.each( function( reelView ) { reelView.drawFadedBackgroundIfDirty( ctx ); } );
	};

	ReelViewGroup.prototype.drawReelband = function( ctx ) {
		this.each( function( reelView ) { reelView.drawReelband( ctx ); } );
	};

	ReelViewGroup.prototype.isDirtyReelband = function() {
		return this.reelViews.some( function( reelView ) { return reelView.isDirtyReelband(); } );
	};

	ReelViewGroup.prototype.drawReelbandIfDirty = function( ctx ) {
		this.each( function( reelView ) { reelView.drawReelbandIfDirty( ctx ); } );
	};

	ReelViewGroup.prototype.drawAnimation = function( ctx ) {
		this.each( function( reelView ) { reelView.drawAnimation( ctx ); } );
	};

	ReelViewGroup.prototype.isDirtyAnimation = function() {
		return this.reelViews.some( function( reelView ) { return reelView.isDirtyAnimation(); } );
	};

	ReelViewGroup.prototype.drawAnimationIfDirty = function( ctx ) {
		this.each( function( reelView ) { reelView.drawAnimationIfDirty( ctx ); } );
	};


	libs.client.reels.ReelView = ReelView;
	libs.client.reels.ReelViewGroup = ReelViewGroup;

}());
