goog.provide( 'libs.client.reels.ReelsMachineModel' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.serverInstance' );
goog.require( 'libs.StateMachine' );
goog.require( 'libs.Timeout' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.reels.ReelModel' );
goog.require( 'libs.client.reels.ReelsMachineAnimationNormal' );
goog.require( 'libs.client.reels.ReelsMachineAnimationBlurred' );
goog.require( 'libs.client.reels.ReelsDefaultLandingBehaviour' );

goog.require( 'game.logic' );


(function() {
	"use strict";

	var defaultSettings = {
		minimumQuickSpinTime: 650,
		numberOfSymbolRows: game.logic.GameData.numberOfSymbolRows,
		verticalSymbolSpacing: undefined
	};

	var ReelsMachineModel = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._handlePreReelsLanding = null;
		this._handleReelsLanding = null;

		this._reelModels = [];
		this._preReelsLandingHasStarted = false;
		this._reelsAtSpeed = false;
		this._betResult = null;
		this._stateMachine = new libs.StateMachine();
		this._elapsedSpinTime = 0;
		this._spinParameters = {};

		var reelStrips = libs.serverInstance.getReels();
		var startingReelSet = libs.serverInstance.getStartingReelSet();
		var numberOfReels = reelStrips[0].length;
		var reelsAnimationType = this._selectReelsAnimation();
		for ( var i = 0; i < numberOfReels; i++ ) {
			this._createAReel( i, reelsAnimationType, reelStrips, startingReelSet );
		}

		if ( reelsAnimationType === 'normal' ) {
			this._reelsAnimation = new libs.client.reels.ReelsMachineAnimationNormal( this._settings, this._reelModels );
		} else {
			this._reelsAnimation = new libs.client.reels.ReelsMachineAnimationBlurred( this._settings, this._reelModels );
		}

		this._defaultLandingBehaviour = new libs.client.reels.ReelsDefaultLandingBehaviour( this._settings );

		libs.globalEventBus.connectActivityStart( 'libs.activity.reelsState.reelsStarting', function( activity ) {
			that._onReelsStarting( activity );
		} );
		// this._preReelsLandingHasStarted makes sure the libs.activity.reelsState.preReelsLanding activity has started before the reels attempts to land.
		libs.globalEventBus.connectActivityStart( 'libs.activity.reelsState.preReelsLanding', function( activity ) {
			that._onPreReelsLandingStart( activity );
		} );

		libs.globalEventBus.connectActivityStart( 'libs.activity.betState.preBet', function( activity ) {
			that._onPreBetStart( activity );
		} );

		libs.globalEventBus.connectActivityStart( 'libs.activity.reelsState.reelsLanding', function( activity ) {
			that._onReelsLandingStart( activity );
		} );

		libs.globalEventBus.connect( 'libs.event.notify.betResult', function( betResult ) {
			that._onNotifyBetResult( betResult );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.serverBetAborted', function() {
			that._onNotifyServerBetAborted();
		} );
		libs.globalEventBus.connect( 'libs.event.notify.reelsAtSpeed', function() {
			that._onNotifyReelsAtSpeed();
		} );
		libs.globalEventBus.connect( 'libs.event.notify.allReelsStopped', function() {
			that._onNotifyAllReelsStopped();
		} );
		libs.globalEventBus.connect( 'libs.event.notify.spinParameters', function( spinParameters ) {
			that._spinParameters = spinParameters;
		} );
	};

	ReelsMachineModel.prototype._selectReelsAnimation = function() {
		if ( libs.client.core.clientInformation.useBlurredReels() ) {
			return 'blurred';
		} else {
			return 'normal';
		}
	};

	ReelsMachineModel.prototype.getReelModels = function() {
		return this._reelModels;
	};

	ReelsMachineModel.prototype.update = function( timeDelta ) {
		this._reelModels.forEach( function( reel ) {
			reel.update( timeDelta );
		} );
		this._stateMachine.update( timeDelta );
		this._reelsAnimation.update( timeDelta );
	};

	ReelsMachineModel.prototype._createAReel = function( index, reelsAnimationType, reelStrips, startingReelSet ) {
		var stripsForReel = reelStrips.map( function( stripSet ) {
			return stripSet[index];
		} );

		var model = new libs.client.reels.ReelModel( this._settings.reelModel, reelsAnimationType, index, stripsForReel, this._settings.numberOfSymbolRows, this._settings.verticalSymbolSpacing, startingReelSet );

		this._reelModels.push( model );
	};


	ReelsMachineModel.prototype._onNotifyBetResult = function( betResult ) {
		this._betResult = betResult;
	};

	ReelsMachineModel.prototype._onNotifyServerBetAborted = function() {
		this._reelAnimation.abortAnim();
		this._reelModels.forEach( function( reel ) {
			if ( reel ) {
				reel.abortAnim();
			}
		} );
	};

	ReelsMachineModel.prototype._onPreBetStart = function( activity ) {
		this._betResult = null;
	};

	ReelsMachineModel.prototype._onReelsStarting = function( activity ) {
		var that = this;
		this._preReelsLandingHasStarted = false;
		this._reelsAtSpeed = false;
		this._stateMachine.stateGoto( function( timeDelta ) { that._stateSpin( timeDelta ); } );
	};

	ReelsMachineModel.prototype._stateSpin = function( timeDelta ) {

		if ( this._stateMachine.phaseOnce() ) {
			this._reelsAnimation.startSpin();
			this._elapsedSpinTime = 0;
		}

		this._elapsedSpinTime += timeDelta;

		// Note: This code might be tidier if we connected to & joined the preReelsLanding activity ourselves
		// and then left it once we have a bet result and reels at speed received instead of checking isActivityRunning()
		if ( this._stateMachine.phaseManual() ) {
			// wait for bet result to come in and that the reels are all at full speed
			if ( this._reelsAtSpeed && this._betResult !== null ) {
				this._stateMachine.phaseStep();
			}
		}

		// then wait for pre-reels landing activity to end, as a feature may be extending the time the reels spin
		if ( this._stateMachine.phaseManual() ) {
			if ( this._preReelsLandingHasStarted ) {
				this._handlePreReelsLanding.leave();
				this._handlePreReelsLanding = null;
				this._stateMachine.phaseStep();
			}
		}

		if ( this._stateMachine.phaseManual() ) {
			if ( !libs.globalEventBus.isActivityRunning( 'libs.activity.reelsState.preReelsLanding' ) ) {
				this._stateMachine.phaseStep();
			}
		}

		if ( this._stateMachine.phaseManual() ) {
			if ( !this._settings.minimumQuickSpinTime || this._elapsedSpinTime >= this._settings.minimumQuickSpinTime ) {
				this._stateMachine.phaseStep();
			}
		}

		if ( this._stateMachine.phaseManual() ) {
			if ( !this._spinParameters.minimumSpinTime || this._elapsedSpinTime >= this._spinParameters.minimumSpinTime ) {
				this._stateMachine.phaseStep();
			}
		}

//		if (this._animationStateMachine.phaseOnce()) {
//			return; // Wait a frame for the 'libs.activity.reelsState.preReelsLanding' activity to end if in turbo mode
//		}

		// Then make sure the reelsLanding activity has been started by the reels state controller
		if ( this._stateMachine.phaseManual() ) {
			if ( libs.globalEventBus.isActivityRunning( 'libs.activity.reelsState.reelsLanding' ) ) {
				this._stateMachine.phaseStep();
			}
		}

		// then tell the animation to complete as soon as possible
		if ( this._stateMachine.phaseOnce() ) {
			libs.globalEventBus.invoke( 'libs.event.notify.reelsStopping' );
			this._staggerReelsToValidPosition();
			this._reelsAnimation.completeSpin();
		}
	};

	ReelsMachineModel.prototype._onPreReelsLandingStart = function( activity ) {
		this._preReelsLandingHasStarted = true;
		this._handlePreReelsLanding = activity.join();
	};

	ReelsMachineModel.prototype._onReelsLandingStart = function( activity ) {
		this._handleReelsLanding = activity.join();
	};

	ReelsMachineModel.prototype._onNotifyReelsAtSpeed = function() {
		this._reelsAtSpeed = true;
	};

	ReelsMachineModel.prototype._onNotifyAllReelsStopped = function() {
		this._handleReelsLanding.leave();
		this._handleReelsLanding = null;
	};


	ReelsMachineModel.prototype._staggerReelsToValidPosition = function() {

		// end all reels at the same time, insert an increasing number of symbols before they land to create a staggered landing effect
		var multiplier = 2;
		var extraSymbols = {};
		for ( var reelIndex = 0; reelIndex < game.logic.GameData.reels.nreels; ++reelIndex ) {
			extraSymbols[reelIndex.toString()] = reelIndex * multiplier;
		}

		var testFunc = function( reelModel, index ) {
			return reelModel.testImposeLandingSymbols( extraSymbols[index] );
		};

		// check if the current landing position will be valid and will not add any extra symbols
		var isValid = false;
		for ( var attempt = 0; attempt < 100 && !isValid; ++attempt ) {
			isValid = this._reelModels.every( testFunc );

			if ( !isValid ) {
				// If it is not valid, add an extra symbol to be inserted into the new band
				for ( var i = 0; i < game.logic.GameData.reels.nreels; ++i ) {
					extraSymbols[i.toString()]++;
				}
			}
		}

		if ( isValid ) {
			libs.globalEventBus.invoke( 'libs.event.request.updateSpinParameters', { insertExtraSymbolsOnLand: extraSymbols } );
		}
	};

	libs.client.reels.ReelsMachineModel = ReelsMachineModel;

}());