goog.provide( 'libs.client.reels.SlotsGameModel' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.serverInstance' );
goog.require( 'libs.client.reels.WinAnimation' );

goog.require( 'game.logic' );


(function() {
	"use strict";

	var defaultSettings = {};

	var SlotsGameModel = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._numberOfWinlines = 1;
		if ( game.logic.GameData.reels.winLineDefinitionArray && game.logic.GameData.reels.winLineDefinitionArray.length > 0 ) {
			this._numberOfWinlines = game.logic.GameData.reels.winLineDefinitionArray.length;
		}

		this._betRecoveryInfo = null;

		this._balance = libs.serverInstance.balance;
		this._availableStakesPerLine = libs.serverInstance.availableStakesPerLine;
		this._stakePerLine = libs.serverInstance.startingStake;

		// we need the current stake per line to be a reference to something in the availableStakesPerLine array,
		// otherwise comparisons fail.
		// TODO: Replace .indexOf calls in ButtonModel.js which rely on this to reliably find a BigNumber within an array
		for ( var i = 0; i < this._availableStakesPerLine.length; ++i ) {
			if ( libs.serverInstance.startingStake.equals( this._availableStakesPerLine[i] ) ) {
				this._stakePerLine = this._availableStakesPerLine[i];
				break;
			}
		}

		this._jackpots = libs.serverInstance.jackpots.slice( 0 );
		this._lastJackpots = libs.serverInstance.jackpots.slice( 0 );

		this._serverBetModificationCallback = null;

		this._betResult = null;
		this._isInBet = false;
		this._postBackData = null;

		this._winMultiplier = 1.0;
		this._winTotal = new BigNumber( 0 );
		this._winTotalDisplayedSoFar = new BigNumber( 0 );
		this._winTotalToDisplay = new BigNumber( 0 );
		this._jackpotWins = [];
		this._inBetSequence = false;
		this._turboModeEnabled = false;

		this._autoplayState = {
			active: false,
			remainingPlays: 0,
			decrementing: true // set false when spin not to decrement remaining, like in freeSpins
		};

		this._handlePreBet = null;
		this._handleBetSequence = null;
		this._handleAutoplay = null;

		libs.globalEventBus.connectActivityEnd( 'libs.activity.systemState.gameInit', function() { that._onGameInitEnd(); } );

		libs.globalEventBus.connect( 'libs.event.notify.serverBetResult', function( serverBetResult ) { that._onNotifyServerBetResult( serverBetResult ); } );
		libs.globalEventBus.connect( 'libs.event.notify.serverBetFinished', function() { that._onNotifyServerBetFinished(); } );
		libs.globalEventBus.connect( 'libs.event.notify.serverBetAborted', function() { that._onNotifyServerBetAborted(); } );
		libs.globalEventBus.connect( 'libs.event.notify.serverPolled', function() { that._onNotifyServerPolled(); } );

		libs.globalEventBus.connectActivityStart( 'libs.activity.betState.preBet', function( activity ) { that._onPreBetStart( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.betState.postBet', function( activity ) { that._onPostBetStart( activity ); } );

		libs.globalEventBus.connect( 'libs.event.request.showMoneyWin', function( winAmount ) { that._onRequestShowMoneyWin( winAmount ); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.reelsState.displayReelsWin', function() { that._onDisplayReelsWinEnd(); } );

		libs.globalEventBus.connectActivityStart( 'libs.activity.gameMode.autoplay', function( activity ) { that._onAutoplayStart( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.gameMode.betSequence', function( activity ) { that._onBetSequenceStart( activity ); } );

		libs.globalEventBus.connect( 'libs.event.notify.betRecoveryInfo', function( betRecoveryInfo ) { that._onNotifyBetRecoveryInfo( betRecoveryInfo ); } );

		libs.globalEventBus.connect( 'libs.event.request.changeStakePerLine', function( stakePerLine ) { that._onRequestChangeStakePerLine( stakePerLine ); } );
		libs.globalEventBus.connect( 'libs.event.request.openHistory', function() { that._onRequestOpenHistory(); } );

		libs.globalEventBus.connect( 'libs.event.request.enableTurboMode', function() { that._onRequestEnableTurboMode(); } );
		libs.globalEventBus.connect( 'libs.event.request.disableTurboMode', function() { that._onRequestDisableTurboMode(); } );

		libs.globalEventBus.connect( 'libs.event.request.startAutoplay', function( numberOfAutoplays ) { that._onRequestStartAutoplay( numberOfAutoplays ); } );
		libs.globalEventBus.connect( 'libs.event.request.stopAutoplay', function() { that._onRequestStopAutoplay(); } );

		libs.globalEventBus.connectActivityStart( 'libs.activity.display.error', function( activity, isFatal ) { that._onDisplayErrorStart( activity, isFatal ); } );
		libs.globalEventBus.connect( 'libs.event.request.startBetPostData', function( data ) { that._onRequestStartBetPostData( data ); } );
		libs.globalEventBus.connect( 'libs.event.request.changeWinMultiplier', function( multiplier ) { that._onRequestChangeWinMultiplier( multiplier ); } );
	};


	SlotsGameModel.prototype._onRequestChangeWinMultiplier = function( multiplier ) {

		this._winMultiplier = multiplier;
	};

	// Use this event to modify the received bet result, e.g. for analysing reel delays
	SlotsGameModel.prototype.onServerBetModification = function( callback ) {
		this._serverBetModificationCallback = callback;
	};

	SlotsGameModel.prototype._onRequestStartBetPostData = function( data ) {
		this._postBackData = data;
	};

	SlotsGameModel.prototype.isInBet = function() {
		return this._isInBet;
	};

	SlotsGameModel.prototype._onGameInitEnd = function() {
		// setup
		libs.globalEventBus.invoke( 'libs.event.notify.availableStakesPerLine', this._availableStakesPerLine );
		libs.globalEventBus.invoke( 'libs.event.notify.numberOfWinlines', this._numberOfWinlines );

		// financial
		libs.globalEventBus.invoke( 'libs.event.notify.stakePerLine', this._stakePerLine );
		libs.globalEventBus.invoke( 'libs.event.notify.stakeTotal', this._stakePerLine.times( this._numberOfWinlines ) );
		libs.globalEventBus.invoke( 'libs.event.notify.balance', this._balance );
		libs.globalEventBus.invoke( 'libs.event.notify.jackpotValues', this._jackpots );
		libs.globalEventBus.invoke( 'libs.event.notify.totalWin', this._winTotal );
		libs.globalEventBus.invoke( 'libs.event.notify.winTotalToDisplay', this._winTotalToDisplay );
	};

	SlotsGameModel.prototype._onRequestEnableTurboMode = function() {
		this._turboModeEnabled = true;
		libs.globalEventBus.invoke( 'libs.event.notify.turboModeState', true );
	};

	SlotsGameModel.prototype._onRequestDisableTurboMode = function() {
		this._turboModeEnabled = false;
		libs.globalEventBus.invoke( 'libs.event.notify.turboModeState', false );
	};

	SlotsGameModel.prototype._onRequestStartAutoplay = function( numberOfAutoplays ) {
		this._autoplayState.active = true;
		this._autoplayState.remainingPlays = numberOfAutoplays;
		libs.globalEventBus.invoke( 'libs.event.notify.autoplaysRemaining', this._autoplayState.remainingPlays );
		libs.globalEventBus.invokeActivity( 'libs.activity.gameMode.autoplay' );
	};

	SlotsGameModel.prototype._onRequestStopAutoplay = function() {
		this._stopAutoplay();
	};

	SlotsGameModel.prototype._onDisplayErrorStart = function( activity ) {
		this._stopAutoplay();
	};

	SlotsGameModel.prototype._onNotifyServerBetResult = function( serverBetResult ) {
		var that = this;
		this._betResult = serverBetResult;
		if ( this._serverBetModificationCallback ) {
			this._serverBetModificationCallback( this._betResult );
		}

		// Handle win multiplier
		this._betResult['win'] = new BigNumber( this._betResult['win'] ).times( this._winMultiplier );

		// Win multiplier for the reel wins
		if ( this._betResult[ 'reels' ] && this._betResult[ 'reels' ][ 'winline' ] ) {
			var winlineArray = this._betResult[ 'reels' ][ 'winline' ];
			for ( var winlineIndex=0; winlineIndex<winlineArray.length; ++winlineIndex ) {
				var wl = winlineArray[ winlineIndex ];
				wl[ 'win' ] = wl[ 'win' ].times( this._winMultiplier );
			}
		}

		// Win multiplier for bonuses
		if ( this._betResult['bonus'] && this._betResult['bonus'].length > 0 ) {
			this._betResult['bonus'].forEach( function( bonusInfo ) {
				if ( bonusInfo['win'] ) {
					bonusInfo['win'] = new BigNumber( bonusInfo['win'] ).times( that._winMultiplier );
				}
				if ( bonusInfo['accumulativeWin'] ) {
					bonusInfo['accumulativeWin'] = new BigNumber( bonusInfo['accumulativeWin'] ).times( that._winMultiplier );
				}
			} );
		}
		// Although the 'win' property should be a BigNumber instance when we receive it from the server,
		// for reasons beyond my understanding this intermittendly isn't the case. So we double check it's
		// a BigNumber here for robustness
		this._winTotal = new BigNumber( this._betResult['win'] );
		this._jackpotWins = this._betResult['jackpotWins'] || [];

		if ( this._autoplayState.active && this._autoplayState.decrementing ) {
			if ( this._autoplayState.remainingPlays > 0 ) {
				this._autoplayState.remainingPlays--;
				if ( this._autoplayState.remainingPlays === 0 ) {
					this._stopAutoplay();
				} else {
					libs.globalEventBus.invoke( 'libs.event.notify.autoplaysRemaining', this._autoplayState.remainingPlays );
				}
			}
		}

		this._inBetSequence = Boolean( this._betResult['expectCheckpoint'] );
		if ( this._inBetSequence ) {
			this._autoplayState.decrementing = false;
		}

		this._serverSetMoney();

		if ( this._betResult['bonus'] && this._betResult['bonus'].length > 0 ) {
			this._betResult['bonus'].forEach( function( bonusInfo ) {
				libs.globalEventBus.invoke( 'libs.event.notify.betBonusInfo', bonusInfo );
			} );
		}

		libs.globalEventBus.invoke( 'libs.event.notify.betResult', this._betResult );

		libs.globalEventBus.invoke( 'libs.event.notify.jackpotWins', this._jackpotWins );
		libs.globalEventBus.invoke( 'libs.event.notify.totalWin', this._winTotal );
		libs.globalEventBus.invoke( 'libs.event.notify.winTotalToDisplay', this._winTotalToDisplay );

		this._handlePreBet.leave();
		this._handlePreBet = null;
	};

	SlotsGameModel.prototype._onNotifyServerBetFinished = function() {
		this._serverSetMoney();

		if ( this._inBetSequence ) {
			if ( !this._handleBetSequence ) {
				libs.globalEventBus.invokeActivity( 'libs.activity.gameMode.betSequence' );
			}
		} else {
			this._autoplayState.decrementing = true;

			if ( this._handleBetSequence ) {
				this._handleBetSequence.leave();
				this._handleBetSequence = null;
			}
		}

		this._isInBet = false;
	};

	SlotsGameModel.prototype._onNotifyServerBetAborted = function() {
		this._isInBet = false;

		this._stopAutoplay();

		if ( !this._inBetSequence ) {
			this._balance.plus( this._stakePerLine.times( this._numberOfWinlines ) );
		}

		libs.globalEventBus.invoke( 'libs.event.notify.balance', this._balance );
	};

	SlotsGameModel.prototype._onNotifyServerPolled = function() {
		this._serverSetMoney();
	};

	SlotsGameModel.prototype._onPreBetStart = function( activity ) {
		this._handlePreBet = activity.join();

		libs.globalEventBus.invoke( 'libs.event.notify.quickPlayState', this._turboModeEnabled );

		this._setBetState();

		if ( this._betRecoveryInfo ) {
			libs.globalEventBus.invoke( 'libs.event.notify.betRecoveryStarted' );
			libs.globalEventBus.invoke( 'libs.event.notify.serverBetResult', this._betRecoveryInfo );

			if ( this._betRecoveryInfo['bonus'] ) {
				for ( var i = 0; i < this._betRecoveryInfo['bonus'].length; ++i ) {
					var bonusInfo = this._betRecoveryInfo['bonus'][i];
					if ( bonusInfo['isFreeSpinCollect'] ) {
						libs.globalEventBus.invokeActivity( 'libs.activity.gameMode.freespins', true );
						libs.globalEventBus.invokeActivity( 'libs.activity.display.freespinsUI', true );
						break;
					}
				}
			}

			this._betRecoveryInfo = null;
		} else {
			libs.serverInstance.startBet( this._stakePerLine, this._numberOfWinlines, this._postBackData );
			this._postBackData = null;
		}
	};

	SlotsGameModel.prototype._onPostBetStart = function( activity ) {
		libs.serverInstance.betFinished( this._inBetSequence );
	};

	SlotsGameModel.prototype._onRequestShowMoneyWin = function( winAmount ) {
		this._winTotalToDisplay = this._winTotalToDisplay.plus( winAmount );
		this._winTotalDisplayedSoFar = this._winTotalDisplayedSoFar.plus( winAmount );
		libs.globalEventBus.invoke( 'libs.event.notify.winTotalToDisplay', this._winTotalToDisplay );
	};

	SlotsGameModel.prototype._onDisplayReelsWinEnd = function() {
		this._winTotalToDisplay = this._winTotalToDisplay.plus( this._winTotal ).minus( this._winTotalDisplayedSoFar );
		this._balance = this._balance.plus( this._winTotal );

		libs.globalEventBus.invoke( 'libs.event.notify.winTotalToDisplay', this._winTotalToDisplay );
	};

	SlotsGameModel.prototype._onAutoplayStart = function( activity ) {
		this._handleAutoplay = activity.join();
		libs.globalEventBus.invoke( 'libs.event.request.startBet' );
	};

	SlotsGameModel.prototype._onBetSequenceStart = function( activity ) {
		this._handleBetSequence = activity.join();
	};

	SlotsGameModel.prototype._onNotifyBetRecoveryInfo = function( betRecoveryInfo ) {
		this._betRecoveryInfo = betRecoveryInfo;
		if ( this._betRecoveryInfo ) {
			this._stakePerLine = this._betRecoveryInfo['bet_stake'];
			libs.globalEventBus.invoke( 'libs.event.notify.stakePerLine', this._stakePerLine );
			libs.globalEventBus.invoke( 'libs.event.notify.stakeTotal', this._stakePerLine.times( this._numberOfWinlines ) );

			this._winTotalToDisplay = this._betRecoveryInfo['totalWin'].minus( this._betRecoveryInfo['win'] );
			this._inBetSequence = Boolean( this._betRecoveryInfo['expectCheckpoint'] );
			if ( this._betRecoveryInfo['isCheckpoint'] ) {
				// if previous spin was part of a bet sequence, start the libs.activity.gameMode.betSequence activity
				if ( !this._handleBetSequence ) {
					libs.globalEventBus.invokeActivity( 'libs.activity.gameMode.betSequence' );
				}
			}
		}
	};

	SlotsGameModel.prototype._onRequestChangeStakePerLine = function( stakePerLine ) {
		if ( stakePerLine ) {
			this._stakePerLine = stakePerLine;
			this._winTotalToDisplay = new BigNumber( 0 );

			libs.globalEventBus.invoke( 'libs.event.notify.stakePerLine', this._stakePerLine );
			libs.globalEventBus.invoke( 'libs.event.notify.stakeTotal', this._stakePerLine.times( this._numberOfWinlines ) );
			libs.globalEventBus.invoke( 'libs.event.notify.winTotalToDisplay', this._winTotalToDisplay );
		}
	};

	SlotsGameModel.prototype._onRequestOpenHistory = function() {
		libs.serverInstance.loadHistory();
	};

	SlotsGameModel.prototype._stopAutoplay = function() {
		this._autoplayState.active = false;
		this._autoplayState.remainingPlays = 0;

		libs.globalEventBus.invoke( 'libs.event.notify.autoplaysRemaining', 0 );
		if ( this._handleAutoplay ) {
			this._handleAutoplay.leave();
			this._handleAutoplay = null;
		}
	};

	SlotsGameModel.prototype._setBetState = function() {

		this._isInBet = true;
		this._betResult = null;

		if ( !this._inBetSequence && !this._betRecoveryInfo ) {
			this._balance = this._balance.minus( this._stakePerLine.times( this._numberOfWinlines ) );
			this._winTotalToDisplay = new BigNumber( 0 );
		}

		this._autoplayState.decrementing = !this._inBetSequence;

		this._winTotal = new BigNumber( 0 );
		this._winTotalDisplayedSoFar = new BigNumber( 0 );

		libs.globalEventBus.invoke( 'libs.event.notify.balance', this._balance );
		libs.globalEventBus.invoke( 'libs.event.notify.totalWin', this._winTotal );
		libs.globalEventBus.invoke( 'libs.event.notify.winTotalToDisplay', this._winTotalToDisplay );
	};

	SlotsGameModel.prototype._serverSetMoney = function() {
		this._balance = libs.serverInstance.balance;

		this._lastJackpots = this._jackpots.slice( 0 );
		this._jackpots = libs.serverInstance.jackpots.slice( 0 );

		libs.globalEventBus.invoke( 'libs.event.notify.balance', this._balance );
		libs.globalEventBus.invoke( 'libs.event.notify.jackpotValues', this._jackpots );

		if ( this._jackpots.length === this._lastJackpots.length ) {
			for ( var i = 0; i < this._jackpots.length; ++i ) {
				if ( this._jackpots[i].lessThan( this._lastJackpots[i] ) ) {
					libs.globalEventBus.invoke( 'libs.event.notify.jackpotValueDecreased', i );
				}
			}
		}
	};


	libs.client.reels.SlotsGameModel = SlotsGameModel;

}());