goog.provide( 'libs.client.reels.WinAnimation' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.StateMachine' );
goog.require( 'libs.client.utils.misc' );


(function() {
	'use strict';

	var defaultSettings = {
		xOfAKindValue: 5,
		winStateTime: {
			EarlyScatterWin: 1500,
			SingleLineWin: 1750,
			QuickLineWin: 560,
			AllLinesWin: 1750,
			ShowScatterSet: 1500
		}
	};

	var WinAnimation = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._lineMapping = game.logic.GameData.reels.winLineDefinitionArray;

		this._state = null;
		this._nextState = null;

		this._scatterWins = null;
		this._earlyScatter = null;
		this._symbolWins = [];
		this._shownScatter = false;
		this._earlyScatterShown = 0;
		this._scatterWinsShown = 0;
		this._shownSymbols = 0;
		this._shownLines = 0;
		this._animating = [];

		this._xOfAKindSplashStarted = false;
		this._xOfAKindSplashFinished = false;

		this._disabledReels = [];

		this._states = {
			'xOfAKindSplash': {
				time: 0,
				action: function() {
					that._xOfAKindSplashStarted = true;
					libs.globalEventBus.invokeActivity( 'libs.activity.display.xOfAKindSplash', that._countXOfAKind() );
					that._pickNextState();
				},
				checkStop: function() {
					return that._xOfAKindSplashFinished;
				}
			},
			'EarlyScatterWin': {
				time: this._settings.winStateTime.EarlyScatterWin,
				action: function() {
					var scatterWin = that._earlyScatter[that._earlyScatterShown++];
					that._fireSymbolAnimations( scatterWin['symbols'] );

					libs.globalEventBus.invoke( 'libs.event.notify.showingScatterWin', scatterWin[0]['symbol'] );

					if ( that._earlyScatterShown < that._earlyScatter.length ) {
						that._nextState = 'EarlyScatterWin';
					} else {
						that._pickNextState();
					}
				}
			},
			'SingleLineWin': {
				time: this._settings.winStateTime.SingleLineWin,
				action: function() {
					if(that._symbolWins[that._shownSymbols] !== undefined){
						var lineInfo = that._symbolWins[that._shownSymbols][0];
						var animations = that._animationFrames( lineInfo );

						that._fireSymbolAnimations( animations );
						that._fireShowMoneyWin( lineInfo['win'] );
						that._fireShowLines( [lineInfo], -1 );
						libs.globalEventBus.invoke( 'libs.event.notify.showingAllSymbolsOfWin', lineInfo['symbol'], lineInfo['win'] );

						that._shownSymbols++;

					}
					that._pickNextState();
				},
				checkStop: function() {
					return !that._stillAnimating();
				}
			},
			'QuickLineWin': {
				time: this._settings.winStateTime.QuickLineWin,
				action: function() {
					if(that._symbolWins[that._shownSymbols] !== undefined){
						var lineInfo = that._symbolWins[that._shownSymbols][that._shownLines];
						var animations = that._animationFrames( lineInfo );

						animations.forEach( function( anim ) {anim.freezeAtStart = true;} );
						that._fireSymbolAnimations( animations );

						that._fireShowLines( [lineInfo], that._shownLines );

						that._shownLines++;

						if ( that._shownLines < that._symbolWins[that._shownSymbols].length ) {
							that._nextState = 'QuickLineWin';
						} else {
							that._nextState = 'AllLinesWin';
						}
					}
				}
			},
			'AllLinesWin': {
				time: this._settings.winStateTime.AllLinesWin,
				action: function() {
					if(that._symbolWins[that._shownSymbols] !== undefined){					
						var animations = [];
						var symbolInfo = that._symbolWins[that._shownSymbols];
						var symbol = symbolInfo[0]['symbol'];
						var totalWin = new BigNumber( 0 );

						that._shownLines = 0;
						that._fireShowLines( [], -1 );

						symbolInfo.forEach( function( lineInfo ) {
							var lineAnimations;
							totalWin = totalWin.plus( lineInfo['win'] );

							lineAnimations = that._animationFrames( lineInfo );
							animations = animations.concat( lineAnimations );

						} );

						that._fireSymbolAnimations( animations );
						that._fireShowMoneyWin( totalWin );
						libs.globalEventBus.invoke( 'libs.event.notify.showingAllSymbolsOfWin', symbol, totalWin );

						that._shownSymbols++;

					}
					that._pickNextState();
				},
				checkStop: function() {
					return !that._stillAnimating();
				}
			},
			'ShowScatterSet': {
				time: this._settings.winStateTime.ShowScatterSet,
				action: function() {
					var scatterWin = that._scatterWins[that._scatterWinsShown++];
					that._fireShowLines( [], -1 );
					that._fireSymbolAnimations( scatterWin['position'] );
					libs.globalEventBus.invoke( 'libs.event.notify.showingScatterWin', scatterWin['position'][0]['symbol'] );

					if ( that._scatterWinsShown < that._scatterWins.length ) {
						that._nextState = 'ShowScatterSet';
					} else {
						that._shownScatter = true;
						that._pickNextState();
					}
				},
				checkStop: function() {
					return !that._stillAnimating();
				}
			},
			'WinOver': {
				time: 0,
				action: function() {
					that._nextState = null;
					that._fireShowLines( [], -1 );
					that._fireSymbolAnimations( [] );
					if ( that._handleDisplayReelsWin ) {
						that._handleDisplayReelsWin.leave();
						that._handleDisplayReelsWin = null;
						that._xOfAKindSplashStarted = false;
						that._xOfAKindSplashFinished = false;
						that._scatterWins = null;
						that._earlyScatter = null;
						that._symbolWins.length = 0;
						that._shownScatter = false;
						that._earlyScatterShown = 0;
						that._scatterWinsShown = 0;
						that._shownSymbols = 0;
						that._shownLines = 0;
						that._animating.length = 0;
					}
				}
			}
		};

		this._animationStateMachine = new libs.StateMachine();
		this._animationStateMachine.stateGoto( function() { that._stateIdle(); } );

		this._handleDisplayReelsWin = null;

		libs.globalEventBus.connect( 'libs.event.notify.betResult', function( betResult ) { that._onNotifyBetResult( betResult ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.reelsState.displayReelsWin', function( activity ) { that._onDisplayReelsWinStart( activity ); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.display.xOfAKindSplash', function( numberOfWins ) { that._onDisplayXOfAKindSplashEnd( numberOfWins ); } );
		libs.globalEventBus.connect( 'libs.event.request.skipWinAnimations', function() { that._onRequestSkipWinAnimations(); } );
		libs.globalEventBus.connect( 'libs.event.notify.reelSymbolAnimationFinished', function( reelIndex, rowIndex ) { that._onNotifyReelSymbolAnimationFinished( reelIndex, rowIndex ); } );

		libs.globalEventBus.connect( 'libs.event.request.disableWinAnimationsOnReel', function( reelId ) {that._onRequestDisableWinAnimationsOnReel( reelId );} );
		libs.globalEventBus.connect( 'libs.event.request.enableWinAnimationsOnReel', function( reelId ) {that._onRequestEnableWinAnimationsOnReel( reelId ); } );
	};

	WinAnimation.prototype._onNotifyBetResult = function( betResult ) {
		if ( betResult && betResult['reels'] ) {
			this._scatterWins = betResult['reels']['scatter'] || [];
			this._earlyScatter = betResult['earlyScatter'] || [];
			this._symbolWins = this._convertToSymbolWins( betResult['reels']['winline'] || [] );
		}
	};

	WinAnimation.prototype._onDisplayReelsWinStart = function( activity ) {
		if ( !this._handleDisplayReelsWin ) {
			this._handleDisplayReelsWin = activity.join();
			this._pickNextState();
		}
	};

	WinAnimation.prototype._onDisplayXOfAKindSplashEnd = function( numberOfWins ) {
		this._xOfAKindSplashFinished = true;
	};

	WinAnimation.prototype._onRequestSkipWinAnimations = function() {
		if ( this._nextState ) {
			this._state = 'WinOver';
			this._states['WinOver'].action();
		}
	};

	WinAnimation.prototype._onNotifyReelSymbolAnimationFinished = function( reelIndex, rowIndex ) {
		this._animating.forEach( function( anim ) {
			if ( anim.reel === reelIndex && anim.row === rowIndex ) {
				anim.animating = false;
			}
		} );
	};

	WinAnimation.prototype._onRequestDisableWinAnimationsOnReel = function( reelId ) {
		this._disabledReels.push( reelId );
	};

	WinAnimation.prototype._onRequestEnableWinAnimationsOnReel = function( reelId ) {
		var index = this._disabledReels.indexOf( reelId );
		if ( index > -1 ) {
			this._disabledReels.splice( index, 1 );
		}
	};

	WinAnimation.prototype._stateIdle = function() {
		var that = this;
		if ( this._nextState ) {
			if ( this._state === 'AllLinesWin' ) {
				libs.globalEventBus.invoke( 'libs.event.notify.showingAllSymbolsOfWinComplete' );
			}
			this._state = this._nextState;
			this._animationStateMachine.stateGoto( function() { that._stateAnimating(); } );
		}
	};

	WinAnimation.prototype._stateAnimating = function() {
		var state = this._states[this._state];
		var animationFinished = false;

		if ( this._animationStateMachine.phaseOnce() ) {
			state.action();
		}

		if ( this._animationStateMachine.phaseTimed( state.time || 0 ) ) {
		}

		if ( this._animationStateMachine.phaseManual() ) {
			var checkStop = this._states[this._state].checkStop;
			if ( checkStop ) {
				animationFinished = checkStop();
			} else {
				animationFinished = true;
			}

			if ( animationFinished ) {
				var that = this;
				this._animationStateMachine.stateGoto( function() { that._stateIdle(); } );
			}
		}
	};

	WinAnimation.prototype.update = function( timeDelta ) {
		this._animationStateMachine.update( timeDelta );
	};

	WinAnimation.prototype._pickNextState = function() {
		if ( !this._xOfAKindSplashStarted && !this._xOfAKindSplashFinished && this._countXOfAKind() ) {
			this._nextState = 'xOfAKindSplash';
		} else if ( this._earlyScatter && this._earlyScatter.length && !this._earlyScatterShown ) {
			this._nextState = 'EarlyScatterWin';
		} else if ( this._symbolWins && this._symbolWins.length > this._shownSymbols ) {
			this._nextState = this._pickLineState();
		} else if ( this._scatterWins && this._scatterWins.length && !this._shownScatter ) {
			this._nextState = 'ShowScatterSet';
		} else {
			this._nextState = 'WinOver';
		}
	};

	WinAnimation.prototype._countXOfAKind = function() {
		var that = this;

		var xOfAKindCount = 0;
		this._symbolWins.forEach( function( lineWins ) {
			lineWins.forEach( function( win ) {
				if ( win['run'] >= that._settings.xOfAKindValue ) {
					++xOfAKindCount;
				}
			} );
		} );

		return xOfAKindCount;
	};

	WinAnimation.prototype._pickLineState = function() {
		var lineWins = this._symbolWins[this._shownSymbols];
		if ( lineWins.length === 1 ) {
			return 'SingleLineWin';
		} else {
			return 'QuickLineWin';
		}
	};

	WinAnimation.prototype._animationFrames = function( lineInfo ) {
		var i;
		var animations = [];
		var linePositions = this._lineMapping[lineInfo['line']];
		for ( i = 0; i < lineInfo['run']; i++ ) {
			if ( this._disabledReels.indexOf( i ) === -1 ) {
				animations.push( { 'reel': i, 'row': linePositions[i], 'symbol': lineInfo['symbol'] } );
			}
		}

		return animations;
	};

	WinAnimation.prototype._stillAnimating = function() {
		return this._animating.some( function( anim ) {return anim.animating;} );
	};

	WinAnimation.prototype._fireShowLines = function( lineInfo, positionInRun ) {
		libs.globalEventBus.invoke( 'libs.event.request.showWinlines', lineInfo, positionInRun );
	};

	WinAnimation.prototype._fireSymbolAnimations = function( symbolsToAnimate ) {
		this._animating = symbolsToAnimate.map( function( sym ) {
			return { reel: sym['reel'], row: sym['row'], animating: true };
		} );
		libs.globalEventBus.invoke( 'libs.event.request.startSymbolAnimations', symbolsToAnimate );
	};

	WinAnimation.prototype._fireShowMoneyWin = function( winAmount ) {
		libs.globalEventBus.invoke( 'libs.event.request.showMoneyWin', winAmount );
	};

	WinAnimation.prototype._convertToSymbolWins = function( winLines ) {
		var symbolWins = [];

		winLines.forEach( function( lineWin ) {
			var symIndex = game.logic.GameData.reels.symbols.shortNameToIndex( lineWin['symbol'] );
			symbolWins[symIndex] = symbolWins[symIndex] || [];
			symbolWins[symIndex].push( lineWin );

		} );

		symbolWins = symbolWins.filter( function( elm ) {return elm;} );
		symbolWins.sort( function( lhs, rhs ) {
			var lhsIndex = game.logic.GameData.reels.symbols.shortNameToIndex( lhs[0]['symbol'] );
			var rhsIndex = game.logic.GameData.reels.symbols.shortNameToIndex( rhs[0]['symbol'] );
			return lhsIndex - rhsIndex;
		} );

		return symbolWins;
	};


	libs.client.reels.WinAnimation = WinAnimation;

}());