goog.provide( 'libs.client.reels.WinlinesView' );

goog.require( 'libs.xmlUtils' );

goog.require( 'game.logic' );


(function() {
	"use strict";

	var defaultSettings = {
		dataXML: 'assets/data/WinLineData.xml',
		dataScreenWidth: 960, // winline data is based on 960x540 resolution, do not change for desktop, despite different canvas
		dataScreenHeight: 540,
		dataOffset: {
			x: 8,
			y: 0
		},
		numberOfSymbolRows: game.logic.GameData.numberOfSymbolRows,
		highlightBoxes: { // defaults are for 5x4 symbol layout
			reelPositions: undefined,
			drawOffset: {
				x: 31,
				y: 83
			},
			width: 135,
			height: 92,
			vSpacing: 0
		},
		backgroundImage: null // super impose the background strips behind the win animation, in case of a feature has an image between the reels and the winanimation
	};

	var threeRowsDefaultSetting = {
		highlightBoxes: {
			drawOffset: {
				y: 68
			},
			height: 119,
			vSpacing: 5
		}
	};

	var fourRowsDefaultSetting = {
		highlightBoxes: {
			drawOffset: {
				y: 83
			},
			height: 92,
			vSpacing: 0
		}
	};

	var WinlinesView = function( overrideSettings, winlinesModel ) {
		var that = this;
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		if ( this._settings.numberOfSymbolRows === 3 ) {
			jQuery.extend( true, this._settings, threeRowsDefaultSetting );
		} else if ( this._settings.numberOfSymbolRows === 4 ) {
			jQuery.extend( true, this._settings, fourRowsDefaultSetting );
		}
		jQuery.extend( true, this._settings, overrideSettings );

		this._forceRenderBackground = false;
		this._winlinesModel = winlinesModel;
		this._loadXml();

		libs.globalEventBus.connect( 'libs.event.request.enableWinlineBoxOverEntireReel', function( reelId ) {that._onRequestEnableWinlineBoxOverEntireReel( reelId );} );
		libs.globalEventBus.connect( 'libs.event.request.disableWinlineBoxOverEntireReel', function( reelId ) {that._onRequestDisableWinlineBoxOverEntireReel( reelId );} );

		this._disabledReels = [];

		libs.globalEventBus.connect( 'libs.event.request.startForcedRenderReelBackground', function( imageName ) { that._onRequestStartForcedRenderReelBackground( imageName ); } );
		libs.globalEventBus.connect( 'libs.event.request.stopForcedRenderReelBackground', function() { that._onRequestStopForcedRenderReelBackground(); } );
	};

	WinlinesView.prototype._loadXml = function() {
		var xmlhttp = new XMLHttpRequest();
		xmlhttp.open( "GET", libs.client.assetUrlPrefix + this._settings.dataXML, false );
		xmlhttp.send();
		var xmlDoc = xmlhttp.responseXML;

		var docRoot = libs.xmlUtils.getElementsByTagNameSafe( xmlDoc, "WinlineXml" );

		// read in vertical guides
		var verticalGuidesRoot = libs.xmlUtils.getElementsByTagNameSafe( docRoot[0], "VerticalGuides" );
		var verticalGuideElements = libs.xmlUtils.getElementsByTagNameSafe( verticalGuidesRoot[0], "VerticalGuide" );

		var verticalGuides = [];
		verticalGuides.length = verticalGuideElements.length;

		for ( var guideIndex = 0; guideIndex < verticalGuideElements.length; ++guideIndex ) {
			var guideElem = verticalGuideElements[guideIndex];
			var guideId = libs.xmlUtils.getAttributeSafe( guideElem, "id" );
			var guideXOffset = libs.xmlUtils.getTextContent( guideElem );

			verticalGuides[guideId] = guideXOffset;
		}

		// read in winlines
		var winlineRoot = libs.xmlUtils.getElementsByTagNameSafe( docRoot[0], "Winlines" );
		var winlineElements = libs.xmlUtils.getElementsByTagNameSafe( winlineRoot[0], "Winline" );

		this.winlines = [];
		this.winlines.length = winlineElements.length;

		for ( var index = 0; index < winlineElements.length; ++index ) {
			var elem = winlineElements[index];
			var winlineIndex = libs.xmlUtils.getAttributeSafe( elem, "number" );
			winlineIndex -= 1; // winlineIndex is 1-based in XML, make it zero based
			var colourHex = this._removeAlphaFromColourString( libs.xmlUtils.getAttributeSafe( elem, "colour" ) );
			var outerColourHex = this._removeAlphaFromColourString( libs.xmlUtils.getAttributeSafe( elem, "outer_colour" ) );
			var isShown = libs.xmlUtils.getAttributeSafe( elem, "shown" );

			var pointElements = libs.xmlUtils.getElementsByTagNameSafe( elem, "Point" );
			var points = [];

			for ( var pointIndex = 0; pointIndex < pointElements.length; ++pointIndex ) {

				var pointElem = pointElements[pointIndex];
				var verticalGuideId = libs.xmlUtils.getAttributeSafe( pointElem, "vgid" );
				var normalisedY = this._normaliseOffset( libs.xmlUtils.getTextContent( pointElem ) );
				var normalisedX = this._normaliseOffset( verticalGuides[verticalGuideId] );

				var pointX = Math.floor( normalisedX * this._settings.dataScreenWidth );
				var pointY = Math.floor( normalisedY * this._settings.dataScreenHeight );

				points.push( { x: pointX + this._settings.dataOffset.x, y: pointY + this._settings.dataOffset.y } );
			}

			if ( !!isShown ) { // double !! converts to bool
				this.winlines[winlineIndex] = { colourHex: colourHex, outerColourHex: outerColourHex, points: points };
			}
		}

		// setup square boxes to indicate winning symbols
		this.winBoxes = [];

		//Set up square boxes to go around each reel
		this.reelWinBoxes = [];

		var reelDimensionsElem = libs.xmlUtils.getElementsByTagNameSafe( docRoot[0], "ReelDimensions" );
		var data = libs.xmlUtils.getTextContent( reelDimensionsElem[0] );
		var numOfReels = parseInt( data.charAt( 0 ), 10 );
		var numOfRows = parseInt( data.charAt( data.length - 1 ), 10 );

		var boxSettings = this._settings.highlightBoxes;
		var reel;

		var createBox = function( reel, yTop, height ) {
			var yBottom = yTop + height;
			var xLeft = boxSettings.reelPositions[reel].x + boxSettings.drawOffset.x;
			var xRight = xLeft + boxSettings.width;
			var box = [];
			box.push( { x: xLeft, y: yTop } );
			box.push( { x: xRight, y: yTop } );
			box.push( { x: xRight, y: yBottom } );
			box.push( { x: xLeft, y: yBottom } );
			return box;
		};

		for ( reel = 0; reel < numOfReels; ++reel ) {
			var boxes = [];
			for ( var row = 0; row < numOfRows; ++row ) {

				boxes.push( createBox(
						reel,
						boxSettings.reelPositions[reel].y + boxSettings.drawOffset.y + (boxSettings.height * row) + (boxSettings.vSpacing * row),
						boxSettings.height )
				);
			}
			this.winBoxes.push( boxes );
		}

		for ( reel = 0; reel < numOfReels; reel++ ) {
			this.reelWinBoxes.push( createBox(
					reel,
					boxSettings.reelPositions[reel].y + boxSettings.drawOffset.y + boxSettings.vSpacing,
					(boxSettings.height * 3 ) )
			);
		}

	};


	WinlinesView.prototype._onRequestStartForcedRenderReelBackground = function( imageName ) {
		this._forceRenderBackground = true;
		this._backgroundImageName = imageName || this._settings.backgroundImage;
	};


	WinlinesView.prototype._onRequestStopForcedRenderReelBackground = function() {
		this._forceRenderBackground = false;
	};

	WinlinesView.prototype._onRequestDisableWinlineBoxOverEntireReel = function( reelId ) {
		var index = this._disabledReels.indexOf( reelId );
		if ( index > -1 ) {
			this._disabledReels.splice( index, 1 );
		}
	};

	WinlinesView.prototype._onRequestEnableWinlineBoxOverEntireReel = function( reelId ) {
		this._disabledReels.push( reelId );
	};

	WinlinesView.prototype._removeAlphaFromColourString = function( colourString ) {
		if ( colourString.charAt( 0 ) === '#' && colourString.length === 9 ) {
			var partStr = colourString.substr( 3, 6 );
			return '#' + partStr;
		} else {
			return colourString;
		}
	};

	WinlinesView.prototype._normaliseOffset = function( offset ) {
		var normalised = ( parseFloat( offset ) + 1.0 ) / 2.0;
		return normalised;
	};

	WinlinesView.prototype.draw = function( ctx ) {
		this._drawLineIf( ctx, function( line ) {
			return line.visible;
		} );

		this._winlinesModel.lines.forEach( function( line ) {line.dirty = false;} );
	};

	WinlinesView.prototype.isDirty = function() {
		var dirtyLine = false;

		this._winlinesModel.lines.forEach( function( line ) {
			if ( line.dirty === true ) {
				dirtyLine = true;
			}
		} );

		return dirtyLine;
	};

	WinlinesView.prototype.drawIfDirty = function( ctx ) {

		this._drawLineIf( ctx, function( line ) {
			return line.dirty && line.visible;
		} );

		this._winlinesModel.lines.forEach( function( line ) {line.dirty = false;} );
	};

	WinlinesView.prototype._drawLineIf = function( ctx, toCall ) {
		var numLines = this._winlinesModel.numLines;

		for ( var i = 0; i < numLines; i++ ) {
			var line = this._winlinesModel.lines[i];
			if ( toCall( line ) ) {

				var winlineData = this.winlines[i];
				if ( winlineData !== undefined ) {

					this._drawCanvasLine( ctx, winlineData.points, winlineData.outerColourHex, 5, false );
					this._drawCanvasLine( ctx, winlineData.points, winlineData.colourHex, 3, false );

					for ( var reel = 0; reel < line.run; ++reel ) {
						//If reel is in the output
						var winBox;
						if ( this._disabledReels.indexOf( reel ) === -1 ) {
							winBox = this.winBoxes[reel][this._winlinesModel.lineMapping[i][reel]];
						} else {
							winBox = this.reelWinBoxes[reel];
						}
						ctx.clearRect( winBox[0].x, winBox[0].y, this._settings.highlightBoxes.width, (winBox[3].y - winBox[0].y) );
						this._drawBackground( ctx, winBox );
						this._drawCanvasLine( ctx, winBox, winlineData.outerColourHex, 5, true );
						this._drawCanvasLine( ctx, winBox, winlineData.colourHex, 3, true );
					}
				}
			}
		}
	};

	WinlinesView.prototype._drawCanvasLine = function( ctx, points, colourHex, lineWidth, closePath ) {
		ctx.strokeStyle = colourHex;
		ctx.lineWidth = lineWidth;

		ctx.beginPath();

		for ( var pointIndex = 0; pointIndex < points.length; ++pointIndex ) {
			var point = points[pointIndex];

			if ( pointIndex === 0 ) {
				ctx.moveTo( point.x, point.y );
			} else {
				ctx.lineTo( point.x, point.y );
			}
		}
		if ( closePath === true ) {
			ctx.closePath();
		}
		ctx.stroke();
	};


	WinlinesView.prototype._drawBackground = function( ctx, winBox ) {

		if ( this._forceRenderBackground ) {
			var bgImage = libs.loadedImages[this._backgroundImageName];
			var width = winBox[1].x - winBox[0].x;
			var height = winBox[2].y - winBox[0].y;
			var boxSettings = this._settings.highlightBoxes;

			if ( bgImage['type'] !== "metagu_sub_image" ) {
				ctx.drawImage( bgImage, Math.floor( winBox.x ), Math.floor( winBox.y ), width, height );
			} else {
				var x = winBox[0].x;
				var y = winBox[0].y;
				var xSrcOffset = bgImage.x;
				var ySrcOffset = bgImage.y;
				var xoffset = boxSettings.reelPositions[0].x + boxSettings.drawOffset.x;
				var yoffset = boxSettings.reelPositions[0].y + boxSettings.drawOffset.y;
				ctx.drawImage( bgImage.image,
					Math.floor( x + xSrcOffset - xoffset ),
					Math.floor( y + ySrcOffset - yoffset ),
					Math.floor( width ),
					Math.floor( height ),
					Math.floor( x ),
					Math.floor( y ),
					Math.floor( width ),
					Math.floor( height ) );
			}
		}
	};

	libs.client.reels.WinlinesView = WinlinesView;

}());