goog.provide( 'libs.client.rendering.LegacyAnimatedSprite' );

goog.require( 'libs.StateMachine' );


(function() {
	"use strict";

	//Sprite to display a sequence of images.
	var LegacyAnimatedSprite = function( imgs, animSettings, drawSettings ) {
		this.image = imgs[0];
		this.numberOfFrames = imgs.length;

		this.x = drawSettings !== undefined && drawSettings.x !== undefined ? drawSettings.x : 0;
		this.y = drawSettings !== undefined && drawSettings.y !== undefined ? drawSettings.y : 0;
		this.alpha = drawSettings !== undefined && drawSettings.alpha !== undefined ? drawSettings.alpha : 1.0;

		this.width = imgs[0].width;
		this.height = imgs[0].height;

		this.trimmedOffsetX = imgs[0].trimmedOffsetX || 0;
		this.trimmedOffsetY = imgs[0].trimmedOffsetY || 0;

		// where the image is actually drawn; using the x and y, but adjusting it by how the image may be trimmed
		this._drawAtCalculatedX = this.x - (this.trimmedOffsetX - this.image.trimmedOffsetX);
		this._drawAtCalculatedY = this.y - (this.trimmedOffsetY - this.image.trimmedOffsetY);

		this._animationStateMachine = new libs.StateMachine();
		this._fadeStateMachine = new libs.StateMachine();
		this._fadeTime = 0;
		this._isFading = false;

		this._allImages = imgs;
		this._currentFrame = 0;
		this._fps = animSettings.fps || 25;
		this._timesToLoop = animSettings.timesToLoop || 0;
		this._reverseLoop = animSettings.reverseLoop !== undefined ? animSettings.reverseLoop : false;
		this._autoplay = animSettings.autoplay !== undefined ? animSettings.autoplay : false;
		this._resetToFrameZeroOnEnd = animSettings.resetToFrameZeroOnEnd !== undefined ? animSettings.resetToFrameZeroOnEnd : false;
		this._drawWhenNotPlaying = animSettings.drawWhenNotPlaying !== undefined ? animSettings.drawWhenNotPlaying : true;
		this._playForward = true;

		this._lastX = null;
		this._lastY = null;
		this._timesLooped = 0;
		this._isDirty = true;

		this.isPlaying = false;

		if ( this._autoplay === true ) {
			this.play();
		}

		this._loopedCallback = null;
	};

	LegacyAnimatedSprite.prototype.onLooped = function( callback ) {
		this._loopedCallback = callback;
	};

	LegacyAnimatedSprite.prototype.update = function( timeDelta ) {
		this._animationStateMachine.update( timeDelta );
		this._fadeStateMachine.update( timeDelta );
	};

	LegacyAnimatedSprite.prototype._statePlaying = function() {
		if ( this._animationStateMachine.phaseTimed( this.getAnimTime() ) ) {
			if ( this._playForward ) {
				this._currentFrame = Math.round( (this.numberOfFrames - 1) * this._animationStateMachine.phaseTimedGetProgress() );
			} else {
				this._currentFrame = Math.round( (this.numberOfFrames - 1) * this._animationStateMachine.phaseTimedGetRevProgress() );
			}
		}

		if ( this._animationStateMachine.phaseOnce() ) {
			if ( this._timesToLoop === -1 || (this._timesLooped < this._timesToLoop) ) {
				this._timesLooped++;
				if ( this._reverseLoop ) {
					this._playForward = !this._playForward;
				}
				this._animationStateMachine.phaseReset();
			} else {
				this.stop( this._resetToFrameZeroOnEnd );
			}
			if ( this._loopedCallback ) {
				this._loopedCallback();
			}
		}
	};

	LegacyAnimatedSprite.prototype._stateFadeIn = function() {
		if ( this._fadeStateMachine.phaseTimed( this._fadeTime ) ) {
			this._isFading = true;
			this.alpha = this._fadeStateMachine.phaseTimedGetProgress();
		}

		if ( this._fadeStateMachine.phaseOnce() ) {
			this._isFading = false;
		}
	};

	LegacyAnimatedSprite.prototype._stateFadeOut = function() {
		if ( this._fadeStateMachine.phaseTimed( this._fadeTime ) ) {
			this._isFading = true;
			this.alpha = this._fadeStateMachine.phaseTimedGetRevProgress();
		}

		if ( this._fadeStateMachine.phaseOnce() ) {
			this._isFading = false;
		}
	};

	LegacyAnimatedSprite.prototype.updateAnimationSettings = function( animSettings ) {
		if ( animSettings.fps ) {
			this._fps = animSettings.fps;
		}
		if ( animSettings.timesToLoop ) {
			this._timesToLoop = animSettings.timesToLoop;
		}
		if ( animSettings.reverseLoop ) {
			this._reverseLoop = animSettings.reverseLoop;
		}
		if ( animSettings.autoplay ) {
			this._autoplay = animSettings.autoplay;
		}
		if ( animSettings.resetToFrameZeroOnEnd ) {
			this._resetToFrameZeroOnEnd = animSettings.resetToFrameZeroOnEnd;
		}
		if ( animSettings.playForward ) {
			this._playForward = animSettings.playForward;
		}
	};

	LegacyAnimatedSprite.prototype.play = function() {
		var that = this;
		this.isPlaying = true;
		this._timesLooped = 0;
		this._currentFrame = 0;
		this._playForward = true;
		this._isDirty = true;

		this._animationStateMachine.stateGoto( function() { that._statePlaying(); } );
	};

	LegacyAnimatedSprite.prototype.stop = function( resetToFrameZero ) {
		resetToFrameZero = resetToFrameZero !== undefined ? resetToFrameZero : true;
		this._timesLooped = 0;
		if ( resetToFrameZero ) {
			this._currentFrame = 0;
		}
		this.isPlaying = false;
		this._playForward = true;
		this._isDirty = true;

		this._animationStateMachine.stateGoto( null );
	};

	LegacyAnimatedSprite.prototype.getAnimTime = function() {
		return (this.numberOfFrames / this._fps) * 1000; // * 1000 as statemachine is in ms
	};

	LegacyAnimatedSprite.prototype.isDirty = function() {
		return ( this.isPlaying || this._isDirty || (this._lastX !== this.x) || (this._lastY !== this.y) || this._isFading );
	};

	LegacyAnimatedSprite.prototype.draw = function( ctx, width, height, offsetX, offsetY ) {
		if ( !this._drawWhenNotPlaying && !this.isPlaying ) {
			return;
		}

		this.image = this._allImages[this._currentFrame];

		width = width || this.image.width;
		height = height || this.image.height;
		offsetX = offsetX || 0;
		offsetY = offsetY || 0;

		// animation frames are cropped, so some images may be a different
		// size than base frame; adjust the position here to keep them centred
		if ( this.isDirty() ) {
			this._drawAtCalculatedX = this.x - (this.trimmedOffsetX - this.image.trimmedOffsetX);
			this._drawAtCalculatedY = this.y - (this.trimmedOffsetY - this.image.trimmedOffsetY);
		}

		if ( this.alpha > 0 ) {
			if ( this.alpha < 1 ) {
				ctx.save();
				ctx.globalAlpha = this.alpha;
			}

			if ( this.image.type === "metagu_sub_image" ) {
				ctx.drawImage( this.image.image,
					this.image.x + offsetX, this.image.y + offsetY, width, height,
					this._drawAtCalculatedX, this._drawAtCalculatedY, width, height );
			} else {
				ctx.drawImage( this.image, this._drawAtCalculatedX, this._drawAtCalculatedY );
			}

			if ( this.alpha < 1 ) {
				ctx.restore();
			}
		}

		this._lastX = this.x;
		this._lastY = this.y;

		this._isDirty = false;
	};

	LegacyAnimatedSprite.prototype.drawIfDirty = function( ctx ) {
		if ( this.isDirty() ) {
			this.draw( ctx );
		}
	};

	LegacyAnimatedSprite.prototype.fadeIn = function( time ) {
		var that = this;
		this._fadeTime = time || 1000;
		this._fadeStateMachine.stateGoto( function() { that._stateFadeIn();} );
	};

	LegacyAnimatedSprite.prototype.fadeOut = function( time ) {
		var that = this;
		this._fadeTime = time || 1000;
		this._fadeStateMachine.stateGoto( function() { that._stateFadeOut();} );
	};

	LegacyAnimatedSprite.prototype.getCurrentFrame = function() {
		return this._currentFrame;
	};

	libs.client.rendering.LegacyAnimatedSprite = LegacyAnimatedSprite;

}());