goog.provide( 'libs.client.rendering.NewCanvasText' );


(function() {

	"use strict";

	var defaultSettings = {
		text: '',
		drawSettings: {
			x: 0,
			y: 0,
			offsetX: 0,
			offsetY: 0,
			scaleXY: 1
		},
		fontSettings: {
			fontSize: 40,
			font: 'Arial',
			colors: ["#ffffff"]
		}
	};

	var NewCanvasText = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );


		this._position = { x: this._settings.drawSettings.x, y: this._settings.drawSettings.y };
		this._scale = this._settings.drawSettings.scaleXY;
		this._offset = { x: this._settings.drawSettings.offsetX, y: this._settings.drawSettings.offsetY };
		this._maxWidth = this._settings.drawSettings.width;
		this._gradientHeight = this._settings.drawSettings.height;

		this._fontSize = this._settings.fontSettings.fontSize;
		this._font = this._settings.fontSettings.font;
		this._colors = this._settings.fontSettings.colors;
		this._text = this._settings.text;
	};

	/**
	 * Getter/Setters for common variables
	 */

	NewCanvasText.prototype.getText = function() {
		return this._text;
	};

	NewCanvasText.prototype.setText = function( text ) {
		this._text = text || ' ';
	};

	NewCanvasText.prototype.getFont = function() {
		return this._font;
	};

	NewCanvasText.prototype.setFont = function( font ) {
		this._font = font || this._settings.fontSettings.font;
	};

	NewCanvasText.prototype.getColors = function() {
		return this._colors;
	};

	NewCanvasText.prototype.setColors = function( colors ) {
		this._colors = colors || this._settings.fontSettings.colors;
	};

	NewCanvasText.prototype.getPosition = function() {
		return this._position;
	};

	NewCanvasText.prototype.getFontSize = function() {
		return this._fontSize;
	};

	NewCanvasText.prototype.getScale = function() {
		return this._scale;
	};

	NewCanvasText.prototype.getOffset = function() {
		return this._offset;
	};

	/**
	 * Animation functions
	 */

	NewCanvasText.prototype.moveTo = function( position ) {
		this._position.x = position.x;
		this._position.y = position.y;

		this._isDirty = true;
	};

	NewCanvasText.prototype.moveBy = function( position ) {
		this._position.x += position.x;
		this._position.y += position.y;

		this._isDirty = true;
	};

	NewCanvasText.prototype.scaleTo = function( scaleParameters ) {
		this._scale = scaleParameters.xy;
	};

	NewCanvasText.prototype.scaleBy = function( scaleParameters ) {
		this._scale += scaleParameters.xy;
	};

	/**
	 * Draw functions
	 */

	NewCanvasText.prototype.draw = function( ctx ) {

		//Apply the offset
		var xPosition = this._offset.x ? this._position.x + this._offset.x : this._position.x;
		var yPosition = this._offset.y ? this._position.y + this._offset.y : this._position.y;

		var centerX = xPosition + (this._settings.drawSettings.width / 2);
		var centerY = yPosition + (this._settings.drawSettings.height / 2);

		ctx.save();

		//Set the gradient up if any
		var numOfColors = this._colors.length;
		if ( numOfColors > 1 ) {
			var grd = ctx.createLinearGradient( centerX, this._position.y - (this._settings.drawSettings.height / 2), centerX, centerY );

			var highestIndex = numOfColors - 1;
			for ( var i = 0; i < numOfColors; ++i ) {
				var distance = i / highestIndex;
				grd.addColorStop( distance, this._colors[i] );
			}
			ctx.fillStyle = grd;
		} else {
			ctx.fillStyle = this._colors[0];
		}

		ctx.textBaseline = "center";
		ctx.textAlign = "center";

		//Use the width value to scale text down by the max width. 
		var computedFont = (this._fontSize * this._scale) + 'px' + ' ' + this._font;

		ctx.font = computedFont;

		var textSize = ctx.measureText( this._text );
		computedFont = this._scaleDownByTextSize( textSize );

		ctx.fillText( this._text, centerX, centerY, this._settings.drawSettings.width );
		ctx.restore();

		this._isDirty = false;
	};

	/**
	 * Function scales down the size
	 * @param  {Number} textWidth Width the text will be
	 * @return {String}
	 */
	NewCanvasText.prototype._scaleDownByTextSize = function( textWidth ) {
		if ( textWidth > this._maxWidth ) {
			var scale = this._maxWidth / textWidth;
			return (this._fontSize * scale) + 'px' + ' ' + this._font;
		} else {
			return (this._fontSize * this._scale) + 'px' + ' ' + this._font;
		}
	};

	NewCanvasText.prototype.isDirty = function() {
		return this._isDirty;
	};

	NewCanvasText.prototype.drawIfDirty = function( ctx ) {
		if ( this._isDirty ) {
			this.draw( ctx );
		}
	};

	libs.client.rendering.NewCanvasText = NewCanvasText;

}());