goog.provide( 'libs.client.rendering.Sprite' );

goog.require( 'libs.StateMachine' );
goog.require( 'libs.client.utils.misc' );
goog.require( 'libs.client.utils.tweens' );


(function() {
	'use strict';

	var defaultSettings = {
		imageName: undefined,
		enableDebug: false,
		loadAnimationFrames: true,
		callbacks: {
			onClick: null,
			onAnimFinished: null,
			onAnimLooped: null,
			onFrameChange: null
		},
		animSettings: {
			startFrame: null,
			framesCount: null, // will be accurately set at creation of a Sprite object
			fps: 25,
			startPlayForwards: true,
			timesToLoop: 0,
			reverseLoop: false,
			resetToStartFrameOnEnd: false,
			drawWhenNotPlaying: true
		},
		customAnims: null,
		drawSettings: {
			x: 0,
			y: 0,
			scaleX: 1,
			scaleY: 1,
			alpha: 1,
			rotation: 0,
			offsetX: 0,
			offsetY: 0,
			clipRegion: null
		}
	};

	var defaultTransformationStep = {
		duration: 0,
		progress: false,
		isAbsolute: true,
		tween: libs.client.utils.tweens.linear,
		callback: null
	};

	var radiansPerDegree = Math.PI / 180;

	var Sprite = function( overrideSettings, thisContext ) {
		this.reset( overrideSettings, thisContext );

		var that = this;

		// state machines
		this._animationStateMachine = new libs.StateMachine();
		this._positionStateMachine = new libs.StateMachine();
		this._scaleStateMachine = new libs.StateMachine();
		this._rotationStateMachine = new libs.StateMachine();
		this._alphaStateMachine = new libs.StateMachine();

		this._positionStateMachine.stateGoto( function() { that._statePositioning(); } );
		this._scaleStateMachine.stateGoto( function() { that._stateScaling(); } );
		this._rotationStateMachine.stateGoto( function() { that._stateRotating(); } );
		this._alphaStateMachine.stateGoto( function() { that._stateFading(); } );

		libs.globalEventBus.connect( 'libs.event.notify.tick', function( timeDelta ) { that._onNotifyTick( timeDelta ); } );
		if ( this._settings.callbacks.onClick ) {
			libs.globalEventBus.connect( 'libs.event.notify.canvasPointerClick', function( point ) { that._onNotifyCanvasPointerClick( point ); } );
		}
	};

	Sprite.prototype.reset = function( overrideSettings, thisContext ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		this._thisContext = thisContext || this;

		if ( this._settings.loadAnimationFrames ) {
			this._imageFrames = this._loadAnimationFrames( this._settings.imageName );
		} else {
			this._imageFrames = [libs.loadedImages[this._settings.imageName]];
		}

		if ( this._imageFrames.length === 0 ) {
			libs.logError( 'Sprite: no images found for name: ' + this._settings.imageName );
		}

		this._settings.animSettings.framesCount = this._imageFrames.length;
		this._isAnimation = this._imageFrames.length > 1;

		this._image = this._imageFrames[0];

		this._drawSettings = this._settings.drawSettings;

		this._imageDimensions = {
			width: this._image.width,
			height: this._image.height
		};

		// To allow backwards compatibility for texture data JSON formats that do not contained the 'untrimmedSize' object (was added 22/09/2014)
		this._useSpritesUntrimmedSize = this._image.untrimmedSize !== undefined;

		if ( !this._useSpritesUntrimmedSize ) {
			this._trimmedOffset = {
				x: this._image.trimmedOffsetX || 0,
				y: this._image.trimmedOffsetY || 0
			};
		}

		// set draw settings
		this._position = {
			x: this._drawSettings.x,
			y: this._drawSettings.y
		};

		this._scale = {
			x: this._drawSettings.scaleX,
			y: this._drawSettings.scaleY
		};

		this._rotation = this._drawSettings.rotation;
		this._alpha = this._drawSettings.alpha;

		this._offset = { // offset used to place image on offest and fnd new relative centerpoint
			x: this._drawSettings.offsetX,
			y: this._drawSettings.offsetY
		};

		this._drawAtCalculatedPos = { x: 0, y: 0 };
		this._calculateDrawPosition();

		this._currentAnimTime = 0;
		this._activeAnimInfo = {
			currentFrame: 0,
			timesLooped: 0,
			isPlaying: false,
			playForwards: true
		};
		this._currentAnimSettings = null;
		this._setAnimation();

		this._dirtyFlag = false;

		this._debugFillStyle = 'rgba(' + libs.client.utils.misc.getRandomInt( 0, 255 ) + ',' + libs.client.utils.misc.getRandomInt( 0, 255 ) + ',' + libs.client.utils.misc.getRandomInt( 0, 255 ) + ',' + 0.5 + ')';

		var Transformation = function( type ) {
			this.type = type;
			this.isAbsolute = false;
			this.sequenceArray = [];
		};

		this._positionTransform = new Transformation( 'position' );
		this._positionTransform.startX = 0;
		this._positionTransform.startY = 0;

		this._scaleTransform = new Transformation( 'scale' );
		this._scaleTransform.startX = 1;
		this._scaleTransform.startY = 1;

		this._rotationTransform = new Transformation( 'rotation' );
		this._rotationTransform.startVal = 0;

		this._alphaTransform = new Transformation( 'alpha' );
		this._alphaTransform.startVal = 1;

		this._playCallback = null;
	};

	Sprite.prototype._calculateDrawPosition = function() {
		if ( !this._useSpritesUntrimmedSize ) {
			this._drawAtCalculatedPos.x = this._position.x - (this._trimmedOffset.x - this._image.trimmedOffsetX) + this._offset.x;
			this._drawAtCalculatedPos.y = this._position.y - (this._trimmedOffset.y - this._image.trimmedOffsetY) + this._offset.y;
		} else {
			this._drawAtCalculatedPos.x = this._position.x + this._image.trimmedOffsetX + this._offset.x;
			this._drawAtCalculatedPos.y = this._position.y + this._image.trimmedOffsetY + this._offset.y;
		}
	};


	Sprite.prototype.setClipRegion = function( clipRegion ) {
		this._drawSettings.clipRegion = clipRegion;
	};


	Sprite.prototype._calculateDrawDimensions = function() {

		// NOTE: Does not take into account scaling at the moment
		var cr = this._drawSettings.clipRegion;
		if ( cr ) {
			var dstX = this._drawAtCalculatedPos.x;
			var dstY = this._drawAtCalculatedPos.y;

			var clipRight = cr.x + cr.width;
			var clipBottom = cr.y + cr.height;
			var rightPoint = dstX + this._image.width;
			var bottomPoint = dstY + this._image.height;

			var leftInside = cr.x <= dstX && cr.x + cr.width >= dstX;
			var rightInside = cr.x <= rightPoint && cr.x + cr.width >= rightPoint;
			var topInside = cr.y <= dstY && cr.y + cr.height >= dstY;
			var bottomInside = cr.y <= bottomPoint && cr.y + cr.height >= bottomPoint;

			var noOverlap = ( clipBottom < dstY || cr.y > bottomPoint || clipRight < dstX || cr.x > rightPoint );

			if ( noOverlap ) {
				// completely outside the clip region
				return null;
			} else {
				// image is partially inside clip region
				var leftOffsetX = 0;
				var rightOffsetX = 0;
				var topOffsetY = 0;
				var bottomOffsetY = 0;

				if ( !leftInside ) {
					leftOffsetX = cr.x - dstX;
				}
				if ( !rightInside ) {
					rightOffsetX = this._image.width - ( cr.x + cr.width - dstX );
				}
				if ( !topInside ) {
					topOffsetY = cr.y - dstY;
				}
				if ( !bottomInside ) {
					bottomOffsetY = this._image.height - ( cr.y + cr.height - dstY );
				}

				var newWidth = this._image.width - leftOffsetX - rightOffsetX;
				var newHeight = this._image.height - topOffsetY - bottomOffsetY;

				if ( newWidth <= 0 || newHeight <= 0 ) {
					return null;
				}

				return {
					src: {
						x: this._image.x + leftOffsetX,
						y: this._image.y + topOffsetY,
						width: newWidth,
						height: newHeight
					},
					dst: {
						x: this._drawAtCalculatedPos.x + leftOffsetX,
						y: this._drawAtCalculatedPos.y + topOffsetY,
						width: newWidth,
						height: newHeight
					}
				};
			}
		} else {
			// Clipping disabled
			return {
				src: {
					x: this._image.x,
					y: this._image.y,
					width: this._image.width,
					height: this._image.height
				},
				dst: {
					x: this._drawAtCalculatedPos.x,
					y: this._drawAtCalculatedPos.y,
					width: this._image.width,
					height: this._image.height
				}
			};
		}
	};


	/**
	 *    Draw image to context
	 * @param ctx - Context for canvas to draw onto
	 */
	Sprite.prototype.draw = function( ctx ) {
		if ( !this._currentAnimSettings.drawWhenNotPlaying && !this._activeAnimInfo.isPlaying ) {
			return;
		}

		if ( this._isAnimation ) {
			this._image = this._imageFrames[this._activeAnimInfo.currentFrame];
		}

		var isScale = this._scale.x > 1 || this._scale.x < 1 || this._scale.y > 1 || this._scale.y < 1;
		var isAlpha = this._alpha < 1;
		var isRotate = this._rotation > 0 || this._rotation < 0;

		this._calculateDrawPosition();

		if ( this._alpha > 0 ) {

			if ( isAlpha || isScale || isRotate || this._settings.enableDebug ) {
				ctx.save();
				ctx.globalAlpha = this._alpha;

				var imgCenterX = this._drawAtCalculatedPos.x + ( this._image.width / 2 );
				var imgCenterY = this._drawAtCalculatedPos.y + ( this._image.height / 2 );

				this._drawAtCalculatedPos.x -= imgCenterX;
				this._drawAtCalculatedPos.y -= imgCenterY;

				ctx.translate( imgCenterX, imgCenterY );
			}

			if ( isRotate ) {
				ctx.rotate( this._rotation * radiansPerDegree );
			}

			if ( isScale ) {
				ctx.scale( this._scale.x, this._scale.y );
			}

			var drawDimensions = this._calculateDrawDimensions();

			if ( drawDimensions ) {
				ctx.drawImage(
					this._image.image,
					drawDimensions.src.x,
					drawDimensions.src.y,
					drawDimensions.src.width,
					drawDimensions.src.height,
					drawDimensions.dst.x,
					drawDimensions.dst.y,
					drawDimensions.dst.width,
					drawDimensions.dst.height
				);

				if ( this._settings.enableDebug ) {
					ctx.beginPath();
					ctx.rect( this._drawAtCalculatedPos.x, this._drawAtCalculatedPos.y, this._image.width, this._image.height );
					ctx.fillStyle = this._debugFillStyle;
					ctx.fill();
				}
			}

			if ( isAlpha || isScale || isRotate || this._settings.enableDebug ) {
				ctx.restore();
			}
		}

		this._lastX = this._position.x;
		this._lastY = this._position.y;

		this._dirtyFlag = false;
	};

	/**
	 * Check if image is dirty
	 * @returns {boolean|*}
	 */
	Sprite.prototype.isDirty = function() {
		return (this._activeAnimInfo.isPlaying || this._dirtyFlag);
	};

	/**
	 * Draws if it's dirty
	 * @param ctx
	 */
	Sprite.prototype.drawIfDirty = function( ctx ) {
		if ( this.isDirty() ) {
			this.draw( ctx );
		}
	};

	/**
	 * Returns the total frame count of a sprite
	 */
	Sprite.prototype.getFrameCount = function() {
		return this._imageFrames.length;
	};

	/**
	 * Returns position of sprite
	 * @returns {object}
	 */
	Sprite.prototype.getPosition = function() {
		return this._position;
	};

	/**
	 * Returns current scale
	 * @returns {object}
	 */
	Sprite.prototype.getScale = function() {
		return this._scale;
	};

	/**
	 * Returns current image alpha
	 * @returns {object}
	 */
	Sprite.prototype.getAlpha = function() {
		return this._alpha;
	};

	/**
	 * Returns current rotation
	 * @returns {object}
	 */
	Sprite.prototype.getRotation = function() {
		return this._rotation;
	};

	/**
	 * Returns last drawn position
	 * @returns {object}
	 */
	Sprite.prototype.getDrawnPosition = function() {
		return this._drawAtCalculatedPos;
	};

	/**
	 * Returns last drawn dimensions
	 * @returns {Object}
	 */
	Sprite.prototype.getDrawnDimensions = function() {
		return this._imageDimensions;
	};

	/**
	 * Returns position transforms
	 * @returns {Object}
	 */
	Sprite.prototype.getPositionTransforms = function() {
		return this._positionTransform;
	};

	/**
	 * Returns scale transforms
	 * @returns {Object}
	 */
	Sprite.prototype.getScaleTransforms = function() {
		return this._scaleTransform;
	};

	/**
	 * Returns alpha transforms
	 * @returns {Object}
	 */
	Sprite.prototype.getAlphaTransforms = function() {
		return this._alphaTransform;
	};

	/**
	 * Returns rotation transforms
	 * @returns {Object}
	 */
	Sprite.prototype.getRotationTransforms = function() {
		return this._rotationTransform;
	};

	/**
	 * Play animation
	 * @method play
	 * @public
	 * @param {String} animName
	 * @param {Object=} settings
	 */
	Sprite.prototype.play = function( animName, settings ) {
		var that = this;
		if ( this._isAnimation ) {
			if ( this._setAnimation( animName ) ) {
				this._activeAnimInfo.isPlaying = true;
				this._isDirty = true;

				if ( settings && settings.callback ) {
					this._playCallback = settings.callback;
				}

				if ( this._settings.callbacks.onFrameChange ) {
					this._settings.callbacks.onFrameChange.call( this._thisContext, this._activeAnimInfo.currentFrame );
				}

				this._currentAnimTime = this._getAnimTime();
				this._animationStateMachine.stateGoto( function() {
					that._stateAnimation();
				} );
			}
		}
	};

	/**
	 * Stop animation
	 */
	Sprite.prototype.stop = function() {
		if ( this._isAnimation ) {
			this._activeAnimInfo.isPlaying = false;
			this._activeAnimInfo.timesLooped = 0;
			this._activeAnimInfo.playForwards = this._currentAnimSettings.startPlayForwards;
			this._isDirty = true;

			if ( this._currentAnimSettings.resetToStartFrameOnEnd ) {
				this._activeAnimInfo.currentFrame = this._currentAnimSettings.startFrame;
				if ( this._settings.callbacks.onFrameChange ) {
					this._settings.callbacks.onFrameChange.call( this._thisContext, this._activeAnimInfo.currentFrame );
				}
			}

			if ( this._playCallback ) {
				this._playCallback.call( this._thisContext );
				this._playCallback = null;
			}

			if ( this._settings.callbacks.onAnimFinished ) {
				this._settings.callbacks.onAnimFinished.call( this._thisContext );
			}

			this._animationStateMachine.stateGoto( null );
		}
	};

	/**
	 * Function to tell if sprite is currently playing an animation
	 * @returns {boolean}
	 */
	Sprite.prototype.isPlaying = function() {
		return this._activeAnimInfo.isPlaying;
	};

	/**
	 * Move Sprite to a point on the canvas
	 * @param options - Object or Array of Objects - { duration: int, x: int, y: int, callback: function, tween: function }
	 */
	Sprite.prototype.moveTo = function( options ) {
		this._doMove( options, true );
		return this;
	};

	/**
	 * Move Sprite by distance from it's current point
	 * @param options - Object or Array of Objects - { duration: int, x: int, y: int, callback: function, tween: function }
	 */
	Sprite.prototype.moveBy = function( options ) {
		this._doMove( options, false );
		return this;
	};

	/**
	 * Rotate Sprite to a certain value
	 * @param options - Object or Array of Objects - { duration: int, deg: int, callback: function, tween: function }
	 */
	Sprite.prototype.rotateTo = function( options ) {
		this._doRotate( options, true );
		return this;
	};

	/**
	 * Rotate Sprite by a certain amount from its current rotation
	 * @param options - Object or Array of Objects - { duration: int, deg: int, callback: function, tween: function }
	 */
	Sprite.prototype.rotateBy = function( options ) {
		this._doRotate( options, false );
		return this;
	};

	/**
	 * Scale Sprite to the value given
	 * @param options - Object or Array of Objects - { duration: int, x: int, y: int, callback: function, tween: function }
	 */
	Sprite.prototype.scaleTo = function( options ) {
		this._doScale( options, true );
		return this;
	};

	/**
	 * Scale Sprite an amount from the current scaling
	 * @param options - Object or Array of Objects - { duration: int, x: int, y: int, callback: function, tween: function }
	 */
	Sprite.prototype.scaleBy = function( options ) {
		this._doScale( options, false );
		return this;
	};

	/**
	 * Fade Sprite to a certain value
	 * @param options - Object or Array of Objects - { duration: int, alpha: int, callback: function, tween: function }
	 */
	Sprite.prototype.fadeTo = function( options ) {
		this._doFade( options, true );
		return this;
	};

	/**
	 * Fade Sprite by a certain amount from its current alpha
	 * @param options - Object or Array of Objects - { duration: int, alpha: int, callback: function, tween: function }
	 */
	Sprite.prototype.fadeBy = function( options ) {
		this._doFade( options, false );
		return this;
	};

	Sprite.prototype._doMove = function( options, isAbsolute ) {
		var that = this;

		var baseDefaultStep = {
			x: isAbsolute ? that._position.x : 0,
			y: isAbsolute ? that._position.y : 0
		};
		var startValFunc = function( transformObject ) {
			transformObject.startX = that._position.x;
			transformObject.startY = that._position.y;
		};
		var transformObject = this._positionTransform;

		this._createTransformationSteps( baseDefaultStep, startValFunc, transformObject, options, isAbsolute,
			this._positionIsAbsoluteFunc, this._positionTransformationFunc, this._positionEndFunc );
	};

	Sprite.prototype._doRotate = function( options, isAbsolute ) {
		var that = this;

		var baseDefaultStep = {
			deg: isAbsolute ? that._rotation : 0
		};
		var startValFunc = function( transformObject ) {
			transformObject.startVal = that._rotation;
		};
		var transformObject = this._rotationTransform;

		this._createTransformationSteps( baseDefaultStep, startValFunc, transformObject, options, isAbsolute,
			this._rotationIsAbsoluteFunc, this._rotationTransformationFunc, this._rotationEndFunc );
	};


	Sprite.prototype._doScale = function( options, isAbsolute ) {
		var that = this;

		var baseDefaultStep = {
			x: isAbsolute ? that._scale.x : 0,
			y: isAbsolute ? that._scale.y : 0
		};
		var startValFunc = function( transformObject ) {
			transformObject.startX = that._scale.x;
			transformObject.startY = that._scale.y;
		};
		var transformObject = this._scaleTransform;

		this._createTransformationSteps( baseDefaultStep, startValFunc, transformObject, options, isAbsolute,
			this._scaleIsAbsoluteFunc, this._scaleTransformationFunc, this._scaleEndFunc );
	};

	Sprite.prototype._doFade = function( options, isAbsolute ) {
		var that = this;

		var baseDefaultStep = {
			alpha: isAbsolute ? that._alpha : 0
		};
		var startValFunc = function( transformObject ) {
			transformObject.startVal = that._alpha;
		};
		var transformObject = this._alphaTransform;

		this._createTransformationSteps( baseDefaultStep, startValFunc, transformObject, options, isAbsolute,
			this._alphaIsAbsoluteFunc, this._alphaTransformationFunc, this._alphaEndFunc );
	};

	Sprite.prototype._createTransformationSteps = function( baseDefaultStep, startValFunc, transformObject, options, isAbsolute,
															isAbsoluteFunc, transformationFunc, endFunc ) {
		var defaultStep = jQuery.extend( true, {}, baseDefaultStep );
		defaultStep.isAbsolute = isAbsolute;

		var optionsArray = [];

		if ( options instanceof Array ) {
			optionsArray = options;
		} else {
			optionsArray.push( options );
		}

		var transformationSteps = [];
		for ( var i = 0; i < optionsArray.length; ++i ) {
			var option = optionsArray[i];
			transformationSteps.push( jQuery.extend( true, {}, defaultTransformationStep, defaultStep, option ) );
		}

		var sequenceLength = transformObject.sequenceArray.length;
		if ( sequenceLength === 0 ) {
			startValFunc( transformObject );
		}

		// if no currently queued anims, and current anim has duration of 0, just do it
		// immediately rather than have to wait for slower state machine
		if ( sequenceLength === 0 && transformationSteps.length === 1 && transformationSteps[0].duration === 0 ) {
			var currentTransformation = transformationSteps[0];
			if ( currentTransformation.isAbsolute ) {
				isAbsoluteFunc.call( this, currentTransformation );
			}

			transformationFunc.call( this, currentTransformation, transformObject, 1.0 );
			this._dirtyFlag = true;

			currentTransformation.progress = 1;

			if ( currentTransformation.callback ) {
				currentTransformation.callback.call( this._thisContext );
			}

			endFunc.call( this, currentTransformation, transformObject );
		} else {
			Array.prototype.push.apply( transformObject.sequenceArray, transformationSteps );
		}
	};

	Sprite.prototype._onNotifyTick = function( timeDelta ) {
		if ( this._activeAnimInfo.isPlaying ) {
			this._animationStateMachine.update( timeDelta );
		}
		if ( this._positionTransform.sequenceArray.length > 0 ) {
			this._positionStateMachine.update( timeDelta );
		}
		if ( this._alphaTransform.sequenceArray.length > 0 ) {
			this._alphaStateMachine.update( timeDelta );
		}
		if ( this._rotationTransform.sequenceArray.length > 0 ) {
			this._rotationStateMachine.update( timeDelta );
		}
		if ( this._scaleTransform.sequenceArray.length > 0 ) {
			this._scaleStateMachine.update( timeDelta );
		}
	};

	Sprite.prototype._onNotifyCanvasPointerClick = function( point ) {
		if ( this._settings.callbacks.onClick ) {
			if ( (point.x > this._position.x && point.x < (this._position.x + this._imageDimensions.width)) &&
				(point.y > this._position.y && point.y < (this._position.y + this._imageDimensions.height)) ) {
				this._settings.callbacks.onClick.call( this._thisContext, point );
			}
		}
	};

	/**
	 * Gets animation frames from loaded library images
	 * @param imageBaseName
	 * @returns {Array}
	 * @private
	 */
	Sprite.prototype._loadAnimationFrames = function( imageBaseName ) {
		var padToFive = function( number ) {
			if ( number <= 99999 ) {
				number = ("0000" + number).slice( -5 );
			}
			return number;
		};

		var assertImageFormat = function( img ) {
			// Images using the new sprite class should be loaded using the 'atlas' method in the manifest, using a data:[] member
			if ( img.image === undefined ) {
				throw new Error( "Images loaded using the new sprite class should use a 'data: []' member in their manifest!" );
			}
		};

		var imageList = [];
		for ( var i = 0; i < 500; ++i ) {
			var frameName = imageBaseName + '_' + padToFive( i );
			if ( libs.loadedImages[frameName] ) {
				assertImageFormat( libs.loadedImages[frameName] );
				imageList.push( libs.loadedImages[frameName] );
			} else if ( libs.loadedImages[imageBaseName] && i < 1 ) {
				assertImageFormat( libs.loadedImages[imageBaseName] );
				imageList.push( libs.loadedImages[imageBaseName] );
			} else if ( i > 1 ) {
				break;
			}
		}
		return imageList;
	};

	Sprite.prototype._setAnimation = function( animName ) {
		if ( animName ) {
			if ( this._settings.customAnims[animName] ) {
				this._currentAnimSettings = jQuery.extend( true, {}, this._settings.animSettings, this._settings.customAnims[animName] );
			} else {
				return false;
			}
		} else {
			this._currentAnimSettings = jQuery.extend( true, {}, this._settings.animSettings );
		}
		if ( this._currentAnimSettings.framesCount === undefined || this._currentAnimSettings.framesCount === null ) {
			this._currentAnimSettings.framesCount = this._imageFrames.length;
		}
		if ( this._currentAnimSettings.startFrame === undefined || this._currentAnimSettings.startFrame === null ) {
			this._currentAnimSettings.startFrame = this._currentAnimSettings.startPlayForwards ? 0 : this._currentAnimSettings.framesCount - 1;
		}
		this._activeAnimInfo.currentFrame = this._currentAnimSettings.startFrame;
		this._activeAnimInfo.timesLooped = 0;
		this._activeAnimInfo.isPlaying = false;
		this._activeAnimInfo.playForwards = this._currentAnimSettings.startPlayForwards;

		return true;
	};

	/**
	 * Animation playing state
	 * @private
	 */
	Sprite.prototype._stateAnimation = function() {
		if ( this._animationStateMachine.phaseTimed( this._currentAnimTime ) ) {
			this._isDirty = true;
			var frameToShow = 0;
			if ( this._activeAnimInfo.playForwards ) {
				frameToShow = this._currentAnimSettings.startFrame + Math.round( (this._currentAnimSettings.framesCount - 1) * this._animationStateMachine.phaseTimedGetProgress() );
			} else {
				if ( this._currentAnimSettings.reverseLoop ) {
					frameToShow = this._currentAnimSettings.startFrame + Math.round( (this._currentAnimSettings.framesCount - 1) * this._animationStateMachine.phaseTimedGetRevProgress() );
				} else {
					frameToShow = this._currentAnimSettings.startFrame - Math.round( (this._currentAnimSettings.framesCount - 1) * this._animationStateMachine.phaseTimedGetProgress() );
				}
			}

			if ( this._activeAnimInfo.currentFrame !== frameToShow ) {
				this._activeAnimInfo.currentFrame = frameToShow;
				if ( this._settings.callbacks.onFrameChange ) {
					this._settings.callbacks.onFrameChange.call( this._thisContext, this._activeAnimInfo.currentFrame );
				}
			}
		}

		if ( this._animationStateMachine.phaseOnce() ) {
			if ( this._currentAnimSettings.timesToLoop === -1 || (this._activeAnimInfo.timesLooped < this._currentAnimSettings.timesToLoop) ) {
				this._activeAnimInfo.timesLooped++;
				if ( this._currentAnimSettings.reverseLoop ) {
					this._activeAnimInfo.playForwards = !this._activeAnimInfo.playForwards;
					this._activeAnimInfo.startFrame = this._activeAnimInfo.currentFrame;
				}
				this._animationStateMachine.phaseReset();
			} else {
				this.stop( this._currentAnimSettings.resetToStartFrameOnEnd );
			}

			if ( this._settings.callbacks.onAnimLooped ) {
				this._settings.callbacks.onAnimLooped.call( this._thisContext );
			}
		}
	};

	Sprite.prototype._stateTransformation = function( stateMachine, isAbsoluteFunc, transformationFunc, endFunc, transformSettings ) {
		if ( transformSettings.sequenceArray.length === 0 ) {
			return;
		}

		var currentTransformation = transformSettings.sequenceArray[0];
		var duration = currentTransformation.duration;

		if ( stateMachine.phaseOnce() ) {
			if ( currentTransformation.isAbsolute ) {
				isAbsoluteFunc.call( this, currentTransformation );
			}
		}
		if ( stateMachine.phaseTimed( duration ) ) {
			currentTransformation.progress = stateMachine.phaseTimedGetProgress();
			transformationFunc.call( this, currentTransformation, transformSettings, currentTransformation.progress );
			this._dirtyFlag = true;
		}
		if ( stateMachine.phaseOnce() ) {
			if ( currentTransformation.callback ) {
				currentTransformation.callback.call( this._thisContext );
			}

			endFunc.call( this, currentTransformation, transformSettings );

			transformSettings.sequenceArray.shift();
			stateMachine.phaseReset();
		}
	};

	Sprite.prototype._positionIsAbsoluteFunc = function( currentTransformation ) {
		currentTransformation.x -= this._position.x;
		currentTransformation.y -= this._position.y;
	};

	Sprite.prototype._positionTransformationFunc = function( currentTransformation, transformSettings, progress ) {
		this._position.x = transformSettings.startX + currentTransformation.x * currentTransformation.tween( progress );
		this._position.y = transformSettings.startY + currentTransformation.y * currentTransformation.tween( progress );
	};

	Sprite.prototype._positionEndFunc = function( currentTransformation, transformSettings ) {
		transformSettings.startX = this._position.x;
		transformSettings.startY = this._position.y;
	};

	Sprite.prototype._scaleIsAbsoluteFunc = function( currentTransformation ) {
		currentTransformation.x -= this._scale.x;
		currentTransformation.y -= this._scale.y;
	};

	Sprite.prototype._scaleTransformationFunc = function( currentTransformation, transformSettings, progress ) {
		this._scale.x = transformSettings.startX + currentTransformation.x * currentTransformation.tween( progress );
		this._scale.y = transformSettings.startY + currentTransformation.y * currentTransformation.tween( progress );
	};

	Sprite.prototype._scaleEndFunc = function( currentTransformation, transformSettings ) {
		transformSettings.startX = this._scale.x;
		transformSettings.startY = this._scale.y;
	};

	Sprite.prototype._rotationIsAbsoluteFunc = function( currentTransformation ) {
		currentTransformation.deg -= this._rotation;
	};

	Sprite.prototype._rotationTransformationFunc = function( currentTransformation, transformSettings, progress ) {
		this._rotation = transformSettings.startVal + currentTransformation.deg * currentTransformation.tween( progress );
	};

	Sprite.prototype._rotationEndFunc = function( currentTransformation, transformSettings ) {
		transformSettings.startVal = this._rotation;
	};

	Sprite.prototype._alphaIsAbsoluteFunc = function( currentTransformation ) {
		currentTransformation.alpha -= this._alpha;
	};

	Sprite.prototype._alphaTransformationFunc = function( currentTransformation, transformSettings, progress ) {
		this._alpha = transformSettings.startVal + currentTransformation.alpha * currentTransformation.tween( progress );
	};

	Sprite.prototype._alphaEndFunc = function( currentTransformation, transformSettings ) {
		transformSettings.startVal = this._alpha;
	};

	Sprite.prototype._statePositioning = function() {
		this._stateTransformation( this._positionStateMachine, this._positionIsAbsoluteFunc, this._positionTransformationFunc, this._positionEndFunc, this._positionTransform );
	};

	Sprite.prototype._stateRotating = function() {
		this._stateTransformation( this._rotationStateMachine, this._rotationIsAbsoluteFunc, this._rotationTransformationFunc, this._rotationEndFunc, this._rotationTransform );
	};

	Sprite.prototype._stateScaling = function() {
		this._stateTransformation( this._scaleStateMachine, this._scaleIsAbsoluteFunc, this._scaleTransformationFunc, this._scaleEndFunc, this._scaleTransform );
	};

	Sprite.prototype._stateFading = function() {
		this._stateTransformation( this._alphaStateMachine, this._alphaIsAbsoluteFunc, this._alphaTransformationFunc, this._alphaEndFunc, this._alphaTransform );
	};

	/**
	 * Get total animation time
	 * @returns {number}
	 */
	Sprite.prototype._getAnimTime = function() {
		return (this._currentAnimSettings.framesCount / this._currentAnimSettings.fps) * 1000; // * 1000 as statemachine is in ms
	};


	/**
	 * Sets the image frame that is displayed for animation frame (if it's not playing)
	 * This can be used by games that wish to control animations themselves without calling .play()
	 */
	Sprite.prototype.setAnimationFrame = function( frameIndex ) {
		if ( !this._isAnimation ) {
			throw new Error( "Sprite.setAnimationFrame() can only be called on animation sprites!" );
		}
		if ( frameIndex < 0 || frameIndex >= this._currentAnimSettings.framesCount ) {
			throw new Error( "Invalid argument for Sprite.setAnimationFrame()" );
		}

		this._activeAnimInfo.currentFrame = frameIndex;
	};


	libs.client.rendering.Sprite = Sprite;

}());