goog.provide( 'libs.client.rendering.SurfaceManager' );
goog.provide( 'libs.surfaceBase' );
goog.provide( 'libs.surfaceTop' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.rendering.Surface' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.utils.misc' );


(function() {
	"use strict";

	var defaultSettings = {
		gameWidth: undefined,
		gameHeight: undefined
	};

	var selectors = {
		canvasBase: $( '#canvas-base' ),
		canvasTop: $( '#canvas-top' )
	};

	var clientInfo = libs.client.core.clientInformation;
	var isNativeAndroid = clientInfo.isNativeAndroidBrowser();

	var eatMouse = function( e ) {
		e.preventDefault();
	};

	var SurfaceManager = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		libs.surfaceBase = null;
		libs.surfaceTop = null;

		libs.surfaceSettings = {
			base: {
				zIndexNormal: 200,
				zIndexMovedToFront: 200
			},
			top: {
				zIndexNormal: 220,
				zIndexMovedToFront: 390
			}
		};

		this._currentSurfaceBaseZindex = libs.surfaceSettings.base.zIndexNormal;
		this._currentSurfaceTopZindex = libs.surfaceSettings.top.zIndexNormal;

		this._androidRequiresZIndexTrick = false;
		this._blankCanvasData = null;

		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.systemInit', function( activity ) { that._onSystemInitStart( activity ); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.systemState.systemInit', function() { that._onSystemInitEnd(); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gameInit', function( activity ) { that._onGameInitStart( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gamePlaying', function( activity ) { that._onGamePlayingStart( activity ); } );
		libs.globalEventBus.connect( 'libs.event.notify.tick', function( timeDelta ) { that._onNotifyTick( timeDelta ); } );
		libs.globalEventBus.connect( 'libs.event.request.changeTopCanvasZIndex', function( zIndex ) { that._onRequestChangeTopCanvasZIndex( zIndex ); } );
		libs.globalEventBus.connect( 'libs.event.request.enableZIndexHack', function() { that._onRequestEnableZIndexHack(); } );
		libs.globalEventBus.connect( 'libs.event.request.disableZIndexHack', function() { that._onRequestDisableZIndexHack(); } );
	};

	SurfaceManager.prototype._onSystemInitStart = function( activity ) {
		libs.surfaceBase = new libs.client.rendering.Surface( selectors.canvasBase, this._settings.gameWidth, this._settings.gameHeight );
		libs.surfaceTop = new libs.client.rendering.Surface( selectors.canvasTop, this._settings.gameWidth, this._settings.gameHeight );

		this._blankCanvasData = libs.surfaceBase.ctx.getImageData( 0, 0, this._settings.gameWidth, this._settings.gameHeight );
		libs.surfaceBase.setBlankCanvasData( this._blankCanvasData );
		libs.surfaceTop.setBlankCanvasData( this._blankCanvasData );

		libs.surfaceBase.canvas.style.zIndex = this._currentSurfaceBaseZindex;
		libs.surfaceTop.canvas.style.zIndex = this._currentSurfaceTopZindex;
	};

	SurfaceManager.prototype._onSystemInitEnd = function() {
		var that = this;

		for ( var i = 0; i < 2; ++i ) {
			var jCanvas = i === 0 ? selectors.canvasBase : selectors.canvasTop;
			var canvas = jCanvas.get( 0 );
			canvas.style.width = "100%";
			canvas.style.height = "100%";
			canvas.width = this._settings.gameWidth;
			canvas.height = this._settings.gameHeight;
		}

		selectors.canvasTop.bind( "vmousemove", eatMouse );

		var jWin = $( window );
		jWin.resize( function() {that._resize(); } );
		jWin.focus( function() { libs.globalEventBus.invoke( 'libs.event.request.performFullRender', 1 ); } );
		jWin.scroll( function() { libs.globalEventBus.invoke( 'libs.event.request.performFullRender', 1 ); } );

		libs.globalEventBus.connect( 'libs.event.notify.displayHeightChanged', function() { that._resize(); } );

		libs.globalEventBus.invoke( 'libs.event.notify.interactiveSurface', libs.surfaceTop );
	};

	SurfaceManager.prototype._onGameInitStart = function( activity ) {
		selectors.canvasTop.unbind( "vmousemove", eatMouse );
	};

	SurfaceManager.prototype._onGamePlayingStart = function( activity ) {
		this._resize();
	};

	SurfaceManager.prototype._onNotifyTick = function( timeDelta ) {
		if ( this._androidRequiresZIndexTrick ) {
			this._zIndexTrick();
		}
	};

	SurfaceManager.prototype._onRequestEnableZIndexHack = function() {
		if ( isNativeAndroid ) {
			this._androidRequiresZIndexTrick = true;
		}
	};

	SurfaceManager.prototype._onRequestDisableZIndexHack = function() {
		if ( isNativeAndroid ) {
			this._androidRequiresZIndexTrick = false;
			$( '#viewport' ).redrawGentle();
		}
	};

	SurfaceManager.prototype._onRequestChangeTopCanvasZIndex = function( zIndex ) {
		this._currentSurfaceTopZindex = zIndex;
		selectors.canvasTop.css( 'z-index', zIndex );
	};

	// changing z-index allows clearrect to work at all times on samsung android devices
	SurfaceManager.prototype._zIndexTrick = function( doDOMRedraw ) {
		var that = this;

		libs.surfaceBase.canvas.style.zIndex = that._currentSurfaceBaseZindex + 1;
		libs.surfaceTop.canvas.style.zIndex = that._currentSurfaceTopZindex + 1;
		libs.globalEventBus.invoke( 'libs.event.request.canvasFullDraw', true, true );
		if ( doDOMRedraw ) {
			$( '#viewport' ).redrawGentle();
		}

		libs.setTimeout( function() {
			libs.surfaceBase.canvas.style.zIndex = that._currentSurfaceBaseZindex;
			libs.surfaceTop.canvas.style.zIndex = that._currentSurfaceTopZindex;
			libs.globalEventBus.invoke( 'libs.event.request.canvasFullDraw', true, true );
			if ( doDOMRedraw ) {
				$( '#viewport' ).redrawGentle();
			}
		}, 1 );
	};

	SurfaceManager.prototype._resize = function() {
		libs.globalEventBus.invoke( 'libs.event.request.performFullRender', 3 );
		libs.setTimeout( function() {
			var screen = libs.client.utils.misc.screenSize();
			libs.surfaceBase.resizeWithAspect( screen.width, screen.height );
			libs.surfaceTop.resizeWithAspect( screen.width, screen.height );
			libs.globalEventBus.invoke( 'libs.event.request.performFullRender', 3 );
		}, 100 );
	};


	libs.client.rendering.SurfaceManager = SurfaceManager;

}());