goog.provide( 'libs.client.sound.SoundPlayer' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.core.clientInformation' );


(function() {
	"use strict";

	var defaultSettings = {
		touchEventSound: 'SndSilence',
		playSound: {
			loopCount: 0,
			loopCallback: null,
			pan: 0
		}
	};

	var enableLogging = !CLOSURE_COMPILE;

	var clientInfo = libs.client.core.clientInformation;

	var SoundPlayerBase = function( overrideSettings ) {
		if ( this.constructor === SoundPlayerBase ) {
			libs.logError( "Can't instantiate abstract SoundPlayerBase class" );
		}

		var that = this;

		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		this._audioQuality = clientInfo.getAudioQuality();
		this._assetsLoaded = false;
		this._availableSounds = [];
		this._soundInfoByName = null;

		for ( var i = 0; i < game.manifestAudio.length; ++i ) {
			this._availableSounds.push( game.manifestAudio[i].id );
		}

		var initFunc = function() {
			if ( that._assetsLoaded ) {
				if ( that._audioQuality === 'MQ' ) {
					// have to play each sound first within a touch event to let them play outside touch events in future
					for ( var i = 0; i < game.manifestAudio.length; ++i ) {
						that._play( game.manifestAudio[i].id );
						that._stop( game.manifestAudio[i].id );
					}
				} else if ( that._audioQuality === 'HQ' ) {
					that._play( that._settings.touchEventSound );
					that._stop( that._settings.touchEventSound );
				}

				$( window ).off( 'pointerdown MSPointerDown touchstart mousedown', '*', initFunc );
				$( '.button' ).off( 'pointerdown MSPointerDown touchstart mousedown', initFunc );
				// $( '#button-spin-container' ).off( 'pointerdown MSPointerDown touchstart mousedown', initFunc );
			}
		};

		if ( clientInfo.getDeviceType() === 'mobile' ) {
			$( window ).on( 'pointerdown MSPointerDown touchstart mousedown', '*', initFunc );
			$( '.button' ).on( 'pointerdown MSPointerDown touchstart mousedown', initFunc );
			// $( '#button-spin-container' ).on( 'pointerdown MSPointerDown touchstart mousedown', initFunc );

			// initialise the list of sounds by name.
			if (this._soundInfoByName === null) {
				this._soundInfoByName = {};
				$.each(game.manifestAudio, function(manifestIndex, soundInfo) {
					that._soundInfoByName[soundInfo.id] = soundInfo;
				});
			}
		}
	};

	SoundPlayerBase.prototype._initialise = function() {
		var that = this;

		if ( this._audioQuality !== "LQ" ) {
			$( '#tag-sound-player' ).remove();
		}

		this._deviceType = clientInfo.getDeviceType();
		this._enabled = true;

		libs.globalEventBus.connectActivityEnd( 'libs.activity.systemState.gameLoading', function() { that._onGameLoadingEnd(); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.systemState.gameInit', function() { that._onGameInitEnd(); } );

		libs.globalEventBus.connect( 'libs.event.request.enableSounds', function() { that._onRequestEnableSounds(); } );
		libs.globalEventBus.connect( 'libs.event.request.disableSounds', function() { that._onRequestDisableSounds(); } );
		libs.globalEventBus.connect( 'libs.event.request.playSound', function( quality, name, settings ) { that._onRequestPlaySound( quality, name, settings ); } );
		libs.globalEventBus.connect( 'libs.event.request.stopSound', function( quality, name ) { that._onRequestStopSound( quality, name ); } );
	};

	SoundPlayerBase.prototype._getSoundInfo = function(name) {
		var soundInfo = null;

		if (this._soundInfoByName !== null && (name in this._soundInfoByName)) {
			soundInfo = this._soundInfoByName[name];
		}

		return soundInfo;
	};

	SoundPlayerBase.prototype._onGameLoadingEnd = function() {
		this._assetsLoaded = true;
	};

	SoundPlayerBase.prototype._onGameInitEnd = function() {
		libs.globalEventBus.invoke( 'libs.event.notify.soundsState', this._enabled );
	};

	SoundPlayerBase.prototype._onRequestEnableSounds = function() {
		this._enabled = true;
		this._enableSounds( this._enabled );

		libs.globalEventBus.invoke( 'libs.event.notify.soundsState', this._enabled );
	};

	SoundPlayerBase.prototype._onRequestDisableSounds = function() {
		this._enabled = false;
		this._enableSounds( this._enabled );

		libs.globalEventBus.invoke( 'libs.event.notify.soundsState', this._enabled );
	};

	SoundPlayerBase.prototype._onRequestPlaySound = function( quality, name, settings ) {
		if ( this._doesQualityMatch( quality ) ) {
			if ( !settings ) {
				settings = jQuery.extend( true, {}, this._settings.playSound );
			}

			var doesSoundExist = this._availableSounds.indexOf( name ) > -1;
			if ( this._audioQuality === 'LQ' ) {
				doesSoundExist = true;
			}

			if ( doesSoundExist && this._canPlaySound() ) {
				settings = jQuery.extend( true, {}, this._settings.playSound, settings );

				if ( settings.loopCount === -1 ) {
					settings.loopCount = 99999;
				}

				if ( ('Media' in window)) {
					this._playWithCordova(name, settings);
				}
				else {
					this._play( name, settings );
				}
			} else {
				if ( settings.loopCallback ) {
					settings.loopCallback();
				}
			}
		}
	};

	SoundPlayerBase.prototype._onRequestStopSound = function( quality, name ) {
		if ( this._shouldHandleRequest( quality ) ) {
			this._stop( name );
		}
	};

	SoundPlayerBase.prototype._shouldHandleRequest = function( quality ) {
		return this._doesQualityMatch( quality ) && this._canPlaySound();
	};

	SoundPlayerBase.prototype._doesQualityMatch = function( quality ) {
		var qualityArray = [];

		if ( quality instanceof Array ) {
			qualityArray = quality;
		} else {
			qualityArray.push( quality );
		}

		return (qualityArray.indexOf( this._audioQuality ) > -1);
	};

	SoundPlayerBase.prototype._canPlaySound = function() {
		return (this._enabled || this._deviceType === 'desktop');
	};

	var SoundJsPlayer = function( overrideSettings ) {
		SoundPlayerBase.apply( this, arguments );
		this._initialise();

		this.soundCache = [];

		this._getInstance = function( name ) {
			// allow 2 instance of each sound
			var earliestInstanceIndex = null;
			var numberOfInstances = 0;

			var len = this.soundCache.length;
			for ( var i = 0; i < len; ++i ) {
				if ( this.soundCache[i].name === name ) {
					if ( earliestInstanceIndex === null ) {
						earliestInstanceIndex = i;
					}
					++numberOfInstances;
				}
			}

			if ( numberOfInstances < 4 ) {
				return null;
			} else {
				this.soundCache.move( earliestInstanceIndex, len - 1 );
				if ('media' in this.soundCache[len - 1]) {
					this.soundCache[len - 1].media.stop();
				}
				else {
					this.soundCache[len - 1].stop();
					this.soundCache[len - 1].removeAllEventListeners();
				}
				return this.soundCache[len - 1];
			}
		};

		this._handleComplete = function( ev, name ) {
			if ( enableLogging ) {
				libs.log( "SoundJsPlayer: _handleComplete(" + name + ")" );
			}
		};

		this._handleFailed = function( ev, name ) {
			if ( enableLogging ) {
				libs.logError( "SoundJsPlayer ERROR: Sound " + name + " failed to play" );
			}
		};

		this._playWithCordova = function(name, settings) {
			if ( !settings ) {
				settings = jQuery.extend( true, {}, this._settings.playSound );
			}

			var instance = this._getInstance(name);
			if (!instance) {
				var soundInfo = this._getSoundInfo(name);
				var src = null;

				if (libs.client.core.clientInformation.isAndroid()) {
					src = '/android_asset/www/' + soundInfo.src;
				}
				else {
					src = soundInfo.src;
				}

				instance = {
					name: name,
					media: new Media(src),
					loopsRemaining: null
				};

				this.soundCache.push(instance);
			}

			if (instance) {
				if (settings.loopCount > 1) {
					instance.loopsRemaining = parseInt(settings.loopCount, 10);
				}

				instance.media.successCallback = function() {
					if (('loopsRemaining' in instance) && instance.loopsRemaining !== null) {
						if (instance.loopsRemaining > 1) {
							instance.loopsRemaining--;
							instance.media.play();
						}
						else if (instance.loopsRemaining === 0) {
							if (settings.loopCallback) {
								settings.loopCallback();
							}
							instance.loopsRemaining = null;
							instance.media.release();
							instance.media.errorCallback = null;
							instance.media.successCallback = null;
						}
					}
					else {
						instance.media.release();
						instance.media.errorCallback = null;
						instance.media.successCallback = null;
					}
				};

				instance.media.errorCallback = function(e) {
					console.log(e);
				}

				instance.media.play();
			}
		};

		this._play = function( name, settings ) {
			if ( !settings ) {
				settings = jQuery.extend( true, {}, this._settings.playSound );
			}

			try {
				var instance = this._getInstance( name );
				var playSettings = {
					interrupt: createjs.Sound.INTERRUPT_LATE,
					loop: settings.loopCount || 0,
					pan: settings.pan || 0
				};
				if ( instance ) {
					instance.play( playSettings );
				} else {
					instance = createjs.Sound.play( name, playSettings );
					instance.name = name;
					this.soundCache.push( instance );
				}

				if ( settings.loopCallback ) {
					instance.addEventListener( 'loop', settings.loopCallback );
				}

				//instance.addEventListener('complete', createjs.proxy(this._handleComplete, this, name));
				//instance.addEventListener('failed', createjs.proxy(this._handleFailed, this, name));

				if ( instance.playState === createjs.Sound.PLAY_FAILED ) {
					if ( enableLogging ) {
						libs.logError( "SoundJsPlayer ERROR: Sound " + name + " failed to play" );
					}
				}
			}
			catch ( err ) {
				if ( enableLogging ) {
					libs.logError( "SoundJsPlayer ERROR: " + err.message );
				}
			}
		};

		this._stop = function( name ) {
			for ( var i = 0, len = this.soundCache.length; i < len; ++i ) {
				if ( this.soundCache[i].name === name ) {
					if ('media' in this.soundCache[i]) {
						this.soundCache[i].loopsRemaining = null;
						this.soundCache[i].media.stop();
					}
					else {
						this.soundCache[i].stop();
						this.soundCache[i].removeAllEventListeners();
					}
				}
			}
		};

		this._enableSounds = function( enabled ) {
			if ( this._deviceType === 'mobile' ) {
				if ( !enabled ) {
					createjs.Sound.stop();
				}
			} else {
				createjs.Sound.setMute( !enabled );
			}
		};
	};

	SoundJsPlayer.prototype = Object.create( SoundPlayerBase.prototype );
	SoundJsPlayer.prototype.constructor = SoundJsPlayer;


	var SoundTagPlayer = function( overrideSettings ) {
		SoundPlayerBase.apply( this, arguments );
		this._initialise();

		this._audioTag = $( '#tag-sound-player' )[0];

		this._play = function( name, settings ) {
			this._audioTag.currentTime = 0;

			this._audioTag.loop = settings.loopCount > 0;
			this._audioTag.play();
		};

		this._stop = function() {
			this._audioTag.pause();
		};

		this._enableSounds = function( enabled ) {
			if ( !enabled ) {
				this._stop();
			}
		};
	};

	SoundTagPlayer.prototype = Object.create( SoundPlayerBase.prototype );
	SoundTagPlayer.prototype.constructor = SoundTagPlayer;


	if ( clientInfo.getAudioQuality() === "LQ" ) {
		libs.client.sound.SoundPlayer = SoundTagPlayer;
	} else {
		libs.client.sound.SoundPlayer = SoundJsPlayer;
	}

}());