goog.provide( 'libs.client.utils.DOM' );

goog.require( 'libs.client.core.clientInformation' );


(function() {
	"use strict";

	var selectors = {
		textHelper: $( '#text-helper' )
	};

	var clientInfo = libs.client.core.clientInformation;
	var isMobileDeviceType = clientInfo.getDeviceType() === 'mobile';

	var gameWidth = 800;
	libs.setTimeout( function() {
		// give game a chance to sort it's width out
		gameWidth = $( '#canvas-base' )[0].width;
	}, 250 );

	var defaultSettings = {
		text: {
			fontSize: 16, //initial font size
			parentClass: 'multiline-parent', //class to add to the parent container
			prefix: null, //id prefix for each line
			marginTop: 0, //amount to move the button down by, in pixels
			multiline: {
				reduceSpaceBetweenLinesBy: 0, //amount to reduce the space between lines, in % of a line
				marginTop: 0, //amount to reduce the top space, in % of a line
				consistentFontSize: true,
				numWordsToGroup: 0,
				maxLines: null
			}
		}
	};

	var useTextContent = 'textContent' in document.body;
	var bodyFontScale = 1;

	var langCode = 'en_GB';

	libs.globalEventBus.connect( 'libs.event.notify.fontScaleChanged', function( fontScale ) { bodyFontScale = fontScale; } );

	libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( code ) {
		langCode = code;
	} );

	/**
	 * Sets text on an element, bypassing jQuery if possible to make the operation as fast as can be
	 * @param {Object} element - jQuery element
	 * @param {string} text - Text to add into the element
	 */
	var setTextOnElem = function( element, text ) {
		if ( element[0] && !element[1] ) {
			if ( useTextContent ) {
				element[0].textContent = text;
			} else {
				element[0].innerText = text;
			}
		} else {
			element.text( text );
		}
	};

	/**
	 * Sets the font size on an element, bypassing jQuery if possible to make the operation as fast as can be
	 * @param {Object} element - jQuery element
	 * @param {string} fontSize - Size to set the text to
	 */
	var setFontSizeOnElem = function( element, fontSize ) {
		if ( element[0] && !element[1] ) {
			element[0].style.fontSize = fontSize;
		} else {
			element.css( 'font-size', fontSize );
		}
	};

	//Use for sketchy browsers to ensure that the element is redrawn
	/**
	 * A very harsh method to force sketchy browsers (like native Android browsers) to redraw an element
	 * @param {Object} element - The element (jQuery or native) to force the redraw on
	 * @param {boolean} forceOnAllDevices - By default, this only works on native Android browsers, this flag makes it always work
	 */
	var forceDOMRedraw = function( element, forceOnAllDevices ) {
		if ( !element || (!clientInfo.isNativeAndroidBrowser() && !forceOnAllDevices) ) {
			return;
		}

		element = element instanceof jQuery ? element[0] : element;

		var n = document.createTextNode( ' ' );
		var disp = element.style.display;

		element.appendChild( n );
		element.style.display = 'none';

		setTimeout( function() {
			element.style.display = disp;
			n.parentNode.removeChild( n );
		}, 20 );
	};

	/**
	 * A small public function to access the base fontSize that equates to 1em
	 * @returns {number}
	 */
	var getBaseFontSize = function() {
		return defaultSettings.text.fontSize;
	};

	/**
	 * Convert a pixel font size into an em value
	 * @param {number} fontSize - The font size in pixels
	 * @param {booleon=} justReturnNumberVal - Only return the em number, not the string val used for css
	 * @returns {number|string}
	 */
	var pem = function( fontSize, justReturnNumberVal ) {
		if ( justReturnNumberVal ) {
			return fontSize / getBaseFontSize();
		} else {
			return fontSize / getBaseFontSize() + 'em';
		}
	};

	/**
	 * Use a dedicated hidden div called #text-helper to check the size of a piece of text, and then adjust the font size
	 * if the calculated width is great than the maxWidth and/or maxHeight in the settings
	 * @param {string} text
	 * @param {Object} settings
	 * @param {boolean=} returnInPixels
	 * @returns {string|number}
	 */
	var calculateFontSize = function( text, settings, returnInPixels ) {
		text = text || "";
		var fontSize = settings.fontSize || getBaseFontSize;

		if ( settings.maxWidth || settings.maxHeight ) {
			setTextOnElem( selectors.textHelper, text );

			if ( settings.maxHeight ) {
				selectors.textHelper[0].style.fontSize = pem( fontSize );
				selectors.textHelper.css( 'width', settings.maxWidth / gameWidth * 100 + '%' );
				var height = selectors.textHelper[0].clientHeight / bodyFontScale;
				if ( height > settings.maxHeight ) {
					fontSize *= settings.maxHeight / height;
				}
				selectors.textHelper.css( 'width', '' );
			} else {
				if ( settings.maxWidth ) {
					selectors.textHelper[0].style.fontSize = pem( fontSize );
					var width = selectors.textHelper[0].clientWidth / bodyFontScale;
					if ( width > settings.maxWidth ) {
						fontSize *= settings.maxWidth / width;
					}
				}
			}

			setTextOnElem( selectors.textHelper, '' );
		} else if ( settings.maxCharacters ) {
			var maxCharacters = settings.maxCharacters || 10;
			var fontSizeDecreasePerExtraChar = settings.fontSizeDecreasePerExtraChar || 1;

			if ( text.length > maxCharacters ) {
				var extraCharacters = text.length - maxCharacters;
				fontSize -= extraCharacters * fontSizeDecreasePerExtraChar;
			}
		}

		if ( fontSize < 1 ) {
			libs.logError( 'calculateFontSize error: font now too small' );
			fontSize = 1;
		}

		if ( returnInPixels ) {
			return fontSize;
		} else {
			return pem( fontSize );
		}
	};

	/**
	 * Adds text to the element, resizing the text and adding gradient info if necessary
	 * @param {Object} element - jQuery element
	 * @param {string} text - Text to add to the element
	 * @param {Object} settings - Configuration on how the text should resize / position itself
	 * @param {boolean=} hasGradient - Does the text require the use of gradient tags
	 */
	var setText = function( element, text, settings, hasGradient ) {
		if ( text === undefined ) {
			text = "\xa0";
		}
		if ( settings ) {
			setFontSizeOnElem( element, libs.client.utils.DOM.calculateFontSize( text, settings ) );
			if ( settings.maxCharacters && BUILD_TYPE === 'dev' ) {
				libs.logError( 'libs.client.utils.DOM.setText incorrectly used with old settings object - selector: ' + element.selector + ' - text: ' + text );
			}
		} else if ( BUILD_TYPE === 'dev' ) {
			libs.logError( 'libs.client.utils.DOM.setText incorrectly used with no settings object - selector: ' + element.selector + ' - text: ' + text );
		}

		if ( hasGradient ) {
			if ( clientInfo.isWebkitBrowser() && !clientInfo.isNativeAndroidBrowser() ) {
				element.addClass( 'gradient-text' );
				element.attr( 'data-gradient-text', text );
			} else {
				element.addClass( 'gradient-text-normal-shadow' );
			}
		}

		setTextOnElem( element, text );
	};

	/**
	 * Wraps a tag around specific text
	 * @param {Object} parentElem - jquery element
	 * @param {Object} options - Configuration on how the text should resize / position itself
	 */

	var wrapTextInTag = function( element, options ) {
		if ( element && options ) {
			//options is an array
			if ( options instanceof Array ) {
				//loop through options
				for ( var i = 0; options[i]; i++ ) {
					var option = options[i];

					//check if is a valid option
					if ( (option.useDiv || option.useSpan) && option.replaceText ) {
						var text = element.text();
						var wrapWithFront, wrapWithBack;
						var className = option.className || '';
						var elemType = option.useDiv ? 'div' : 'span';

						//build elem
						wrapWithFront = '<' + elemType + ' class="' + className + '">';
						wrapWithBack = '</' + elemType + '>';

						//replace in text string
						text = text.replace( option.replaceText, wrapWithFront + option.replaceText + wrapWithBack );

						//updtae element with html
						element.html( text );
					}
				}
			}
		}
	};

	/**
	 * Adds text to the element, which could vary in the number of lines and therefore divs, respacing these as necessary
	 * @param {Object} parentElem - jquery element
	 * @param {string} text - Text to add to the element
	 * @param {Object} settings - Configuration on how the text should resize / position itself
	 * @param {boolean=} numSpacesToIgnore - Deprecated, to be removed after debugging
	 */
	var setMultiLineText = function( parentElem, text, settings, numSpacesToIgnore ) {
		if ( numSpacesToIgnore ) {
			libs.logError( 'libs.client.utils.DOM.setMultiLineText incorrectly used numSpacesToIgnore - selector: ' + parentElem.selector + ' - text: ' + text );
		}

		//get textSettings by merging defaultSettings with settings
		var textSettings = libs.client.utils.misc.mergeSettings( defaultSettings.text, settings );
		//get the words in an array
		var words = text.split( ' ' );
		//create selector prefix
		var prefix = (textSettings.prefix || parentElem.attr( 'id' )) + '-word-element-';
		//dont create a line break
		if ( textSettings.multiline.numWordsToGroup > 0 ) {
			//increment to match number of words
			++textSettings.multiline.numWordsToGroup;
			//get the words that make up the first line
			var firstLine = words.slice( 0, textSettings.multiline.numWordsToGroup ).join( ' ' );
			//add the new first line as the first entry in the words array
			words.splice( 0, textSettings.multiline.numWordsToGroup, firstLine );
		}

		if ( textSettings.multiline.maxLines && words.length > 1 ) {
			while ( words.length > textSettings.multiline.maxLines ) {
				var lastLine = words[words.length - 1];
				words[words.length - 2] = words[words.length - 2] + ' ' + lastLine;
				words.pop();
			}
		}

		if ( parentElem.data( 'lang-code' ) !== langCode || parentElem.data( 'num-lines' ) !== words.length ) {
			//check if button is hidden
			var shouldShowHide = parentElem.css( 'display' ) === 'none';
			//empty button
			parentElem.empty();

			//show button if hidden
			if ( shouldShowHide ) {
				parentElem.show();
			}
			var totalChildrenHeight = 0;
			//get parent height, this will be modified later
			var parentHeight = parentElem.height();
			//create a div that will be used for vertical spacing
			var spacerDiv = $( '<div>' ).addClass( 'vertical-space' );

			if ( textSettings.multiline.consistentFontSize ) {
				//find the longest word so font size matches each line
				for ( var i = 0; i < words.length; i++ ) {
					var fontSize = calculateFontSize( words[i], textSettings, true );
					if ( fontSize < textSettings.fontSize ) {
						textSettings.fontSize = fontSize;
					}
				}
			}

			//iterate through words
			for ( var j = 0, len = words.length; j < len; j++ ) {
				//create a div for every word
				var wordDiv = $( '<div>' ).attr( 'id', prefix + j );

				//add div to the button
				parentElem.append( wordDiv );

				//set the word as text - adding after appending to dom so can get proper css properties from it
				setText( wordDiv, words[j], textSettings );

				//track accumulative height
				var divHeight = wordDiv.height();
				totalChildrenHeight += divHeight;

				//apply space between lines, only for multiple lines
				if ( len > 1 && defaultSettings.text.multiline ) {
					if ( j ) {
						wordDiv.css( 'marginTop', libs.client.utils.misc.getValueAsPercent( -(libs.client.utils.misc.getPercentOfTotal( textSettings.multiline.reduceSpaceBetweenLinesBy, divHeight )), parentHeight, true ) );
					} else {
						wordDiv.css( 'marginTop', libs.client.utils.misc.getValueAsPercent( libs.client.utils.misc.getPercentOfTotal( textSettings.multiline.marginTop, divHeight ), parentHeight, true ) );
					}
				}
			}

			//set the spacing div height to half of the remaing height to horizontally align the words
			spacerDiv.css( {
				'height': libs.client.utils.misc.getValueAsPercent( (parentHeight - totalChildrenHeight) / 2, parentHeight, true ),
				'marginTop': ( textSettings.marginTop ? textSettings.marginTop : 0 ) + '%'
			} );

			//add spacing div to the begin of the button
			parentElem.prepend( spacerDiv );
			parentElem.addClass( textSettings.parentClass );
			parentElem.data( 'num-lines', words.length );
			parentElem.data('lang-code', langCode);

			//show button if it was hidden
			if ( shouldShowHide ) {
				parentElem.hide();
			}
		} else {
			//iterate through words
			for ( var x = 0; words[x]; x++ ) {
				setText( $( '#' + prefix + x ), words[x], textSettings );
			}
		}
	};

	var fadeIn = function( element, time, callback, hardwareAccelerated ) {
		if ( (clientInfo.getGraphicalQuality() === 'HQ' || hardwareAccelerated) && time > 0 ) {
			element
				.velocity( 'stop', true )
				.velocity( { 'opacity': 1 }, {
					'display': 'block',
					'duration': time,
					'complete': callback,
					'mobileHA': !!hardwareAccelerated
				} );
		} else {
			element.show();
			if ( callback ) {
				callback();
			}
		}
	};

	var fadeOut = function( element, time, callback, hardwareAccelerated ) {
		if ( (clientInfo.getGraphicalQuality() === 'HQ' || hardwareAccelerated) && time > 0 ) {
			element
				.velocity( 'stop', true )
				.velocity( { 'opacity': 0 }, {
					'display': 'none',
					'duration': time,
					'complete': callback,
					'mobileHA': !!hardwareAccelerated
				} );
		} else {
			element.hide();
			if ( callback ) {
				callback();
			}
		}
	};

	var showBackground = function( element, time, hardwareAccelerated ) {
		time = clientInfo.isIOS6() ? 0 : time;
		if ( typeof hardwareAccelerated === 'undefined' ) {
			hardwareAccelerated = true;
		}

		element
			.css( 'z-index', 0 )
			.velocity( 'stop', true )
			.velocity( { 'opacity': 1 }, { 'duration': time, 'mobileHA': !!hardwareAccelerated } );
	};

	var hideBackground = function( element, time, hardwareAccelerated ) {
		time = clientInfo.isIOS6() ? 0 : time;
		if ( typeof hardwareAccelerated === 'undefined' ) {
			hardwareAccelerated = true;
		}

		element
			.css( 'z-index', 0 )
			.velocity( 'stop', true )
			.velocity( { 'opacity': 0 }, { 'duration': time, 'mobileHA': !!hardwareAccelerated } )
			.css( 'z-index', -1 );
	};

	var enableCSSHardwareAcceleration = function( element, requireCentered ) {
		if ( isMobileDeviceType ) {
			element.addClass( requireCentered ? 'forceCSSHardwareAccelerationCentered' : 'forceCSSHardwareAcceleration' );
		}
	};

	var disableCSSHardwareAcceleration = function( element ) {
		if ( isMobileDeviceType ) {
			element.removeClass( 'forceCSSHardwareAcceleration' );
			element.removeClass( 'forceCSSHardwareAccelerationCentered' );
		}
	};

	libs.client.utils.DOM.forceDOMRedraw = forceDOMRedraw;
	libs.client.utils.DOM.calculateFontSize = calculateFontSize;
	libs.client.utils.DOM.getBaseFontSize = getBaseFontSize;
	libs.client.utils.DOM.pem = pem;
	libs.client.utils.DOM.setText = setText;
	libs.client.utils.DOM.wrapTextInTag = wrapTextInTag;
	libs.client.utils.DOM.setMultiLineText = setMultiLineText;
	libs.client.utils.DOM.fadeIn = fadeIn;
	libs.client.utils.DOM.fadeOut = fadeOut;
	libs.client.utils.DOM.showBackground = showBackground;
	libs.client.utils.DOM.hideBackground = hideBackground;
	libs.client.utils.DOM.enableCSSHardwareAcceleration = enableCSSHardwareAcceleration;
	libs.client.utils.DOM.disableCSSHardwareAcceleration = disableCSSHardwareAcceleration;

}());