goog.provide( 'libs.client.virgo.Menu' );

//goog.requireThirdParty('addToHome'); as only for mobile
goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.utils.misc' );
goog.require( 'libs.client.utils.DOM' );

goog.require( 'game.logic' );


( function() {
	"use strict";
	var externalLinks = null;
	libs.globalEventBus.connect( 'libs.event.notify.externalMenuLinks', function( links ) {
		externalLinks = links;
	} );

	//menu default settings
	var defaultSettings = {
		turboButton: {
			create: true
		},
		helpButton: {
			url: 'replace'
		},
		payTableButton: true,
		addToHomeScreenButton: true,
		lobbyUrl: 'https://mobile.williamhill.com/vegas',
		bankingUrl: 'https://mobilegaming.williamhill.com/session/account/withdraw-funds?clientId=mobileVegasSA&returnUrl=https://mobile.williamhill.com/vegas/close',
		depositUrl: 'https://mobilegaming.williamhill.com/session/account/deposit-funds?clientId=mobileVegasSA&returnUrl=https://mobile.williamhill.com/vegas/close',
		activeClass: 'active',
		menuActions: {
			'SOUND': 'SOUND',
			'TURBO': 'TURBO',
			'PAYTABLE': 'PAYTABLE',
			'HELP': 'HELP',
			'ADDTOHOMESCREEN': 'ADDTOHOMESCREEN',
			'LOBBY': 'LOBBY',
			'DEPOSIT': 'DEPOSIT',
			'BANKING': 'BANKING'
		},
		menuGraphics: {
			'SOUND': 'SOUND',
			'TURBO': 'TURBO',
			'PAYTABLE': 'PAYTABLE',
			'HELP': 'HELP',
			'ADDTOHOMESCREEN': 'ADDTOHOMESCREEN',
			'LOBBY': 'LOBBY',
			'DEPOSIT': 'DEPOSIT',
			'BANKING': 'BANKING'
		},
		topMenuItemClass: 'menu-button',
		menuItemClass: 'menu-item',
		menuItemIdPrefix: 'virgo-menu-item-',
		rowClassPrefix: 'rows-',
		noGraphicClass: 'nographic',
		numItemsPerRow: 6,
		menuAnimate: {
			top: '1%',
			duration: 1000
		}
	};

	var desktopSettigs = {
		helpUrl: libs.client.assetUrlPrefix + 'assets/data/Desktop/help_{{languageCode}}.html'
	};

	var mobileSettings = {
		helpUrl: libs.client.assetUrlPrefix + 'assets/data/Mobile/help_{{languageCode}}.html'
	};

	var clientInfo = libs.client.core.clientInformation;

	//platform prefix
	var platformPrefix = 'platform-virgo-';

	//menu DOM elements
	var selectors = {
		closeButton: $( '#' + platformPrefix + 'menu-button-close' ),
		container: $( '#' + platformPrefix + 'menu-container' ),
		helpContainer: $( '#' + platformPrefix + 'menu-help-container' ),
		topContainer: $( '#' + platformPrefix + 'top-menu-items-container' ),
		menuItemContainer: $( '#' + platformPrefix + 'menu-items-container' ),
		middleMenu: $( '#' + platformPrefix + 'middle-menu' )
	};

	var Menu = function( overrideSettings ) {
		var that = this;
		defaultSettings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings.isDesktop ? desktopSettigs : mobileSettings );
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		this._isMenuOpen = false; //track menu being open or closed
		this._soundEnabled = false; //track sound being enabled or disabled
		this._turboEnabled = false; //track turbo being enabled or disabled
		this._shownMenu = false;
		this._helpOpen = false;
		this._helpModal = null;
		this._handleDisplayMenu = null;
		this._topContainerMarginTop = null;
		var namespace = 'virgo.' + (this._settings.isDesktop ? 'desktop' : 'mobile');
		this._menuConfig = libs.getExternalArray( namespace + '.menuItems', [] );
		this._topMenuConfig = libs.getExternalArray( namespace + '.topMenuItems', [] );
		this._graphicFolder = libs.getExternalSetting( 'virgo.graphicFolder', [] );
		this._hideMenuHackTimeoutId = null;
		this._currentLanguage = null;

		this._setExternalMenuLinks( externalLinks );

		//load and setup menu
		this._loadMenu();
		$( '.menu-item' ).hide();
		$( '.menu-button' ).hide();
		selectors.container.hide();
		selectors.helpContainer.hide();

		//close menu when window is re-sized
		$( window ).resize( function() {
			that._onWindowResize();
		} );

		//events and activities
		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );
		libs.globalEventBus.connect( 'libs.event.request.openMenu', function() { that._onRequestOpenMenu(); } );
		libs.globalEventBus.connect( 'libs.event.notify.soundsState', function( enabled ) { that._onNotifySoundsState( enabled ); } );
		libs.globalEventBus.connect( 'libs.event.notify.orientationChanged', function( orientation ) { that._onNotifyOrientationChanged( orientation ); } );
		libs.globalEventBus.connect( 'libs.event.notify.turboModeState', function( enabled ) { that._onNotifyTurboModeState( enabled ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.error', function( activity, isFatal ) { that._onDisplayErrorStart( activity, isFatal ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.menu', function( activity ) { that._onDisplayMenuStart( activity ); } );
	};

	Menu.prototype._onNotifyLanguageChanged = function( langCode ) {
		var that = this;

		if ( this._helpModal && langCode !== this._currentLanguage ) {
			// give a little time for rest of game to sort out languages before loading new help text in background
			libs.setTimeout( function() {
				that._loadHelpText();
			}, 100 );
		}

		this._currentLanguage = langCode;
	};

	Menu.prototype._onDisplayMenuStart = function( activity ) {
		this._handleDisplayMenu = activity.join();
	};

	/**
	 * Get and set external links
	 * @method _onExternalMenuLinks
	 * @param {Object} links - externals links to be used by the menu
	 */

	Menu.prototype._setExternalMenuLinks = function( links ) {
		if ( links ) {
			if ( links['helpLink'] ) {
				this._settings.helpUrl = links['helpLink'];
			}
			if ( links['lobbyLink'] ) {
				this._settings.lobbyUrl = links['lobbyLink'];
			}
			if ( links['depositLink'] ) {
				this._settings.depositUrl = links['depositLink'];
			}
			if ( links['bankingLink'] ) {
				this._settings.bankingUrl = links['bankingLink'];
			}
		}
	};

	/**
	 * Setup and show menu items
	 * @method _loadMenu
	 */

	Menu.prototype._loadMenu = function() {
		//close button
		this._setupCloseButton();

		//top menu buttons
		this._createMenuItems( this._topMenuConfig, selectors.topContainer, this._settings.topMenuItemClass );

		//menu items
		this._createMenuItems( this._menuConfig, selectors.menuItemContainer, this._settings.menuItemClass );
	};

	/**
	 * Setup the close button
	 * @method _setupCloseButton
	 */

	Menu.prototype._setupCloseButton = function() {
		//add event for closing the menu
		var btnClose = selectors.closeButton;
		var that = this;
		btnClose.downEvent( function() {
			if ( !that._helpOpen ) {
				that._closeMenu( 0, 500 );
			}
		} );
	};

	Menu.prototype._onDisplayErrorStart = function( activity ) {
		this._closeMenu( 0, 0 );
	};

	/**
	 * Create configurable menu items
	 * @method _createMenuItems
	 */

	Menu.prototype._createMenuItems = function( config, container, itemPrefix ) {
		var numItems = 0;
		for ( var i = 0, len = config.length; i < len; i++ ) {
			var menuItemConfig = config[i];
			var menuAction = menuItemConfig['action'];
			var menuGraphic = menuItemConfig['graphic'];

			if ( menuItemConfig['showItem'] && menuAction && menuGraphic ) {
				//count how many items are created
				numItems++;

				var menuItem = this._createMenuItem( i, itemPrefix, menuAction );
				if ( menuItem ) {
					this._addMenuItemAction( menuItem, menuAction, menuItemConfig['newTab'] );
					this._addMenuGraphic( menuItem.elem, menuGraphic, itemPrefix );

					container.append( menuItem.elem );
				}
			}
		}

		container.addClass( this._graphicFolder );

		//handle more than one row
		var numRows = Math.ceil( numItems / this._settings.numItemsPerRow );
		selectors.middleMenu.addClass( this._settings.rowClassPrefix + numRows );
	};

	/**
	 * Create a menu item
	 * @method _createMenuItem
	 * @param {Number} index - item index
	 * @param {String} className - class name to add
	 * @param {String} action - action to take
	 */

	Menu.prototype._createMenuItem = function( index, className, action ) {
		var menuActions = this._settings.menuActions;
		var item = {};
		//create element
		if ( action === menuActions['ADDTOHOMESCREEN'] && !clientInfo.isIOS() ) {
			return null;
		}

		if ( action === menuActions['HELP'] ) {
			item.modal = this._createHelpAction( menuActions['HELP'], className );
		}

		var elem = $( '<div>' );
		elem.addClass( className );
		elem.attr( 'id', className + '-' + index );
		item.elem = elem;
		return item;
	};

	/**
	 * Add a graphic to the menu item
	 * @method _addMenuItemAction
	 * @param {jQuery} item - element to update
	 * @param {String} action - action to execute
	 * @param {Boolean} newTab - open link in new tab
	 */

	Menu.prototype._addMenuItemAction = function( item, action, newTab ) {
		var menuAction = this._settings.menuActions;
		var that = this;
		var elem = item.elem;
		switch ( action ) {
			case menuAction['SOUND'] :
				selectors.soundButton = elem;
				elem.downEvent( function() {
					if ( !that._helpOpen ) {
						if ( that._soundEnabled ) {
							libs.globalEventBus.invoke( 'libs.event.request.disableSounds' );
						} else {
							libs.globalEventBus.invoke( 'libs.event.request.enableSounds' );
						}
					}
				} );
				break;
			case menuAction['TURBO'] :
				selectors.turboButton = elem;
				elem.downEvent( function() {
					if ( !that._helpOpen ) {
						if ( that._turboEnabled ) {
							libs.globalEventBus.invoke( 'libs.event.request.disableTurboMode' );
						} else {
							libs.globalEventBus.invoke( 'libs.event.request.enableTurboMode' );
						}
					}
				} );
				break;
			case menuAction['PAYTABLE'] :
				elem.downEvent( function() {
					if ( !that._helpOpen ) {
						that._showPaytables();
					}
				} );
				break;
			case menuAction['HELP'] :
				elem.downEvent( function() {
					if ( !that._helpOpen ) {
						that._helpOpen = true;
						libs.setTimeoutTick( function() {
							libs.clearTimeoutTick( that._hideMenuHackTimeoutId );
							item.modal['parent'].show();
							item.modal['container'].show();
							item.modal['parent'].css( 'visibility', 'visible' );
							item.modal['container'].css( 'visibility', 'visible' );
							item.modal['inner'].scrollTop( 0 );
						}, that._settings.menuAnimate.duration );

						that._topContainerMarginTop = selectors.topContainer.css( 'marginTop' );
						selectors.topContainer.velocity( 'stop', true ).velocity( {
							'marginTop': that._settings.menuAnimate.top,
							'opacity': 0.2
						}, that._settings.menuAnimate.duration );
						$( '#platform-virgo-menu-items-container .menu-item' ).velocity( 'stop', true ).velocity( { 'opacity': 0.2 }, that._settings.menuAnimate.duration );
						selectors.closeButton.velocity( 'stop', true ).velocity( { 'opacity': 0 }, that._settings.menuAnimate.duration );

						libs.setTimeoutTick( function() {
							selectors.container.redrawHarsh();
						}, 1 );
					}
				} );

				item.modal['closeBtn'].downEvent( function() {
					that._closeHelpModal( item.modal, true );
				} );

				break;
			case menuAction['ADDTOHOMESCREEN'] :
				elem.downEvent( function() {
					if ( !that._helpOpen ) {
						that._addToHomeScreen();
					}
				} );
				break;
			case menuAction['LOBBY'] :
				elem.downEvent( function() {
					if ( !that._helpOpen ) {
						that._openExternalUrl( that._settings.lobbyUrl, newTab );
					}
				} );
				break;
			case menuAction['DEPOSIT'] :
				elem.downEvent( function() {
					if ( !that._helpOpen ) {
						that._openExternalUrl( that._settings.depositUrl, newTab );
					}
				} );
				break;
			case menuAction['BANKING'] :
				elem.downEvent( function() {
					if ( !that._helpOpen ) {
						that._openExternalUrl( that._settings.bankingUrl, newTab );
					}
				} );
				break;
			default :
				elem.downEvent( function() {
					if ( !that._helpOpen ) {
						that._openExternalUrl( action, newTab );
					}
				} );
				break;
		}
	};

	/**
	 * Close the help modal
	 * @method _closeHelpModal
	 * @param {Object} modal - modal items
	 */

	Menu.prototype._closeHelpModal = function( modal, animate ) {
		var that = this;

		// need to delay actually hiding the containers, as HTC One crashes if set off a massive scroll then hide the
		// parent div. thus, we set it to invisible

		modal['parent'].css( 'visibility', 'hidden' );
		modal['container'].css( 'visibility', 'hidden' );

		this._hideMenuHackTimeoutId = libs.setTimeoutTick( function() {
			modal['parent'].hide();
			modal['container'].hide();
		}, clientInfo.isNativeAndroidBrowser() ? 1000 : 0 );

		if ( animate ) {
			selectors.topContainer.velocity( 'stop', true ).velocity( {
				'marginTop': this._topContainerMarginTop,
				'opacity': 1
			}, this._settings.menuAnimate.duration );
			$( '.menu-item' ).velocity( 'stop', true ).velocity( { 'opacity': 1 }, this._settings.menuAnimate.duration );
			selectors.closeButton.velocity( { 'opacity': 1 }, this._settings.menuAnimate.duration );

			libs.setTimeoutTick( function() {
				selectors.container.redrawHarsh();
			}, 1 );

			libs.setTimeoutTick( function() {
				that._helpOpen = false;
			}, this._settings.menuAnimate.duration );
		} else {
			selectors.topContainer.velocity( 'stop', true ).removeAttr( 'style' );
			$( '#platform-virgo-menu-items-container .menu-item' ).velocity( 'stop', true ).removeAttr( 'style' );
			selectors.closeButton.velocity( 'stop', true ).removeAttr( 'style' );
			this._helpOpen = false;
		}


	};

	/**
	 * Add a graphic to the menu item
	 * @method _addMenuGraphic
	 * @param {jQuery} elem - element to update
	 * @param {String} graphic - graphic to display
	 * @param {String} prefix - class prefix
	 */

	Menu.prototype._addMenuGraphic = function( elem, graphic, prefix ) {
		var menuGraphics = this._settings.menuGraphics;
		var lowerCaseGraphic = graphic.toLowerCase();
		var classPrefix = prefix + '-';

		//check if a valid graphic
		if ( menuGraphics[graphic] ) {
			elem.addClass( classPrefix + lowerCaseGraphic );
		} else {
			elem.addClass( classPrefix + this._settings.noGraphicClass );
		}
	};

	/**
	 * Create the help component
	 * @method _createHelpAction
	 * @param {String} action - action to perform
	 * @param {String} className - class name to add
	 */

	Menu.prototype._createHelpAction = function( action, className ) {
		var helpModal = $( '<div>' );
		helpModal.addClass( className + '-' + action.toLowerCase() + '-modal' );
		helpModal.addClass( className + '-modal' );
		helpModal.addClass( 'enableTouchMove' );
		var helpInner = $( '<div>' );
		helpInner.addClass( className + '-modal-inner' );

		helpModal.append( helpInner );

		var closeBtn = $( '<div>' );
		closeBtn.addClass( 'menu-item-close' );
		helpModal.append( closeBtn );
		selectors.helpContainer.append( helpModal );

		this._helpModal = {
			'parent': selectors.helpContainer,
			'inner': helpInner,
			'container': helpModal,
			'closeBtn': closeBtn
		};

		this._loadHelpText();

		return this._helpModal;
	};

	/**
	 * Handle menu open request
	 * @method _onRequestOpenMenu
	 */

	Menu.prototype._onRequestOpenMenu = function() {
		if ( !this._shownMenu ) {
			$( '.menu-item' ).show();
			$( '.menu-button' ).show();
			this._shownMenu = true;
		}

		if ( !this._isMenuOpen ) {
			//open menu if not opened
			this._isMenuOpen = true;
			libs.globalEventBus.invokeActivity( 'libs.activity.display.menu' );
			this._showMenu( 500 );
		}
	};

	/**
	 * Close the menu
	 * @method _closeMenu
	 * @param {Number} delay - close menu after specific delay
	 * @param {Number} duration - duration of animation
	 */

	Menu.prototype._closeMenu = function( delay, duration ) {
		var that = this;

		//close if menu is opened
		if ( this._isMenuOpen ) {
			this._isMenuOpen = false;
			this._helpOpen = false;

			if ( this._helpModal ) {
				this._closeHelpModal( this._helpModal );
			}

			if ( delay ) {
				//hide menu after a delay
				libs.setTimeoutTick( function() {
					that._hideMenu( duration );
				}, delay );
			} else {
				//hide menu immediately
				that._hideMenu( duration );
			}

			this._handleDisplayMenu.leave();
			this._handleDisplayMenu = null;
		}
	};

	/**
	 * Handle sound state update
	 * @method _onNotifySoundsState
	 * @param {Boolean} enabled - whether sound is enabled or disabled
	 */

	Menu.prototype._onNotifySoundsState = function( enabled ) {
		var btnSound = selectors.soundButton;
		if ( btnSound ) {
			this._soundEnabled = enabled;
			this._updateToggleButton( btnSound, this._soundEnabled );
		}
	};

	/**
	 * Handle turbo state update
	 * @method _onNotifyTurboModeState
	 * @param {Boolean} enabled - whether turbo is enabled or disabled
	 */

	Menu.prototype._onNotifyTurboModeState = function( enabled ) {
		var btnTurbo = selectors.turboButton;
		if ( btnTurbo ) {
			this._turboEnabled = enabled;
			this._updateToggleButton( btnTurbo, this._turboEnabled );
		}
	};

	/**
	 * Handle orientation change
	 * @method _onNotifyOrientationChanged
	 * @param {String} orientation - device orientation
	 */

	Menu.prototype._onNotifyOrientationChanged = function( orientation ) {
		//if menu is opened and orientation is portrait close menu
		if ( this._isMenuOpen && orientation === 'portrait' ) {
			this._closeMenu( 0, 0 );
		}
	};

	/**
	 * Handle window resize
	 * @method _onWindowResize
	 */

	Menu.prototype._onWindowResize = function() {
		//if menu is open and widow is re sized, close menu
		if ( this._isMenuOpen ) {
			this._closeMenu( 0, 0 );
		}
	};

	/**
	 * Show pay tables
	 * @method _showPaytables
	 */

	Menu.prototype._showPaytables = function() {
		//if menu is opened, close menu and show pay tables
		if ( this._isMenuOpen ) {
			this._closeMenu( 0, 0 );
			libs.globalEventBus.invoke( 'libs.event.request.openPaytable' );
		}
	};

	/**
	 * Add game to home screen
	 * @method _addToHomeScreen
	 */

	Menu.prototype._addToHomeScreen = function() {
		if ( this._isMenuOpen ) {
			addToHome.show( true );
		}
	};

	/**
	 * Open external urls
	 * @method _openExternalUrl
	 * @param {String} url - url to open
	 * @param {Boolean} newWindow - open in new window or the same wiindow
	 */

	Menu.prototype._openExternalUrl = function( url, newWindow ) {
		if ( this._isMenuOpen ) {
			this._closeMenu( 0, 0 );
			libs.client.utils.misc.openUrl( url, newWindow );
		}
	};
	/**
	 * Update toggle button
	 * @method _updateToggleButton
	 * @param {Object} elem - element to be updated
	 * @param {Boolean} enabled - button state
	 */

	Menu.prototype._updateToggleButton = function( elem, enabled ) {
		var activeClass = this._settings.activeClass;
		if ( enabled ) {
			elem.addClass( activeClass );
		} else {
			elem.removeClass( activeClass );
		}
	};

	/* Helper functions */

	/**
	 * Show the menu
	 * @method showMenu
	 * @param {Number} duration - duration of the animation
	 */

	Menu.prototype._showMenu = function( duration ) {
		libs.globalEventBus.invoke( 'libs.event.request.showBackdrop', 'body', {
			backdropClass: 'backdrop-menu',
			zIndex: 538,
			animationTime: duration
		} );
		libs.client.utils.DOM.fadeIn( selectors.container, duration || 0 );
	};

	/**
	 * Hide the menu
	 * @method hideMenu
	 * @param {Number} duration - duration of the animation
	 */

	Menu.prototype._hideMenu = function( duration ) {
		$( '.menu-item-modal' ).hide();
		libs.globalEventBus.invoke( 'libs.event.request.hideBackdrop', 'body', 'backdrop-menu' );
		libs.client.utils.DOM.fadeOut( selectors.container, duration || 0 );
	};

	Menu.prototype._loadHelpText = function() {
		var that = this;

		var helpUrl = libs.client.utils.misc.replaceLanguageCodeInText( that._settings.helpUrl );
		$.ajax( { url: helpUrl, type: 'GET' } ).done( function( data ) {
			data = libs.client.utils.misc.replaceAssetPathsInText( data );
			that._helpModal['inner'].html( data );

			//fix scroll links within the modal
			$( '.scrollLink' ).downEvent( function( e ) {
				var href = $( e.target ).attr( 'href' );
				if ( href ) {
					that._helpModal['inner'].scrollTop( that._helpModal['inner'].scrollTop() + $( href ).position().top );
				}
			} );

		} ).fail( function() {
			if ( BUILD_TYPE === 'dev' ) {
				libs.logError( 'Virgo menu: failed to load help file for language ' + that._currentLanguage );
			}
		} );
	};

	//expose Menu
	libs.client.virgo.Menu = Menu;

}());