/*global VIRGO,GAME*/

goog.provide( 'libs.client.virgo.createLiveServer' );

goog.require( 'libs.getDefaultFormatForCurrencyCode' );
goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );

goog.require( 'game.logic' );


/******************************************************************************
 *
 * THE "GAME" OBJECT
 * =================
 *
 * Virgo expects that an object named "GAME" be placed in the global namespace.
 * That object must have the following visible properties:
 *
 * - UID
 *
 * This is a string with the game's unique identifier.  The string exposed here
 * must exactly match the UID expressed in the server-side code (the Java).
 *
 * - interfaceVersion
 *
 * This is a string containing the version number of Virgo Remote API the game
 * has been coded for.  The string exposed here must exactly match the version
 * expressed in the server-side code (the Java).
 *
 * - postHandshake
 *
 * This is a function, which will be invoked by Virgo and passed a single
 * argument (the "handshake result").  Virgo guarantees that postHandshake
 * will not be invoked until the preHandshake function has finished executing.
 * The single argument is an object which provides the Player's persisted state,
 * balance, localization, preferences, etc..
 *
 * After the postHandshake function has returned, the game can invoke Virgo API
 * commands in response to Player actions.  Also, the game must be ready to
 * receive notifications (e.g. balance updates).
 *
 * - onPause
 *
 * This is a function, which will be invoked by Virgo and passed no arguments.
 * When this is called, you should immediately halt all animations and audio
 * playback.  All user input should be ignored.
 *
 * - onResume
 *
 * This is a function, which will be invoked by Virgo and passed no arguments.
 * This is called to undo the effect of the "pause" function.  All activity
 * should resume, as if no interruption had occurred.
 *
 * - onBalanceUpdate
 *
 * This is a function, which will be invoked by Virgo and passed a single
 * argument.  The argument is an object containing a new balance from Virgo.
 * Virgo guarantees that this will NOT be invoked if the game is waiting for
 * a response to a Virgo API command invocation that will mutate the balance
 * (e.g. StartGame).
 *
 *****************************************************************************/

this['GAME'] = null;

(function() {
	"use strict";

	var progressiveValuesRefreshInterval = 5000;


	var LiveServer = function() {

		if ( typeof game.logic.GameData.virgo.gameUID !== 'string' || game.logic.GameData.virgo.gameUID.length <= 0 ) {
			throw new Error( "Virgo games require game.logic.GameData.virgo.gameUID to be defined" );
		}

		var that = this;
		this._onInitCallback = null;
		this._startingReelSet = 0;
		this._languageCode = 'en_GB';
		this._currencyCode = 'GBP';
		this._currencyFormatting = libs.getDefaultFormatForCurrencyCode( this._currencyCode ); // Format to GBP until we receive handshake response

		this.jackpots = [];
		var jpCount = ( game.logic.GameData.jackpots && game.logic.GameData.jackpots.list ) ? game.logic.GameData.jackpots.list.length : 0;
		for ( var i = 0; i < jpCount; ++i ) {
			this.jackpots.push( new BigNumber( 0 ) );
		}

		this._betRecoveryResult = null;
		this._handshakeComplete = false;
		this._inBetSequence = false;
		this._initPersistentData = null;
		this.balance = new BigNumber( 0 );
		this._balanceLastUpdatedTimestamp = new BigNumber( 0 );

		this.startingStake = new BigNumber( 0 );
		this.availableStakesPerLine = [];
		libs.globalEventBus.connect( 'libs.event.notify.betRecoveryStarted', function() { that._onNotifyBetRecoveryStarted(); } );
	};


	LiveServer.prototype.gameInit = function() {

	};


	// Functions expected by Virgo

	LiveServer.prototype['UID'] = game.logic.GameData.virgo.gameUID;
	LiveServer.prototype['interfaceVersion'] = "0.1";

	LiveServer.prototype['onPause'] = function() {};
	LiveServer.prototype['onResume'] = function() {};

	LiveServer.prototype['postHandshake'] = function( handshakeResult ) {

		try {
			this.progressiveValueUpdate( handshakeResult );
			this['onBalanceUpdate']( handshakeResult, true, true );

			if ( handshakeResult['Localization'] && handshakeResult['Localization']['LocaleString'] ) {
				this._languageCode = handshakeResult['Localization']['LocaleString'];
			}

			// This is where we receive a state if we are running from bet recovery.
			if ( handshakeResult['State'] ) {
				var rawStateJson = handshakeResult['State']['value'];
				var stateJson = libs.xmlUtils.jsonParse( rawStateJson );
				if ( stateJson['serverState'] && stateJson['serverState']['player'] ) {
					// check if persistent data has been stored
					this._initPersistentData = stateJson['serverState']['player'];
				}
				if ( stateJson && stateJson['lastGame'] ) {
					// Bet recovery state
					this._betRecoveryResult = this._constructBetResult( stateJson['lastGame'] );
					// restore jackpot values
					if ( stateJson['jackpotState'] && stateJson['jackpotState']['jackpotList'] ) {
						var jpList = stateJson['jackpotState']['jackpotList'];
						for ( var jp = 0; jp < jpList.length; ++jp ) {
							this.jackpots[jp] = new BigNumber( jpList[jp].value );
						}
					}
					libs.log( "Starting Virgo bet recovery using object " + JSON.stringify( this._betRecoveryResult ) );
					this._inBetSequence = true;
				}
			} else {
				this._betRecoveryResult = null;
			}

			// To get around that Virgo has no 'init' message or equivalent, we call the 'init' method to get reel band data 
			// manually in our local copy of the game logic.
			var initObject = game.logic.BuildInitResponse( this._currencyCode, { 'serverState': { 'player': this._initPersistentData } } );
			this._reelSets = initObject.reel_set;
			this._startingReelSet = initObject.startingReelSet || 0;
			this.availableStakesPerLine = initObject.stake.steps.map( function( stake ) { return new BigNumber( stake ); } );
			this.startingStake = new BigNumber( initObject.stake.starting );
			this._setExternalMenuLinks();
		}
		catch ( err ) {
			libs.error( err );
		}

		this._handshakeComplete = true;
	};


	LiveServer.prototype['onBalanceUpdate'] = function( params, updateCurrencyFormat, invokeServerPolled ) {

		try {
			if ( params && params["Balance"] && params["Balance"]["Banks"] && params["Balance"]["Banks"]["CREDIT"] ) {

				if ( updateCurrencyFormat ) {
					var newCurrencyCode = params["Balance"]["Banks"]["CREDIT"]['CurrencyCode'];
					if ( this._currencyCode !== newCurrencyCode ) {
						this._currencyCode = newCurrencyCode;
						this._currencyFormatting = params["Balance"]["Banks"]["CREDIT"]['CurrencyFormat'] || libs.getDefaultFormatForCurrencyCode( this._currencyCode );
						libs.globalEventBus.invoke( 'libs.event.request.changeCurrency', this._currencyCode, this._currencyFormatting );
					}
				}

				// only update balance if we have been sent a message with a more recent timestamp than the last update we had
				var messageTimestamp = new BigNumber( params["Balance"]["Timestamp"] );
				if ( this._balanceLastUpdatedTimestamp.lessThan( messageTimestamp ) ) {
					this._balanceLastUpdatedTimestamp = messageTimestamp;
					this.balance = new BigNumber( params["Balance"]["Banks"]["CREDIT"]["Amount"] );

					if ( invokeServerPolled ) {
						libs.globalEventBus.invoke( 'libs.event.notify.serverPolled' );
					}
				}
			}
		}
		catch ( err ) {
			console.error( err );
		}
	};

	/* PW: This comment was in the example game, but i dont think this is true looking at the client side API:
	 *
	 * This function is invoked by Virgo whenever an ad hoc balance update is
	 * received.  Because this example game has simple logic, we will also
	 * invoke this function when we receive the handshake response, or any
	 * response to a call such as startGame (those also give new balances).
	 */
	LiveServer.prototype.progressiveValueUpdate = function( state ) {

		try {
			if ( state['State'] && state['State']['value'] ) {
				var rawStateJson = state['State']['value'];
				var stateJson = libs.xmlUtils.jsonParse( rawStateJson );
				if ( stateJson['jackpotState'] ) {
					var prgState = stateJson['jackpotState'];
					if ( prgState !== undefined && prgState['jackpotList'] ) {
						for ( var i = 0; i < prgState['jackpotList'].length; ++i ) {
							this.jackpots[i] = new BigNumber( prgState['jackpotList'][i].value );
						}
					}
				}
			}
			libs.globalEventBus.invoke( 'libs.event.notify.serverPolled' );
		}
		catch ( err ) {
			console.error( err );
		}
	};

	// Functions expected by the rest of the game


	LiveServer.prototype._onNotifyBetRecoveryStarted = function() {
		this._inBetSequence = true;
	};


	LiveServer.prototype.onInit = function( callback ) {
		this._onInitCallback = callback;
	};


	LiveServer.prototype.connect = function() {

		this._callInitOnceHandshakeIsComplete();
	};


	LiveServer.prototype._callInitOnceHandshakeIsComplete = function() {
		var that = this;
		libs.setTimeout( function() {
			if ( that._handshakeComplete ) {
				that._refreshProgressiveValues(); // start progressive refresh command
				if ( that._onInitCallback ) {
					libs.globalEventBus.invoke( 'libs.event.request.changeLanguage', that._languageCode );
					libs.globalEventBus.invoke( 'libs.event.request.changeCurrency', that._currencyCode, that._currencyFormatting );
					that._onInitCallback( that._betRecoveryResult, that._initPersistentData );
				}
			} else {
				that._callInitOnceHandshakeIsComplete();
			}
		}, 100 );
	};


	LiveServer.prototype._refreshProgressiveValues = function() {

		var that = this;

		if ( !this._inBetSequence ) {
			VIRGO['api']['refreshProgressive']( {},
				function( result ) {
					that['onBalanceUpdate']( result, false, false );
					that.progressiveValueUpdate( result );
					libs.setTimeout( function() { that._refreshProgressiveValues(); }, progressiveValuesRefreshInterval );
				},
				function( error ) {
					console.error( error );
					libs.setTimeout( function() { that._refreshProgressiveValues(); }, progressiveValuesRefreshInterval );
				} );
		} else {
			libs.setTimeout( function() { that._refreshProgressiveValues(); }, progressiveValuesRefreshInterval );
		}
	};


	LiveServer.prototype.getReels = function() {
		return this._reelSets;
	};


	LiveServer.prototype.getStartingReelSet = function() {
		return this._startingReelSet;
	};


	LiveServer.prototype.startBet = function( stake, lines, postData ) {

		// show balance being artificially decremented.
		// This gives the illusion of the stake being taken instantly.
		// This will be updated on the next balanceUpdate call
		this.balance = this.balance.minus( stake );

		var that = this;

		var callStartGame = !this._inBetSequence;
		this._inBetSequence = true;
		if ( callStartGame ) {
			VIRGO['api']['startGame'](
				stake.times( lines ).toN(),
				this._currencyCode,
				/*
				 * An object describing the request.
				 * In this example game, there's only one thing you can request: a spin.
				 * A more complex game might say "$2 on red", or similar.
				 */
				/*{"Command" : "SPIN"}*/
				{ 'postData': libs.xmlUtils.jsonStringify( postData ), 'lines': lines },
				function( result ) { that._onServerResult( result ); },
				function( error ) { that._onServerError( error ); }
			);
		} else {
			VIRGO['api']['checkpoint'](
				{ 'postData': libs.xmlUtils.jsonStringify( postData ) },
				function( result ) { that._onServerResult( result ); },
				function( error ) { that._onServerError( error ); }
			);
		}
	};


	LiveServer.prototype._constructBetResult = function( betResult ) {

		betResult['win'] = new BigNumber( betResult['win'] ); // Make sure 'win' element is a bignumber instance
		betResult.inBetSequence = betResult['expectCheckpoint'];
		betResult.reelDelays = [];

		return betResult;
	};


	LiveServer.prototype._onServerResult = function( virgoServerResult ) {

		try {
			this.progressiveValueUpdate( virgoServerResult );

			// // Remove the artificial decrement from the balance.
			// GAME.onBalanceUpdate(result);
			// GAME.progressiveValueUpdate(result);
			// stateMachine.newState(constants.state.SPINNING);
			// spin(result);
			var that = this;
			this['onBalanceUpdate']( virgoServerResult, false, true );

			// Convert from virgo object to betResult object the game is expecting
			var rawStateJson = virgoServerResult['State']['value'];
			var stateJson = libs.xmlUtils.jsonParse( rawStateJson );
			var serverBetResults = stateJson['lastGame'];

			var betResult = this._constructBetResult( serverBetResults );
			libs.globalEventBus.invoke( 'libs.event.notify.serverBetResult', betResult );
		}
		catch ( err ) {
			libs.globalEventBus.invoke( 'libs.event.request.showError', err.toString(), false );
		}
	};


	LiveServer.prototype._onServerError = function( virgoServerError ) {
		// // TODO what can go wrong here?
		// renderBalance(trueBalance);
		// VIRGO.api.error("GAME ERROR", "Error returned invoking \"startGame\".");
		// stateMachine.newState(constants.state.IDLE);
		this._inBetSequence = false;
		libs.globalEventBus.invoke( 'libs.event.notify.serverBetAborted' );
	};


	LiveServer.prototype._onBetFinishedComplete = function( gameCycleEnd, result ) {

		this['onBalanceUpdate']( result, false, true );
		this._inBetSequence = !gameCycleEnd;
		if ( result ) {
			this.progressiveValueUpdate( result );
		}
		libs.globalEventBus.invoke( 'libs.event.notify.serverBetFinished' );
	};


	LiveServer.prototype.betFinished = function( inFreeSpin ) {

		var that = this;

		if ( !inFreeSpin ) {
			VIRGO['api']['endGame'](
				/*{"Command": "SPIN_COMPLETE"}*/ {},
				function( result ) { that._onBetFinishedComplete( true, result ); },
				function( error ) { libs.globalEventBus.invoke( 'libs.event.request.showError', error.toString(), false ); }
			);
		} else {
			this._onBetFinishedComplete( false, null );
		}
	};


	LiveServer.prototype.loadHistory = function() {

	};

	LiveServer.prototype._setExternalMenuLinks = function() {
		var externalLinks = {};

		if ( VIRGO['getOperatorUrl'] ) {
			var helpLink = VIRGO['getOperatorUrl']( 'help' );
			var lobbyLink = VIRGO['getOperatorUrl']( 'home' );
			var depositLink = VIRGO['getOperatorUrl']( 'deposit' );
			var bankingLink = VIRGO['getOperatorUrl']( 'withdraw' );

			externalLinks = {
				'helpLink': helpLink,
				'lobbyLink': lobbyLink,
				'depositLink': depositLink,
				'bankingLink': bankingLink
			};
		}

		libs.globalEventBus.invoke( 'libs.event.notify.externalMenuLinks', externalLinks );
	};

	GAME = new LiveServer();

	libs.client.virgo.createLiveServer = function() {
		GAME.gameInit();
		return GAME;
	};

}());

