goog.require( 'libs' );
goog.provide( 'libs.coreUtils' );

goog.provide( 'libs.log' );
goog.provide( 'libs.logError' );
goog.provide( 'libs.logException' );
goog.provide( 'libs.onLogException' );

goog.provide( 'libs.getDateString' );
goog.provide( 'libs.getTimeString' );

goog.provide( 'libs.callback' );
goog.provide( 'libs.setTimeout' );
goog.provide( 'libs.clearTimeout' );
goog.provide( 'libs.setInterval' );
goog.provide( 'libs.clearInterval' );
goog.provide( 'libs.setImmediate' );
goog.provide( 'libs.clearImmediate' );


(function() {
	"use strict";

	var canConsoleLog = !(typeof console === "undefined" || typeof console.log === "undefined" );
	var canConsoleError = !(typeof console === "undefined" || typeof console.error === "undefined" );
	var canAlert = !(typeof window === "undefined" || typeof window.alert === "undefined" );

	var logCallbackList = [];
	var log = function( message, level ) {
		if ( canConsoleLog ) {
			if ( level ) {
				console.log( libs.getTimeString( true ) + ' [' + level + '] - ' + message );
			} else {
				console.log( libs.getTimeString( true ) + ' - ' + message );
			}
		}
		logCallbackList.forEach( function( callback ) {
			if ( level ) {
				callback( '[' + level + '] - ' + message );
			} else {
				callback( message );
			}
		} );
	};

	var onLog = function( func ) {
		logCallbackList.push( func );
	};

	var logError = function( message, level ) {
		if ( canConsoleError ) {
			console.error( libs.getTimeString( true ) + ' - ' + message );
			logCallbackList.forEach( function( callback ) {
				callback( message );
			} );
		} else {
			libs.log( message, level );
		}

		if ( canAlert && BUILD_TYPE === 'dev' ) {
//			debugger;
		}
	};

	var logWarn = function( message, level ) {
		if ( canConsoleError ) {
			console.warn( libs.getTimeString( true ) + ' - ' + message );
		} else {
			libs.log( message, level );
		}
	};

	var logSettings = function( settings ) {
		if ( BUILD_TYPE !== 'release' ) {
			libs.log( JSON.stringify( settings, null, "\t" ) );
		}
	};

	var logExceptionCallbackList = [];
	var logException = function( e ) {

		var msg = '';
		if ( e.name ) {
			msg += e.name + ":";
		}
		if ( e.message ) {
			msg += e.message;
		} else {
			msg += e.toString();
		}

		if ( e.fileName ) {
			msg += " at " + e.fileName + ":" + e.lineNumber;
		}
		libs.logError( "Exception caught: " + msg );
		if ( e.stack ) {
			// Extract Firefox stack information.
			libs.log( "Stacktrace: " + e.stack );
		}

		logExceptionCallbackList.forEach( function( callback ) {
			callback( msg, e );
		} );
	};

	var onLogException = function( func ) {
		logExceptionCallbackList.push( func );
	};

	var getTimeString = function( useSeperationCharacters ) {
		var time = "";
		var now = new Date();
		var val = now.getUTCHours();
		if ( val < 10 ) { time += "0"; }
		time += val;
		if ( useSeperationCharacters ) {
			time += ":";
		}
		val = now.getUTCMinutes();
		if ( val < 10 ) { time += "0"; }
		time += val;
		if ( useSeperationCharacters ) {
			time += ":";
		}
		val = now.getUTCSeconds();
		if ( val < 10 ) { time += "0"; }
		time += val;
		if ( useSeperationCharacters ) {
			time += ":";
		}
		val = now.getUTCMilliseconds();
		if ( val < 10 ) { time += "0"; }
		if ( val < 100 ) { time += "0"; }
		time += val;
		return time;
	};

	var getDateString = function( useSeperationCharacters ) {
		var date = "";
		var now = new Date();
		date += now.getUTCFullYear();
		if ( useSeperationCharacters ) {
			date += "-";
		}
		var val = now.getUTCMonth() + 1;
		if ( val < 10 ) { date += "0"; }
		date += val;
		if ( useSeperationCharacters ) {
			date += "-";
		}
		val = now.getUTCDate();
		if ( val < 10 ) { date += "0"; }
		date += val;
		if ( useSeperationCharacters ) {
			date += " ";
		}
		val = now.getUTCHours();
		if ( val < 10 ) { date += "0"; }
		date += val;
		if ( useSeperationCharacters ) {
			date += ":";
		}
		val = now.getUTCMinutes();
		if ( val < 10 ) { date += "0"; }
		date += val;
		if ( useSeperationCharacters ) {
			date += ":";
		}
		val = now.getUTCSeconds();
		if ( val < 10 ) { date += "0"; }
		date += val;
		return date;
	};

	var aArgs = [];
	var callback = function( func, closure ) {
		aArgs.length = 0;
		if ( arguments.length > 2 ) {
			aArgs = Array.prototype.slice.call( arguments, 2 );
		}
		var decorated = function() {
			try {
				if ( func ) {
					return func.apply( closure || this, Array.prototype.slice.call( arguments, 0 ).concat( aArgs ) );
				}
			} catch ( exception ) {
				libs.logException( exception );
				throw exception;
			}
		};
		return decorated;
	};

	var libsSetTimeout = function( func, time ) {
		return setTimeout( libs.callback( func ), time );
	};

	var libsClearTimeout = function( timeoutId ) {
		clearTimeout( timeoutId );
		return null;
	};

	var libsSetInterval = function( func, time ) {
		return setInterval( libs.callback( func ), time );
	};

	var libsClearInterval = function( intervalId ) {
		clearInterval( intervalId );
		return null;
	};

	var libsSetImmediate = function( func ) {
		return window['setImmediate']( libs.callback( func ) );
	};

	var libsClearImmediate = function( immediateId ) {
		window['clearImmediate']( immediateId );
		return null;
	};

	var libsSetIntervalCount = function( func, intervalTime, totalIterations ) {
		intervalTime = intervalTime || 0;
		totalIterations = totalIterations || 0;
		var iterationCount = 0;

		var intervalId = libs.setInterval( function() {
			iterationCount++;
			if ( iterationCount < totalIterations ) {
				func( iterationCount );
			} else {
				libs.clearInterval( intervalId );
			}
		}, intervalTime );

		return intervalId;
	};

	var libsClearIntervalCount = function( intervalCountId ) {
		libs.clearInterval( intervalCountId );
		return null;
	};

	var validateOptionsObject = function( name, defaults, options ) {
		// This method ensures that the options object only contains keys that exist in the defaults object.
		// This should catch errors introduced by the closure compiler when element names are mangled and string literals are not.
		var defaultKeys = Object.keys( defaults );
		var optionKeys = Object.keys( options );

		var ok = optionKeys.every( function( okey ) {
			return defaultKeys.indexOf( okey ) >= 0;
		} );

		if ( !ok ) {
			throw new Error( "validateOptionsObject failed on '" + name + "'" );
		}
	};

	/**
	 * Format string using {0} format
	 * @param key - String
	 * @param args - Array
	 * @returns String
	 */
	var formatString = function( key, args ) {
		var s = key;

		args = args instanceof Array ? args : [args];

		for ( var i = 0, len = args.length; i < len; i++ ) {
			s = s.replace( new RegExp( "\\{" + i + "\\}", "gm" ), args[i] );
		}
		return s;
	};


	libs.log = log;
	libs.logError = logError;
	libs.logWarn = logWarn;
	libs.logException = logException;
	libs.logSettings = logSettings;
	libs.onLog = onLog;
	libs.onLogException = onLogException;

	libs.getDateString = getDateString;
	libs.getTimeString = getTimeString;

	libs.callback = callback;

	libs.setTimeout = libsSetTimeout;
	libs.clearTimeout = libsClearTimeout;
	libs.setInterval = libsSetInterval;
	libs.clearInterval = libsClearInterval;
	libs.setImmediate = libsSetImmediate;
	libs.clearImmediate = libsClearImmediate;
	libs.setIntervalCount = libsSetIntervalCount;
	libs.clearIntervalCount = libsClearIntervalCount;

	libs.validateOptionsObject = validateOptionsObject;
	libs.formatString = formatString;

}());