goog.provide( 'libs.EventBus' );
goog.provide( 'libs.globalEventBus' );

goog.require( 'libs.globalEventBusNames' );


(function() {
	"use strict";

	var replaceBigNumber = function( key, value ) {
		if ( value instanceof BigNumber ) {
			return value.toString();
		}
		return value;
	};

	var enableLogging = function( scope, name ) {
		return !libs.eventsDontLog.some( function( rhs ) { return name === rhs; } ) && scope === libs.globalEventBus && BUILD_TYPE !== 'release';
	};

	var ActivityHandle = function( activity ) {
		this._activity = activity;
	};

	ActivityHandle.prototype.leave = function() {
		if ( this._activity ) {
			this._activity._onHandleLeave();
			this._activity = null;
		}

		return null;
	};

	ActivityHandle.prototype.isJoined = function() {
		return this._activity !== null;
	};


	var Activity = function( activityName, args, eventBus ) {
		this._name = activityName;
		this._arguments = Array.prototype.slice.call( args, 0 );
		this._eventBus = eventBus;
		this._handleCount = 0;
		this._ended = false;
		this._inJoinPhase = true;
	};

	Activity.prototype._onHandleLeave = function() {
		this._handleCount--;
		this._checkIfFinished();
	};

	Activity.prototype._checkIfFinished = function() {
		// If no one joined the activity, this'll end it
		if ( !this._inJoinPhase && !this._ended && this._handleCount <= 0 ) {
			this._ended = true;
			this._handleCount = 0;
			this._eventBus._invokeActivityEnd( this._name, this._arguments );
		}
	};

	Activity.prototype._joinPhaseComplete = function() {
		this._inJoinPhase = false;
		this._checkIfFinished();
	};

	Activity.prototype.join = function() {
		if ( this._ended ) {
			throw new Error( "Logic error: join() called on activity " + this._name + " after it has completed!" );
		}
		this._handleCount++;
		return new ActivityHandle( this );
	};

	Activity.prototype.isActive = function() {
		return !this._ended;
	};


	var SignalArray = function( eventNames, regex ) {
		this._eventNames = this._flattenObject( eventNames );
		this._signals = {};
		this._verifyValidEventNames( this._eventNames, regex );
	};

	SignalArray.prototype._verifyValidEventNames = function( names, regex ) {
		if ( regex ) {
			var isValid = names.every( function( name ) {
				return name.match( regex );
			} );
			if ( !isValid ) {
				throw new Error( "Invalid event name passed to EventBus() object: All event must match " + regex.toString() +
				' - event passed: ' + names );
			}
		}
	};

	SignalArray.prototype.registerEventNames = function( eventNames, regex ) {

		var newEventNames = this._flattenObject( eventNames );
		this._verifyValidEventNames( newEventNames, regex );
		this._eventNames = this._eventNames.concat( newEventNames );
	};

	SignalArray.prototype._flattenObject = function( obj ) {
		if ( Array.isArray( obj ) ) {
			return obj.slice( 0 );
		} else {
			// converts object of arrays into 1 array
			var ret = [];
			for ( var prop in obj ) {
				if ( obj.hasOwnProperty( prop ) && Array.isArray( obj[prop] ) ) {
					ret = ret.concat( obj[prop] );
				}
			}
			return ret;
		}
	};

	SignalArray.prototype.containsEvent = function( eventName ) {
		if ( !this._eventNames ) {
			return false;
		}
		return this._eventNames.some( function( ev ) {
			return ev === eventName;
		} );
	};

	SignalArray.prototype._verifyEventName = function( eventName ) {
		if ( !this.containsEvent( eventName ) ) {
			throw new Error( 'EventBus: Event name "' + eventName + '" does not exist!' );
		}
	};

	SignalArray.prototype.connect = function( eventName, callback ) {
		this._verifyEventName( eventName );
		if ( !this._signals[eventName] ) {
			this._signals[eventName] = [];
		}
		this._signals[eventName].push( callback );
	};


	SignalArray.prototype.invoke = function( eventName ) {
		var eventArgs = [];
		this._verifyEventName( eventName );
		var eventArray = this._signals[eventName];
		if ( eventArray ) {
			eventArgs.length = 0;
			if ( arguments.length > 1 ) {
				eventArgs = Array.prototype.slice.call( arguments, 1 );
			}
			eventArray.forEach( function( ev ) {
				ev.apply( this, eventArgs );
			} );
		}
	};


	var EventBus = function( configObject ) {

		this._registerEventNamesCalled = false;
		this._configObject = configObject;
		if ( !this._configObject || ( !this._configObject.eventNames && !this._configObject.activityNames ) ) {
			throw new Error( 'EventBus: Config object passed to constructor must contain an "eventNames" and/or "activityNames" property containing all event names!' );
		}
		this._configObject.eventNames = this._configObject.eventNames || {};
		this._configObject.activityNames = this._configObject.activityNames || {};
		if ( this._configObject.doNameVerification === undefined ) {
			this._configObject.doNameVerification = true;
		}

		// Ensure all event / activity names registered in the constructor are prefixed with lib.*
		// (any names registered through .registerEventNames() can only be prefixed by game.*. .registerEventNames() can only be called once per game.
		// this keeps all event names in one place)
		var libsRegex = this._configObject.doNameVerification ? /^libs\./ : null;

		this._normalEventSignals = new SignalArray( this._configObject.eventNames, libsRegex );
		this._activityStartSignals = new SignalArray( this._configObject.activityNames, libsRegex );
		this._activityEndSignals = new SignalArray( this._configObject.activityNames, libsRegex );

		this._runningActivities = [];
		this._runningActivitiesChangedCallbacks = [];
		this._subscribedActivities = {};
	};

	EventBus.prototype.registerEventNames = function( eventNames, activityNames ) {

		if ( this._registerEventNamesCalled ) {
			throw new Error( "EventBus.prototype.registerEventNames() can only be called once per project" );
		}

		var gameRegex = this._configObject.doNameVerification ? /^game\./ : null;

		this._normalEventSignals.registerEventNames( eventNames, gameRegex );
		this._activityStartSignals.registerEventNames( activityNames, gameRegex );
		this._activityEndSignals.registerEventNames( activityNames, gameRegex );

		this._registerEventNamesCalled = true;
	};

	EventBus.prototype.containsEvent = function( eventName ) {
		return this._normalEventSignals.containsEvent( eventName );
	};

	EventBus.prototype.containsActivity = function( eventName ) {
		return this._activityStartSignals.containsEvent( eventName );
	};

	EventBus.prototype.connect = function( eventName, callback ) {
		this._normalEventSignals.connect( eventName, callback );
	};

	EventBus.prototype.invoke = function( eventName ) {
		if ( enableLogging( this, eventName ) ) {
			try {
				libs.log( 'Event Invoked: ' + JSON.stringify( arguments, replaceBigNumber ) );
			} catch ( e ) {
				libs.log( 'Event Invoked: {"0":"' + eventName + '"} - (could not parse arguments)"' );
			}
		}

		this._normalEventSignals.invoke.apply( this._normalEventSignals, arguments );
	};

	EventBus.prototype.invokeAndJoinActivity = function( activityName ) {

		var that = this;
		if ( !this._subscribedActivities[activityName] ) {
			this.connectActivityStart( activityName, function( activity ) {
				that._subscribedActivities[activityName] = activity.join();
			} );
		}
		this.invokeActivity( activityName );
		return this._subscribedActivities[activityName];
	};

	EventBus.prototype.invokeActivity = function( activityName ) {
		if ( enableLogging( this, activityName ) ) {
			libs.log( 'Activity Start: ' + JSON.stringify( arguments, replaceBigNumber ) );
		}

		var args = Array.prototype.slice.call( arguments, 0 );
		var activity = new Activity( activityName, args, this );
		args.splice( 1, 0, activity );

		this._addRunningActivity( activityName );
		this._activityStartSignals.invoke.apply( this._activityStartSignals, args );

		// Notifies activity that the 'join phase' is finished and that if all subscribers leave
		// then it should fire end event. Prevents one subscriber joining then immediately leaving to cause end event to fire before
		// other subscribers have a chance to join.
		activity._joinPhaseComplete();
	};

	EventBus.prototype._invokeActivityEnd = function( activityName, args ) {
		if ( enableLogging( this, activityName ) ) {
			// convert to object just to make logging match
			libs.log( 'Activity End: ' + JSON.stringify( $.extend( {}, args ), replaceBigNumber ) );
		}

		this._removeRunningActivity( activityName );
		this._activityEndSignals.invoke.apply( this._activityEndSignals, args );
	};

	EventBus.prototype._addRunningActivity = function( activityName ) {
		this._runningActivities.push( activityName );
		this._runningActivitiesChangedCallbacks.forEach( function( ev ) {
			ev.apply( this );
		} );
	};

	EventBus.prototype._removeRunningActivity = function( activityName ) {
		var index = this._runningActivities.indexOf( activityName );
		if ( index !== -1 ) {
			this._runningActivities.splice( index, 1 );
			this._runningActivitiesChangedCallbacks.forEach( function( ev ) {
				ev.apply( this );
			} );
		} else {
			libs.logError( 'Activity tried to be removed from running list, but was not on it' );
		}
	};

	EventBus.prototype.isActivityRunning = function( activityName ) {
		return this._runningActivities.indexOf( activityName ) > -1;
	};

	EventBus.prototype.connectActivityStart = function( activityName, callback ) {
		this._activityStartSignals.connect( activityName, callback );
	};

	EventBus.prototype.connectActivityEnd = function( activityName, callback ) {
		this._activityEndSignals.connect( activityName, callback );
	};

	EventBus.prototype.connectRunningActivitiesChanged = function( callback ) {
		this._runningActivitiesChangedCallbacks.push( callback );
	};


	libs.EventBus = EventBus;
	libs.globalEventBus = new EventBus( { eventNames: libs.eventNames, activityNames: libs.activityNames } );

}());
 