goog.provide( 'libs.StateMachine' );

goog.require( 'libs.UpdateTimer' );


(function() {
	"use strict";

	var StateMachine = function() {
		this._currentPhase = 0;
		this._checkingPhase = 0;
		this._stateCallback = null;

		this._timer = new libs.UpdateTimer();
		this._timerId = 0;
		this._timerInitialised = false;
		this._timerFinished = true;
	};

	StateMachine.prototype.update = function( timeDelta ) {

		if ( !isFinite( timeDelta ) ) {
			throw( 'StateMachine.prototype.update: timeDelta must be a finite number' );
		}
		this._checkingPhase = 0;
		this._timer.update( timeDelta );
		if ( this._stateCallback ) {
			this._stateCallback( timeDelta );
		}
	};

	StateMachine.prototype.stateGoto = function( stateCallback ) {
		this._stateCallback = stateCallback;
		this._timer.stopTimer( this._timerId );
		this._timerInitialised = false;
		this._checkingPhase = 0;
		this._currentPhase = 0;
	};

	StateMachine.prototype.phaseOnce = function() {
		var result = this._currentPhase === this._checkingPhase;
		this._checkingPhase++;
		if ( result ) {
			this.phaseStep();
		}
		return result;
	};

	StateMachine.prototype.phaseManual = function() {
		var result = this._currentPhase === this._checkingPhase;
		this._checkingPhase++;
		return result;
	};

	StateMachine.prototype.phaseTimed = function( duration_ms ) {
		if ( !isFinite( duration_ms ) ) {
			throw( 'StateMachine.prototype.phaseTimed: duration_ms must be a finite number' );
		}
		var that = this;
		var result = this._currentPhase === this._checkingPhase;
		this._checkingPhase++;
		if ( result ) {
			if ( !this._timerInitialised ) {
				this._timerFinished = false;
				this._timerId = this._timer.addTimeout( duration_ms, function() { that._timerFinished = true; } );
				this._timerInitialised = true;
			}

			if ( this._timerFinished ) {
				this.phaseStep();
			}
		}

		return result;
	};

	StateMachine.prototype.phaseStep = function() {
		this._timer.stopTimer( this._timerId );
		this._timerInitialised = false;
		this._currentPhase++;
	};

	StateMachine.prototype.phaseReset = function() {
		this._timer.stopTimer( this._timerId );
		this._timerInitialised = false;
		this._currentPhase = 0;
		this._checkingPhase = 0;
	};

	StateMachine.prototype.phaseTimedGetProgress = function() {
		if ( this._timerInitialised ) {
			return this._timer.getProgress( this._timerId );
		} else if ( this._timerFinished ) {
			return 1.0;
		} else {
			return 0.0;
		}
	};

	StateMachine.prototype.phaseTimedGetRevProgress = function() {
		return 1.0 - this.phaseTimedGetProgress();
	};


	libs.StateMachine = StateMachine;

}());