goog.provide( 'libs.xmlUtils' );


(function() {
	"use strict";

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// XML / JSON conversion
	var createXmlDoc = function( xmlDocStr, customDOM ) {
		var useMSDOM = false;
		var xmlDoc = null;
		try {
			var Dom = customDOM || window.DOMParser;
			if ( Dom ) {
				var parser = new Dom();
				var parsererrorNS = parser.parseFromString( 'INVALID', 'text/xml' ).childNodes[0].namespaceURI;
				xmlDoc = parser.parseFromString( xmlDocStr, "text/xml" );
				if ( xmlDoc.getElementsByTagNameNS( parsererrorNS, 'parsererror' ).length > 0 ) {
					//throw new Error('Error parsing XML: '+xmlDocStr);
					xmlDoc = null;
				}
			} else {
				useMSDOM = true;
			}
		}
		catch ( err ) {
			useMSDOM = true;
		}

		if ( useMSDOM ) {
			// IE :(
			if ( xmlDocStr.indexOf( "<?" ) === 0 ) {
				xmlDocStr = xmlDocStr.substr( xmlDocStr.indexOf( "?>" ) + 2 );
			}
			xmlDoc = new ActiveXObject( "Microsoft.XMLDOM" );
			xmlDoc.async = "false";
			xmlDoc.loadXML( xmlDocStr );
		}
		if ( xmlDoc && typeof xmlDoc.setProperty === 'function' ) {
			xmlDoc.setProperty( "ProhibitDTD", true ); // Disallows DTDs, resolves potential security issues
		}
		return xmlDoc;
	};

	// V1.1.4 of xml2json lib
	function X2JS( config ) {
		var VERSION = "1.1.4";

		config = config || {};
		initConfigDefaults();
		initRequiredPolyfills();

		function initConfigDefaults() {
			if ( !config.escapeMode ) {
				config.escapeMode = true;
			}
			config.attributePrefix = config.attributePrefix || "_";
			config.arrayAccessForm = config.arrayAccessForm || "none";
			config.emptyNodeForm = config.emptyNodeForm || "text";
			if ( !config.enableToStringFunc ) {
				config.enableToStringFunc = true;
			}
			config.arrayAccessFormPaths = config.arrayAccessFormPaths || [];
			if ( !config.skipEmptyTextNodesForObj ) {
				config.skipEmptyTextNodesForObj = true;
			}
			if ( !config.stripWhitespaces ) {
				config.stripWhitespaces = true;
			}
			config.datetimeAccessFormPaths = config.datetimeAccessFormPaths || [];
		}

		var DOMNodeTypes = {
			ELEMENT_NODE: 1,
			TEXT_NODE: 3,
			CDATA_SECTION_NODE: 4,
			COMMENT_NODE: 8,
			DOCUMENT_NODE: 9
		};

		function initRequiredPolyfills() {
			function pad( number ) {
				var r = String( number );
				if ( r.length === 1 ) {
					r = '0' + r;
				}
				return r;
			}

			// Hello IE8-
			if ( typeof String.prototype.trim !== 'function' ) {
				String.prototype.trim = function() {
					return this.replace( /^\s+|^\n+|(\s|\n)+$/g, '' );
				};
			}
			if ( typeof Date.prototype.toISOString !== 'function' ) {
				// Implementation from http://stackoverflow.com/questions/2573521/how-do-i-output-an-iso-8601-formatted-string-in-javascript
				Date.prototype.toISOString = function() {
					return this.getUTCFullYear() + '-' + pad( this.getUTCMonth() + 1 ) + '-' + pad( this.getUTCDate() ) + 'T' + pad( this.getUTCHours() ) +
						':' + pad( this.getUTCMinutes() ) + ':' + pad( this.getUTCSeconds() ) + '.' + String( (this.getUTCMilliseconds() / 1000).toFixed( 3 ) ).slice( 2, 5 ) + 'Z';
				};
			}
		}

		function getNodeLocalName( node ) {
			var nodeLocalName = node.localName;
			if ( !nodeLocalName ) { // Yeah, this is IE!!
				nodeLocalName = node.baseName;
			}
			if ( !nodeLocalName || nodeLocalName.length === 0 ) { // =="" is IE too
				nodeLocalName = node.nodeName;
			}
			return nodeLocalName;
		}

		function getNodePrefix( node ) {
			return node.prefix;
		}

		function escapeXmlChars( str ) {
			if ( typeof(str) === "string" ) {
				return str.replace( /&/g, '&amp;' ).replace( /</g, '&lt;' ).replace( />/g, '&gt;' ).replace( /"/g, '&quot;' ).replace( /'/g, '&#x27;' ).replace( /\//g, '&#x2F;' );
			} else {
				return str;
			}
		}

		function unescapeXmlChars( str ) {
			return str.replace( /&amp;/g, '&' ).replace( /&lt;/g, '<' ).replace( /&gt;/g, '>' ).replace( /&quot;/g, '"' ).replace( /&#x27;/g, "'" ).replace( /&#x2F;/g, '\/' );
		}

		function toArrayAccessForm( obj, childName, path ) {
			if ( config.arrayAccessForm === 'property' ) {
				if ( !(obj[childName] instanceof Array) ) {
					obj[childName + "_asArray"] = [obj[childName]];
				} else {
					obj[childName + "_asArray"] = obj[childName];
				}
			}

			if ( !(obj[childName] instanceof Array) && config.arrayAccessFormPaths.length > 0 ) {
				var idx = 0;
				for ( ; idx < config.arrayAccessFormPaths.length; idx++ ) {
					var arrayPath = config.arrayAccessFormPaths[idx];
					if ( typeof arrayPath === "string" ) {
						if ( arrayPath === path ) {
							break;
						}
					}
					else if ( arrayPath instanceof RegExp ) {
						if ( arrayPath.test( path ) ) {
							break;
						}
					}
					else if ( typeof arrayPath === "function" ) {
						if ( arrayPath( obj, childName, path ) ) {
							break;
						}
					}
				}
				if ( idx !== config.arrayAccessFormPaths.length ) {
					obj[childName] = [obj[childName]];
				}
			}
		}

		function fromXmlDateTime( prop ) {
			// Implementation based up on http://stackoverflow.com/questions/8178598/xml-datetime-to-javascript-date-object
			// Improved to support full spec and optional parts
			var bits = prop.split( /[\-T\:\+Z]/g );

			var d = new Date( bits[0], bits[1] - 1, bits[2] );
			var secondBits = bits[5].split( '.' );
			d.setHours( bits[3], bits[4], secondBits[0] );
			if ( secondBits.length > 1 ) {
				d.setMilliseconds( secondBits[1] );
			}

			// Get supplied time zone offset in minutes
			if ( bits[6] && bits[7] ) {
				var offsetMinutes = bits[6] * 60 + Number( bits[7] );
				var sign = /\d\d-\d\d:\d\d$/.test( prop ) ? '-' : '+';

				// Apply the sign
				offsetMinutes = 0 + (sign === '-' ? -1 * offsetMinutes : offsetMinutes);

				// Apply offset and local timezone
				d.setMinutes( d.getMinutes() - offsetMinutes - d.getTimezoneOffset() );
			}
			else if ( prop.indexOf( "Z", prop.length - 1 ) !== -1 ) {
				d = new Date( Date.UTC( d.getFullYear(), d.getMonth(), d.getDate(), d.getHours(), d.getMinutes(), d.getSeconds(), d.getMilliseconds() ) );
			}

			// d is now a local time equivalent to the supplied time
			return d;
		}

		function checkFromXmlDateTimePaths( value, childName, fullPath ) {
			if ( config.datetimeAccessFormPaths.length > 0 ) {
				var path = fullPath.split( ".#" )[0];
				var idx = 0;
				for ( ; idx < config.datetimeAccessFormPaths.length; idx++ ) {
					var dtPath = config.datetimeAccessFormPaths[idx];
					if ( typeof dtPath === "string" ) {
						if ( dtPath === path ) {
							break;
						}
					}
					else if ( dtPath instanceof RegExp ) {
						if ( dtPath.test( path ) ) {
							break;
						}
					}
					else if ( typeof dtPath === "function" ) {
						if ( dtPath( undefined, childName, path ) ) {
							break;
						}
					}
				}
				if ( idx !== config.datetimeAccessFormPaths.length ) {
					return fromXmlDateTime( value );
				}
				else {
					return value;
				}
			}
			else {
				return value;
			}
		}

		function parseDOMChildren( node, path ) {
			var result;
			var nodeChildren;
			var cidx;
			var child;
			var childName;

			if ( node.nodeType === DOMNodeTypes.DOCUMENT_NODE ) {
				result = {};
				nodeChildren = node.childNodes;
				// Alternative for firstElementChild which is not supported in some environments
				for ( cidx = 0; cidx < nodeChildren.length; cidx++ ) {
					child = nodeChildren.item( cidx );
					if ( child.nodeType === DOMNodeTypes.ELEMENT_NODE ) {
						childName = getNodeLocalName( child );
						result[childName] = parseDOMChildren( child, childName );
					}
				}
				return result;
			}
			else if ( node.nodeType === DOMNodeTypes.ELEMENT_NODE ) {
				result = {};
				result.__cnt = 0;

				nodeChildren = node.childNodes;

				// Children nodes
				for ( cidx = 0; cidx < nodeChildren.length; cidx++ ) {
					child = nodeChildren.item( cidx ); // nodeChildren[cidx];
					childName = getNodeLocalName( child );

					if ( child.nodeType !== DOMNodeTypes.COMMENT_NODE ) {
						result.__cnt++;
						if ( !result[childName] ) {
							result[childName] = parseDOMChildren( child, path + "." + childName );
							toArrayAccessForm( result, childName, path + "." + childName );
						}
						else {
							if ( result[childName] ) {
								if ( !(result[childName] instanceof Array) ) {
									result[childName] = [result[childName]];
									toArrayAccessForm( result, childName, path + "." + childName );
								}
							}
							(result[childName])[result[childName].length] = parseDOMChildren( child, path + "." + childName );
						}
					}
				}

				// Attributes
				for ( var aidx = 0; aidx < node.attributes.length; aidx++ ) {
					var attr = node.attributes.item( aidx ); // [aidx];
					result.__cnt++;
					result[config.attributePrefix + attr.name] = attr.value;
				}

				// Node namespace prefix
				var nodePrefix = getNodePrefix( node );
				if ( nodePrefix && nodePrefix.length > 0 ) {
					result.__cnt++;
					result.__prefix = nodePrefix;
				}

				if ( result["#text"] ) {
					result.__text = result["#text"];
					if ( result.__text instanceof Array ) {
						result.__text = result.__text.join( "\n" );
					}
					if ( config.escapeMode ) {
						result.__text = unescapeXmlChars( result.__text );
					}
					if ( config.stripWhitespaces ) {
						result.__text = result.__text.trim();
					}
					delete result["#text"];
					if ( config.arrayAccessForm === "property" ) {
						delete result["#text_asArray"];
					}
					result.__text = checkFromXmlDateTimePaths( result.__text, childName, path + "." + childName );
				}
				if ( result["#cdata-section"] ) {
					result.__cdata = result["#cdata-section"];
					delete result["#cdata-section"];
					if ( config.arrayAccessForm === "property" ) {
						delete result["#cdata-section_asArray"];
					}
				}

				if ( result.__cnt === 1 && result.__text ) {
					result = result.__text;
				}
				else if ( result.__cnt === 0 && config.emptyNodeForm === "text" ) {
					result = '';
				}
				else if ( result.__cnt > 1 && result.__text && config.skipEmptyTextNodesForObj ) {
					if ( (config.stripWhitespaces && result.__text.length === 0) || (result.__text.trim().length === 0) ) {
						delete result.__text;
					}
				}
				delete result.__cnt;

				if ( config.enableToStringFunc && (result.__text || result.__cdata ) ) {
					result.toString = function() {
						return (this.__text ? this.__text : '') + ( this.__cdata ? this.__cdata : '');
					};
				}

				return result;
			}
			else if ( node.nodeType === DOMNodeTypes.TEXT_NODE || node.nodeType === DOMNodeTypes.CDATA_SECTION_NODE ) {
				return node.nodeValue;
			}
		}

		function startTag( jsonObj, element, attrList, closed ) {
			var resultStr = "<" + ( (jsonObj && jsonObj.__prefix) ? (jsonObj.__prefix + ":") : "") + element;
			if ( attrList ) {
				for ( var aidx = 0; aidx < attrList.length; aidx++ ) {
					var attrName = attrList[aidx];
					var attrVal = jsonObj[attrName];
					if ( config.escapeMode ) {
						attrVal = escapeXmlChars( attrVal );
					}
					resultStr += " " + attrName.substr( config.attributePrefix.length ) + "='" + attrVal + "'";
				}
			}
			if ( !closed ) {
				resultStr += ">";
			} else {
				resultStr += "/>";
			}
			return resultStr;
		}

		function endTag( jsonObj, elementName ) {
			return "</" + (jsonObj.__prefix ? (jsonObj.__prefix + ":") : "") + elementName + ">";
		}

		function endsWith( str, suffix ) {
			return str.indexOf( suffix, str.length - suffix.length ) !== -1;
		}

		function jsonXmlSpecialElem( jsonObj, jsonObjField ) {
			if ( (config.arrayAccessForm === "property" && endsWith( jsonObjField.toString(), ("_asArray") )) ||
				jsonObjField.toString().indexOf( config.attributePrefix ) === 0 || jsonObjField.toString().indexOf( "__" ) === 0 || (jsonObj[jsonObjField] instanceof Function) ) {
				return true;
			} else {
				return false;
			}
		}

		function jsonXmlElemCount( jsonObj ) {
			var elementsCnt = 0;
			if ( jsonObj instanceof Object ) {
				for ( var it in jsonObj ) {
					if ( jsonXmlSpecialElem( jsonObj, it ) ) {
						continue;
					}
					elementsCnt++;
				}
			}
			return elementsCnt;
		}

		function parseJSONAttributes( jsonObj ) {
			var attrList = [];
			if ( jsonObj instanceof Object ) {
				for ( var ait in jsonObj ) {
					if ( ait.toString().indexOf( "__" ) === -1 && ait.toString().indexOf( config.attributePrefix ) === 0 ) {
						attrList.push( ait );
					}
				}
			}
			return attrList;
		}

		function parseJSONTextAttrs( jsonTxtObj ) {
			var result = "";

			if ( jsonTxtObj.__cdata ) {
				result += "<![CDATA[" + jsonTxtObj.__cdata + "]]>";
			}

			if ( jsonTxtObj.__text ) {
				if ( config.escapeMode ) {
					result += escapeXmlChars( jsonTxtObj.__text );
				} else {
					result += jsonTxtObj.__text;
				}
			}
			return result;
		}

		function parseJSONTextObject( jsonTxtObj ) {
			var result = "";

			if ( jsonTxtObj instanceof Object ) {
				result += parseJSONTextAttrs( jsonTxtObj );
			}
			else if ( jsonTxtObj ) {
				if ( config.escapeMode ) {
					result += escapeXmlChars( jsonTxtObj );
				} else {
					result += jsonTxtObj;
				}
			}

			return result;
		}

		function parseJSONArray( jsonArrRoot, jsonArrObj, attrList ) {
			var result = "";
			if ( jsonArrRoot.length === 0 ) {
				result += startTag( jsonArrRoot, jsonArrObj, attrList, true );
			}
			else {
				for ( var arIdx = 0; arIdx < jsonArrRoot.length; arIdx++ ) {
					result += startTag( jsonArrRoot[arIdx], jsonArrObj, parseJSONAttributes( jsonArrRoot[arIdx] ), false );
					result += parseJSONObject( jsonArrRoot[arIdx] );
					result += endTag( jsonArrRoot[arIdx], jsonArrObj );
				}
			}
			return result;
		}

		function parseJSONObject( jsonObj ) {
			var result = "";

			var elementsCnt = jsonXmlElemCount( jsonObj );

			if ( elementsCnt > 0 ) {
				for ( var it in jsonObj ) {

					if ( jsonXmlSpecialElem( jsonObj, it ) ) {
						continue;
					}

					var subObj = jsonObj[it];

					var attrList = parseJSONAttributes( subObj );

					if ( !subObj ) {
						result += startTag( subObj, it, attrList, true );
					}
					else if ( subObj instanceof Object ) {

						if ( subObj instanceof Array ) {
							result += parseJSONArray( subObj, it, attrList );
						}
						else if ( subObj instanceof Date ) {
							result += startTag( subObj, it, attrList, false );
							result += subObj.toISOString();
							result += endTag( subObj, it );
						}
						else {
							var subObjElementsCnt = jsonXmlElemCount( subObj );
							if ( subObjElementsCnt > 0 || subObj.__text || subObj.__cdata ) {
								result += startTag( subObj, it, attrList, false );
								result += parseJSONObject( subObj );
								result += endTag( subObj, it );
							}
							else {
								result += startTag( subObj, it, attrList, true );
							}
						}
					}
					else {
						result += startTag( subObj, it, attrList, false );
						result += parseJSONTextObject( subObj );
						result += endTag( subObj, it );
					}
				}
			}
			result += parseJSONTextObject( jsonObj );

			return result;
		}

		this.parseXmlString = function( xmlDocStr, customDOM ) {
			if ( !xmlDocStr ) {
				return null;
			}
			var xmlDoc = createXmlDoc( xmlDocStr, customDOM );
			return xmlDoc;
		};

		this.asArray = function( prop ) {
			if ( prop instanceof Array ) {
				return prop;
			} else {
				return [prop];
			}
		};

		this.toXmlDateTime = function( dt ) {
			if ( dt instanceof Date ) {
				return dt.toISOString();
			} else if ( typeof(dt) === 'number' ) {
				return new Date( dt ).toISOString();
			} else {
				return null;
			}
		};

		this.asDateTime = function( prop ) {
			if ( typeof(prop) === "string" ) {
				return fromXmlDateTime( prop );
			} else {
				return prop;
			}
		};

		this.xml2json = function( xmlDoc ) {
			return parseDOMChildren( xmlDoc );
		};

		this.xml_str2json = function( xmlDocStr, customDOM ) {
			var xmlDoc = this.parseXmlString( xmlDocStr, customDOM );
			if ( xmlDoc ) {
				return this.xml2json( xmlDoc );
			} else {
				return null;
			}
		};

		this.json2xml_str = function( jsonObj ) {
			return parseJSONObject( jsonObj );
		};

		this.json2xml = function( jsonObj, customDOM ) {
			var xmlDocStr = this.json2xml_str( jsonObj );
			return this.parseXmlString( xmlDocStr, customDOM );
		};

		this.getVersion = function() {
			return VERSION;
		};

	}


	libs.xmlUtils.getElementsByTagNameSafe = function( root, name ) {
		var elements = root.getElementsByTagName( name );
		if ( elements.length === 0 ) {
			throw('XML Parsing Error: No Element: ' + name );
		}

		return elements;
	};

	libs.xmlUtils.getAttributeSafe = function( element, name ) {
		var attribute = element.getAttribute( name );
		if ( attribute === null ) {
			throw('XML Parsing Error: No Attribute: ' + name );
		}

		return attribute;
	};

	libs.xmlUtils.getTextContent = function( element ) {
		return element.textContent || element.text;
	};

	// Converts a standard javascript object to an XML fragment
	libs.xmlUtils.jsonObjectToXml = function( anObject ) {
		var x2js = new X2JS();
		var xmlDocStr = x2js.json2xml_str( anObject );
		return xmlDocStr;
	};

	// Converts a standard javascript object to an XML fragment
	libs.xmlUtils.xmlToJsonObject = function( xmlString, arrayAccessFormPaths, customDOM ) {
		var x2js = new X2JS( { arrayAccessFormPaths: arrayAccessFormPaths } );
		var jsonObject = x2js.xml_str2json( xmlString, customDOM ) || {};
		return jsonObject;
	};


	var escapeCDataTags = function( str ) {
		// check for ']]>' string in json object (shouldn't be, but check anyway)
		str = str.replace( /\]\]\>/g, '&#93&#93&#62' );
		return str;
	};


	var unEscapeCDataTags = function( str ) {
		str = str.replace( /\&\#93\&\#93\&\#62/g, ']]>' );
		return str;
	};


	var bigNumberPrefix = "_+BN+_";


	libs.xmlUtils.jsonStringify = function( jsonObject ) {
		var replacer = function( key, value ) {
			if ( value instanceof BigNumber ) {
				return bigNumberPrefix + value.toString();
			} else if ( typeof value === 'string' ) {
				return escapeCDataTags( value );
			} else {
				return value;
			}
		};
		return JSON.stringify( jsonObject, replacer );
	};


	libs.xmlUtils.jsonParse = function( jsonString ) {
		var replacer = function( key, value ) {
			try {
				if ( typeof value === 'string' ) {
					if ( value.length > bigNumberPrefix.length && value.slice( 0, bigNumberPrefix.length ) === bigNumberPrefix ) {
						var strNum = value.slice( bigNumberPrefix.length );
						return new BigNumber( strNum );
					} else {
						return unEscapeCDataTags( value );
					}
				}
			}
			catch ( err ) {
			}
			return value;
		};
		return JSON.parse( jsonString, replacer );
	};


	// Used by William Hill components: Wraps a JSON object into an XML fragment for transmission.
	libs.xmlUtils.wrapJsonInXml = function( tagName, jsonObject ) {

		var jsonStr = '';
		try {
			jsonStr = libs.xmlUtils.jsonStringify( jsonObject );
		}
		catch ( err ) {
			throw new Error( "libs.xmlUtils.wrapJsonInXml: Invalid input data" );
		}
		// Note: If any ]]> strings still exist in the string, throw an error
		if ( jsonStr.indexOf( ']]>' ) >= 0 ) {
			throw new Error( "libs.xmlUtils.wrapJsonInXml: Invalid characters ']]>' found in JSON string" );
		}
		return "<" + tagName + "><![CDATA[" + jsonStr + "]]></" + tagName + ">";

	};


	libs.xmlUtils.unwrapJsonFromXml = function( tagName, xmlstr, customDOM ) {

		try {
			var doc = createXmlDoc( xmlstr, customDOM );
			var jsonResponseNode = doc.getElementsByTagName( tagName )[0];
			var jsonString = jsonResponseNode.childNodes[0].nodeValue;
			var jsonObject = libs.xmlUtils.jsonParse( jsonString );
			return jsonObject;
		}
		catch ( err ) {
			throw new Error( "libs.xmlUtils.unwrapJsonFromXml: Invalid input data" );
		}
	};


}());