// BetSelection object validates data sent by the client to ensure it is valid.
// Used by ASP server only
goog.provide( 'libs.server.validateBetSelectionRequest' );
goog.provide( 'libs.server.validateStateObject' );


(function() {
	"use strict";

	var isValidNumber = function( num, low, high ) {
		if ( typeof num === 'number' ) {
			return !isNaN( num ) &&
				isFinite( num ) &&
				( low !== undefined ? num >= low : true ) &&
				( high !== undefined ? num <= high : true );
		}
		if ( num instanceof BigNumber ) {
			return !num.isNaN() &&
				num.isFinite() &&
				( low !== undefined ? num.greaterThanOrEqualTo( low ) : true ) &&
				( high !== undefined ? num.lessThanOrEqualTo( high ) : true );
		}
		return false;
	};

	var isObject = function( obj ) {
		return obj && typeof obj === 'object';
	};


	libs.server.validateBetSelectionRequest = function( betSelectionData, balance, currencyMultiplier, expectedJackpotsObject ) {

		var FatalError = 0;
		var NotEnoughBalance = 2;

		if ( !isValidNumber( betSelectionData['checkpointId'], 0 ) ) {
			throw( {
				message: "Sequence " + betSelectionData['checkpointId'] + " is invalid.",
				errorType: FatalError,
				suspicious: true
			} );
		}

		if ( !isValidNumber( betSelectionData['stake'], 0 ) ) {
			throw( {
				message: "Stake " + betSelectionData['stake'] + " is invalid.",
				errorType: FatalError,
				suspicious: true
			} );
		}

		var stakeExists = game.logic.GameData.financial.availableStakesPerLine.some( function( stake ) {
			return betSelectionData['stake'].equals( stake * currencyMultiplier );
		} );
		if ( !stakeExists ) {
			throw( {
				message: "Stake " + betSelectionData['stake'].toString() + " is not one of acceptable values.",
				errorType: FatalError,
				suspicious: true
			} );
		}

		if ( !isValidNumber( betSelectionData['lines'], 1, game.logic.GameData.reels.winLineDefinitionArray.length ) ) {
			throw( {
				message: "Lines " + betSelectionData['lines'] + " is invalid.",
				errorType: FatalError,
				suspicious: true
			} );
		}

		if ( game.logic.GameData.reels.valid_number_lines_arr ) {
			var linesValid = game.logic.GameData.reels.valid_number_lines_arr.some( function( line ) {
				return line === betSelectionData['lines'];
			} );
			if ( !linesValid ) {
				throw( {
					message: "Number of lines " + betSelectionData['lines'] + " is not one of acceptable values.",
					errorType: FatalError,
					suspicious: true
				} );
			}
		}

		betSelectionData['total_stake'] = betSelectionData['stake'].times( betSelectionData['lines'] );

		// If stake and lines valid can validate total stake
		// Stake can be greater than balance if this is a later part of a bet.
		if ( balance >= 0 ) {
			var isFirstOperation = betSelectionData['checkpointId'] === 0;
			if ( betSelectionData['total_stake'].greaterThan( balance ) && isFirstOperation ) {
				throw( {
					message: "You have insufficient funds to place this stake. Please reduce your stake.",
					errorType: NotEnoughBalance,
					suspicious: true
				} );
			}
		}

		// Validate jackpots - there should be same amount defined in game data
		if ( expectedJackpotsObject && Array.isArray( expectedJackpotsObject ) && expectedJackpotsObject.length > 0 ) {

			var expectedJackpotCount = game.logic.GameData.jackpots && game.logic.GameData.jackpots.list ? game.logic.GameData.jackpots.list.length : 0;

			if ( Array.isArray( expectedJackpotsObject ) ) {
				if ( expectedJackpotsObject.length !== expectedJackpotCount ) {
					throw( {
						message: "There should be " + expectedJackpotCount + " (not " + expectedJackpotsObject.length + ") jackpots.",
						errorType: FatalError,
						suspicious: true
					} );
				}
				// Each jackpot should have a valid ID and money value
				var validJackpots = expectedJackpotsObject.some( function( jpInfo ) {
					return isValidNumber( jpInfo['id'], 0, expectedJackpotsObject.length - 1 ) &&
						isValidNumber( jpInfo['money'], 0 );
				} );
				if ( !validJackpots ) {
					throw( {
						message: "Provided jackpot has invalid value.",
						errorType: FatalError,
						suspicious: true
					} );
				}
			} else {
				if ( expectedJackpotCount > 0 ) {
					throw( {
						message: "There should be " + expectedJackpotCount + " (not 0) jackpots.",
						errorType: FatalError,
						suspicious: true
					} );
				}
			}
		}
	};


	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	libs.server.validateStateObject = function( stateData, currentSequence ) {

		var stateExists = isObject( stateData );

		if ( stateExists && currentSequence === 0 ) {
			throw( { message: "State found when starting bet" } );
		}
		if ( !stateExists && currentSequence !== 0 ) {
			throw( { message: "No state found when continuing bet" } );
		}

		if ( stateExists ) {
			if ( !isObject( stateData['betSelection'] ) || !isObject( stateData['lastGame'] ) || !isValidNumber( stateData['betSelection']['checkpointId'], 0 )
			) {
				throw( { message: "Invalid state found" } );
			}
			if ( !stateData['lastGame']['expectCheckpoint'] ) {
				throw( {
					message: "Bet not expected",
					suspiciousMessage: "Bet: sequence=" + currentSequence + " suspicious: Bet not expected"
				} );
			}
			var stateCheckpointId = stateData['betSelection']['checkpointId'];
			if ( ( stateCheckpointId + 1 ) !== currentSequence ) {
				throw( { message: "Unexpected sequence (" + currentSequence + ") when last was " + stateCheckpointId } );
			}
		}
	};

}());
