/*jslint node: true */

"use strict";

var vm = require( 'vm' );
var fs = require( 'fs' );
var path = require( 'path' );


var NodeJsBootstrap = function() {
	this._vmContext = null;
	this._runInThisContext = false;
};


NodeJsBootstrap.prototype.initVm = function( logicPath, additionalFiles, runInThisContext ) {

	this._runInThisContext = runInThisContext;
	var include = function( filePath, failIfDoesntExist ) {

		if ( fs.existsSync( filePath ) ) {
			try {
				var fileCode = fs.readFileSync( filePath, 'utf-8' );
				if ( !runInThisContext ) {
					vm.runInContext( fileCode, that._vmContext );
				} else {
					vm.runInThisContext( fileCode );
				}
			}
			catch ( err ) {
				throw new Error( 'Could not load game logic file "' + filePath + '" : ' + err );
			}
		} else {
			if ( failIfDoesntExist ) {
				throw new Error( 'Could not load game logic file "' + filePath + '"' );
			}
		}
	};

	if ( !runInThisContext ) {
		var context = {
			initSettings: null,
			ret_initResponse: null,
			ret_betResult: null,
			ret_betResults: []
		};
		this._vmContext = vm.createContext( context );
	}

	var commonFiles = [
		'../Libs_Common/js/namespace.js',
		'../bignumber.js/bignumber.js',
		'../Libs_Common/js/coreUtils.js',
		'../Libs_Common/js/arrayPolyfills.js',
		'../Libs_Common/js/mersenneTwister.js',
		'../Libs_Common/js/xmlUtils.js',
		'../Libs_ServerReels/js/outcomeTable.js',
		'../Libs_ServerReels/js/awardsTable.js',
		'../Libs_ServerReels/js/reelBandSet.js',
		'../Libs_ServerReels/js/symbolMap.js',
		'js/gamedata.js',

		'../Libs_ServerCore/js/datavalidation.js',
		'../Libs_ServerCore/js/platformUtils.js',
		'../Libs_ServerReels/js/slotgamelogic.js',

		'js/buildbetresults.js',
		'js/buildinitresponse.js'
	];

	var that = this;

	commonFiles.forEach( function( p ) {
		include( path.join( logicPath, p ), true );
	} );

	if ( additionalFiles ) {
		additionalFiles.forEach( function( p ) {
			include( path.join( logicPath, p ), false );
		} );
	}

	include( path.join( __dirname, 'nodeJsLogicBootstrapContext.js' ) );
};


NodeJsBootstrap.prototype.getGameDataObject = function() {
	if ( !this._runInThisContext ) {
		if ( !this._vmContext ) {
			throw new Error( 'NodeJsBootstrap.getGameDataObject: Must call doInit() first!' );
		}
		return this._vmContext.game.logic.GameData;
	} else {
		return game.logic.GameData;
	}
};


// Settings object is format { currencyCode: settings.currencyCode, jackpotMode: settings.jackpotMode } - passed to local platform interface object
NodeJsBootstrap.prototype.doInit = function( initSettings ) {
	if ( !this._runInThisContext ) {
		this._vmContext.initSettings = initSettings;
		vm.runInContext( 'ret_initResponse = do_init( initSettings );', this._vmContext );
		return this._vmContext.ret_initResponse;
	} else {
		var ret_initResponse = do_init( initSettings );
		return ret_initResponse;
	}
};


// Performs an entire game cycle (all checkpoints) - collating all bet result objects into an array
NodeJsBootstrap.prototype.doEntireGameCycle = function( stakePerLine, lineCount, forcedRngString ) {

	if ( !this._runInThisContext ) {
		if ( forcedRngString && forcedRngString.length > 0 ) {
			vm.runInContext( 'ret_betResults = do_entire_game_cycle( ' + stakePerLine + ', ' + lineCount + ', "' + forcedRngString + '" )', this._vmContext );
		} else {
			vm.runInContext( 'ret_betResults = do_entire_game_cycle( ' + stakePerLine + ', ' + lineCount + ' )', this._vmContext );
		}
		return this._vmContext.ret_betResults;
	} else {
		var ret_betResults = do_entire_game_cycle( stakePerLine, lineCount, forcedRngString );
		return ret_betResults;
	}
};


// Performs a single checkpoint on a game. State is maintained in the VM context. Returns the betresult generated
NodeJsBootstrap.prototype.doCheckpointGameCycle = function( stakePerLine, lineCount, forcedRngString ) {

	if ( !this._runInThisContext ) {
		if ( forcedRngString && forcedRngString.length > 0 ) {
			vm.runInContext( 'ret_betResult = do_checkpoint_game_cycle( ' + stakePerLine + ', ' + lineCount + ', "' + forcedRngString + '" )', this._vmContext );
		} else {
			vm.runInContext( 'ret_betResult = do_checkpoint_game_cycle( ' + stakePerLine + ', ' + lineCount + ' )', this._vmContext );
		}
		return this._vmContext.ret_betResult;
	} else {
		var ret_betResult = do_checkpoint_game_cycle( stakePerLine, lineCount, forcedRngString );
		return ret_betResult;
	}
};


module.exports = new NodeJsBootstrap();

