goog.provide( 'libs.server.OutcomeTable' );


(function() {
	"use strict";

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// OutcomeTable - helper class to access outcome tables

	var OutcomeTable = function( name, outcomeTable, stakeSpecificTable, fallbackOnDefaultTableIfStakeNotFound ) {

		this._name = name;
		this._outcomeTable = outcomeTable || {};
		this._stakeSpecificTable = {};
		this._useStakeSpecificTable = false;

		// for ease of use, the class merges the default outcome table with the stake-specific one to create a gigantic stake-specific table
		// which contains a merge between the stake-specific provided table and the default one
		if ( stakeSpecificTable ) {
			this._useStakeSpecificTable = !fallbackOnDefaultTableIfStakeNotFound;
			for ( var stake in stakeSpecificTable ) {
				if ( stakeSpecificTable.hasOwnProperty( stake ) ) {

					this._stakeSpecificTable[stake] = {};
					var stakeOutcomeObj = stakeSpecificTable[stake];

					// Add the default stake outcomes
					for ( var outcome in this._outcomeTable ) {
						if ( this._outcomeTable.hasOwnProperty( outcome ) ) {
							this._stakeSpecificTable[stake][outcome] = this._outcomeTable[outcome];
						}
					}

					// add outcomes that appear in the stake-specific table
					for ( var stakeOutcome in stakeOutcomeObj ) {
						if ( stakeOutcomeObj.hasOwnProperty( stakeOutcome ) ) {
							this._stakeSpecificTable[stake][stakeOutcome] = stakeOutcomeObj[stakeOutcome];
						}
					}
				}
			}
		}
	};


	OutcomeTable.prototype._getStakeOutcomeTable = function( stake ) {

		if ( stake && this._stakeSpecificTable && this._stakeSpecificTable.hasOwnProperty( stake ) ) {
			return this._stakeSpecificTable[stake];
		}
		if ( this._useStakeSpecificTable ) {
			throw { message: 'Invalid stake passed to outcome table ' + this._name + ' (' + stake + ')' };
		}
		if ( !this._outcomeTable ) {
			throw { message: 'Default outcome table for ' + this._name + ' has not been defined. Is stake ' + stake + ' correct?' };
		}
		return this._outcomeTable;
	};


	OutcomeTable.prototype.getReelBandsUsedForStake = function( stake ) {

		var table = this._getStakeOutcomeTable( stake );
		return Object.keys( table );
	};


	OutcomeTable.prototype.getSumWeightingForReelSet = function( stake, reelBandId ) {

		var table = this._getStakeOutcomeTable( stake );
		var totalWgt = 0;
		for ( var outcome in table ) {
			if ( table.hasOwnProperty( outcome ) ) {
				var outcomeWeight = Number( table[outcome] );
				if ( isFinite( outcomeWeight ) ) {
					if ( outcome === reelBandId ) {
						return totalWgt;
					}
					totalWgt += outcomeWeight;
				} else {
					throw { message: 'OutcomeTable.getSumWeightingForReelSet: Weight for outcome ' + outcome + ' has invalid value ' + outcomeWeight + ' in table ' + this._name };
				}
			}
		}
		throw { message: 'Reel band index ' + reelBandId + ' not found in table ' + this._name };
	};


	OutcomeTable.prototype.assertTotal = function( supposedTotal, stake ) {

		var actualTotal = this.getTotalWeighting( stake );
		if ( actualTotal !== supposedTotal ) {
			throw { message: 'OutcomeTable.assertTotal: Total weighting for table ' + this._name + ' does not match asserted total (actual: ' + actualTotal + ', asserted: ' + supposedTotal + ' stake: ' + stake + ')' };
		}
		if ( actualTotal > 100000 ) {
			// William Hill Vegas requires that any outcome totals above 99999 are divisible by 1000
			if ( ( actualTotal % 1000 ) !== 0 ) {
				throw { message: 'OutcomeTable.assertTotal: Total weighting for table ' + this._name + ' has invalid total for William Hill Vegas platform - any totals above 99999 must be divisible by 1000 (actual: ' + actualTotal + ' stake: ' + stake + ')' };
			}
		}
	};


	OutcomeTable.prototype.assertTotalAllStakes = function( supposedTotal, allStakesArray ) {

		for ( var stakeIndex = 0; stakeIndex < allStakesArray.length; ++stakeIndex ) {

			var stake = allStakesArray[stakeIndex];
			this.assertTotal( supposedTotal, stake );
		}
	};


	OutcomeTable.prototype.getTotalWeighting = function( stake ) {

		var table = this._getStakeOutcomeTable( stake );

		var totalWgt = 0;
		for ( var outcome in table ) {
			if ( table.hasOwnProperty( outcome ) ) {
				var outcomeWeight = Number( table[outcome] );
				if ( isFinite( outcomeWeight ) ) {
					totalWgt += outcomeWeight;
				} else {
					throw { message: 'OutcomeTable.getTotalWeighting: Weight for outcome ' + outcome + ' has invalid value ' + outcomeWeight + ' in table ' + this._name };
				}
			}
		}
		return totalWgt;
	};


	OutcomeTable.prototype.pickOutcomeUsingRandomNumber = function( randomChance, stake ) {

		var totalWeight = this.getTotalWeighting( stake );
		var table = this._getStakeOutcomeTable( stake );

		var runningWeight = 0;
		for ( var outcome in table ) {
			if ( table.hasOwnProperty( outcome ) ) {
				runningWeight += Number( table[outcome] );
				if ( randomChance < runningWeight ) {
					return outcome;
				}
			}
		}
		throw { message: 'OutcomeTable.pickOutcomeUsingRandomNumber failed on table ' + this._name + ' as random chance provided ' + randomChance + ' is larger than total weight ' + totalWeight };
	};


	OutcomeTable.prototype.pickOutcome = function( platformInterface, stake ) {

		var totalWgt = this.getTotalWeighting( stake );
		var chance = platformInterface.getRandomNumber( this._name, 0, totalWgt - 1 );
		return this.pickOutcomeUsingRandomNumber( chance, stake );
	};


	libs.server.OutcomeTable = OutcomeTable;


	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


}());

