goog.provide( 'libs.server.ReelBand' );
goog.provide( 'libs.server.ReelBandSet' );


(function() {
	"use strict";


	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Reel band class manages a single band of reel symbols.

	var ReelBand = function( data ) {

		this._data = data;
	};


	ReelBand.createReelBand = function( symbolReelset, symbolMap, reelSetName, bandIndex ) {
		reelSetName = reelSetName || "Unknown";
		for ( var symbolIndex = 0; symbolIndex < symbolReelset.length; symbolIndex++ ) {
			var symbolShortName = symbolReelset[symbolIndex];
			// make sure it's a valid symbol short name
			if ( !symbolMap.shortNameExists( symbolShortName ) ) {
				throw( {
					friendlyMessage: 'Invalid game data, please reload the game',
					message: 'Invalid symbol encountered in reel set (name=' + reelSetName + ', bandIndex=' + bandIndex + ', symbolIndex=' + symbolIndex + ', symbol=' + symbolShortName + ')'
				} );
			}
		}
		return new ReelBand( symbolReelset.slice( 0 ) );
	};


	ReelBand.prototype.clampPosition = function( position ) {
		if ( !isFinite( position ) ) {
			throw { message: 'ReelBand.prototype.clampPosition: distance must be a finite number' };
		}
		var safeValue = position % this._data.length;
		while ( safeValue < 0 ) {
			safeValue += this._data.length;
		}
		return safeValue;
	};


	ReelBand.prototype.indexExistsInView = function( testIndex, startIndex, rowCount ) {
		for ( var windowIndex = 0; windowIndex < rowCount; ++windowIndex ) {
			var absoluteReelPosition = testIndex + windowIndex;
			var bandPos = this.clampPosition( absoluteReelPosition );
			if ( bandPos === startIndex ) {
				return windowIndex;
			}
		}
		return -1;
	};


	ReelBand.prototype.getSymbol = function( position ) {

		return this._data[this.clampPosition( position )];
	};


	ReelBand.prototype.setSymbol = function( position, symbol ) {

		this._data[this.clampPosition( position )] = symbol;
	};


	ReelBand.prototype.getLength = function() {

		return this._data.length;
	};


	ReelBand.copyTo = function( sourcePos, sourceBand, targetPos, targetBand, copyLength ) {

		copyLength = copyLength || sourceBand.getLength();
		for ( var index = 0; index < copyLength; ++index ) {
			var sourceSymPos = sourceBand.clampPosition( sourcePos - index );
			var targetSymPos = targetBand.clampPosition( targetPos - index );
			targetBand.setSymbol( targetSymPos, sourceBand.getSymbol( sourceSymPos ) );
		}
	};


	ReelBand.prototype.copyTo = function( sourcePos, targetPos, targetBand, copyLength ) {

		ReelBand.copyTo( sourcePos, this, targetPos, targetBand, copyLength );
	};


	ReelBand.prototype.clone = function() {

		return new ReelBand( this._data.slice( 0 ) );
	};


	ReelBand.prototype.replaceAllSymbols = function( oldSymbol, newSymbol ) {

		for ( var stop = 0; stop < this._data.length; stop++ ) {
			if ( this._data[stop] === oldSymbol ) {
				this._data[stop] = newSymbol;
			}
		}
	};


	ReelBand.prototype.toString = function() {

		return this._data.join( "|" );
	};


	ReelBand.prototype.toArray = function() {

		return this._data.slice( 0 );
	};


	ReelBand.prototype.getRows = function( position, rowCount ) {

		var rows = [];
		for ( var index = 0; index < rowCount; ++index ) {
			rows.push( this.getSymbol( position + index ) );
		}
		return rows;
	};


	ReelBand.prototype.isEqual = function( rhs ) {

		if ( rhs._data.length !== this._data.length ) {
			return false;
		}

		for ( var stop = 0; stop < this._data.length; stop++ ) {
			if ( this._data[stop] !== rhs._data[stop] ) {
				return false;
			}
		}
		return true;
	};

	ReelBand.prototype.removeRowIndexFromBand = function( rowIndexToRemove ) {
		this._data.splice( this.clampPosition( rowIndexToRemove ), 1 );
	};

	// This checks the given reel set to make sure certain symbols do not appear within a certain distance of other symbols (e.g. For most games, you do not want progressive jackpot symbols within aperture distance
	// of each other on a reel band). If this fails it is probably a mistake in the spreadsheet.

	// Accepts the rules parameter in the form (this'll make sure that only the S, E or R symbols are next to the WK symbol. Fails if another symbol is found next to the WK).
	// symbol_adjacent_limit = [ { symbol:symbol_codes_ids_map.WK, only:[ symbol_codes_ids_map.S, symbol_codes_ids_map.E, symbol_codes_ids_map.R ], distance: 1 } ];

	// You can do the opposite and make sure that certain symbols don't appear next to certain symbols using the 'not' parameter:
	// This makes sure the WK symbol does not appear next to another WK symbol within 3 symbol spaces, fails if it finds one:
	// symbol_adjacent_limit = [ { symbol:symbol_codes_ids_map.WK, not:[ symbol_codes_ids_map.WK ], distance: 3 } ];
	ReelBand.prototype.verifySymbolAdjacencyRules = function( symbol_adjacent_limit, reelSetName, bandIndex ) {
		reelSetName = reelSetName || "Unknown";

		for ( var symbolIndex = 0; symbolIndex < this._data.length; ++symbolIndex ) {
			var testSymbol = this.getSymbol( symbolIndex );

			for ( var limitedSymbolIndex = 0; limitedSymbolIndex < symbol_adjacent_limit.length; ++limitedSymbolIndex ) {
				var limitedSymbol = symbol_adjacent_limit[limitedSymbolIndex];
				// found a symbol which has adjacent limitations
				if ( limitedSymbol.symbol === testSymbol ) {
					var testDistance = limitedSymbol.distance || 1;

					// check adjacent symbols (above and below)
					for ( var distance = 1; distance <= testDistance; ++distance ) {

						var aboveSym = this.getSymbol( symbolIndex - distance );
						var belowSym = this.getSymbol( symbolIndex + distance );

						if ( limitedSymbol.only ) {
							// 'only' method specified. This'll test to make sure ONLY the symbols in the 'only' array appear next to the test symbol.
							if ( limitedSymbol.only.indexOf( aboveSym ) < 0 || limitedSymbol.only.indexOf( belowSym ) < 0 ) {
								throw( {
									friendlyMessage: 'Invalid game data, please reload the game',
									message: 'Reel set has invalid symbol combination at (name=' + reelSetName + ', bandIndex=' + bandIndex + ', symbolIndex=' + symbolIndex + ')'
								} );
							}
						}

						if ( limitedSymbol.not ) {
							// 'not' method specified. This'll test to make sure that the symbols in the 'not' array do not appear next to the test symbol.
							if ( limitedSymbol.not.indexOf( aboveSym ) >= 0 || limitedSymbol.not.indexOf( belowSym ) >= 0 ) {
								throw( {
									friendlyMessage: 'Invalid game data, please reload the game',
									message: 'Reel set has invalid symbol combination at (name=' + reelSetName + ', bandIndex=' + bandIndex + ', symbolIndex=' + symbolIndex + ')'
								} );
							}
						}
					}
				}
			}
		}
	};

	libs.server.ReelBand = ReelBand;


	var ReelBandSet = function( dataArray, reelSetName ) {
		this._reelSetName = reelSetName;
		if ( dataArray ) {
			this._bands = dataArray.map( function( data ) { return new ReelBand( data ); } );
		}
	};


	ReelBandSet.prototype.getName = function() {
		return this._reelSetName;
	};


	// This converts a given reel set from letter codes to numbers according to the provided map
	ReelBandSet.createReelBandSet = function( symbolReelset, symbolMap, reelSetName ) {

		var rbs = new ReelBandSet( null, reelSetName );
		var bandIndex = 0;
		rbs._bands = symbolReelset.map( function( band ) { return ReelBand.createReelBand( band, symbolMap, reelSetName, bandIndex++ ); } );
		return rbs;
	};


	ReelBandSet.prototype.verifySymbolAdjacencyRules = function( symbol_adjacent_limit ) {

		var name = this._reelSetName;
		var bandIndex = 0;
		this._bands.forEach( function( rb ) { rb.verifySymbolAdjacencyRules( symbol_adjacent_limit, name, bandIndex++ ); } );
	};


	ReelBandSet.prototype.replaceAllSymbols = function( oldSymbol, newSymbol ) {

		this._bands.forEach( function( rb ) { rb.replaceAllSymbols( oldSymbol, newSymbol ); } );
	};


	ReelBandSet.prototype.clone = function() {

		var rbs = new ReelBandSet();
		rbs._bands = this._bands.map( function( rb ) { return rb.clone(); } );
		rbs._reelSetName = this._reelSetName;
		return rbs;
	};


	ReelBandSet.prototype.getScreen = function( position, rowCount ) {

		return this._bands.map( function( rb ) { return rb.getRows( position, rowCount ); } );
	};


	ReelBandSet.prototype.isEqual = function( rhs ) {

		if ( this._bands.length !== rhs._bands.length ) {
			return false;
		}

		for ( var reelIndex = 0; reelIndex < this._bands.length; ++reelIndex ) {
			if ( !this._bands[reelIndex].isEqual( rhs._bands[reelIndex] ) ) {
				return false;
			}
		}
		return true;
	};


	ReelBandSet.prototype.toStringArray = function() {

		return this._bands.map( function( band ) { return band.toString(); } );
	};


	ReelBandSet.prototype.toArray = function() {

		return this._bands.map( function( band ) { return band.toArray(); } );
	};


	ReelBandSet.prototype.getBandCount = function() {

		return this._bands.length;
	};


	ReelBandSet.prototype.getBand = function( bandIndex ) {

		return this._bands[bandIndex];
	};


	ReelBandSet.prototype.setBand = function( bandIndex, band ) {

		this._bands[bandIndex] = band;
	};


	libs.server.ReelBandSet = ReelBandSet;


}());
