/**************************************************************************
 SlotGameLogic (top level 'class')
 properties:
 nreels, nrows                        - screen display size - nreels (columns) and nrows(across the screen)
 slots_symbol_set                    - symbol-set for the slot game - i.e. an array[nreels] of reel-symbol-sets (each reel may have its own length)
 Note:
 - symbols are indexed from 0+ (?) - their symbol-id - there will also (prob) be a lookup array to map these symbol-ids to symbol-strings
 - a symbol-set is an array of symbol-ids
 - a reel-symbol-sets gives the symbol-ids along the reel strip
 - a slots-symbol-set is an array[nreels] of reel-symbol-sets (for each reel)
 slots_position                        - array[nreels] of indexes (0+) into the reel-symbol-sets for each reel
 Note:
 - the index for a reel is the location of the reel-symbol-set on the top (irow=0) [[[[<<<< or 2nd (usually middle) (irow=1)]]]
 screen_symbols                        - a 2D array of [nreels][nrows] (note cols before rows) of symbol-ids
 methods:
 setScreenSize(nreels, nrows)    - sets the screen display size - nreels (columns) and nrows(across the screen)
 setSymbolSet(slots_symbol_set)    - sets the slots_symbol_set
 setPosition(set_reel_positions, type)    - sets the slot position - the set_reel_positions allows you control how specific positions are set
 - set_reel_positions a list for each reel - if number sets that as the posn for that reel, if '-' dont change it, if '*' do RNG for that reel
 - set_reel_positions undefined (or not above) - use RNG for all reels - i.e. what would typically be used for a spin
 - type - text used for logging the RNG on server (if undefined )
 setScreenSymbols(position_arr)        - set the set of positions explicitly (for sticky symbols) - use after setPosition
 - position_arr is an array of position objects : {reel:<reel>, row:<row>, symbol:<symbol_id>}
 lineWinLR(lineindex, line_definition_arr, minsymbols, matchable_symbol_set, wild_symbol_set) -> testresult
 - tests for a line win of adjacent same symbols LR with support for
 - lineindex is the index number of the line (0+) in the line_definitions
 - line_definition_arr is an array of line-definitions
 where a line-definition is an array[nreels] of row indexes (a typical V line would be [0, 1, 2, 1, 0])
 - minsymbols is the minimum that makes the test pass
 - matchable_symbol_set - a set of symbols which are to be tested for
 - wild_symbol_set - set of symbols which are wild (can match any matchable symbol)
 -> testresult .pass (=true or false) [if true] .symbol (=symbol-id) .run (=nsymbolsgot) .linesymbols (=symbol_set)
 screenScatterWin(minsymbols, scatter_symbol_set) -> testresult
 - tests for a scatter win of any of scatter_symbol_set anywhere on the visible screen
 - minsymbols is the minimum that makes the test pass
 - scatter_symbol_set - a set of symbols which are to be tested for
 -> testresult .pass (=true or false) [if true] .got (=nsymbolsgot)] .positions (=position_arr)
 where position_arr[nsymbolsgot] of position objects : {reel:<reel>, row:<row>, symbol:<symbol_id>}
 columnsScatterWin(minsymbols, column_symbol_set_arr) -> testresult
 - tests each reel(=column) one of the symbol_set assigned for each column
 - i.e. column_symbol_set_arr [nreels] is the allowed symbol_set for that column
 - to permit any symbol in a column(=reel) specify an empty array in the symbol_set
 - minsymbols is the minimum that makes the test pass
 -> testresult .pass (=true or false) [if true] .got (=nsymbolsgot)] .positions (=position_arr)
 where position_arr[nsymbolsgot] of position objects : {reel:<reel>, row:<row>, symbol:<symbol_id>}
 **************************************************************************/

goog.provide( 'libs.server.SlotGameLogic' );


(function() {
	"use strict";

	var SlotGameLogic = function( reelCount, rowCount ) {
		this.nreels = Number( reelCount );
		this.nrows = Number( rowCount );
		this.slots_symbol_set = null;
		this.slots_position = [];
		for ( var reel = 0; reel < this.nreels; reel++ ) {
			this.slots_position[reel] = 0;
		}
		this.screen_symbols = null;
	};


	SlotGameLogic.prototype.buildReelsScreenString = function() {

		var str = '';
		for ( var row = 0; row < this.nrows; row++ ) {
			for ( var reel = 0; reel < this.nreels; reel++ ) {
				str += this.screen_symbols[reel][row];
				if ( reel < this.nreels - 1 ) {
					str += "|"; // between symbols on a row
				}
			}
			if ( row < this.nrows - 1 ) {
				str += "|"; // between rows
			}
		}
		return str;
	};


	SlotGameLogic.prototype.buildSlotsString = function() {

		return this.slots_position.join( "|" );
	};


	SlotGameLogic.prototype.setSymbolSet = function( slots_symbol_set ) {
		if ( (typeof(slots_symbol_set) !== 'object') || (slots_symbol_set.constructor !== libs.server.ReelBandSet) ) {
			throw { name: 'SlotGameLogic.Error', message: 'SlotGameLogic.setSymbolSet invalid parameters' };
		}
		this.slots_symbol_set = slots_symbol_set.clone();
	};

	SlotGameLogic.prototype.setPosition = function( platformInterface, set_reel_positions, type ) {
		var reel_options = [];
		var option;
		var reel;
		var reellen;
		var reelposn;
		if ( (set_reel_positions) && (typeof(set_reel_positions) === 'object') && (set_reel_positions.constructor === Array) ) {
			reel_options = set_reel_positions;
		}
		for ( reel = 0; reel < this.nreels; reel++ ) {
			reellen = this.slots_symbol_set.getBand( reel ).getLength();
			option = '*';
			if ( reel_options.length > reel ) {
				option = reel_options[reel];
			}
			if ( option === '*' ) {
				reelposn = platformInterface.getRandomNumber( type || "SlotGameLogic.setPosition", 0, reellen - 1 );
			}
			else if ( option === '-' ) {
				reelposn = this.slots_position[reel];
			}
			else {
				reelposn = Number( option );
			}
			this.slots_position[reel] = Number( reelposn );
		}

		this.recalculateScreenSymbols();
	};

	SlotGameLogic.prototype.setScreenSymbols = function( position_arr ) {
		var i;
		var reel;
		var row;
		for ( i = 0; i < position_arr.length; i++ ) {
			reel = Number( position_arr[i].reel );
			row = Number( position_arr[i].row );
			this.screen_symbols[reel][row] = Number( position_arr[i].symbol );
		}
	};

	SlotGameLogic.prototype.removeSymbolsFromReelBand = function( reel, rowsToRemove ) {
		if ( rowsToRemove.length > 0 ) {
			var reelBand = this.slots_symbol_set.getBand( reel );

			var rowIndexesToRemove = [];
			for ( var row = 0; row < rowsToRemove.length; ++row ) {
				rowIndexesToRemove.push( reelBand.clampPosition( this.slots_position[reel] + rowsToRemove[row] ) );
			}

			// sort into ascending order
			rowIndexesToRemove.sort( function( a, b ) {
				return a - b;
			} );

			var slotPositionsToMove = rowIndexesToRemove.length;
			for ( var i = rowIndexesToRemove.length - 1; i >= 0; --i ) {
				reelBand.removeRowIndexFromBand( rowIndexesToRemove[i] );

				// usually, wind back the reelpos to the amount of symbols you pop to keep the same reelpos relative to
				// the less number of symbols you have. However, if the row popped is before the current reelpos, need
				// to adjust the reelpos even more, as it's been taken away before your current index rather than after
				if ( rowIndexesToRemove[i] < this.slots_position[reel] ) {
					++slotPositionsToMove;
				}
			}

			this.slots_position[reel] = reelBand.clampPosition( this.slots_position[reel] - slotPositionsToMove );

			this.recalculateScreenSymbols();
		}
	};

	SlotGameLogic.prototype.recalculateScreenSymbols = function() {
		this.screen_symbols = [];
		for ( var reel = 0; reel < this.nreels; reel++ ) {
			this.screen_symbols[reel] = [];
			for ( var row = 0; row < this.nrows; row++ ) {
				var posn = this.slots_position[reel] + row; // i.e. based from top
				if ( posn < 0 ) {
					posn = this.slots_symbol_set.getBand( reel ).getLength() + posn;
				}
				if ( posn >= this.slots_symbol_set.getBand( reel ).getLength() ) {
					posn = posn - this.slots_symbol_set.getBand( reel ).getLength();
				}
				this.screen_symbols[reel][row] = this.slots_symbol_set.getBand( reel ).getSymbol( posn );
			}
		}
	};

	var _indexin = function( item, array ) {
		var posn = -1;
		for ( var i = 0; i < array.length; i++ ) {
			if ( array[i] === item ) {
				posn = i;
				break;
			}
		}
		return posn;
	};


	SlotGameLogic.analyseAllLineWins = function( screen_symbols, awardTable, stakePerLine, lineCount, line_definition_arr, minsymbols, matchable_symbols_list, wild_symbol_set,
		replacePureWildWinlinesWithThisSymbol ) {

		var winlines = [];
		for ( var line = 0; line < lineCount; line++ ) {
			var lineDefinition = line_definition_arr[line];
			// test if the 'pure-wild' winline is worth more than the regular win if 2 wins are available on the same winline
			var wildsWinamount = new BigNumber( 0 );
			var testWildsResult = SlotGameLogic.lineWinLR( screen_symbols, lineDefinition, minsymbols, wild_symbol_set, [] );
			var wildWinSymbol = replacePureWildWinlinesWithThisSymbol ? replacePureWildWinlinesWithThisSymbol : testWildsResult.symbol;
			if ( testWildsResult.pass ) {
				// Note: don't need to multiply by currency rate as stakePerLine is already multiplied
				wildsWinamount = awardTable.calculateAward( stakePerLine, testWildsResult.symbol, testWildsResult.run );
			}
			var testResult = SlotGameLogic.lineWinLR( screen_symbols, lineDefinition, minsymbols, matchable_symbols_list, wild_symbol_set );
			if ( testResult.pass ) {
				var winamount = awardTable.calculateAward( stakePerLine, testResult.symbol, testResult.run );
				if ( winamount.gt( 0 ) ) {
					if ( wildsWinamount.equals( 0 ) || winamount.greaterThanOrEqualTo( wildsWinamount ) ) {
						var sym = testResult.symbol;
						if ( replacePureWildWinlinesWithThisSymbol && wild_symbol_set.indexOf( testResult.symbol ) >= 0 ) {
							sym = replacePureWildWinlinesWithThisSymbol;
						}
						winlines.push( {
							'line': line,
							'win': winamount,
							'run': testResult.run,
							'symbol': sym
						} );
					}
					else {
						winlines.push( {
							'line': line,
							'win': wildsWinamount,
							'run': testWildsResult.run,
							'symbol': wildWinSymbol
						} );
					}
				}
			} else {
				if ( wildsWinamount.greaterThan( 0 ) ) {	// wild-only win
					winlines.push( {
						'line': line,
						'win': wildsWinamount,
						'run': testWildsResult.run,
						'symbol': wildWinSymbol
					} );
				}
			}
		}
		return winlines;
	};


	var _getLineSymbols = function( screen_symbols, lineDefinition ) {
		var linesymbols = [];
		for ( var reel = 0; reel < screen_symbols.length; reel++ ) {
			var lineOffset = lineDefinition[reel];
			linesymbols.push( screen_symbols[reel][lineOffset] );
		}
		return linesymbols;
	};


	// Static version of lineWinLR.
	SlotGameLogic.lineWinLR = function( screen_symbols, lineDefinition, minsymbols, matchable_symbol_set, wild_symbol_set ) {

		var testresult = { pass: false, run: 0, symbol: -1 };
		var linesymbols = _getLineSymbols( screen_symbols, lineDefinition );
		var symbol_id = linesymbols[0]; // may be wild
		var symbol_wild = (_indexin( symbol_id, wild_symbol_set ) !== -1);
		if ( (_indexin( symbol_id, matchable_symbol_set ) !== -1) || symbol_wild ) {
			var nsymbolsgot = 1;
			var reel;
			var current_symbol;
			var current_wild;
			for ( reel = 1; reel < screen_symbols.length; reel++ ) {
				current_symbol = linesymbols[reel];
				current_wild = (_indexin( current_symbol, wild_symbol_set ) !== -1);
				if ( symbol_wild ) {
					if ( current_wild ) {
						nsymbolsgot++;
					}
					else if ( _indexin( current_symbol, matchable_symbol_set ) !== -1 ) {
						nsymbolsgot++;
						symbol_wild = false;
						symbol_id = current_symbol;
					}
					else {
						break;
					}
				}
				else if ( (current_symbol === symbol_id) || current_wild ) {
					nsymbolsgot++;
				}
				else {
					break;
				}
			}
			if ( nsymbolsgot >= minsymbols ) {
				testresult.pass = true;
			}
			testresult.run = nsymbolsgot;
			testresult.symbol = symbol_id; // could still be wild
			testresult.linesymbols = linesymbols;
		}
		return testresult;
	};


	SlotGameLogic.screenScatterWin = function( screen_symbols, minsymbols, scatter_symbol_set ) {
		var testresult = { pass: false };
		var nsymbolsgot = 0;
		var positions_arr = [];

		var reel;
		var row;
		var symbol_id;

		for ( reel = 0; reel < screen_symbols.length; reel++ ) {
			for ( row = 0; row < screen_symbols[reel].length; row++ ) {
				symbol_id = screen_symbols[reel][row];
				if ( _indexin( symbol_id, scatter_symbol_set ) !== -1 ) {
					nsymbolsgot++;
					positions_arr.push( { 'reel': reel, 'row': row, 'symbol': symbol_id } );
				}
			}
		}

		if ( nsymbolsgot >= minsymbols ) {
			testresult.pass = true;
		}
		testresult.got = nsymbolsgot;
		testresult.positions = positions_arr;
		return testresult;
	};

	libs.server.SlotGameLogic = SlotGameLogic;

}());
