/*jslint node: true */

"use strict";

var sync = require( 'synchronize' );
var log = require( './log.js' );

// ClientConnection validates the request JSON received, translates it into a virgo message and passes onto
// processing by the VirgoMessageHandler

var VirgoMessageHandler = require( './virgoMessageHandler.js' ).VirgoMessageHandler;


var VirgoMethodTypes = {

	// Common methods
	getGameType: "VIRGO_GAME_TYPE",
	getGameIdentifier: "VIRGO_GAME_IDENTIFIER",
	getRemoteApiVersion: "VIRGO_REMOTE_API_VERSION",
	getJsonRPC2Version: "VIRGO_JSONRPC2_API_VERSION",

	checkpoint: "CHECKPOINT",
	walletType: "VIRGO_WALLET_TYPE",
	getVirginState: "GET_VIRGIN_STATE",

	// Progressive jackpots
	refreshProgressive: "REFRESH_PROGRESSIVE",

	// Early pay specific
	earlyPayCheckpoint: "EARLY_PAY_CHECKPOINT",
	earlyPayStartGame: "EARLY_PAY_START_GAME",
	earlyPayEndGame: "EARLY_PAY_END_GAME",

	// Late pay specific
	latePayStartGame: "LATE_PAY_START_GAME",
	latePayEndGame: "LATE_PAY_END_GAME",
	latePayGamble: "LATE_PAY_GAMBLE",
	latePayContinueGame: "LATE_PAY_CONTINUE_GAME",
	latePayAwardWinnings: "LATE_PAY_AWARD_WIN"

};


var ClientConnection = function( commandLineArgs ) {

	this._logCallback = null;
	this._commandLineArgs = commandLineArgs;
	this._completeCallback = null;
	this._completeCalled = false;
};


ClientConnection.prototype.onLog = function( logCallback ) {
	this._logCallback = logCallback;
};


ClientConnection.prototype.onComplete = function( completeCallback ) {
	this._completeCallback = completeCallback;
};


ClientConnection.prototype._executeLog = function( err, msgType, msg ) {
	if ( this._logCallback ) {
		this._logCallback( err, msgType, msg );
	}
};


ClientConnection.prototype._executeComplete = function( err, responseJSON, httpCode ) {

	if ( !this._completeCalled ) {
		this._completeCalled = true;
		if ( this._completeCallback ) {
			this._completeCallback( err, responseJSON, httpCode );
		}
	} else {
		if ( err ) {
			// complete called back with error even though it's already been executed, call log with nonfatal error
			this._executeLog( err, 'NONFATAL' );
		}
	}
};


ClientConnection.prototype.handleRequest = function( requestBody ) {

	var requestJSON;
	try {
		//console.log("Request data:" + requestBody);
		requestJSON = JSON.parse( requestBody );

		if ( this._commandLineArgs.debug ) {
			log.debug( "Request received: " + JSON.stringify( requestJSON, null, '\t' ) );
		}

		this._validateRequestJSON( requestJSON );
	}
	catch ( err ) {
		// invalid request
		this._executeComplete( err, null, 400 /* Invalid request, either invalid JSON in request body or unexpected values within */ );
		return;
	}

	try {
		var that = this;
		// All client code is ran in a synchronize fiber to allow for the asynchronous http requests sent to virgo to be
		// made synchronous
		sync.fiber( function() {
			try {
				that._handleMessage( requestJSON );
			}
			catch ( err ) {
				that._executeComplete( err );
			}
		} );
	}
	catch ( err ) {
		// generic error has occurred
		this._executeComplete( err );
	}
};


ClientConnection.prototype._validateRequestJSON = function( requestJSON ) {

	if ( !requestJSON['jsonrpc'] || requestJSON['jsonrpc'].length === 0 ) {
		throw new Error( "Invalid or missing jsonrpc parameter in request JSON" );
	}

	if ( !requestJSON['id'] || requestJSON['id'].length === 0 ) {
		throw new Error( "Invalid or missing id parameter in request JSON" );
	}

	if ( !requestJSON['method'] || requestJSON['method'].length === 0 ) {
		throw new Error( "Invalid or missing method parameter in request JSON" );
	}

	// make sure method name exists in VirgoMethodTypes object (disabled for now), will just return 501 message if unsupported command sent
	// if ( !Object.keys( VirgoMethodTypes ).some( function( methodNameKey ) { return requestJSON['method'].toUpperCase() === VirgoMethodTypes[ methodNameKey ].toUpperCase(); } ) ) {
	// throw new Error( "Method parameter '" + requestJSON['method'] + "' unrecognised" );
	// }
};


ClientConnection.prototype._handleMessage = function( requestJSON ) {

	var that = this;
	var completeCallback = function( err, responseJSON, httpCode ) {
		that._executeComplete( err, {
			'jsonrpc': requestJSON['jsonrpc'],
			'id': requestJSON['id'],
			'result': responseJSON
		}, httpCode );
	};

	var virgoMessageHandler = new VirgoMessageHandler( this._commandLineArgs, requestJSON['params'] );
	virgoMessageHandler.onLog( function( err, msgType, msg ) { that._executeLog( err, msgType, msg ); } );

	switch ( requestJSON['method'].toUpperCase() ) {

		case VirgoMethodTypes.getGameType :
			virgoMessageHandler.getGameType( completeCallback );
			break;
		case VirgoMethodTypes.getGameIdentifier :
			virgoMessageHandler.getGameIdentifier( completeCallback );
			break;

		case VirgoMethodTypes.getRemoteApiVersion:
			virgoMessageHandler.getRemoteApiVersion( completeCallback );
			break;
		case VirgoMethodTypes.getJsonRPC2Version:
			virgoMessageHandler.getJsonRPC2Version( completeCallback );
			break;

		case VirgoMethodTypes.checkpoint:
		case VirgoMethodTypes.earlyPayCheckpoint:
			virgoMessageHandler.checkpoint( completeCallback );
			break;
		case VirgoMethodTypes.walletType :
			virgoMessageHandler.walletType( completeCallback );
			break;
		case VirgoMethodTypes.getVirginState :
			virgoMessageHandler.getVirginState( completeCallback );
			break;

		// Progressive jackpots
		case VirgoMethodTypes.refreshProgressive :
			virgoMessageHandler.refreshProgressive( completeCallback );
			break;

		// Early pay specific
		case VirgoMethodTypes.earlyPayStartGame :
			virgoMessageHandler.earlyPayStartGame( completeCallback );
			break;
		case VirgoMethodTypes.earlyPayEndGame :
			virgoMessageHandler.earlyPayEndGame( completeCallback );
			break;

		// Late pay specific
		case VirgoMethodTypes.latePayStartGame :
		case VirgoMethodTypes.latePayEndGame :
		case VirgoMethodTypes.latePayGamble :
		case VirgoMethodTypes.latePayContinueGame :
		case VirgoMethodTypes.latePayAwardWinnings :
			this._executeComplete( 'Late pay unsupported', null, 501 );
			break;
		default:
			this._executeComplete( 'Unsupported command "' + requestJSON['method'] + '"', null, 501 );
			break;
	}
};


exports.ClientConnection = ClientConnection;

