/*jslint node: true */

// Manages the HTTP server game clients connect to, then creates instances of ClientConnection to handle the messages

"use strict";

var http = require( 'http' );
var os = require( "os" );
var log = require( './log.js' );
var ClientConnection = require( './clientConnection.js' ).ClientConnection;


var ClientManager = function( port, host, commandLineArgs ) {

	var that = this;
	this._commandLineArgs = commandLineArgs;
	this._httpServer = http.createServer( function( req, res ) { that._onHttpRequest( req, res ); } );
	this._httpServer.listen( port, host, function() {
		var host = os.hostname() || 'localhost';
		console.log( 'Server running at http://' + host + ':' + port + '/' );
	} );
};


ClientManager.prototype._onClientLog = function( err, msgType, msg ) {

	if ( err ) {
		msgType = msgType || 'FATAL';
		msg = msg || '';
	}

	var finalMsg = "[" + msgType + "] " + msg;
	console.log( finalMsg );

	if ( err ) {
		log.error( finalMsg );

		if ( typeof err === 'object' ) {
			if ( err.stack ) {
				log.error( err.stack );
			}
			if ( err.msg ) {
				log.error( err.msg );
			}
			if ( err.message ) {
				log.error( err.message );
			}
			try {
				log.error( JSON.stringify( err ) );
			}
			catch ( err ) {}
		} else {
			log.error( err.toString() );
		}
	} else {
		log.info( finalMsg );
	}
};


ClientManager.prototype._onClientComplete = function( err, requestBody, response, responseJSON, httpCode ) {

	if ( !err ) {
		response.writeHeader( httpCode || 200 /* HTTP OK */, {
			"Content-Type": "application/json",
			'Content-Encoding': 'gzip'
		} );
		var responseString = JSON.stringify( responseJSON );

		if ( this._commandLineArgs.debug ) {
			log.debug( "Response sent: " + JSON.stringify( responseJSON, null, '\t' ) );
		}

		response.write( responseString );
	} else {
		this._onClientLog( err );
		this._onClientLog( "Error from request: " + requestBody );
		response.writeHeader( httpCode || 500 /* internal server error */, {
			"Content-Type": "application/json",
			'Content-Encoding': 'gzip'
		} );
		// TODO: is this the correct response on error?
		response.write( JSON.stringify( { error: 'Error has occurred' } ) );
	}
	response.end();
};


ClientManager.prototype._onHttpRequest = function( request, response ) {

	var that = this;
	if ( request.method === 'POST' ) {
		request.setEncoding( 'utf-8' );

		request.on( 'error', function( err ) {
			that._onClientLog( err );
		} );

		var requestBody = '';
		request.on( 'data', function( chunk ) {
			requestBody += chunk;
		} );

		request.on( 'end', function() {
			try {
				var client = new ClientConnection( that._commandLineArgs );
				client.onLog( function( err, msgType, msg ) { that._onClientLog( err, msgType, msg ); } );
				client.onComplete( function( err, responseJSON, httpCode ) { that._onClientComplete( err, requestBody, response, responseJSON, httpCode ); } );
				client.handleRequest( requestBody );
			}
			catch ( err ) {
				// shouldn't happen, handle anyway, write back error to client
				that._onClientComplete( err, requestBody, response );
			}
		} );
	}
};


exports.ClientManager = ClientManager;

