/*jslint node: true */

"use strict";

var UUID = require( 'node-uuid' );
var path = require( 'path' );
var log = require( './log.js' );
var VirgoPlatformInterface = require( './virgoPlatformInterface.js' ).VirgoPlatformInterface;


var getCurrencyCode = function( requestParams, gameState ) {
	// Extract currency code from transaction amount if available.
	if ( requestParams['TransactionAmount'] && requestParams['TransactionAmount']['CurrencyCode'] ) {
		return requestParams['TransactionAmount']['CurrencyCode'];
	} else {
		// if not available from transaction amount, try to retreive from saved value
		if ( gameState && gameState['currencyCode'] ) {
			return gameState['currencyCode'];
		}
	}
	return '';
};


var VirgoMessageHandler = function( commandLineArgs, requestParams ) {

	var that = this;
	this._logCallback = null;
	this._commandLineArgs = commandLineArgs;
	this._requestParams = requestParams;
	this._customRequestParams = {};

	this._importGameLogicCode();

	this._stake = new BigNumber( 0 );
	this._lines = 20;
	if ( this._requestParams['TransactionAmount'] && this._requestParams['TransactionAmount']['Amount'] ) {
		this._stake = new BigNumber( this._requestParams['TransactionAmount']['Amount'] );
	}

	if ( this._requestParams && this._requestParams['RequestJSON'] && this._requestParams['RequestJSON']['RawJSON'] ) {
		try {
			this._customRequestParams = libs.xmlUtils.jsonParse( this._requestParams['RequestJSON']['RawJSON'] );
			if ( isFinite( this._customRequestParams['lines'] ) ) {
				this._lines = this._customRequestParams['lines'];
			}
		}
		catch ( err ) {
		}
	}

	this._currentState = null;
	this._betSelection = {};

	if ( this._requestParams['StateJSON'] && this._requestParams['StateJSON']['RawJSON'] ) {
		var rawStateJson = libs.xmlUtils.jsonParse( this._requestParams['StateJSON']['RawJSON'] );
		if ( rawStateJson['value'] ) {
			var stateJson = libs.xmlUtils.jsonParse( rawStateJson['value'] );
			if ( stateJson ) {
				this._currentState = stateJson;
				if ( this._currentState['betSelection'] ) {
					this._betSelection = this._currentState['betSelection'];
					this._stake = this._betSelection['total_stake'];
					this._lines = this._betSelection['lines'];
				}
			}
		}
	}

	if ( !this._currentState ) {
		this._currentState = {};
	}
	if ( !this._currentState['serverState'] ) {
		this._currentState['serverState'] = {};
	}
	if ( !this._currentState['serverState']['player'] ) {
		this._currentState['serverState']['player'] = {};
	}
	if ( !this._currentState['serverState']['game'] ) {
		this._currentState['serverState']['game'] = {};
	}

	this._playerId = null;

	if ( this._currentState['serverState']['player'] && this._currentState['serverState']['player']['uniqueVirgoId'] ) {
		this._playerId = this._currentState['serverState']['player']['uniqueVirgoId'];
	}

	if ( !this._playerId ) {
		this._playerId = UUID.v1().toString();
		this._currentState['serverState']['player']['uniqueVirgoId'] = this._playerId;
	}

	// Extract currency code, save it as part of the game's state
	var currencyCode = getCurrencyCode( requestParams, this._currentState['serverState']['game'] );
	this._currentState['serverState']['game']['currencyCode'] = currencyCode;
	this._currencyMultiplier = game.logic.GameData.williamHill.currencyCodeRate[currencyCode] || 1;

	this._platformInterface = new VirgoPlatformInterface( requestParams, currencyCode );
	this._platformInterface.onExternalHttpRequest( function( url, json ) { that._onExternalHttpRequest( url, json ); } );
	this._platformInterface.onExternalHttpResponse( function( url, json ) { that._onExternalHttpResponse( url, json ); } );
};


VirgoMessageHandler.prototype._onExternalHttpRequest = function( toUrl, requestJson ) {

	if ( this._commandLineArgs.debug ) {
		log.debug( "Request sent to '" + toUrl + "' : " + libs.xmlUtils.jsonStringify( requestJson, null, '\t' ) );
	}
};


VirgoMessageHandler.prototype._onExternalHttpResponse = function( fromUrl, responseJson ) {

	if ( this._commandLineArgs.debug ) {
		log.debug( "Response received from '" + fromUrl + "' : " + libs.xmlUtils.jsonStringify( responseJson, null, '\t' ) );
	}
};


VirgoMessageHandler.prototype.onLog = function( logCallback ) {
	this._logCallback = logCallback;
};


VirgoMessageHandler.prototype._executeLog = function( err, msgType, msg ) {
	if ( this._logCallback ) {
		this._logCallback( err, msgType, msg );
	}
};


VirgoMessageHandler.prototype.getGameType = function( completeCallback ) {

	//'LATE_PAY_SLOT'
	completeCallback( null, 'EARLY_PAY_SLOT' );
};


VirgoMessageHandler.prototype.getGameIdentifier = function( completeCallback ) {

	completeCallback( null, {
		'GameUID': game.logic.GameData.virgo.gameUID || 'EXAMPLE-VIRGO-GAME',
		'GameName': game.logic.GameData.friendlyGameName || 'Example Virgo Slot Game'
	} );
};


VirgoMessageHandler.prototype.getRemoteApiVersion = function( completeCallback ) {

	completeCallback( null, '0.1' );
};


VirgoMessageHandler.prototype.getJsonRPC2Version = function( completeCallback ) {

	completeCallback( null, '2.0' );
};


VirgoMessageHandler.prototype.checkpoint = function( completeCallback ) {

	this._platformInterface.updateJackpotValues();
	this._betSelection['checkpointId'] = parseInt( this._betSelection['checkpointId'], 10 ) + 1;
	var checkpointJson = this._getCheckpointJson();
	completeCallback( null, checkpointJson );
};


VirgoMessageHandler.prototype.walletType = function( completeCallback ) {

	completeCallback( null, {
		'Identifier': 'GENERIC-SINGLE-METER', // UK-CAT-D-DUAL-METER
		'ILLEGAL_CHARACTER_REGEX': {
			'pattern': '[^\\dA-Z\\-]+',
			'flags': 0
		}
	} );
};


VirgoMessageHandler.prototype._importGameLogicCode = function() {

	this._bootstrap = require( path.join( this._commandLineArgs.logicPath, '../Libs_ServerCore/js/nodeJsLogicBootstrap.js' ) );
	this._bootstrap.initVm( this._commandLineArgs.logicPath, null, true );
};


VirgoMessageHandler.prototype.getVirginState = function( completeCallback ) {

	this._currentState['jackpotState'] = this._getJackpotStateJson();
	var reallyRawJson = libs.xmlUtils.jsonStringify( this._currentState );
	var rawJson = JSON.stringify( { 'value': reallyRawJson } );

	completeCallback( null, {
		"MutatedState": {
			"RawJSON": rawJson
		}
	} );
};


VirgoMessageHandler.prototype.refreshProgressive = function( completeCallback ) {

	this._platformInterface.updateJackpotValues();
	this._currentState['jackpotState'] = this._getJackpotStateJson();
	var reallyRawJson = libs.xmlUtils.jsonStringify( this._currentState );
	var rawJson = JSON.stringify( { 'value': reallyRawJson } );

	completeCallback( null, {
		"MutatedState": {
			"RawJSON": rawJson
		}
	} );
};


VirgoMessageHandler.prototype._calculateResult = function() {

	var serverBetResults;

	try {
		var isFirstOperation = this._betSelection['checkpointId'] === 0;

		// if ( isFirstOperation ) {
		// var forceStringAlreadySet = VirgoPlatformInterface.forcedRngString && VirgoPlatformInterface.forcedRngString.length > 0;
		// if ( !forceStringAlreadySet ) {
		// var forcedRng = this._commandLineArgs.forcedRNG;
		// if ( forcedRng && forcedRng.length > 0 ) {
		// VirgoPlatformInterface.forcedRngString = forcedRng;
		// }
		// }
		// }

		if ( !this._currentState ) {
			this._currentState = {};
		}

		if ( isFirstOperation ) {
			this._currentState['betSelection'] = this._betSelection;
		}

		var stateDesc = '';
		try {
			stateDesc = libs.xmlUtils.jsonStringify( this._currentState );
		}
		catch ( err ) {
			stateDesc = 'Could not parse state';
		}

		// check if post data exists
		if ( this._customRequestParams['postData'] ) {
			this._currentState['postData'] = libs.xmlUtils.jsonParse( this._customRequestParams['postData'] );
		}

		log.info( "[" + this._playerId + "] Playing spin... [State: " + stateDesc + "]" );

		serverBetResults = game.logic.BuildBetResults( this._platformInterface, this._currentState );
		serverBetResults['isCheckpoint'] = !isFirstOperation;

		var resultDesc = '';
		try {
			resultDesc = libs.xmlUtils.jsonStringify( serverBetResults );
		}
		catch ( err ) {
			resultDesc = 'Could not parse result';
		}

		this._currentState['lastGame'] = serverBetResults;
		if ( isFirstOperation ) {
			this._currentState['jackpotState'] = this._getJackpotStateJson();
		}

		log.info( "[" + this._playerId + "] Last spin played result: " + resultDesc );
		log.info( "[" + this._playerId + "] Last spin played RNG: [Checkpoint: " + this._betSelection['checkpointId'] + "]: " + this._platformInterface.getPreviousRngString() );
	}
	catch ( err ) {
		log.info( "[" + this._playerId + "] ERROR: Last spin played RNG: [Checkpoint: " + this._betSelection['checkpointId'] + "]: " + this._platformInterface.getPreviousRngString() );
		throw err;
	}

	return serverBetResults;
};


VirgoMessageHandler.prototype._getPrizesJson = function( resultJson ) {

	var that = this;
	var currencyCode = this._platformInterface.getCurrencyCode();
	var prizes = [];
	var prizesArrayTotal = new BigNumber( 0 );

	var convertWinToNum = function( n ) {
		if ( n instanceof BigNumber ) {
			return n.toN();
		} else {
			return parseInt( n, 10 );
		}
	};

	if ( resultJson['reels'] ) {

		// log reel line wins first
		var wl = resultJson['reels'].winline;
		if ( wl ) {
			for ( var i = 0; i < wl.length; ++i ) {
				var win = wl[i];
				var amount = convertWinToNum( win['win'] );
				if ( amount > 0 ) {
					prizes.push( { 'MoneyAmount': amount, 'WinType': 'REELS' } );
					prizesArrayTotal = prizesArrayTotal.plus( win['win'] );
				}
			}
		}

		// then scatters
		var scatters = resultJson['reels'].scatter;
		if ( scatters ) {
			for ( var j = 0; j < scatters.length; ++j ) {
				var scat = scatters[j];
				var amount = convertWinToNum( scat['win'] );
				if ( amount > 0 ) {
					prizes.push( { 'MoneyAmount': amount, 'WinType': 'REELS' } );
					prizesArrayTotal = prizesArrayTotal.plus( scat['win'] );
				}
			}
		}
	}

	// feature bonuses
	if ( resultJson['bonus'] ) {
		resultJson['bonus'].forEach( function( bonus ) {
			if ( bonus['additionalWin'] && bonus['additionalWin'].gte( 0 ) ) {
				prizes.push( {
					'MoneyAmount': convertWinToNum( bonus['additionalWin'] ),
					'WinType': 'REELS'
				} );
				prizesArrayTotal = prizesArrayTotal.plus( bonus['additionalWin'] );
			}
		} );
	}

	// jackpot wins
	if ( resultJson['jackpotWins'] ) {
		resultJson['jackpotWins'].forEach( function( jp ) {
			if ( jp['win'] && jp['win'].gte( 0 ) ) {
				// lookup claim token required by virgo
				var claimToken = that._platformInterface.getJackpotClaimToken( jp['id'] );
				prizes.push( {
					'MoneyAmount': convertWinToNum( jp['win'] ),
					'WinType': 'PROGRESSIVE',
					'JackpotClaimToken': claimToken
				} );
				prizesArrayTotal = prizesArrayTotal.plus( jp['win'] );
			}
		} );
	}

	// Drop-It special case: Analyse the drops and add up the total won
	if ( resultJson['drops'] ) {
		resultJson['drops'].forEach( function( drop ) {
			if ( drop['winGroups'] ) {
				drop['winGroups'].forEach( function( winGroup ) {
					prizes.push( {
						'MoneyAmount': convertWinToNum( winGroup['win'] ),
						'WinType': 'REELS'
					} );
					prizesArrayTotal = prizesArrayTotal.plus( winGroup['win'] );
				} );
			}
		} );
	}

	if ( !prizesArrayTotal.equals( resultJson['win'] ) ) {
		throw new Error( "Virgo logic error: Total amount awarded within bet results does not match overall 'win' parameter" );
	}

	return prizes;
};


VirgoMessageHandler.prototype._getJackpotStateJson = function() {

	var result = [];
	for ( var i = 0; i < this._platformInterface.getJackpotCount(); ++i ) {
		var jp = this._platformInterface.getJackpotValue( i );
		result.push( { 'id': i, 'value': jp } );
	}
	return { 'jackpotList': result };
};


VirgoMessageHandler.prototype._getCheckpointJson = function() {

	libs.server.validateBetSelectionRequest( this._betSelection, -1, this._currencyMultiplier, false );
	//libs.server.validateStateObject( this._currentState, this._betSelection[ 'checkpointId' ] );

	var resultJson = this._calculateResult();
	var prizesJson = this._getPrizesJson( resultJson );

	var reallyRawJson = libs.xmlUtils.jsonStringify( this._currentState );
	var rawJson = JSON.stringify( { 'value': reallyRawJson } );

	return {
		"Prizes": prizesJson,
		"Rtp": resultJson['RTP'],
		"MutatedState": {
			"RawJSON": rawJson
		}
	};
};


VirgoMessageHandler.prototype.earlyPayStartGame = function( completeCallback ) {

	this._platformInterface.startFirstBetOperation();

	if ( !this._currentState ) {
		this._currentState = {};
	}

	this._currentState['betSelection'] = {
		'total_stake': this._stake,
		'lines': this._lines,
		'stake': this._stake.dividedBy( this._lines ),
		'checkpointId': 0
	};
	this._betSelection = this._currentState['betSelection'];

	this._platformInterface.updateJackpotValues();

	// Enter all jackpots - this may need to be restricted on a per game basis
	// This is for centrally-triggered jackpots only
	// for ( var i=0; i<this._platformInterface.getJackpotCount(); ++i ) {
	// this._platformInterface.jackpotMarkEligible( i );
	// }

	var checkpointJson = this._getCheckpointJson();

	completeCallback( null, checkpointJson );
};


VirgoMessageHandler.prototype.earlyPayEndGame = function( completeCallback ) {

	this._platformInterface.closeBet();

	delete this._currentState['lastGame'];
	delete this._currentState['betSelection'];
	this._currentState['serverState']['game'] = {};
	this._currentState['jackpotState'] = this._getJackpotStateJson();
	var reallyRawJson = libs.xmlUtils.jsonStringify( this._currentState );
	var rawJson = JSON.stringify( { 'value': reallyRawJson } );

	completeCallback( null, {
		"MutatedState": {
			"RawJSON": rawJson
		}
	} );
};


exports.VirgoMessageHandler = VirgoMessageHandler;



