goog.provide( 'game.Freespins' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.rendering.LegacyAnimatedSprite' );
goog.require( 'libs.client.rendering.SpriteGroup' );
goog.require( 'libs.client.utils.DOM' );
goog.require( 'libs.client.utils.misc' );

goog.require( 'game.logic' );


( function() {
	"use strict";

	var defaultSettings = {
		numberOfReels: game.logic.GameData.numberOfReels,
		numberOfSymbolRows: game.logic.GameData.numberOfSymbolRows,
		useAlternateBar: false,
		fireball: {},
		heavyDropDust: {
			reelPositions: null,
			drawOffset: {
				x: 8,
				y: [
					155,
					247,
					339,
					434
				]
			}
		},
		reelBandGlow: {
			animSettings: {
				autoplay: true,
				timesToLoop: -1,
				fps: 10
			},
			drawSettings: [ {
				x: 40,
				y: 90
			}, {
				x: 160,
				y: 90
			}, {
				x: 190,
				y: 90
			}, {
				x: 305,
				y: 90
			}, {
				x: 335,
				y: 90
			}, {
				x: 450,
				y: 90
			}, {
				x: 485,
				y: 90
			}, {
				x: 600,
				y: 90
			}, {
				x: 635,
				y: 90
			}, {
				x: 750,
				y: 90
			} ]
		},
		wildParticle: {
			animSettings: {
				autoplay: false,
				timesToLoop: 0,
				fps: 15,
				drawWhenNotPlaying: false
			}
		},
		text: {
			remainingText: {
				fontSize: 20,
				maxWidth: 240
			},
			skipAnimationsText: {
				fontSize:20,
				maxWidth: 750
			}

		}
	};

	var defaultSettingsMobile = {
		progressBarWidthPercentage: 27
	};

	var defaultSettingsDesktop = {
		progressBarWidthPercentage: 33
	};

	var selectors = {
		progressContainer: $( '#freespins-progress-container' ),
		progressTitle: $( '#freespins-progress-title' ),
		progressValue: $( '#freespins-progress-value' ),
		progressBars: $( '.freespins-progress-bars' ),
		progressBarFront: $( '#freespins-progressbar-front' ),
		tooltip: $( '#freespins-tooltip-text' )
	};


	selectors.progressContainer.hide();
	selectors.tooltip.hide();

	var fireballSymsToLookAhead = 1;

	var clientInfo = libs.client.core.clientInformation;
	var isDesktopDeviceType = clientInfo.getDeviceType() === 'desktop';
	defaultSettings = libs.client.utils.misc.mergeSettings( defaultSettings, isDesktopDeviceType ? defaultSettingsDesktop : defaultSettingsMobile, true );

	var Freespins = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._isFreespinsUIActive = false;
		this._bonusInfo = null;

		this._isQuickPlay = false;

		this._fireballAnimArray = [];
		this._glowAnims = null;
		this._particleAnims = null;
		
		this._freespinsLeft = 0;

		for ( var reelIndex = 0; reelIndex < 5; ++reelIndex ) {
			this._fireballAnimArray.push( new game.FreespinsFireball( this._settings.fireball ) );
		}

		var glowFrames = libs.client.utils.misc.loadAnimationFrames( libs.loadedImages, 'FS_ReelBandGlow' );
		if ( glowFrames && glowFrames.length > 0 ) {
			this._glowAnims = new libs.client.rendering.SpriteGroup( [] );
			for ( var i = 0; i < glowFrames.length; ++i ) {
				var glowAnim = new libs.client.rendering.LegacyAnimatedSprite( glowFrames, this._settings.reelBandGlow.animSettings, this._settings.reelBandGlow.drawSettings[ i ] );
				this._glowAnims.sprites.push( glowAnim );
			}
		}

		var particleFrames = libs.client.utils.misc.loadAnimationFrames( libs.loadedImages, 'RS_WildParticle' );
		if ( particleFrames && particleFrames.length > 0 ) {
			this._particleAnims = new libs.client.rendering.SpriteGroup( [] );
			for ( var reel = 0; reel < this._settings.numberOfReels; ++reel ) {
				for ( var row = 0; row < this._settings.numberOfSymbolRows; ++row ) {
					var posX = this._settings.heavyDropDust.reelPositions[ reel ].x + this._settings.heavyDropDust.drawOffset.x;
					var posY = this._settings.heavyDropDust.reelPositions[ reel ].y + this._settings.heavyDropDust.drawOffset.y[ row ];
					var particleAnim = new libs.client.rendering.LegacyAnimatedSprite( particleFrames, this._settings.wildParticle.animSettings, {
						x: posX,
						y: posY
					} );
					this._particleAnims.sprites.push( particleAnim );
				}
			}
		}


		libs.globalEventBus.connect( 'libs.event.notify.tick', function( timeDelta ) {
			that._onNotifyTick( timeDelta );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.betBonusInfo', function( bonusInfo ) {
			that._onNotifyBetBonusInfo( bonusInfo );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.betRecoveryStarted', function() {
			that._onNotifyBetRecoveryStarted();
		} );
		libs.globalEventBus.connect( 'libs.event.notify.quickPlayState', function( enabled ) {
			that._isQuickPlay = enabled;
		} );
		libs.globalEventBus.connect( 'libs.event.notify.betResult', function( betResult ) {
			that._onNotifyBetResult( betResult );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.symbolPassedWindow', function( reelIndex, symbolId, reelStrip, reelPos, targetStopPos ) {
			that._onNotifySymbolPassedWindow( reelIndex, symbolId, reelStrip, reelPos, targetStopPos );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) {
			that._onNotifyLanguageChanged( langCode );
		} );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.freespinsUI', function( activity, inBetRecovery ) {
			that._onFreespinsUIStart( activity, inBetRecovery );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.display.freespinsUI', function() {
			that._onFreespinsUIEnd();
		} );
		libs.globalEventBus.connectActivityStart( 'libs.activity.reels.heavyDrop', function( activity, reelIndex ) {
			that._onHeavyDropStart( activity, reelIndex );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.reels.heavyDrop', function( reelIndex ) {
			that._onHeavyDropEnd( reelIndex );
		} );
		libs.globalEventBus.connectActivityStart( 'libs.activity.betState.preBet', function( activity ) {
			that._onPreBetStart( activity );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.betState.playingBet', function() {
			that._onPlayingBetEnd();
		} );
	};

	Freespins.prototype._onNotifyTick = function( timeDelta ) {
		if ( this._isFreespinsUIActive ) {
			if ( this._glowAnims ) {
				this._glowAnims.update( timeDelta );
			}
			this._fireballAnimArray.forEach( function( anim ) {
				anim.update( timeDelta );
			} );

			if ( this._particleAnims ) {
				this._particleAnims.update( timeDelta );
			}
		}
	};

	Freespins.prototype._onNotifyBetBonusInfo = function( bonusInfo ) {
		if ( bonusInfo[ 'name' ] === "freeSpinWin" || bonusInfo[ 'name' ] === "freeSpinCollect" ) {
			this._bonusInfo = bonusInfo;
		}
	};

	Freespins.prototype._onNotifyBetRecoveryStarted = function() {
		this._inBetRecovery = true;
	};

	Freespins.prototype._onNotifyBetResult = function( betResult ) {
		this._betResult = betResult;
	};

	Freespins.prototype._onNotifySymbolPassedWindow = function( reelIndex, symbolId, reelStrip, reelPos, targetStopPos ) {
		if ( this._isFreespinsUIActive && !this._isQuickPlay ) {
			// TODO: replace 10 with wild symbol solid symbol
			// look 2 symbols ahead to see if it's a block of wilds
			var wildSymbolId = 'FW';
			var examinePos = Math.floor( reelPos ) - fireballSymsToLookAhead;
			if ( examinePos < 0 ) {
				examinePos += reelStrip.length;
			}
			var examineSymbol = reelStrip[ examinePos ];
			if ( examineSymbol === wildSymbolId ) {
				// see if this is the first wild symbol part of a block of wilds
				var symAbovePos = ( examinePos - 1 ) < 0 ? examinePos - 1 + reelStrip.length : examinePos - 1;
				var symAbovePos2 = ( examinePos - 2 ) < 0 ? examinePos - 2 + reelStrip.length : examinePos - 2;
				var symBelowPos = ( examinePos + 1 ) >= reelStrip.length ? 0 : examinePos + 1;
				var symAbove = reelStrip[ symAbovePos ];
				var symAbove2 = reelStrip[ symAbovePos2 ];
				var symBelow = reelStrip[ symBelowPos ];
				if ( symAbove === wildSymbolId && symAbove2 === wildSymbolId && symBelow !== wildSymbolId ) {
					// start of a block of wilds - make sure it's not above the landing position (otherwise we just see a sole fireball drop down the screen after the
					// reels have landed)
					var abort = false;
					if ( targetStopPos !== null && isFinite( targetStopPos ) ) {
						for ( var i = 0; i <= fireballSymsToLookAhead + 1; ++i ) {
							var lookpos = examinePos + i;
							if ( lookpos < 0 ) {
								lookpos += reelStrip.length;
							} else if ( lookpos >= reelStrip.length ) {
								lookpos -= reelStrip.length;
							}
							if ( lookpos === targetStopPos ) {
								abort = true;
								break;
							}
						}
					}

					if ( !abort ) {
						// trigger fireball animation if it's not already active
						libs.globalEventBus.invoke( 'libs.event.request.playSound', [ 'HQ' ], 'SndFireballWhoosh' );
						this._fireballAnimArray[ reelIndex ].start( reelIndex );
					}
				}
			}
		}
	};

	Freespins.prototype._onNotifyLanguageChanged = function( langCode ) {
		if ( this._settings.useAlternateBar ) {
			if ( this._freespinsLeft === 1 ) {
				libs.client.utils.DOM.setMultiLineText( selectors.progressContainer, libs.getTranslation( 'X_FREESPIN_REMAINING', this._freespinsLeft ), this._settings.text.remainingText );
			} else {
				libs.client.utils.DOM.setMultiLineText( selectors.progressContainer, libs.getTranslation( 'X_FREESPINS_REMAINING', this._freespinsLeft ), this._settings.text.remainingText );
			}
		} else {
			if ( this._freespinsLeft === 1 ) {
				libs.client.utils.DOM.setText( selectors.progressTitle, libs.getTranslation( 'X_FREESPIN_REMAINING', this._freespinsLeft ), this._settings.text.remainingText );
			} else {
				libs.client.utils.DOM.setText( selectors.progressTitle, libs.getTranslation( 'X_FREESPINS_REMAINING', this._freespinsLeft ), this._settings.text.remainingText );
			}
		}
	
		libs.client.utils.DOM.setText( selectors.tooltip, libs.getTranslation( 'PRESS_SPACEBAR_TO_SKIP_WINLINE_ANIMATIONS' ), this._settings.text.skipAnimationsText);
	};

	Freespins.prototype._onFreespinsUIStart = function( activity, inBetRecovery ) {
		this._isFreespinsUIActive = true;
		if ( !this._settings.useAlternateBar ) {
			libs.client.utils.DOM.fadeIn( selectors.progressBars, 2500 );
		}
		libs.client.utils.DOM.fadeIn( selectors.progressContainer, 2500, function() {
			//HACK: Forces a redraw on buttons for Android devices
			libs.client.utils.DOM.forceDOMRedraw( $( '#buttons-container' ) );
		} );


		if ( isDesktopDeviceType ) {
			libs.client.utils.DOM.fadeIn( selectors.tooltip, 500 );
		}
	};

	Freespins.prototype._onFreespinsUIEnd = function() {
		this._isFreespinsUIActive = false;

		this._bonusInfo = null;

		// reset progress bar width
		libs.client.utils.DOM.fadeOut( selectors.progressBars, 1000 );
		libs.client.utils.DOM.fadeOut( selectors.progressContainer, 1000 );

		if ( isDesktopDeviceType ) {
			libs.client.utils.DOM.fadeOut( selectors.tooltip, 500 );
		}

		//Reset the width later on so that it doesnt show during the hiding of the UI. P
		libs.setTimeout(function(){
			selectors.progressBarFront.width( this._settings.progressBarWidthPercentage + "%" );
		},  500)
	};

	Freespins.prototype._onHeavyDropStart = function( activity, reelIndex ) {
		libs.globalEventBus.invoke( 'libs.event.request.playSound', [ 'MQ', 'HQ' ], 'SndRockSlide' );
	};

	Freespins.prototype._onHeavyDropEnd = function( reelIndex ) {
		// display dust particle animation on each wild symbol in reel that has moved
		if ( this._particleAnims ) {
			var rowCount = this._settings.numberOfSymbolRows;
			for ( var row = 0; row < rowCount; ++row ) {
				if ( this._betResult[ 'reels' ][ 'screen' ][ reelIndex ][ row ] === 'WI' ) {
					var spriteIndex = row + ( reelIndex * rowCount );
					//this._particleAnims.sprites[t].updateAnimationSettings({timesToLoop:1});
					this._particleAnims.sprites[ spriteIndex ].play();
				}
			}
		}
	};

	Freespins.prototype._onPreBetStart = function( activity ) {
		if ( this._bonusInfo ) {
			if ( this._settings.useAlternateBar ) {
				this._updateAlternateProgressBarUI( false );
			} else {
				this._updateProgressBarUI( false );
			}
		}

		this._inBetRecovery = false;
	};

	Freespins.prototype._onPlayingBetEnd = function() {
		if ( this._bonusInfo && this._bonusInfo[ 'name' ] === 'freeSpinWin' ) {
			if ( this._settings.useAlternateBar ) {
				this._updateAlternateProgressBarUI();
			} else {
				this._updateProgressBarUI( true );
			}
		}
	};

	Freespins.prototype._updateProgressBarUI = function( initialEntry ) {
		var numberofFreespinsAwarded = this._bonusInfo[ 'awarded' ] || 0;
		var freeSpinsUsed = this._bonusInfo[ 'count' ] || 0;
		if ( !initialEntry && !this._inBetRecovery ) {
			freeSpinsUsed++;
		}

		this._freespinsLeft = numberofFreespinsAwarded - freeSpinsUsed;
		var singleSpinReductionAmount = this._settings.progressBarWidthPercentage / numberofFreespinsAwarded;
		var newFreespinsRemainingWidth = Math.round( singleSpinReductionAmount * this._freespinsLeft );

		selectors.progressBarFront.animate( {
			width: ( newFreespinsRemainingWidth + "%" )
		}, 500 );

		// setting value
		if ( !freeSpinsUsed ) {
			libs.client.utils.DOM.setText( selectors.progressTitle, libs.getTranslation( 'X_FREESPINS_REMAINING', numberofFreespinsAwarded ), this._settings.text.remainingText );
		} else {
			if ( this._freespinsLeft === 1 ) {
				libs.client.utils.DOM.setText( selectors.progressTitle, libs.getTranslation( 'X_FREESPIN_REMAINING', this._freespinsLeft ), this._settings.text.remainingText );
			} else {
				libs.client.utils.DOM.setText( selectors.progressTitle, libs.getTranslation( 'X_FREESPINS_REMAINING', this._freespinsLeft ), this._settings.text.remainingText );
			}
		}
	};


	Freespins.prototype._updateAlternateProgressBarUI = function( initialEntry ) {
		var numberofFreespinsAwarded = this._bonusInfo[ 'awarded' ] || 0;
		var freeSpinsUsed = this._bonusInfo[ 'count' ] || 0;
		if ( !initialEntry && !this._inBetRecovery ) {
			freeSpinsUsed++;
		}

		this._freespinsLeft = numberofFreespinsAwarded - freeSpinsUsed;
		var singleSpinReductionAmount = this._settings.progressBarWidthPercentage / numberofFreespinsAwarded;
		var newFreespinsRemainingWidth = Math.round( singleSpinReductionAmount * this._freespinsLeft );

		selectors.progressBarFront.animate( {
			width: ( newFreespinsRemainingWidth + "%" )
		}, 500 );

		// setting value
		if ( !freeSpinsUsed ) {
			libs.client.utils.DOM.setMultiLineText( selectors.progressContainer, libs.getTranslation( 'X_FREESPINS_REMAINING', numberofFreespinsAwarded ), this._settings.text.remainingText );
		} else {
			if ( this._freespinsLeft === 1 ) {
				libs.client.utils.DOM.setMultiLineText( selectors.progressContainer, libs.getTranslation( 'X_FREESPIN_REMAINING', this._freespinsLeft ), this._settings.text.remainingText );
			} else {
				libs.client.utils.DOM.setMultiLineText( selectors.progressContainer, libs.getTranslation( 'X_FREESPINS_REMAINING', this._freespinsLeft ), this._settings.text.remainingText );
			}
		}
	};

	Freespins.prototype.draw = function( ctx ) {
		if ( this._particleAnims && !this._isQuickPlay ) {
			this._particleAnims.draw( ctx );
			
		}
		this._fireballAnimArray.forEach( function( anim ) {
				anim.draw( ctx );
			} );

	};

	Freespins.prototype.isDirty = function() {
		if ( this._glowAnims && this._glowAnims.isDirty() ) {
			return true;
		}
		if ( this._fireballAnimArray.some( function( anim ) {
			return anim && anim.isDirty();
		} ) ) {
			return true;
		}

		return this._isFreespinsUIActive;
	};

	Freespins.prototype.drawIfDirty = function( ctx ) {
		if ( this.isDirty() ) {
			this.draw( ctx );
		}
	};

	Freespins.prototype.reelsLocked = function() {
		if ( this._bonusInfo && this._bonusInfo[ 'lockedReels' ] ) {
			return {
				reels: this._bonusInfo[ 'lockedReels' ],
				set: this._bonusInfo[ 'lockedReelSet' ]
			};
		}
		return null;
	};


	game.Freespins = Freespins;

}() );