goog.provide( 'game.FreespinsCharacter' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.rendering.LegacyAnimatedSprite' );
goog.require( 'libs.client.utils.misc' );


(function() {
	"use strict";

	var defaultSettings = {
		animName: 'RS_FreeSpinsGoddess',
		singleFrame: {
			animSettings: {
				fps: 1,
				resetToFrameZeroOnEnd: false
			},
			drawSettings: {
				alpha: 0
			},
			trimmedOffsetX: 0
		},
		multiFrame: {
			animSettings: {
				fps: 15,
				resetToFrameZeroOnEnd: false
			},
			drawSettings: {
				alpha: 0
			},
			trimmedOffsetX: 0
		}
	};

	var defaultSettingsMobile = {
		singleFrame: {
			drawSettings: {
				x: 788,
				y: 145,
				width: 177,
				height: 349
			}
		},
		multiFrame: {
			drawSettings: {
				x: 788,
				y: 145,
				width: 177,
				height: 349
			}
		}
	};

	var defaultSettingsDesktop = {
		singleFrame: {
			drawSettings: {
				x: 765,
				y: 75,
				width: 192,
				height: 424
			}
		},
		multiFrame: {
			animSettings: {
				fps: 20
			},
			drawSettings: {
				x: 320,
				y: 120,
				width: 388,
				height: 596
			}
		}
	};

	var isDesktopDeviceType = libs.client.core.clientInformation.getDeviceType() === 'desktop';
	var isMobileDeviceType = libs.client.core.clientInformation.getDeviceType() === 'mobile';
	defaultSettings = libs.client.utils.misc.mergeSettings( defaultSettings, isDesktopDeviceType ? defaultSettingsDesktop : defaultSettingsMobile, true );

	var FreespinsCharacter = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._isFreespinsUIActive = false;
		this._spinsSinceLastAnimated = 0;

		this._handleCharacterAnimating = null;

		this._animation = null;
		var animationFrames = libs.client.utils.misc.loadAnimationFrames( libs.loadedImages, this._settings.animName );
		this._animVars = this._settings.singleFrame;
		if ( animationFrames.length > 1 ) {
			this._animVars = this._settings.multiFrame;
		}
		this._animation = new libs.client.rendering.LegacyAnimatedSprite( animationFrames, this._animVars.animSettings, this._animVars.drawSettings );


		this._animation.play();
		this._animation.stop();
		this._animation.onLooped( function() { that._onCharacterAnimationFinished(); } );

		libs.globalEventBus.connectActivityStart( 'game.activity.character.animating', function( activity ) { that._onCharacterAnimatingStart( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.freespinsUI', function( activity, inBetRecovery ) { that._onFreespinsUIStart( activity, inBetRecovery ); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.display.freespinsUI', function() { that._onFreespinsUIEnd(); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.betState.preBet', function( activity ) { that._onPreBetStart( activity ); } );
		libs.globalEventBus.connect( 'libs.event.notify.tick', function( timeDelta ) { that._onNotifyTick( timeDelta ); } );
		libs.globalEventBus.connect( 'game.event.request.playCharacterAnimation', function() { that._onRequestPlayCharacterAnimation(); } );
		libs.globalEventBus.connect( 'game.event.request.fadeOutCharacterAnimation', function( timeToFade ) { that._onRequestFadeOutCharacterAnimation( timeToFade ); } );
		libs.globalEventBus.connect( 'game.event.request.fadeInCharacterAnimation', function( timeToFade ) { that._onRequestFadeInCharacterAnimation( timeToFade ); } );
	};

	FreespinsCharacter.prototype._onCharacterAnimatingStart = function( activity ) {
		this._handleCharacterAnimating = activity.join();
	};

	FreespinsCharacter.prototype._onFreespinsUIStart = function( activity, inBetRecovery ) {
		this._isFreespinsUIActive = true;

		if ( inBetRecovery ) {
			libs.setTimeoutTick( function() {
				if ( isMobileDeviceType ) {
					libs.globalEventBus.invoke( 'game.event.request.playCharacterAnimation' );
				}
			}, 250 );
		}
	};

	FreespinsCharacter.prototype._onFreespinsUIEnd = function() {
		this._isFreespinsUIActive = false;
	};

	FreespinsCharacter.prototype._onPreBetStart = function( activity ) {
		if ( this._isFreespinsUIActive && isMobileDeviceType ) {
			if ( !this._animation.isPlaying ) {
				++this._spinsSinceLastAnimated;
				if ( this._spinsSinceLastAnimated > 3 ) {
					this._spinsSinceLastAnimated = 0;
					libs.globalEventBus.invoke( 'game.event.request.playCharacterAnimation' );
				}
			}
		}
	};

	FreespinsCharacter.prototype._onNotifyTick = function( timeDelta ) {
		this._animation.update( timeDelta );
	};

	FreespinsCharacter.prototype._onRequestPlayCharacterAnimation = function() {
		if ( !this._handleCharacterAnimating ) {
			libs.globalEventBus.invokeActivity( 'game.activity.character.animating' );
			this._animation.play();
			this._spinsSinceLastAnimated = 0;
		}
	};

	FreespinsCharacter.prototype._onRequestFadeOutCharacterAnimation = function( timeToFade ) {
		this._animation.fadeOut( timeToFade );
	};

	FreespinsCharacter.prototype._onRequestFadeInCharacterAnimation = function( timeToFade ) {
		this._animation.fadeIn( timeToFade );
	};

	FreespinsCharacter.prototype._onCharacterAnimationFinished = function() {
		this._handleCharacterAnimating.leave();
		this._handleCharacterAnimating = null;
	};

	FreespinsCharacter.prototype.getDrawSettings = function() {
		return this._animVars.drawSettings;
	};

	FreespinsCharacter.prototype.isDirty = function() {
		return this._animation.isDirty();
	};

	FreespinsCharacter.prototype.draw = function( ctx ) {
		this._animation.draw( ctx );
	};


	game.FreespinsCharacter = FreespinsCharacter;

}());