goog.provide( 'game.FreespinsFireball' );

goog.require( 'libs.StateMachine' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.rendering.LegacyAnimatedSprite' );


(function() {
	"use strict";

	var defaultSettings = {
		reelsFramePosition: null,
		reelPositions: null,
		freeSpinsSpinSpeed: 1.7,
		fireballOffset: {
			x: 18,
			y: 100
		},
		fireballAnim: {
			animName: 'RS_FireBall',
			animSettings: {
				autoplay: false,
				timesToLoop: -1,
				fps: 15
			},
			drawSettings: {
				x: 100,
				y: 100
			}
		},
		symbolsToLookAhead: 1,
		verticalSymbolSpacing: null
	};

	var defaultSettingsMobile = {
		clipAtPosY: 470
	};

	var defaultSettingsDesktop = {
		clipAtPosY: 480
	};

	var fireballFrames = null;
	var isDesktopDeviceType = libs.client.core.clientInformation.getDeviceType() === 'desktop';
	defaultSettings = libs.client.utils.misc.mergeSettings( defaultSettings, isDesktopDeviceType ? defaultSettingsDesktop : defaultSettingsMobile, true );

	var FreespinsFireball = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		var that = this;

		this._animState = new libs.StateMachine();
		this._isDirty = false;
		this._isAnimating = false;
		this._fireballDrawWidth = 200;
		this._fireballDrawHeight = 0;
		this._totalDistanceTravelled = 0;
		this._isQuickSpin = false;

		this._fireballAnim = null;
		if ( !fireballFrames ) {
			fireballFrames = libs.client.utils.misc.loadAnimationFrames( libs.loadedImages, this._settings.fireballAnim.animName );
		}
		if ( fireballFrames.length > 0 ) {
			this._animationHeight = fireballFrames[0].height;
			this._distanceFromWild = this._settings.symbolsToLookAhead * this._settings.verticalSymbolSpacing + this._animationHeight - this._settings.fireballOffset.y;
			this._fireballDrawHeight = this._animationHeight;
			this._fireballAnim = new libs.client.rendering.LegacyAnimatedSprite( fireballFrames, this._settings.fireballAnim.animSettings, this._settings.fireballAnim.drawSettings );
			this._fireballAnim.alpha = 0;
		}

		libs.globalEventBus.connect('libs.event.notify.quickPlayState', function(playState){
			that._onNotifyQuickPlayState(playState);
		});
	};

	FreespinsFireball.prototype._onNotifyQuickPlayState = function(playState) {
		this._isQuickSpin = playState;
	};

	FreespinsFireball.prototype.start = function( reelIndex ) {
		var that = this;

		if ( this._fireballAnim && !this._isAnimating ) {
			this._isAnimating = true;
			this._reelTop = this._settings.reelsFramePosition.y + this._settings.reelPositions[reelIndex].y;
			this._totalDistanceTravelled = 0;
			this._fireballAnim.x = this._settings.reelPositions[reelIndex].x + this._settings.fireballOffset.x;
			this._animState.stateGoto( function( td ) { that._stateAnimate( td ); } );
		}
	};

	FreespinsFireball.prototype._boundaryCalculation = function() {
		this._animationHeight = this._fireballAnim.image.height;
		this._distanceFromWild = 2 * this._settings.verticalSymbolSpacing + this._animationHeight - this._settings.fireballOffset.y;

		this._fireballAnim.y = this._reelTop - this._distanceFromWild + this._totalDistanceTravelled;
		if ( this._fireballAnim.y < this._reelTop ) {
			this._fireballDrawOffsetY = this._reelTop - this._fireballAnim.y;
			this._fireballDrawHeight = this._animationHeight - this._fireballDrawOffsetY;
			this._fireballAnim.y = this._reelTop;
		}

		// clip fireball animation if it moves outside the reels
		var bottomEdge = this._settings.clipAtPosY - this._animationHeight;
		if ( this._fireballAnim.y > bottomEdge ) {
			this._fireballDrawHeight = this._animationHeight - ( this._fireballAnim.y - bottomEdge );
			if ( this._fireballDrawHeight < 0 ) {
				this._fireballDrawHeight = 0;
				this._isAnimating = false;
			}
		}
	};

	FreespinsFireball.prototype.isDirty = function() {
		return this._isAnimating || this._isDirty || ( this._fireballAnim && this._fireballAnim.isDirty() );
	};

	FreespinsFireball.prototype.draw = function( ctx ) {
		this._isDirty = false;
		if ( !this._fireballAnim ) {
			return;
		}

		if ( this._fireballDrawHeight > 0  && !this._isQuickSpin) {
			this._fireballAnim.draw( ctx, this._fireballDrawWidth, this._fireballDrawHeight, undefined, this._fireballDrawOffsetY );
		}
	};

	FreespinsFireball.prototype._clearRect = function(surface) {
		surface.clearRect(this._reelTop,game.settings.gameHeight,  200, game.settings.gameHeight );
	};

	FreespinsFireball.prototype.update = function( timeDelta ) {
		this._animState.update( timeDelta );
		if ( this._fireballAnim ) {
			this._fireballAnim.update( timeDelta );
		}

		if ( this._isAnimating ) {
			this._totalDistanceTravelled += timeDelta * this._settings.freeSpinsSpinSpeed;
			this._boundaryCalculation();
		}
	};

	FreespinsFireball.prototype._stateAnimate = function( timeDelta ) {
		this._isDirty = true;

		if ( this._animState.phaseOnce() ) {
			this._fireballAnim.alpha = 1;
			this._fireballAnim.play();
		}

		if ( this._animState.phaseManual() ) {
			if ( !this._isAnimating ) {
				this._animState.phaseStep();
			}
		}

		if ( this._animState.phaseOnce() ) {
			this._fireballAnim.alpha = 0;
			this._animState.stateGoto( null );
		}
	};


	game.FreespinsFireball = FreespinsFireball;

}());