goog.provide( 'game.HiddenWilds' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.StateMachine' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.rendering.LegacyAnimatedSprite' );
goog.require( 'libs.client.rendering.SpriteGroup' );
goog.require( 'libs.client.utils.misc' );


(function() {
	"use strict";

	var defaultSettings = {
		animName: 'RS_Mystery',
		animationOffset: {
			x: 19,
			y: 80
		},
		animSettings: {
			autoplay: true,
			fps: 15
		},
		reelPositions: null,
		showWildSpeed: 15,
		verticalSymbolSpacing: null
	};

	var isDesktopDeviceType = libs.client.core.clientInformation.getDeviceType() === 'desktop';

	var HiddenWilds = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this.featureActive = false;
		this._bonusInfo = null;

		this._redrawReels = false;
		this._isDirty = false;

		this._covers = new libs.client.rendering.SpriteGroup( [] );
		this._coverImages = libs.client.utils.misc.loadAnimationFrames( libs.loadedImages, this._settings.animName );

		this._symbolChangeState = new libs.StateMachine();

		this._handlePostReelsLanding = null;

		libs.globalEventBus.connect( 'libs.event.notify.tick', function( timeDelta ) { that._onNotifyTick( timeDelta ); } );
		libs.globalEventBus.connect( 'libs.event.notify.betBonusInfo', function( bonusInfo ) { that._onNotifyBetBonusInfo( bonusInfo ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.reelsState.postReelsLanding', function( activity ) { that._onPostReelsLandingStart( activity ); } );
	};

	HiddenWilds.prototype._onNotifyTick = function( timeDelta ) {
		this._covers.update( timeDelta );
		this._symbolChangeState.update( timeDelta );
	};

	HiddenWilds.prototype._onNotifyBetBonusInfo = function( bonusInfo ) {
		if ( bonusInfo['name'] === "hiddenWilds" ) {
			this._bonusInfo = bonusInfo;
		}
	};

	HiddenWilds.prototype._onPostReelsLandingStart = function( activity ) {
		var that = this;

		if ( this._bonusInfo ) {
			this._handlePostReelsLanding = activity.join();

			this._symbolChangeState.stateGoto( function( td ) {that._showHidden( 0, td );} );

			this.featureActive = true;
		}
	};

	HiddenWilds.prototype.isDirty = function() {
		return this._isDirty || this._redrawReels;
	};

	HiddenWilds.prototype.draw = function( ctx ) {
		this._covers.draw( ctx );

		this._isDirty = false;
	};

	HiddenWilds.prototype.drawIfDirty = function( ctx ) {
		if ( this.isDirty() ) {
			this.draw( ctx );
		}
	};

	HiddenWilds.prototype._showHidden = function( clumpIndex, timeDelta ) {
		var i = 0;
		var that = this;
		var vibrateArray = [];
		var symbolChanges = this._bonusInfo['changes'][clumpIndex];

		if ( this._symbolChangeState.phaseTimed( 1000 ) ) {
		}

		if ( this._symbolChangeState.phaseOnce() ) {
			libs.globalEventBus.invoke( 'libs.event.request.playSound', ['HQ'], 'MysterySymbol' );

			vibrateArray.length = 0;
			for ( i = 0; i < this._settings.showWildSpeed; ++i ) {
				vibrateArray.push( 1 );
				vibrateArray.push( 100 );
			}
			libs.globalEventBus.invoke( 'libs.event.request.playHaptic', vibrateArray );

			this._startHidingSymbol( clumpIndex );

			libs.globalEventBus.invoke( 'libs.event.request.forceSymbols', symbolChanges );
			libs.globalEventBus.invoke( 'libs.event.request.canvasFullDraw', true, true );

			if ( isDesktopDeviceType ) { // set buttons z-index to the back
				$( '.button' ).css( 'z-index', 0 );
			}
		}

		if ( this._symbolChangeState.phaseManual() ) { //Wait for animation to finish.
		}

		if ( this._symbolChangeState.phaseTimed( 250 ) ) {
		}

		if ( this._symbolChangeState.phaseOnce() ) {
			if ( this._bonusInfo['changes'].length > clumpIndex + 1 ) {
				// animate next clump
				this._symbolChangeState.stateGoto( function( td ) {that._showHidden( clumpIndex + 1, td );} );
			} else {
				// otherwise finish feature
				this._finishBonus();
			}
		}
	};

	HiddenWilds.prototype._startHidingSymbol = function( clumpIndex ) {
		var that = this;

		var animation = null;
		this._redrawReels = true;

		var change = this._bonusInfo['changes'][clumpIndex];
		this._covers.sprites = [];
		change.forEach( function( pos ) {
			var location = that._symbolLocation( pos );
			that._covers.sprites.push( new libs.client.rendering.LegacyAnimatedSprite( that._coverImages, that._settings.animSettings, location ) );
		} );

		animation = this._covers.sprites[0];
		animation.onLooped( function() {
			that._symbolChangeState.phaseStep();
			animation.onLooped( null );
		} );
	};

	HiddenWilds.prototype._finishBonus = function() {
		this.featureActive = false;

		this._bonusInfo = null;
		this._covers = new libs.client.rendering.SpriteGroup( [] );
		this._redrawReels = false;
		this._symbolChangeState.stateGoto( null );

		if ( isDesktopDeviceType ) { // reset buttons z-index
			$( '.button' ).css( 'z-index', '' );
		}

		if ( this._handlePostReelsLanding ) {
			this._handlePostReelsLanding.leave();
			this._handlePostReelsLanding = null;
		}
	};

	HiddenWilds.prototype._symbolLocation = function( symbolPos ) {
		var reelPos = this._settings.reelPositions[symbolPos['reel']];
		var x = reelPos.x + this._settings.animationOffset.x;
		var y = reelPos.y + this._settings.animationOffset.y + (this._settings.verticalSymbolSpacing * symbolPos['row'] - 1);

		return { x: x, y: y };
	};


	game.HiddenWilds = HiddenWilds;

}());